@@ -9,8 +9,8 @@ called for the first time).
 """
 #============================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/business/gmClinicalRecord.py,v $
-# $Id: gmClinicalRecord.py,v 1.252 2007/10/25 12:26:05 ncq Exp $
-__version__ = "$Revision: 1.252 $"
+# $Id: gmClinicalRecord.py,v 1.270.2.1 2008/10/12 17:02:26 ncq Exp $
+__version__ = "$Revision: 1.270.2.1 $"
 __author__ = "K.Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL"
 
@@ -32,16 +32,21 @@ import sys, string, time, copy, locale
 
 
 # 3rd party
-import mx.DateTime as mxDT, psycopg2
+import mx.DateTime as mxDT, psycopg2, logging
 
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmLog, gmExceptions, gmPG2, gmSignals, gmDispatcher, gmI18N, gmCfg, gmTools
-from Gnumed.business import gmAllergy, gmEMRStructItems, gmClinNarrative
+	from Gnumed.pycommon import gmLog2, gmDateTime, gmI18N
+	gmI18N.activate_locale()
+	gmI18N.install_domain()
+	gmDateTime.init()
+from Gnumed.pycommon import gmExceptions, gmPG2, gmDispatcher, gmI18N, gmCfg, gmTools
+from Gnumed.business import gmAllergy, gmEMRStructItems, gmClinNarrative, gmPathLab
 
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lData, __version__)
+
+_log = logging.getLogger('gm.emr')
+_log.debug(__version__)
 
 _me = None
 _here = None
@@ -102,49 +107,26 @@ select fk_encounter from
 		if not self._register_interests():
 			raise gmExceptions.ConstructorError, "cannot register signal interests"
 
-		_log.Log(gmLog.lData, 'Instantiated clinical record for patient [%s].' % self.pk_patient)
+		_log.debug('Instantiated clinical record for patient [%s].' % self.pk_patient)
 	#--------------------------------------------------------
 	def __del__(self):
 		pass
 	#--------------------------------------------------------
 	def cleanup(self):
-		_log.Log(gmLog.lData, 'cleaning up after clinical record for patient [%s]' % self.pk_patient)
-
-#		sig = "%s:%s" % (gmSignals.health_issue_change_db(), self.pk_patient)
-#		self._conn_pool.Unlisten(service = 'historica', signal = sig, callback = self._health_issues_modified)
-
-#		sig = "%s:%s" % (gmSignals.episode_change_db(), self.pk_patient)
-#		self._conn_pool.Unlisten(service = 'historica', signal = sig, callback = self._db_callback_episodes_modified)
-
-#		sig = "%s:%s" % (gmSignals.vacc_mod_db(), self.pk_patient)
-#		self._conn_pool.Unlisten(service = 'historica', signal = sig, callback = self.db_callback_vaccs_modified)
+		_log.debug('cleaning up after clinical record for patient [%s]' % self.pk_patient)
 
 		return True
 	#--------------------------------------------------------
 	# messaging
 	#--------------------------------------------------------
 	def _register_interests(self):
-		# backend notifications
-#		sig = "%s:%s" % (gmSignals.vacc_mod_db(), self.pk_patient)
-#		if not self._conn_pool.Listen('historica', sig, self.db_callback_vaccs_modified):
-#			return None
-
-#		sig = "%s:%s" % (gmSignals.health_issue_change_db(), self.pk_patient)
-#		if not self._conn_pool.Listen(service = 'historica', signal = sig, callback = self._health_issues_modified):
-#			return None
-
-#		sig = "%s:%s" % (gmSignals.episode_change_db(), self.pk_patient)
-#		if not self._conn_pool.Listen(service = 'historica', signal = sig, callback = self._db_callback_episodes_modified):
-#			return None
-
-		return 1
+		return True
 	#--------------------------------------------------------
 	def db_callback_vaccs_modified(self, **kwds):
 		try:
 			self.__db_cache['vaccinations'] = {}
 		except KeyError:
 			pass
-		gmDispatcher.send(signal = gmSignals.vaccinations_updated(), sender = self.__class__.__name__)
 		return True
 	#--------------------------------------------------------
 	def _health_issues_modified(self):
@@ -156,7 +138,6 @@ select fk_encounter from
 			del self.__db_cache['problems']
 		except KeyError:
 			pass
-		gmDispatcher.send(signal = gmSignals.health_issue_updated(), sender = self.__class__.__name__)
 		return 1
 	#--------------------------------------------------------
 	def _db_callback_episodes_modified(self):
@@ -168,17 +149,16 @@ select fk_encounter from
 			del self.__db_cache['problems']
 		except KeyError:
 			pass
-		gmDispatcher.send(signal = gmSignals.episodes_modified(), sender = self.__class__.__name__)
 		return 1
 	#--------------------------------------------------------
 	def _clin_item_modified(self):
-		_log.Log(gmLog.lData, 'DB: clin_root_item modification')
+		_log.debug('DB: clin_root_item modification')
 	#--------------------------------------------------------
 	# Narrative API
 	#--------------------------------------------------------
 	def add_clin_narrative(self, note='', soap_cat='s', episode=None):
 		if note.strip() == '':
-			_log.Log(gmLog.lInfo, 'will not create empty clinical note')
+			_log.info('will not create empty clinical note')
 			return None
 		status, data = gmClinNarrative.create_clin_narrative (
 			narrative = note,
@@ -187,7 +167,7 @@ select fk_encounter from
 			encounter_id = self.__encounter['pk_encounter']
 		)
 		if not status:
-			_log.Log(gmLog.lErr, str(data))
+			_log.error(str(data))
 			return None
 		return data
 	#--------------------------------------------------------
@@ -213,6 +193,10 @@ select fk_encounter from
 			where pk_patient = %s
 			order by date, soap_rank
 		"""
+
+		#xxxxxxxxxxxxxxx
+		# support row_version in narrative for display in tree
+
 		rows, idx = gmPG2.run_ro_queries(queries=[{'cmd': cmd, 'args': [self.pk_patient]}], get_col_idx=True)
 
 		filtered_narrative = [ gmClinNarrative.cNarrative(row = {'pk_field': 'pk_narrative', 'idx': idx, 'data': row}) for row in rows ]
@@ -276,12 +260,12 @@ where
 		cmd = "select %s from clin.v_pat_items where pk_patient=%%s order by src_table, clin_when" % string.join(fields, ', ')
 		ro_conn = self._conn_pool.GetConnection('historica')
 		curs = ro_conn.cursor()
-		if not gmPG.run_query(curs, None, cmd, self.pk_patient):
-			_log.Log(gmLog.lErr, 'cannot load item links for patient [%s]' % self.pk_patient)
+		if not gmPG2.run_query(curs, None, cmd, self.pk_patient):
+			_log.error('cannot load item links for patient [%s]' % self.pk_patient)
 			curs.close()
 			return None
 		rows = curs.fetchall()
-		view_col_idx = gmPG.get_col_indices(curs)
+		view_col_idx = gmPG2.get_col_indices(curs)
 
 		# aggregate by src_table for item retrieval
 		items_by_table = {}
@@ -308,18 +292,18 @@ where
 			# we don't know anything about the columns of
 			# the source tables but, hey, this is a dump
 			if len(item_ids) == 0:
-				_log.Log(gmLog.lInfo, 'no items in table [%s] ?!?' % src_table)
+				_log.info('no items in table [%s] ?!?' % src_table)
 				continue
 			elif len(item_ids) == 1:
 				cmd = "select * from %s where pk_item=%%s order by modified_when" % src_table
-				if not gmPG.run_query(curs, None, cmd, item_ids[0]):
-					_log.Log(gmLog.lErr, 'cannot load items from table [%s]' % src_table)
+				if not gmPG2.run_query(curs, None, cmd, item_ids[0]):
+					_log.error('cannot load items from table [%s]' % src_table)
 					# skip this table
 					continue
 			elif len(item_ids) > 1:
 				cmd = "select * from %s where pk_item in %%s order by modified_when" % src_table
 				if not gmPG.run_query(curs, None, cmd, (tuple(item_ids),)):
-					_log.Log(gmLog.lErr, 'cannot load items from table [%s]' % src_table)
+					_log.error('cannot load items from table [%s]' % src_table)
 					# skip this table
 					continue
 			rows = curs.fetchall()
@@ -445,7 +429,7 @@ where
 
 		rows, view_col_idx = gmPG.run_ro_query('historica', cmd, 1, params)
 		if rows is None:
-			_log.Log(gmLog.lErr, 'cannot load item links for patient [%s]' % self.pk_patient)
+			_log.error('cannot load item links for patient [%s]' % self.pk_patient)
 			return None
 
 		# -- sort the data --
@@ -477,18 +461,18 @@ where
 			# we don't know anything about the columns of
 			# the source tables but, hey, this is a dump
 			if len(item_ids) == 0:
-				_log.Log(gmLog.lInfo, 'no items in table [%s] ?!?' % src_table)
+				_log.info('no items in table [%s] ?!?' % src_table)
 				continue
 			elif len(item_ids) == 1:
 				cmd = "select * from %s where pk_item=%%s order by modified_when" % src_table
 				if not gmPG.run_query(curs, None, cmd, item_ids[0]):
-					_log.Log(gmLog.lErr, 'cannot load items from table [%s]' % src_table)
+					_log.error('cannot load items from table [%s]' % src_table)
 					# skip this table
 					continue
 			elif len(item_ids) > 1:
 				cmd = "select * from %s where pk_item in %%s order by modified_when" % src_table
 				if not gmPG.run_query(curs, None, cmd, (tuple(item_ids),)):
-					_log.Log(gmLog.lErr, 'cannot load items from table [%s]' % src_table)
+					_log.error('cannot load items from table [%s]' % src_table)
 					# skip this table
 					continue
 			rows = curs.fetchall()
@@ -549,19 +533,75 @@ where
 	def get_patient_ID(self):
 		return self.pk_patient
 	#--------------------------------------------------------
-	def get_summary(self):
-		cmds = [
-			(u'select count(*) from clin.v_problem_list where pk_patient=%s', 'problems'),
-			(u'select count(*) from clin.encounter where fk_patient=%s', 'visits'),
-			(u'select count(*) from clin.v_pat_items where pk_patient=%s', 'items'),
-			(u'select count(*) from blobs.doc_med where fk_identity=%s', 'documents')
-		]
-		summary = {}
-		for cmd in cmds:
-			query, key = cmd
-			rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': query, 'args': [self.pk_patient]}])
-			summary[key] = rows[0][0]
-		return summary
+	def get_statistics(self):
+		union_query = u'\n	union all\n'.join ([
+			u'select count(*) from clin.v_problem_list where pk_patient = %(pat)s',
+			u'select count(*) from clin.encounter where fk_patient = %(pat)s',
+			u'select count(*) from clin.v_pat_items where pk_patient = %(pat)s',
+			u'select count(*) from blobs.doc_med where fk_identity = %(pat)s',
+			u'select count(*) from clin.v_test_results where pk_patient = %(pat)s'
+		])
+
+		rows, idx = gmPG2.run_ro_queries (
+			queries = [{'cmd': union_query, 'args': {'pat': self.pk_patient}}],
+			get_col_idx = False
+		)
+
+		stats = dict (
+			problems = rows[0][0],
+			visits = rows[1][0],
+			items = rows[2][0],
+			documents = rows[3][0],
+			results = rows[4][0]
+		)
+
+		return stats
+	#--------------------------------------------------------
+	def format_statistics(self):
+		return _("""Medical problems: %(problems)s
+Total visits: %(visits)s
+Total EMR entries: %(items)s
+Documents: %(documents)s
+Test results: %(results)s
+
+"""			) % self.get_statistics()
+	#--------------------------------------------------------
+	def format_summary(self):
+
+		stats = self.get_statistics()
+		first = self.get_first_encounter()
+		last = self.get_last_encounter()
+		probs = self.get_problems()
+
+		txt = _('EMR Statistics\n\n')
+		if len(probs) > 0:
+			txt += _(' %s known problems. Clinically relevant thereof:\n') % stats['problems']
+		else:
+			txt += _(' %s known problems\n') % stats['problems']
+		for prob in probs:
+			if not prob['clinically_relevant']:
+				continue
+			txt += u'   \u00BB%s\u00AB (%s)\n' % (
+				prob['problem'],
+				gmTools.bool2subst(prob['problem_active'], _('active'), _('inactive'))
+			)
+		txt += _(' %s encounters from %s to %s\n') % (
+			stats['visits'],
+			first['started'].strftime('%x'),
+			last['started'].strftime('%x')
+		)
+		txt += _(' %s documents\n') % stats['documents']
+		txt += _(' %s test results\n\n') % stats['results']
+
+		if self.allergic_state:
+			txt += _('Allergies and Intolerances\n\n')
+			for allg in self.get_allergies():
+				txt += u' %s: %s\n' % (
+					allg['descriptor'],
+					gmTools.coalesce(allg['reaction'], _('unknown reaction'))
+				)
+
+		return txt
 	#--------------------------------------------------------
 	# allergy API
 	#--------------------------------------------------------
@@ -594,7 +634,7 @@ where
 		if ID_list is not None:
 			filtered_allergies = filter(lambda allg: allg['pk_allergy'] in ID_list, filtered_allergies)
 			if len(filtered_allergies) == 0:
-				_log.Log(gmLog.lErr, 'no allergies of list [%s] found for patient [%s]' % (str(ID_list), self.pk_patient))
+				_log.error('no allergies of list [%s] found for patient [%s]' % (str(ID_list), self.pk_patient))
 				# better fail here contrary to what we do elsewhere
 				return None
 			else:
@@ -721,30 +761,37 @@ where
 			pk_health_issue = pk_health_issue,
 			episode_name = episode_name,
 			patient_id = self.pk_patient,
-			is_open = is_open
+			is_open = is_open,
+			encounter = self.__encounter['pk_encounter']
 		)
 		return episode
 	#--------------------------------------------------------
-	def get_most_recent_episode(issue=None):
+	def get_most_recent_episode(self, issue=None):
 		# try to find the episode with the most recently modified clinical item
+
+		issue_where = gmTools.coalesce(issue, u'', u'and pk_health_issue = %(issue)s')
+
 		cmd = u"""
 select pk
 from clin.episode
-where pk=(
+where pk = (
 	select distinct on(pk_episode) pk_episode
 	from clin.v_pat_items
 	where
-		pk_patient=%s
+		pk_patient = %%(pat)s
 			and
-		modified_when=(
+		modified_when = (
 			select max(vpi.modified_when)
 			from clin.v_pat_items vpi
-			where vpi.pk_patient=%(pat)s
+			where vpi.pk_patient = %%(pat)s
 		)
+		%s
 	-- guard against several episodes created at the same moment of time
 	limit 1
-	)"""
-		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': [self.pk_patient]}])
+	)""" % issue_where
+		rows, idx = gmPG2.run_ro_queries(queries = [
+			{'cmd': cmd, 'args': {'pat': self.pk_patient, 'issue': issue}}
+		])
 		if len(rows) != 0:
 			return gmEMRStructItems.cEpisode(aPK_obj=rows[0][0])
 
@@ -755,14 +802,17 @@ select vpe0.pk_episode
 from
 	clin.v_pat_episodes vpe0
 where
-	vpe0.pk_patient = %s
+	vpe0.pk_patient = %%(pat)s
 		and
 	vpe0.episode_modified_when = (
 		select max(vpe1.episode_modified_when)
 		from clin.v_pat_episodes vpe1
-		where vpe1.pk_episode=vpe0.pk_episode
-	)"""
-		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': [self.pk_patient]}])
+		where vpe1.pk_episode = vpe0.pk_episode
+	)
+	%s""" % issue_where
+		rows, idx = gmPG2.run_ro_queries(queries = [
+			{'cmd': cmd, 'args': {'pat': self.pk_patient, 'issue': issue}}
+		])
 		if len(rows) != 0:
 			return gmEMRStructItems.cEpisode(aPK_obj=rows[0][0])
 
@@ -781,10 +831,6 @@ where
 		episodes - Episodes' PKs to filter problems by
 		issues - Health issues' PKs to filter problems by
 		"""
-#		try:
-#			self.__db_cache['problems']
-#		except KeyError:
-
 		cmd = u"""select pk_health_issue, pk_episode from clin.v_problem_list where pk_patient=%s"""
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': [self.pk_patient]}], get_col_idx=True)
 		self.__db_cache['problems'] = []
@@ -844,7 +890,7 @@ where
 		"""Transform given problem into either episode or health issue instance.
 		"""
 		if not isinstance(problem, gmEMRStructItems.cProblem):
-			_log.Log(gmLog.lData, str(problem))
+			_log.debug(str(problem))
 			raise TypeError, 'cannot reclass [%s] instance to problem' % type(problem)
 		if problem['type'] == 'episode':
 			return self.get_episodes(id_list=[problem['pk_episode']])[0]
@@ -877,9 +923,13 @@ where
 	#------------------------------------------------------------------
 	def add_health_issue(self, issue_name=None):
 		"""Adds patient health issue."""
-		success, issue = gmEMRStructItems.create_health_issue(patient_id=self.pk_patient, description=issue_name)
+		success, issue = gmEMRStructItems.create_health_issue (
+			patient_id = self.pk_patient,
+			description = issue_name,
+			encounter = self.__encounter['pk_encounter']
+		)
 		if not success:
-			_log.Log(gmLog.lErr, 'cannot create health issue [%s] for patient [%s]' % (issue_name, self.pk_patient))
+			_log.error('cannot create health issue [%s] for patient [%s]' % (issue_name, self.pk_patient))
 			return None
 		return issue
 	#--------------------------------------------------------
@@ -907,7 +957,7 @@ where
 					 where pk_patient=%s"""
 			rows = gmPG.run_ro_query('historica', cmd, None, self.pk_patient)
 			if rows is None:
-				_log.Log(gmLog.lErr, 'cannot retrieve scheduled vaccination courses')
+				_log.error('cannot retrieve scheduled vaccination courses')
 				del self.__db_cache['vaccinations']['scheduled regimes']
 				return None
 			# Instantiate vaccination items and keep cache
@@ -920,7 +970,7 @@ where
 		if ID is not None:
 			filtered_regimes = filter(lambda regime: regime['pk_course'] == ID, filtered_regimes)
 			if len(filtered_regimes) == 0:
-				_log.Log(gmLog.lErr, 'no vaccination course [%s] found for patient [%s]' % (ID, self.pk_patient))
+				_log.error('no vaccination course [%s] found for patient [%s]' % (ID, self.pk_patient))
 				return []
 			else:
 				return filtered_regimes[0]
@@ -942,7 +992,7 @@ where
 		# filled (or fill it for nearby use)
 		vaccinations = self.get_vaccinations()
 		if vaccinations is None:
-			_log.Log(gmLog.lErr, 'cannot load vaccinated indications for patient [%s]' % self.pk_patient)
+			_log.error('cannot load vaccinated indications for patient [%s]' % self.pk_patient)
 			return (False, [[_('ERROR: cannot retrieve vaccinated indications'), _('ERROR: cannot retrieve vaccinated indications')]])
 		if len(vaccinations) == 0:
 			return (True, [[_('no vaccinations recorded'), _('no vaccinations recorded')]])
@@ -978,7 +1028,7 @@ where
  					order by indication, date"""
 			rows, idx  = gmPG.run_ro_query('historica', cmd, True, self.pk_patient)
 			if rows is None:
-				_log.Log(gmLog.lErr, 'cannot load given vaccinations for patient [%s]' % self.pk_patient)
+				_log.error('cannot load given vaccinations for patient [%s]' % self.pk_patient)
 				del self.__db_cache['vaccinations']['vaccinated']
 				return None
 			# Instantiate vaccination items
@@ -1019,7 +1069,7 @@ where
 		if ID is not None:
 			filtered_shots = filter(lambda shot: shot['pk_vaccination'] == ID, filtered_shots)
 			if len(filtered_shots) == 0:
-				_log.Log(gmLog.lErr, 'no vaccination [%s] found for patient [%s]' % (ID, self.pk_patient))
+				_log.error('no vaccination [%s] found for patient [%s]' % (ID, self.pk_patient))
 				return None
 			else:
 				return filtered_shots[0]
@@ -1052,7 +1102,7 @@ where
 			cmd = """select * from clin.v_vaccs_scheduled4pat where pk_patient=%s"""
 			rows, idx = gmPG.run_ro_query('historica', cmd, True, self.pk_patient)
 			if rows is None:
-				_log.Log(gmLog.lErr, 'cannot load scheduled vaccinations for patient [%s]' % self.pk_patient)
+				_log.error('cannot load scheduled vaccinations for patient [%s]' % self.pk_patient)
 				del self.__db_cache['vaccinations']['scheduled']
 				return None
 			# Instantiate vaccination items
@@ -1083,7 +1133,7 @@ where
 			cmd = "select indication, seq_no from clin.v_pat_missing_vaccs where pk_patient=%s"
 			rows = gmPG.run_ro_query('historica', cmd, None, self.pk_patient)
 			if rows is None:
-				_log.Log(gmLog.lErr, 'error loading (indication, seq_no) for due/overdue vaccinations for patient [%s]' % self.pk_patient)
+				_log.error('error loading (indication, seq_no) for due/overdue vaccinations for patient [%s]' % self.pk_patient)
 				return None
 			pk_args = {'pat_id': self.pk_patient}
 			if rows is not None:
@@ -1098,7 +1148,7 @@ where
 			cmd = "select indication, seq_no from clin.v_pat_missing_boosters where pk_patient=%s"
 			rows = gmPG.run_ro_query('historica', cmd, None, self.pk_patient)
 			if rows is None:
-				_log.Log(gmLog.lErr, 'error loading indications for missing boosters for patient [%s]' % self.pk_patient)
+				_log.error('error loading indications for missing boosters for patient [%s]' % self.pk_patient)
 				return None
 			pk_args = {'pat_id': self.pk_patient}
 			if rows is not None:
@@ -1170,12 +1220,12 @@ where
 		enc_rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': [self.pk_patient, min_ttl]}])
 		# none found
 		if len(enc_rows) == 0:
-			_log.Log(gmLog.lData, 'no <very recent> encounter (younger than [%s]) found' % min_ttl)
+			_log.debug('no <very recent> encounter (younger than [%s]) found' % min_ttl)
 			return False
 		# attach to existing
 		self.__encounter = gmEMRStructItems.cEncounter(aPK_obj=enc_rows[0][0])
 		self.__encounter.set_active(staff_id = _me['pk_staff'])
-		_log.Log(gmLog.lData, '"very recent" encounter [%s] found and re-activated' % enc_rows[0][0])
+		_log.debug('"very recent" encounter [%s] found and re-activated' % enc_rows[0][0])
 		return True
 	#------------------------------------------------------------------
 	def __activate_fairly_recent_encounter(self):
@@ -1208,7 +1258,7 @@ where
 		enc_rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': [self.pk_patient, max_ttl, min_ttl]}])
 		# none found
 		if len(enc_rows) == 0:
-			_log.Log(gmLog.lData, 'no <fairly recent> encounter (between [%s] and [%s] old) found' % (min_ttl, max_ttl))
+			_log.debug('no <fairly recent> encounter (between [%s] and [%s] old) found' % (min_ttl, max_ttl))
 			return False
 		encounter = gmEMRStructItems.cEncounter(aPK_obj=enc_rows[0][0])
 		# ask user whether to attach or not
@@ -1217,7 +1267,7 @@ where
 			from dem.v_basic_person where pk_identity=%s"""
 		pats, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': [self.pk_patient]}])
 		pat = pats[0]
-		pat_str = u'%s %s %s (%s), %s, #%s' % (
+		pat_str = u'%s %s %s (%s), %s  [#%s]' % (
 			gmTools.coalesce(pat[0], u'')[:5],
 			pat[1][:15],
 			pat[2][:15],
@@ -1227,24 +1277,18 @@ where
 		)
 		enc = gmI18N.get_encoding()
 		msg = _(
-			'Very recently (between "%s" and "%s" ago)\n'
-			'a consultation has been recorded for the patient:\n'
+			'%s\n'
 			'\n'
-			' %s\n'
+			"This patient's chart was worked on only recently:\n"
 			'\n'
-			'with the following details:\n'
+			' %s  %s - %s  (%s)\n'
 			'\n'
-			' date: %s\n'
-			' time: %s - %s\n'
-			' type: %s\n'
-			' request: %s\n'
-			' outcome: %s\n'
+			' Request: %s\n'
+			' Outcome: %s\n'
 			'\n'
-			'Do you want to continue this consultation\n'
+			'Do you want to continue that consultation\n'
 			'or do you want to start a new one ?\n'
 		) % (
-			max_ttl,
-			min_ttl,
 			pat_str,
 			encounter['started'].strftime('%x').decode(enc),
 			encounter['started'].strftime('%H:%M'), encounter['last_affirmed'].strftime('%H:%M'),
@@ -1254,22 +1298,30 @@ where
 		)
 		attach = False
 		try:
-			attach = _func_ask_user(msg = msg, caption = _('starting patient encounter'))
+			attach = _func_ask_user(msg = msg, caption = _('Starting patient encounter'))
 		except:
-			_log.LogException('cannot ask user for guidance')
+			_log.exception('cannot ask user for guidance, not attaching to existing encounter')
 			return False
 		if not attach:
 			return False
 		# attach to existing
 		self.__encounter = encounter
 		self.__encounter.set_active(staff_id = _me['pk_staff'])
-		_log.Log(gmLog.lData, '"fairly recent" encounter [%s] found and re-activated' % enc_rows[0][0])
+		_log.debug('"fairly recent" encounter [%s] found and re-activated' % enc_rows[0][0])
 		return True
 	#------------------------------------------------------------------
 	def start_new_encounter(self):
-		self.__encounter = gmEMRStructItems.create_encounter(fk_patient = self.pk_patient)
+		cfg_db = gmCfg.cCfgSQL()
+		enc_type = cfg_db.get2 (
+			option = u'encounter.default_type',
+			workplace = _here.active_workplace,
+			bias = u'user',
+			default = u'in surgery'
+		)
+		# FIXME: look for MRU/MCU encounter type config here
+		self.__encounter = gmEMRStructItems.create_encounter(fk_patient = self.pk_patient, enc_type = enc_type)
 		self.__encounter.set_active(staff_id = _me['pk_staff'])
-		_log.Log(gmLog.lData, 'new encounter [%s] initiated' % self.__encounter['pk_encounter'])
+		_log.debug('new encounter [%s] initiated' % self.__encounter['pk_encounter'])
 	#------------------------------------------------------------------
 	def get_active_encounter(self):
 		return self.__encounter
@@ -1320,7 +1372,7 @@ where
 
 #			rows = gmPG.run_ro_query('historica', cClinicalRecord._clin_root_item_children_union_query, None, (tuple(issues),))
 #			if rows is None:
-#				_log.Log(gmLog.lErr, 'cannot load encounters for issues [%s] (patient [%s])' % (str(issues), self.pk_patient))
+#				_log.error('cannot load encounters for issues [%s] (patient [%s])' % (str(issues), self.pk_patient))
 #			else:
 #				enc_ids = map(lambda x:x[0], rows)
 #				filtered_encounters = filter(lambda enc: enc['pk_encounter'] in enc_ids, filtered_encounters)
@@ -1399,23 +1451,123 @@ where
 	#------------------------------------------------------------------
 	def remove_empty_encounters(self):
 		# remove empty encounters
+		cfg_db = gmCfg.cCfgSQL()
+		ttl = cfg_db.get2 (
+			option = u'encounter.ttl_if_empty',
+			workplace = _here.active_workplace,
+			bias = u'user',
+			default = u'1 week'
+		)
+
 		# FIXME: this should be done async
 		cmd = u"""
-delete from clin.encounter where 
-	fk_patient = %(pat)s and
-	age(last_affirmed) > '1 week'::interval and
-	pk not in (select fk_encounter from clin.clin_root_item) and
-	pk not in (select fk_encounter from blobs.doc_med) and
-	pk not in (select fk_encounter from clin.operation)
+delete from clin.encounter where
+	clin.encounter.fk_patient = %(pat)s and
+	age(clin.encounter.last_affirmed) > %(ttl)s::interval and
+	not exists (select 1 from clin.clin_root_item where fk_encounter = clin.encounter.pk) and
+	not exists (select 1 from blobs.doc_med where fk_encounter = clin.encounter.pk) and
+	not exists (select 1 from clin.episode where fk_encounter = clin.encounter.pk) and
+	not exists (select 1 from clin.health_issue where fk_encounter = clin.encounter.pk) and
+	not exists (select 1 from clin.operation where fk_encounter = clin.encounter.pk)
 """
 		try:
-			rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': {'pat': self.pk_patient}}])
+			rows, idx = gmPG2.run_rw_queries(queries = [{
+				'cmd': cmd,
+				'args': {'pat': self.pk_patient, 'ttl': ttl}
+			}])
 		except:
-			_log.LogException('error deleting empty encounters')
+			_log.exception('error deleting empty encounters')
 
 		return True
 	#------------------------------------------------------------------
-	# lab data API
+	# measurements API
+	#------------------------------------------------------------------
+	def get_test_types_for_results(self):
+		"""Retrieve data about test types for which this patient has results."""
+		cmd = u"""
+select foo.unified_name, foo.unified_code from (
+	select distinct on (unified_name, unified_code)
+		unified_name,
+		unified_code,
+		clin_when,
+		pk_episode
+	from clin.v_test_results
+	where pk_patient = %(pat)s
+) as foo
+order by foo.clin_when desc, foo.pk_episode, foo.unified_name"""
+		args = {'pat': self.pk_patient}
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
+		return rows
+	#------------------------------------------------------------------
+	def get_test_types_details(self):
+		"""Retrieve details on tests grouped under unified names for this patient's results."""
+		cmd = u"""
+select * from clin.v_unified_test_types where pk_test_type in (
+	select distinct on (unified_name, unified_code) pk_test_type
+	from clin.v_test_results
+	where pk_patient = %(pat)s
+)
+order by unified_name"""
+		args = {'pat': self.pk_patient}
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
+		return rows, idx
+	#------------------------------------------------------------------
+	def get_dates_for_results(self):
+		"""Get the dates for which we have results."""
+		cmd = u"""
+select distinct on (cwhen) date_trunc('day', clin_when) as cwhen
+from clin.v_test_results
+where pk_patient = %(pat)s
+order by cwhen desc"""
+		args = {'pat': self.pk_patient}
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
+		return rows
+	#------------------------------------------------------------------
+	def get_test_results_by_date(self, encounter=None, episode=None):
+		cmd = u"""
+select *, xmin_test_result from clin.v_test_results
+where pk_patient = %(pat)s
+order by clin_when desc, pk_episode, unified_name"""
+		args = {'pat': self.pk_patient}
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
+
+		tests = [ gmPathLab.cTestResult(row = {'pk_field': 'pk_test_result', 'idx': idx, 'data': r}) for r in rows ]
+
+		if episode is not None:
+			tests = [ t for t in tests if t['pk_episode'] == episode ]
+		if encounter is not None:
+			tests = [ t for t in tests if t['pk_encounter'] == encounter ]
+
+		return tests
+	#------------------------------------------------------------------
+	def add_test_result(self, episode=None, type=None, intended_reviewer=None, val_num=None, val_alpha=None, unit=None):
+
+		try:
+			epi = int(episode)
+		except:
+			epi = episode['pk_episode']
+
+		try:
+			type = int(type)
+		except:
+			type = type['pk_test_type']
+
+		if intended_reviewer is None:
+			from Gnumed.business import gmPerson
+			intended_reviewer = _me['pk_staff']
+
+		tr = gmPathLab.create_test_result (
+			encounter = self.__encounter['pk_encounter'],
+			episode = epi,
+			type = type,
+			intended_reviewer = intended_reviewer,
+			val_num = val_num,
+			val_alpha = val_alpha,
+			unit = unit
+		)
+
+		return tr
+	#------------------------------------------------------------------
 	#------------------------------------------------------------------
 	def get_lab_results(self, limit=None, since=None, until=None, encounters=None, episodes=None, issues=None):
 		"""Retrieves lab result clinical items.
@@ -1485,7 +1637,7 @@ delete from clin.encounter where
 			episode_id=episode_id
 		)
 		if not status:
-			_log.Log(gmLog.lErr, str(data))
+			_log.error(str(data))
 			return None
 		return data
 #============================================================
@@ -1500,109 +1652,205 @@ def set_func_ask_user(a_func = None):
 #------------------------------------------------------------
 if __name__ == "__main__":
 
-	gmI18N.activate_locale()
-	gmI18N.install_domain()
-
+	#-----------------------------------------
 	def test_allergic_state():
 		emr = cClinicalRecord(aPKey=1)
 		state = emr.allergic_state
 		print "allergic state is:", state
 		print "setting state to -1"
 		emr.allergic_state = 'abc'
+	#-----------------------------------------
+	def test_get_test_names():
+		emr = cClinicalRecord(aPKey=12)
+		rows = emr.get_test_types_for_results()
+		print "test result names:"
+		for row in rows:
+			print row
+	#-----------------------------------------
+	def test_get_dates_for_results():
+		emr = cClinicalRecord(aPKey=12)
+		rows = emr.get_dates_for_results()
+		print "test result dates:"
+		for row in rows:
+			print row
+	#-----------------------------------------
+	def test_get_measurements():
+		emr = cClinicalRecord(aPKey=12)
+		rows, idx = emr.get_measurements_by_date()
+		print "test results:"
+		for row in rows:
+			print row
+	#-----------------------------------------
+	def test_get_test_results_by_date():
+		emr = cClinicalRecord(aPKey=12)
+		tests = emr.get_test_results_by_date()
+		print "test results:"
+		for test in tests:
+			print test
+	#-----------------------------------------
+	def test_get_test_types_details():
+		emr = cClinicalRecord(aPKey=12)
+		rows, idx = emr.get_test_types_details()
+		print "test type details:"
+		for row in rows:
+			print row
+	#-----------------------------------------
+	def test_get_statistics():
+		emr = cClinicalRecord(aPKey=12)
+		for key, item in emr.get_statistics().iteritems():
+			print key, ":", item
+	#-----------------------------------------
+	def test_add_test_result():
+		emr = cClinicalRecord(aPKey=12)
+		tr = emr.add_test_result (
+			episode = 1,
+			intended_reviewer = 1,
+			type = 1,
+			val_num = 75,
+			val_alpha = u'somewhat obese',
+			unit = u'kg'
+		)
+		print tr
+	#-----------------------------------------
+	def test_get_most_recent_episode():
+		emr = cClinicalRecord(aPKey=12)
+		print emr.get_most_recent_episode(issue = 2)
+	#-----------------------------------------
+	if (len(sys.argv) > 0) and (sys.argv[1] == 'test'):
+		#test_allergic_state()
+		#test_get_test_names()
+		#test_get_dates_for_results()
+		#test_get_measurements()
+		#test_get_test_results_by_date()
+		#test_get_test_types_details()
+		#test_get_statistics()
+		#test_add_test_result()
+		test_get_most_recent_episode()
 
-	test_allergic_state()
 	sys.exit(1)
 
-	import traceback
-	gmLog.gmDefLog.SetAllLogLevels(gmLog.lData)
-	try:
-		emr = cClinicalRecord(aPKey = 12)
+#	emr = cClinicalRecord(aPKey = 12)
 
-		# problems
-		problems = emr.get_problems()
-		print 'Problems:'
-		for a_problem in problems:
-			print '    -[problem:"%s"] [type:"%s"]' % (a_problem['problem'], a_problem['type'])
-			if a_problem['type'] == 'episode':
-				print '      .as episode: %s' % emr.problem2episode(a_problem)
+#	# Vacc regimes
+#	vacc_regimes = emr.get_scheduled_vaccination_regimes(indications = ['tetanus'])
+#	print '\nVaccination regimes: '
+#	for a_regime in vacc_regimes:
+#		pass
+#		#print a_regime
+#	vacc_regime = emr.get_scheduled_vaccination_regimes(ID=10)			
+#	#print vacc_regime
 		
-		# Vacc regimes
-		vacc_regimes = emr.get_scheduled_vaccination_regimes(indications = ['tetanus'])
-		print '\nVaccination regimes: '
-		for a_regime in vacc_regimes:
-			pass
-			#print a_regime
-		vacc_regime = emr.get_scheduled_vaccination_regimes(ID=10)			
-		#print vacc_regime
-		
-		# vaccination regimes and vaccinations for regimes
-		scheduled_vaccs = emr.get_scheduled_vaccinations(indications = ['tetanus'])
-		print 'Vaccinations for the regime:'
-		for a_scheduled_vacc in scheduled_vaccs:
-			pass
-			#print '   %s' %(a_scheduled_vacc)
+#	# vaccination regimes and vaccinations for regimes
+#	scheduled_vaccs = emr.get_scheduled_vaccinations(indications = ['tetanus'])
+#	print 'Vaccinations for the regime:'
+#	for a_scheduled_vacc in scheduled_vaccs:
+#		pass
+#		#print '   %s' %(a_scheduled_vacc)
 
-		# vaccination next shot and booster
-		vaccinations = emr.get_vaccinations()
-		for a_vacc in vaccinations:
-			print '\nVaccination %s , date: %s, booster: %s, seq no: %s' %(a_vacc['batch_no'], a_vacc['date'].strftime('%Y-%m-%d'), a_vacc['is_booster'], a_vacc['seq_no'])
+#	# vaccination next shot and booster
+#	vaccinations = emr.get_vaccinations()
+#	for a_vacc in vaccinations:
+#		print '\nVaccination %s , date: %s, booster: %s, seq no: %s' %(a_vacc['batch_no'], a_vacc['date'].strftime('%Y-%m-%d'), a_vacc['is_booster'], a_vacc['seq_no'])
 
-		# first and last encounters
-		first_encounter = emr.get_first_encounter(issue_id = 1)
-		print '\nFirst encounter: ' + str(first_encounter)
-		last_encounter = emr.get_last_encounter(episode_id = 1)
-		print '\nLast encounter: ' + str(last_encounter)
-		print ''
+#	# first and last encounters
+#	first_encounter = emr.get_first_encounter(issue_id = 1)
+#	print '\nFirst encounter: ' + str(first_encounter)
+#	last_encounter = emr.get_last_encounter(episode_id = 1)
+#	print '\nLast encounter: ' + str(last_encounter)
+#	print ''
 		
-		# lab results
-		lab = emr.get_lab_results()
-		lab_file = open('lab-data.txt', 'wb')
-		for lab_result in lab:
-			lab_file.write(str(lab_result))
-			lab_file.write('\n')
-		lab_file.close()
+#	# lab results
+#	lab = emr.get_lab_results()
+#	lab_file = open('lab-data.txt', 'wb')
+#	for lab_result in lab:
+#		lab_file.write(str(lab_result))
+#		lab_file.write('\n')
+#	lab_file.close()
 		
-		# soap notes
-		narrative = emr.get_clin_narrative(
-			since = mxDT.DateTime(2000, 2, 18),
-			until = mxDT.DateTime(2010, 9, 18),
-			issues = [1],
-			episodes = [1],
-			encounters = [1,2],
-			soap_cats = ['s', 'p']
-		)
-		print '# of clinical notes:', str(len(narrative))
-		for a_narr in narrative:
-			print '%s - %s - %s - %s - %s - %s\n' % (
-				a_narr['date'].strftime('%Y-%m-%d'),
-				str(a_narr['pk_health_issue']),
-				str(a_narr['pk_episode']),
-				str(a_narr['pk_encounter']),
-				a_narr['soap_cat'],
-				a_narr['narrative']
-			)
-			 
-	#	dump = record.get_missing_vaccinations()
-	#	f = open('vaccs.lst', 'wb')
-	#	if dump is not None:
-	#		print "=== due ==="
-	#		f.write("=== due ===\n")
-	#		for row in dump['due']:
-	#			print row
-	#			f.write(repr(row))
-	#			f.write('\n')
-	#		print "=== overdue ==="
-	#		f.write("=== overdue ===\n")
-	#		for row in dump['overdue']:
-	#			print row
-	#			f.write(repr(row))
-	#			f.write('\n')
-	#	f.close()
-	except:
-		traceback.print_exc(file=sys.stdout)
-		_log.LogException('unhandled exception', sys.exc_info(), verbose=1)
+	#dump = record.get_missing_vaccinations()
+	#f = open('vaccs.lst', 'wb')
+	#if dump is not None:
+	#	print "=== due ==="
+	#	f.write("=== due ===\n")
+	#	for row in dump['due']:
+	#		print row
+	#		f.write(repr(row))
+	#		f.write('\n')
+	#	print "=== overdue ==="
+	#	f.write("=== overdue ===\n")
+	#	for row in dump['overdue']:
+	#		print row
+	#		f.write(repr(row))
+	#		f.write('\n')
+	#f.close()
 #============================================================
 # $Log: gmClinicalRecord.py,v $
+# Revision 1.270.2.1  2008/10/12 17:02:26  ncq
+# - better wording for health issue level
+#
+# Revision 1.270  2008/08/15 15:55:41  ncq
+# - comment on row_version
+#
+# Revision 1.269  2008/07/12 15:19:16  ncq
+# - improve summary formatting
+# - fix get_most_recent_episode plus test
+#
+# Revision 1.268  2008/07/07 11:33:15  ncq
+# - improve "continue encounter ?" message
+#
+# Revision 1.267  2008/06/24 16:53:24  ncq
+# - enhance get_test_results_by_date to filter by episode/encounter
+#
+# Revision 1.266  2008/06/16 15:01:01  ncq
+# - test suite cleanup
+# - add_test_result
+#
+# Revision 1.265  2008/05/19 16:22:55  ncq
+# - format_statistics/summary()
+#
+# Revision 1.264  2008/05/19 15:43:17  ncq
+# - get_summary -> _statistics
+# - get all statistics in one database roundtrip
+# - fix test suite
+#
+# Revision 1.263  2008/04/22 21:12:08  ncq
+# - get_test_results_by_date and test
+#
+# Revision 1.262  2008/04/11 23:07:08  ncq
+# - fix remove_empty_encounters()
+#
+# Revision 1.261  2008/03/29 16:04:31  ncq
+# - retrieve test results ordered by day-truncated date
+#
+# Revision 1.260  2008/03/20 15:28:17  ncq
+# - get_test_types_details() w/ test
+#
+# Revision 1.259  2008/03/17 14:53:57  ncq
+# - improve deletion of empty encounters
+# - get_test_types_for_results()
+# - get_dates_for_results()
+# - get_measurements_by_date()
+# - improve tests
+#
+# Revision 1.258  2008/03/05 22:24:31  ncq
+# - support fk_encounter in issue and episode creation
+#
+# Revision 1.257  2008/02/25 16:58:03  ncq
+# - use new logging
+#
+# Revision 1.256  2008/01/30 13:34:49  ncq
+# - switch to std lib logging
+#
+# Revision 1.255  2008/01/16 19:43:28  ncq
+# - use backend configured default encounter type in create_encounter()
+# - configurable empty encounter removal age
+#
+# Revision 1.254  2007/12/26 18:31:53  ncq
+# - remove reference to old PG library
+#
+# Revision 1.253  2007/12/11 12:59:11  ncq
+# - cleanup and explicit signal handling
+#
 # Revision 1.252  2007/10/25 12:26:05  ncq
 # - cleanup
 #
