@@ -1,40 +1,44 @@
+# -*- coding: utf8 -*-
 """GNUmed health related business object.
 
 license: GPL
 """
 #============================================================
-__version__ = "$Revision: 1.103 $"
+__version__ = "$Revision: 1.119.2.4 $"
 __author__ = "Carlos Moro <cfmoro1976@yahoo.es>"
 
-import types, sys, string, datetime
+import types, sys, string, datetime, logging, time
 
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmLog, gmPG2, gmExceptions, gmNull, gmBusinessDBObject, gmDateTime
+from Gnumed.pycommon import gmPG2, gmExceptions, gmNull, gmBusinessDBObject, gmDateTime, gmTools, gmI18N
 from Gnumed.business import gmClinNarrative
 
 
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lInfo, __version__)
+_log = logging.getLogger('gm.emr')
+_log.info(__version__)
+
+#============================================================
+# Foundational Health Issues API
 #============================================================
 class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
-	"""Represents one health issue.
-	"""
+	"""Represents one health issue."""
+
 	_cmd_fetch_payload = u"select *, xmin from clin.health_issue where pk=%s"
 	_cmds_store_payload = [
 		u"""update clin.health_issue set
-				description=%(description)s,
-				age_noted=%(age_noted)s,
-				laterality=%(laterality)s,
+				description = %(description)s,
+				age_noted = %(age_noted)s,
+				laterality = %(laterality)s,
 				is_active = %(is_active)s,
 				clinically_relevant = %(clinically_relevant)s,
 				is_confidential = %(is_confidential)s,
 				is_cause_of_death = %(is_cause_of_death)s
 			where
-				pk=%(pk)s and
-				xmin=%(xmin)s""",
-		u"select xmin from clin.health_issue where pk=%(pk)s"
+				pk = %(pk)s and
+				xmin = %(xmin)s""",
+		u"select xmin from clin.health_issue where pk = %(pk)s"
 	]
 	_updatable_fields = [
 		'description',
@@ -72,7 +76,7 @@ class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 		"""
 		# sanity check
 		if not type(description) in [str, unicode] or description.strip() == '':
-			_log.Log(gmLog.lErr, '<description> must be a non-empty string')
+			_log.error('<description> must be a non-empty string')
 			return False
 		# update the issue description
 		old_description = self._payload[self._idx['description']]
@@ -80,7 +84,7 @@ class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 		self._is_modified = True
 		successful, data = self.save_payload()
 		if not successful:
-			_log.Log(gmLog.lErr, 'cannot rename health issue [%s] with [%s]' % (self, description))
+			_log.error('cannot rename health issue [%s] with [%s]' % (self, description))
 			self._payload[self._idx['description']] = old_description
 			return False
 		return True
@@ -120,12 +124,151 @@ class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 		if len(rows) == 0:
 			return None
 		return cEpisode(aPK_obj=rows[0][0])
+	#--------------------------------------------------------
+	def age_noted_human_readable(self):
+		if self._payload[self._idx['age_noted']] is None:
+			return u'<???>'
+
+		# seemingly silly but convinces PG to "nicely"
+		# format the interval for us
+		cmd = u"""select
+age (
+	(select dob from dem.identity where pk = %(pat)s) + %(issue_age)s,
+	(select dob from dem.identity where pk = %(pat)s)
+)::text
+|| ' (' || age (
+	(select dob from dem.identity where pk = %(pat)s) + %(issue_age)s
+)::text || ' ago)'
+"""
+		args = {
+			'pat': self._payload[self._idx['fk_patient']],
+			'issue_age': self._payload[self._idx['age_noted']]
+		}
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}])
+		return rows[0][0]
+	#--------------------------------------------------------
+	def format(self, left_margin=0, patient=None):
+
+		if patient.ID != self._payload[self._idx['fk_patient']]:
+			msg = '<patient>.ID = %s but health issue %s belongs to patient %s' % (
+				patient.ID,
+				self._payload[self._idx['pk']],
+				self._payload[self._idx['fk_patient']]
+			)
+			raise ValueError(msg)
+
+		lines = []
+
+		lines.append(_('Foundational Health Issue %s%s%s [#%s]') % (
+			u'\u00BB',
+			self._payload[self._idx['description']],
+			u'\u00AB',
+			self._payload[self._idx['pk']]
+		))
+
+		if self._payload[self._idx['is_confidential']]:
+			lines.append('')
+			lines.append(_(' ***** CONFIDENTIAL *****'))
+			lines.append('')
+
+		if self._payload[self._idx['is_cause_of_death']]:
+			lines.append('')
+			lines.append(_(' contributed to death of patient'))
+			lines.append('')
+
+		lines.append(_(' Noted at age: %s') % self.age_noted_human_readable())
+		lines.append(_(' Status: %s, %s') % (
+			gmTools.bool2subst(self._payload[self._idx['is_active']], _('active'), _('inactive')),
+			gmTools.bool2subst(self._payload[self._idx['clinically_relevant']], _('clinically relevant'), _('not clinically relevant'))
+		))
+		lines.append('')
+
+		emr = patient.get_emr()
+
+		epis = emr.get_episodes(issues = [self._payload[self._idx['pk']]])
+		if epis is None:
+			lines.append(_('Error retrieving episodes for this health issue.'))
+		elif len(epis) == 0:
+			lines.append(_('There are no episodes for this health issue.'))
+		else:
+			lines.append(_('Episodes: %s') % len(epis))
+			lines.append(_(' Most recent: %s%s%s') % (
+				u'\u00BB',
+				emr.get_most_recent_episode(issue = self._payload[self._idx['pk']])['description'],
+				u'\u00AB'
+			))
+			lines.append('')
+			for epi in epis:
+				lines.append(u' \u00BB%s\u00AB (%s)' % (
+					epi['description'],
+					gmTools.bool2subst(epi['episode_open'], _('ongoing'), _('closed'))
+				))
+
+		lines.append('')
+
+		first_encounter = emr.get_first_encounter(issue_id = self._payload[self._idx['pk']])
+		last_encounter = emr.get_last_encounter(issue_id = self._payload[self._idx['pk']])
+
+		if first_encounter is None or last_encounter is None:
+			lines.append(_('No encounters found for this health issue.'))
+		else:
+			encs = emr.get_encounters(issues = [self._payload[self._idx['pk']]])
+			lines.append(_('Encounters: %s (%s - %s):') % (
+				len(encs),
+				first_encounter['started_original_tz'].strftime('%m/%Y'),
+				last_encounter['last_affirmed_original_tz'].strftime('%m/%Y')
+			))
+			lines.append(_(' Most recent: %s - %s') % (
+				last_encounter['started_original_tz'].strftime('%Y-%m-%d %H:%M'),
+				last_encounter['last_affirmed_original_tz'].strftime('%H:%M')
+			))
+
+		left_margin = u' ' * left_margin
+		eol_w_margin = u'\n%s' % left_margin
+		return left_margin + eol_w_margin.join(lines) + u'\n'
 #============================================================
+def create_health_issue(patient_id=None, description=None, encounter=None):
+	"""Creates a new health issue for a given patient.
+
+	patient_id - given patient PK
+	description - health issue name
+	"""
+	try:
+		h_issue = cHealthIssue(patient_id=patient_id, name=description)
+		return (True, h_issue)
+	except gmExceptions.ConstructorError:
+		pass
+
+	queries = []
+	cmd = u"insert into clin.health_issue (fk_patient, description, fk_encounter) values (%s, %s, %s)"
+	queries.append({'cmd': cmd, 'args': [patient_id, description, encounter]})
+
+	cmd = u"select currval('clin.health_issue_pk_seq')"
+	queries.append({'cmd': cmd})
+
+	rows, idx = gmPG2.run_rw_queries(queries=queries, return_data=True)
+	h_issue = cHealthIssue(aPK_obj = rows[0][0])
+
+	return (True, h_issue)
+#-----------------------------------------------------------
+def delete_health_issue(health_issue=None):
+	if isinstance(health_issue, cHealthIssue):
+		pk = health_issue['pk']
+	else:
+		pk = int(health_issue)
+
+	try:
+		gmPG2.run_rw_queries(queries = [{'cmd': u'delete from clin.health_issue where pk=%(pk)s', 'args': {'pk': pk}}])
+	except gmPG2.dbapi.IntegrityError:
+		# should be parsing pgcode/and or error message
+		_log.exception('cannot delete health issue')
+		raise gmExceptions.DatabaseObjectInUseError('cannot delete health issue, it is in use')
+#------------------------------------------------------------
 # use as dummy for unassociated episodes
 def get_dummy_health_issue():
 	issue = {
 		'pk': None,
-		'description': _('free-standing episodes'),
+		'description': _('Unattributed episodes'),
 		'age_noted': None,
 		'laterality': u'na',
 		'is_active': True,
@@ -134,6 +277,9 @@ def get_dummy_health_issue():
 		'is_cause_of_death': False
 	}
 	return issue
+
+#============================================================
+# episodes API
 #============================================================
 class cEpisode(gmBusinessDBObject.cBusinessDBObject):
 	"""Represents one clinical episode.
@@ -230,7 +376,7 @@ from (
 		"""
 		# sanity check
 		if description.strip() == '':
-			_log.Log(gmLog.lErr, '<description> must be a non-empty string instance')
+			_log.error('<description> must be a non-empty string instance')
 			return False
 		# update the episode description
 		old_description = self._payload[self._idx['description']]
@@ -238,10 +384,133 @@ from (
 		self._is_modified = True
 		successful, data = self.save_payload()
 		if not successful:
-			_log.Log(gmLog.lErr, 'cannot rename episode [%s] to [%s]' % (self, description))
+			_log.error('cannot rename episode [%s] to [%s]' % (self, description))
 			self._payload[self._idx['description']] = old_description
 			return False
 		return True
+	#--------------------------------------------------------
+	def format(self, left_margin=0, patient=None):
+
+		left_margin = u' ' * left_margin
+
+		if patient.ID != self._payload[self._idx['pk_patient']]:
+			msg = '<patient>.ID = %s but episode %s belongs to patient %s' % (
+				patient.ID,
+				self._payload[self._idx['pk_episode']],
+				self._payload[self._idx['pk_patient']]
+			)
+			raise ValueError(msg)
+
+		lines = []
+
+		lines.append(_('Episode %s%s%s (%s) [#%s]\n') % (
+			u'\u00BB',
+			self._payload[self._idx['description']],
+			u'\u00AB',
+			gmTools.bool2subst(self._payload[self._idx['episode_open']], _('active'), _('finished')),
+			self._payload[self._idx['pk_episode']]
+		))
+
+		emr = patient.get_emr()
+		encs = emr.get_encounters(episodes = [self._payload[self._idx['pk_episode']]])
+		first_encounter = None
+		last_encounter = None
+		if encs is None:
+			lines.append(_('Error retrieving encounters for this episode.'))
+		elif len(encs) == 0:
+			lines.append(_('There are no encounters for this episode.'))
+		else:
+			first_encounter = emr.get_first_encounter(episode_id = self._payload[self._idx['pk_episode']])
+			last_encounter = emr.get_last_encounter(episode_id = self._payload[self._idx['pk_episode']])
+
+			lines.append(_('Last worked on: %s\n') % last_encounter['last_affirmed_original_tz'].strftime('%Y-%m-%d %H:%M'))
+
+			lines.append(_('Encounters: %s (%s - %s):') % (
+				len(encs),
+				first_encounter['started'].strftime('%m/%Y'),
+				last_encounter['last_affirmed'].strftime('%m/%Y')
+			))
+
+			for enc in encs:
+				lines.append(u' %s - %s (%s):%s' % (
+					enc['started_original_tz'].strftime('%Y-%m-%d %H:%M'),
+					enc['last_affirmed_original_tz'].strftime('%H:%M'),
+					enc['l10n_type'],
+					gmTools.coalesce(enc['assessment_of_encounter'], u'', u' \u00BB%s\u00AB')
+				))
+
+		# documents
+		doc_folder = patient.get_document_folder()
+		docs = doc_folder.get_documents (
+			episode = self._payload[self._idx['pk_episode']]
+		)
+
+		if len(docs) > 0:
+			lines.append('')
+			lines.append(_('Documents: %s') % len(docs))
+
+		for d in docs:
+			lines.append(u' %s %s:%s%s' % (
+				d['date'].strftime('%Y-%m-%d'),
+				d['l10n_type'],
+				gmTools.coalesce(d['comment'], u'', u' "%s"'),
+				gmTools.coalesce(d['ext_ref'], u'', u' (%s)')
+			))
+
+		# spell out last encounter
+		if last_encounter is not None:
+			lines.append('')
+			lines.append(_('Progress notes in most recent encounter:'))
+			lines.extend(last_encounter.format_soap (
+				episode = self._payload[self._idx['pk_episode']],
+				left_margin = left_margin,
+				soap_cats = 'soap',
+				emr = emr
+			))
+
+		eol_w_margin = u'\n%s' % left_margin
+		return left_margin + eol_w_margin.join(lines) + u'\n'
+#============================================================
+def create_episode(pk_health_issue=None, episode_name=None, patient_id=None, is_open=False, allow_dupes=False, encounter=None):
+	"""Creates a new episode for a given patient's health issue.
+
+	pk_health_issue - given health issue PK
+	episode_name - name of episode
+	"""
+	if not allow_dupes:
+		try:
+			episode = cEpisode(id_patient=patient_id, name=episode_name, health_issue=pk_health_issue)
+			if episode['episode_open'] != is_open:
+				episode['episode_open'] = is_open
+				episode.save_payload()
+			return episode
+		except gmExceptions.ConstructorError:
+			pass
+
+	queries = []
+	cmd = u"insert into clin.episode (fk_health_issue, fk_patient, description, is_open, fk_encounter) values (%s, %s, %s, %s::boolean, %s)"
+	queries.append({'cmd': cmd, 'args': [pk_health_issue, patient_id, episode_name, is_open, encounter]})
+	queries.append({'cmd': cEpisode._cmd_fetch_payload % u"currval('clin.episode_pk_seq')"})
+	rows, idx = gmPG2.run_rw_queries(queries = queries, return_data=True, get_col_idx=True)
+
+	episode = cEpisode(row={'data': rows[0], 'idx': idx, 'pk_field': 'pk_episode'})
+	return episode
+#-----------------------------------------------------------
+def delete_episode(episode=None):
+	if isinstance(episode, cEpisode):
+		pk = episode['pk_episode']
+	else:
+		pk = int(episode)
+
+	try:
+		gmPG2.run_rw_queries(queries = [{'cmd': u'delete from clin.episode where pk=%(pk)s', 'args': {'pk': pk}}])
+	except gmPG2.dbapi.IntegrityError:
+		# should be parsing pgcode/and or error message
+		_log.exception('cannot delete episode')
+		raise gmExceptions.DatabaseObjectInUseError('cannot delete episode, it is in use')
+
+#============================================================
+# encounter API
 #============================================================
 class cEncounter(gmBusinessDBObject.cBusinessDBObject):
 	"""Represents one encounter."""
@@ -277,11 +546,11 @@ class cEncounter(gmBusinessDBObject.cBusinessDBObject):
 
 		staff_id - Provider's primary key
 		"""
-		self._payload[self._idx['last_affirmed']] = datetime.datetime.now(tz = gmDateTime.cLocalTimezone())
+		self._payload[self._idx['last_affirmed']] = datetime.datetime.now(tz = gmDateTime.gmCurrentLocalTimezone)
 		success, data = self.save_payload()
 		if not success:
-			_log.Log(gmLog.lErr, 'cannot reaffirm encounter [%s]' % self.pk_obj)
-			_log.Log(gmLog.lErr, str(data))
+			_log.error('cannot reaffirm encounter [%s]' % self.pk_obj)
+			_log.error(str(data))
 			return False
 		return True
 	#--------------------------------------------------------
@@ -368,6 +637,264 @@ select exists (
 			}]
 		)
 		return rows[0][0]
+	#--------------------------------------------------------
+	def format_soap(self, episode=None, left_margin=0, soap_cats='soap', emr=None):
+
+		lines = []
+		for soap_cat in soap_cats:
+			soap_cat_narratives = emr.get_clin_narrative (
+				episodes = [episode],
+				encounters = [self._payload[self._idx['pk_encounter']]],
+				soap_cats = [soap_cat]
+			)
+			if soap_cat_narratives is None:
+				continue
+			if len(soap_cat_narratives) == 0:
+				continue
+
+			lines.append('-- %s ----------' % gmClinNarrative.soap_cat2l10n_str[soap_cat])
+			for soap_entry in soap_cat_narratives:
+				txt = gmTools.wrap (
+					text = '%s  %.8s\n%s' % (
+						soap_entry['date'].strftime('%d.%m. %H:%M'),
+						soap_entry['provider'],
+						soap_entry['narrative']
+					),
+					width = 75,
+					initial_indent = u'',
+					subsequent_indent = left_margin + u'   '
+				)
+				lines.append(txt)
+
+		return lines
+	#--------------------------------------------------------
+	def format(self, episode=None, with_soap=False, left_margin=0, patient=None):
+
+		left_margin = u' ' * left_margin
+
+		lines = []
+
+		lines.append(u'%s%s: %s - %s (@%s)%s [#%s]' % (
+			left_margin,
+			self._payload[self._idx['l10n_type']],
+			self._payload[self._idx['started_original_tz']].strftime('%Y-%m-%d %H:%M'),
+			self._payload[self._idx['last_affirmed_original_tz']].strftime('%H:%M'),
+			self._payload[self._idx['source_time_zone']],
+			gmTools.coalesce(self._payload[self._idx['assessment_of_encounter']], u'', u' \u00BB%s\u00AB'),
+			self._payload[self._idx['pk_encounter']]
+		))
+
+		lines.append(_('  your time: %s - %s  (@%s = %s%s)\n') % (
+			self._payload[self._idx['started']].strftime('%Y-%m-%d %H:%M'),
+			self._payload[self._idx['last_affirmed']].strftime('%H:%M'),
+			gmDateTime.current_local_iso_numeric_timezone_string,
+			gmTools.bool2subst (
+				gmDateTime.dst_currently_in_effect,
+				time.tzname[1].decode(gmI18N.get_encoding(), 'replace'),
+				time.tzname[0].decode(gmI18N.get_encoding(), 'replace')
+			),
+			gmTools.bool2subst(gmDateTime.dst_currently_in_effect, u' - ' + _('daylight savings time in effect'), u'')
+		))
+
+		lines.append(u'%s: %s' % (
+			_('RFE'),
+			gmTools.coalesce(self._payload[self._idx['reason_for_encounter']], u'')
+		))
+		lines.append(u'%s: %s' % (
+			_('AOE'),
+			gmTools.coalesce(self._payload[self._idx['assessment_of_encounter']], u'')
+		))
+
+		if with_soap:
+			lines.append(u'')
+
+			if patient.ID != self._payload[self._idx['pk_patient']]:
+				msg = '<patient>.ID = %s but encounter %s belongs to patient %s' % (
+					patient.ID,
+					self._payload[self._idx['pk_encounter']],
+					self._payload[self._idx['pk_patient']]
+				)
+				raise ValueError(msg)
+
+			emr = patient.get_emr()
+
+#			if episode['episode_open']:
+#				template = _('Progress notes for ongoing episode %s%s%s:')
+#			else:
+#				template = _('Progress notes for closed episode %s%s%s:')
+#			lines.append(template % (
+#				u'\u00BB',
+#				episode['description'],
+#				u'\u00AB'
+#			))
+
+			lines.extend(self.format_soap (
+				episode = episode['pk_episode'],
+				left_margin = left_margin,
+				soap_cats = 'soap',
+				emr = emr
+			))
+
+		# test results
+		tests = emr.get_test_results_by_date (
+			episode = episode['pk_episode'],
+			encounter = self._payload[self._idx['pk_encounter']]
+		)
+		if len(tests) > 0:
+			lines.append('')
+			lines.append(_('Measurements and Results:'))
+			for t in tests:
+				lines.extend(t.format())
+
+		# documents
+		doc_folder = patient.get_document_folder()
+		docs = doc_folder.get_documents (
+			episode = episode['pk_episode'],
+			encounter = self._payload[self._idx['pk_encounter']]
+		)
+
+		if len(docs) > 0:
+			lines.append('')
+			lines.append(_('Documents:'))
+
+		for d in docs:
+			lines.append(u' %s %s:%s%s' % (
+				d['date'].strftime('%Y-%m-%d'),
+				d['l10n_type'],
+				gmTools.coalesce(d['comment'], u'', u' "%s"'),
+				gmTools.coalesce(d['ext_ref'], u'', u' (%s)')
+			))
+
+		eol_w_margin = u'\n%s' % left_margin
+		return u'%s\n' % eol_w_margin.join(lines)
+
+		# special items (vaccinations, ...)
+
+#        try:
+ #               filtered_items.extend(emr.get_vaccinations(
+  #                  since=self.__constraints['since'],
+   #                 until=self.__constraints['until'],
+    #                encounters=self.__constraints['encounters'],
+     #               episodes=self.__constraints['episodes'],
+      #              issues=self.__constraints['issues']))
+       # except:
+        #        _log.error("vaccination error? outside regime")
+
+#-----------------------------------------------------------
+def create_encounter(fk_patient=None, fk_location=-1, enc_type=None):
+	"""Creates a new encounter for a patient.
+
+	fk_patient - patient PK
+	fk_location - encounter location
+	enc_type - type of encounter
+
+	FIXME: we don't deal with location yet
+	"""
+	if enc_type is None:
+		enc_type = u'in surgery'
+	# insert new encounter
+	queries = []
+	try:
+		enc_type = int(enc_type)
+		cmd = u"""
+			insert into clin.encounter (
+				fk_patient, fk_location, fk_type
+			) values (
+				%s, -1, %s
+			)"""
+	except ValueError:
+		enc_type = enc_type
+		cmd = u"""
+			insert into clin.encounter (
+				fk_patient, fk_location, fk_type
+			) values (
+				%s, -1,	coalesce((select pk from clin.encounter_type where description=%s), 0)
+			)"""
+	queries.append({'cmd': cmd, 'args': [fk_patient, enc_type]})
+	queries.append({'cmd': cEncounter._cmd_fetch_payload % u"currval('clin.encounter_pk_seq')"})
+	rows, idx = gmPG2.run_rw_queries(queries=queries, return_data=True, get_col_idx=True)
+	encounter = cEncounter(row={'data': rows[0], 'idx': idx, 'pk_field': 'pk_encounter'})
+	return encounter
+#-----------------------------------------------------------
+def update_encounter_type(description=None, l10n_description=None):
+
+	rows, idx = gmPG2.run_rw_queries(queries = [{
+		'cmd': u"select i18n.upd_tx(%(desc)s, %(l10n_desc)s)",
+		'args': {'desc': description, 'l10n_desc': l10n_description}
+	}])
+
+	return {'description': description, 'l10n_description': l10n_description}
+#-----------------------------------------------------------
+def create_encounter_type(description=None, l10n_description=None):
+	"""This will attempt to create a NEW encounter type."""
+
+	# need a system name, so derive one if necessary
+	if description is None:
+		description = l10n_description
+
+	args = {
+		'desc': description,
+		'l10n_desc': l10n_description
+	}
+
+	_log.debug('creating encounter type: %s, %s', description, l10n_description)
+
+	# does it exist already ?
+	cmd = u"select description, _(description) from clin.encounter_type where description = %(desc)s"
+	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}])
+
+	# yes
+	if len(rows) > 0:
+		# both system and l10n name are the same so all is well
+		if (rows[0][0] == description) and (rows[0][1] == l10n_description):
+			_log.info('encounter type [%s] already exists with the proper translation')
+			return {'description': description, 'l10n_description': l10n_description}
+
+		# or maybe there just wasn't a translation to
+		# the current language for this type yet ?
+		cmd = u"select exists (select 1 from i18n.translations where orig = %(desc)s and lang = i18n.get_curr_lang())"
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}])
+
+		# there was, so fail
+		if rows[0][0]:
+			_log.error('encounter type [%s] already exists but with another translation')
+			return None
+
+		# else set it
+		cmd = u"select i18n.upd_tx(%(desc)s, %(l10n_desc)s)"
+		rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
+		return {'description': description, 'l10n_description': l10n_description}
+
+	# no
+	queries = [
+		{'cmd': u"insert into clin.encounter_type (description) values (%(desc)s)", 'args': args},
+		{'cmd': u"select i18n.upd_tx(%(desc)s, %(l10n_desc)s)", 'args': args}
+	]
+	rows, idx = gmPG2.run_rw_queries(queries = queries)
+
+	return {'description': description, 'l10n_description': l10n_description}
+#-----------------------------------------------------------
+def get_encounter_types():
+	cmd = u"select _(description) as l10n_description, description from clin.encounter_type"
+	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd}])
+	return rows
+#-----------------------------------------------------------
+def get_encounter_type(description=None):
+	cmd = u"SELECT * from clin.encounter_type where description = %s"
+	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': [description]}])
+	return rows
+#-----------------------------------------------------------
+def delete_encounter_type(description=None):
+	cmd = u"delete from clin.encounter_type where description = %(desc)s"
+	args = {'desc': description}
+	try:
+		gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
+	except gmPG2.dbapi.IntegrityError, e:
+		if e.pgcode == gmPG2.sql_error_codes.FOREIGN_KEY_VIOLATION:
+			return False
+		raise
+
+	return True
 #============================================================		
 class cProblem(gmBusinessDBObject.cBusinessDBObject):
 	"""Represents one problem.
@@ -415,138 +942,14 @@ class cProblem(gmBusinessDBObject.cBusinessDBObject):
 		The problem's type attribute must be 'episode'
 		"""
 		if self._payload[self._idx['type']] != 'episode':
-			_log.Log(gmLog.lErr, 'cannot convert problem [%s] of type [%s] to episode' % (self._payload[self._idx['problem']], self._payload[self._idx['type']]))
+			_log.error('cannot convert problem [%s] of type [%s] to episode' % (self._payload[self._idx['problem']], self._payload[self._idx['type']]))
 			return None
 		return cEpisode(aPK_obj=self._payload[self._idx['pk_episode']])
 #============================================================
-# convenience functions
-#------------------------------------------------------------
-def create_health_issue(patient_id=None, description=None):
-	"""Creates a new health issue for a given patient.
-
-	patient_id - given patient PK
-	description - health issue name
-	"""
-	try:
-		h_issue = cHealthIssue(patient_id=patient_id, name=description)
-		return (True, h_issue)
-	except gmExceptions.ConstructorError:
-		pass
-
-	queries = []
-	cmd = u"insert into clin.health_issue (fk_patient, description) values (%s, %s)"
-	queries.append({'cmd': cmd, 'args': [patient_id, description]})
-
-	cmd = u"select currval('clin.health_issue_pk_seq')"
-	queries.append({'cmd': cmd})
-
-	rows, idx = gmPG2.run_rw_queries(queries=queries, return_data=True)
-	h_issue = cHealthIssue(aPK_obj = rows[0][0])
-
-	return (True, h_issue)
-#-----------------------------------------------------------
-def delete_health_issue(health_issue=None):
-	if isinstance(health_issue, cHealthIssue):
-		pk = health_issue['pk']
-	else:
-		pk = int(health_issue)
-
-	try:
-		gmPG2.run_rw_queries(queries = [{'cmd': u'delete from clin.health_issue where pk=%(pk)s', 'args': {'pk': pk}}])
-	except gmPG2.dbapi.IntegrityError:
-		# should be parsing pgcode/and or error message
-		_log.LogException('cannot delete health issue')
-		raise gmExceptions.DatabaseObjectInUseError('cannot delete health issue, it is in use')
-#-----------------------------------------------------------
-def create_episode(pk_health_issue=None, episode_name=None, patient_id=None, is_open=False, allow_dupes=False):
-	"""Creates a new episode for a given patient's health issue.
-
-	pk_health_issue - given health issue PK
-	episode_name - name of episode
-	"""
-	if not allow_dupes:
-		try:
-			episode = cEpisode(id_patient=patient_id, name=episode_name, health_issue=pk_health_issue)
-			if episode['episode_open'] != is_open:
-				episode['episode_open'] = is_open
-				episode.save_payload()
-			return episode
-		except gmExceptions.ConstructorError:
-			pass
-
-	queries = []
-	cmd = u"insert into clin.episode (fk_health_issue, fk_patient, description, is_open) values (%s, %s, %s, %s::boolean)"
-	queries.append({'cmd': cmd, 'args': [pk_health_issue, patient_id, episode_name, is_open]})
-	queries.append({'cmd': cEpisode._cmd_fetch_payload % u"currval('clin.episode_pk_seq')"})
-	rows, idx = gmPG2.run_rw_queries(queries = queries, return_data=True, get_col_idx=True)
-
-	episode = cEpisode(row={'data': rows[0], 'idx': idx, 'pk_field': 'pk_episode'})
-	return episode
-#-----------------------------------------------------------
-def delete_episode(episode=None):
-	if isinstance(episode, cEpisode):
-		pk = episode['pk_episode']
-	else:
-		pk = int(episode)
-
-	try:
-		gmPG2.run_rw_queries(queries = [{'cmd': u'delete from clin.episode where pk=%(pk)s', 'args': {'pk': pk}}])
-	except gmPG2.dbapi.IntegrityError:
-		# should be parsing pgcode/and or error message
-		_log.LogException('cannot delete episode')
-		raise gmExceptions.DatabaseObjectInUseError('cannot delete episode, it is in use')
-#-----------------------------------------------------------
-def create_encounter(fk_patient=None, fk_location=-1, enc_type=None):
-	"""Creates a new encounter for a patient.
-
-	fk_patient - patient PK
-	fk_location - encounter location
-	enc_type - type of encounter
-
-	FIXME: we don't deal with location yet
-	"""
-	# FIXME: look for MRU/MCU encounter type config here
-	if enc_type is None:
-		enc_type = u'in surgery'
-	# insert new encounter
-	queries = []
-	try:
-		enc_type = int(enc_type)
-		cmd = u"""
-			insert into clin.encounter (
-				fk_patient, fk_location, fk_type
-			) values (
-				%s, -1, %s
-			)"""
-	except ValueError:
-		enc_type = enc_type
-		cmd = u"""
-			insert into clin.encounter (
-				fk_patient, fk_location, fk_type
-			) values (
-				%s, -1,	coalesce((select pk from clin.encounter_type where description=%s), 0)
-			)"""
-	queries.append({'cmd': cmd, 'args': [fk_patient, enc_type]})
-	queries.append({'cmd': cEncounter._cmd_fetch_payload % u"currval('clin.encounter_pk_seq')"})
-	rows, idx = gmPG2.run_rw_queries(queries=queries, return_data=True, get_col_idx=True)
-	encounter = cEncounter(row={'data': rows[0], 'idx': idx, 'pk_field': 'pk_encounter'})
-	return encounter
-#-----------------------------------------------------------
-def get_encounter_types():
-	cmd = u"SELECT _(description) from clin.encounter_type"
-	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd}])
-	if len(rows) == 0:
-		_log.Log(gmLog.lWarn, 'cannot load consultation types from backend')
-		return [_('in surgery'), _('chart review')]
-	consultation_types = []
-	for cons_type in rows:
-		consultation_types.append(cons_type[0])
-	return consultation_types
-#============================================================
 # main - unit testing
 #------------------------------------------------------------
 if __name__ == '__main__':
-	_log.SetAllLogLevels(gmLog.lData)
+
 	#--------------------------------------------------------
 	# define tests
 	#--------------------------------------------------------
@@ -630,6 +1033,73 @@ if __name__ == '__main__':
 
 #============================================================
 # $Log: gmEMRStructItems.py,v $
+# Revision 1.119.2.4  2009/01/15 08:53:12  ncq
+# - cleanup
+#
+# Revision 1.119.2.3  2008/11/25 11:57:42  ncq
+# - missing gmI18N import
+#
+# Revision 1.119.2.2  2008/11/20 18:23:18  ncq
+# - fix spelling
+#
+# Revision 1.119.2.1  2008/11/11 21:00:12  ncq
+# - need to encode tzname on Windows
+#
+# Revision 1.119  2008/08/17 18:13:39  ncq
+# - add CRLF after date/time/provider in soap formatting as
+#   suggested by Rogerio on the list
+#
+# Revision 1.118  2008/07/24 13:57:51  ncq
+# - update/create/delete_encounter_type
+#
+# Revision 1.117  2008/07/22 13:53:12  ncq
+# - cleanup
+#
+# Revision 1.116  2008/07/14 13:44:38  ncq
+# - add .format to episode
+# - factor out .format_soap from .format on encounter
+# - better visualize soap sections in output as per user request
+#
+# Revision 1.115  2008/07/12 15:20:30  ncq
+# - add format to health issue
+#
+# Revision 1.114  2008/06/26 21:17:59  ncq
+# - episode formatting: include docs
+# - encounter formatting: include results and docs
+#
+# Revision 1.113  2008/06/24 16:53:58  ncq
+# - include test results in encounter formatting such
+#   as to be included in the EMR tree browser :-)
+#
+# Revision 1.112  2008/05/19 15:43:45  ncq
+# - adapt to TZ code changes
+#
+# Revision 1.111  2008/05/13 14:06:17  ncq
+# - remove superfluous \n
+# - add missing .
+#
+# Revision 1.110  2008/04/11 12:20:52  ncq
+# - format() on episode and encounter
+#
+# Revision 1.109  2008/03/05 22:24:31  ncq
+# - support fk_encounter in issue and episode creation
+#
+# Revision 1.108  2008/02/25 17:29:59  ncq
+# - logging cleanup
+#
+# Revision 1.107  2008/01/30 13:34:49  ncq
+# - switch to std lib logging
+#
+# Revision 1.106  2008/01/22 11:49:14  ncq
+# - cleanup
+# - free-standing -> Unattributed as per list
+#
+# Revision 1.105  2008/01/16 19:36:17  ncq
+# - improve get_encounter_types()
+#
+# Revision 1.104  2008/01/13 01:12:53  ncq
+# - age_noted_human_readable()
+#
 # Revision 1.103  2007/10/29 11:04:11  ncq
 # - properly handle NULL pk_health_issue in create_apisode() thereby
 #   finding dupes in free-standing episodes, too
