@@ -1,27 +1,456 @@
-"""GnuMed vaccination related business objects.
+"""GNUmed vaccination related business objects.
 
 license: GPL
 """
 #============================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/business/gmPathLab.py,v $
-# $Id: gmPathLab.py,v 1.56 2007/07/17 11:13:25 ncq Exp $
-__version__ = "$Revision: 1.56 $"
+# $Id: gmPathLab.py,v 1.64 2008/07/14 13:45:08 ncq Exp $
+__version__ = "$Revision: 1.64 $"
 __author__ = "K.Hilbert <Karsten.Hilbert@gmx.net>"
 
-import types, sys
+import types, sys, logging
 
-from Gnumed.pycommon import gmLog, gmExceptions, gmBusinessDBObject
 
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lInfo, __version__)
+if __name__ == '__main__':
+	sys.path.insert(0, '../../')
+	from Gnumed.pycommon import gmLog2, gmDateTime, gmI18N
+	gmDateTime.init()
+	gmI18N.activate_locale()
+from Gnumed.pycommon import gmExceptions, gmBusinessDBObject, gmPG2, gmTools
+
+
+_log = logging.getLogger('gm.lab')
+_log.info(__version__)
 
 # FIXME: use psyopg2 dbapi extension of named cursors - they are *server* side !
 
 #============================================================
-# FIXME: TODO
+class cTestType(gmBusinessDBObject.cBusinessDBObject):
+	"""Represents one test result type."""
+
+	_cmd_fetch_payload = u"""select *, xmin from clin.test_type where pk = %s"""
+
+	_cmds_store_payload = [
+		u"""update clin.test_type set
+				fk_test_org = %(fk_test_org)s,
+				code = %(code)s,
+				coding_system = %(coding_system)s,
+				name = %(name)s,
+				comment = %(comment)s,
+				conversion_unit = %(conversion_unit)s
+			where pk = %(pk)s""",
+		u"""select xmin from clin.test_type where pk = %(pk)"""
+	]
+
+	_updatable_fields = [
+		'fk_test_org',
+		'code',
+		'coding_system',
+		'name',
+		'comment',
+		'conversion_unit'
+	]
+	#--------------------------------------------------------
+	def __setitem__(self, attribute, value):
+
+		# find fk_test_org from name
+		if (attribute == 'fk_test_org') and (value is not None):
+			try:
+				int(value)
+			except:
+				cmd = u"select pk from clin.test_org where internal_name = %(val)s"
+				rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': {'val': value}}])
+				if len(rows) == 0:
+					raise ValueError('[%s]: no test org for [%s], cannot set <%s>' % (self.__class__.__name__, value, attribute))
+				value = rows[0][0]
+
+		gmBusinessDBObject.cBusinessDBObject.__setitem__(self, attribute, value)
+#------------------------------------------------------------
+def find_test_type(lab=None, code=None, name=None):
+
+		where_snippets = []
+
+		if lab is None:
+			where_snippets.append('fk_test_org is null')
+		else:
+			try:
+				int(lab)
+				where_snippets.append('fk_test_org = %(lab)s')
+			except (TypeError, ValueError):
+				where_snippets.append('fk_test_org = (select pk from clin.test_org where internal_name=%(lab)s)')
+
+		if (code is None) and (name is None):
+			raise ArgumentError('must have <code> and/or <name> set')
+
+		if code is not None:
+			where_snippets.append('code = %(code)s')
+
+		if name is not None:
+			where_snippets.append('name = %(name)s')
+
+		where_clause = u' and '.join(where_snippets)
+		cmd = u"select *, xmin from clin.test_type where %s" % where_clause
+		args = {'lab': lab, 'code': code, 'name': name}
+
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
+
+		if len(rows) == 0:
+			return None
+
+		tt = cTestType(row = {'pk_field': 'pk', 'data': rows[0], 'idx': idx})
+		return tt
+#------------------------------------------------------------
+def create_test_type(lab=None, code=None, unit=None, name=None):
+	"""Create or get test type."""
+
+	ttype = find_test_type(lab = lab, code = code, name = name)
+	# found ?
+	if ttype is not None:
+		return ttype
+#		if name is None:
+#			return ttype
+#		db_lname = ttype['name']
+#		# yes but ambigous
+#		if name != db_lname:
+#			_log.error('test type found for [%s:%s] but long name mismatch: expected [%s], in DB [%s]' % (lab, code, name, db_lname))
+#			me = '$RCSfile: gmPathLab.py,v $ $Revision: 1.64 $'
+#			to = 'user'
+#			prob = _('The test type already exists but the long name is different. '
+#					'The test facility may have changed the descriptive name of this test.')
+#			sol = _('Verify with facility and change the old descriptive name to the new one.')
+#			ctxt = _('lab [%s], code [%s], expected long name [%s], existing long name [%s], unit [%s]') % (lab, code, name, db_lname, unit)
+#			cat = 'lab'
+#			status, data = gmPG.add_housekeeping_todo(me, to, prob, sol, ctxt, cat)
+#			return None
+#		return ttype
+
+	_log.debug('creating test type [%s:%s:%s:%s]', lab, code, name, unit)
+
+	# not found, so create it
+	if unit is None:
+		_log.error('need <unit> to create test type: %s:%s:%s:%s' % (lab, code, name, unit))
+		raise ValueError('need <unit> to create test type')
+
+	# make query
+	cols = []
+	val_snippets = []
+	vals = {}
+
+	# lab
+	if lab is not None:
+		cols.append('fk_test_org')
+		try:
+			vals['lab'] = int(lab)
+			val_snippets.append('%(lab)s')
+		except:
+			vals['lab'] = lab
+			val_snippets.append('(select pk from clin.test_org where internal_name = %(lab)s)')
+
+	# code
+	cols.append('code')
+	val_snippets.append('%(code)s')
+	vals['code'] = code
+
+	# unit
+	cols.append('conversion_unit')
+	val_snippets.append('%(unit)s')
+	vals['unit'] = unit
+
+	# name
+	if name is not None:
+		cols.append('name')
+		val_snippets.append('%(name)s')
+		vals['name'] = name
+
+	col_clause = u', '.join(cols)
+	val_clause = u', '.join(val_snippets)
+	queries = [
+		{'cmd': u'insert into clin.test_type (%s) values (%s)' % (col_clause, val_clause), 'args': vals},
+		{'cmd': u"select *, xmin from clin.test_type where pk = currval(pg_get_serial_sequence('clin.test_type', 'pk'))"}
+	]
+	rows, idx = gmPG2.run_rw_queries(queries = queries, get_col_idx = True, return_data = True)
+	ttype = cTestType(row = {'pk_field': 'pk', 'data': rows[0], 'idx': idx})
+
+	return ttype
+#============================================================
 class cTestResult(gmBusinessDBObject.cBusinessDBObject):
-	def link_to_lab_request(self):
-		pass
+	"""Represents one test result."""
+
+	_cmd_fetch_payload = u"select * from clin.v_test_results where pk_test_result = %s"
+
+	_cmds_store_payload = [
+		u"""update clin.test_result set
+				clin_when = %(clin_when)s,
+				narrative = %(comment)s,
+				val_num = %(val_num)s,
+				val_alpha = %(val_alpha)s,
+				val_unit = %(val_unit)s,
+				val_normal_min = %(val_normal_min)s,
+				val_normal_max = %(val_normal_max)s,
+				val_normal_range = %(val_normal_range)s,
+				val_target_min = %(val_target_min)s,
+				val_target_max = %(val_target_max)s,
+				val_target_range = %(val_target_range)s,
+				abnormality_indicator = %(abnormality_indicator)s,
+				norm_ref_group = %(norm_ref_group)s,
+				note_test_org = %(note_test_org)s,
+				material = %(material)s,
+				material_detail = %(material_detail)s,
+				fk_intended_reviewer = %(pk_intended_reviewer)s,
+				fk_encounter = %(pk_encounter)s,
+				fk_episode = %(pk_episode)s,
+				fk_type = %(pk_test_type)s
+			where
+				pk = %(pk_test_result)s and
+				xmin = %(xmin_test_result)s""",
+		u"""select xmin_test_result from clin.v_test_results where pk_test_result = %(pk_test_result)s"""
+	]
+
+	_updatable_fields = [
+		'clin_when',
+		'comment',
+		'val_num',
+		'val_alpha',
+		'val_unit',
+		'val_normal_min',
+		'val_normal_max',
+		'val_normal_range',
+		'val_target_min',
+		'val_target_max',
+		'val_target_range',
+		'abnormality_indicator',
+		'norm_ref_group',
+		'note_test_org',
+		'material',
+		'material_detail',
+		'pk_intended_reviewer',
+		'pk_encounter',
+		'pk_episode',
+		'pk_test_type'
+	]
+	#--------------------------------------------------------
+	def format(self):
+
+		lines = []
+
+		lines.append(u' %s %s (%s): %s %s (%s)' % (
+			self._payload[self._idx['clin_when']].strftime('%d.%m. %H:%M'),
+			self._payload[self._idx['unified_code']],
+			self._payload[self._idx['unified_name']],
+			self._payload[self._idx['unified_val']],
+			self._payload[self._idx['val_unit']],
+			self._payload[self._idx['abnormality_indicator']]
+		))
+		if gmTools.coalesce(self._payload[self._idx['comment']], u'').strip() != u'':
+			lines.append(_('   Doc: %s') % self._payload[self._idx['comment']].strip())
+		if gmTools.coalesce(self._payload[self._idx['comment']], u'').strip() != u'':
+			lines.append(_('   MTA: %s') % self._payload[self._idx['note_test_org']].strip())
+		if self._payload[self._idx['reviewed']]:
+			lines.append(u'   %s @ %s: %s, %s' % (
+				self._payload[self._idx['last_reviewer']],
+				self._payload[self._idx['last_reviewed']].strftime('%Y-%m-%d %H:%M'),
+				gmTools.bool2subst(self._payload[self._idx['is_technically_abnormal']], _('abnormal'), _('normal')),
+				gmTools.bool2subst(self._payload[self._idx['is_clinically_relevant']], _('relevant'), _('not relevant'))
+			))
+
+		return lines
+	#--------------------------------------------------------
+	def _get_reference_ranges(self):
+
+		cmd = u"""
+select
+distinct on (norm_ref_group_str, val_unit, val_normal_min, val_normal_max, val_normal_range, val_target_min, val_target_max, val_target_range)
+	pk_patient,
+	val_unit,
+	val_normal_min, val_normal_max, val_normal_range,
+	val_target_min, val_target_max, val_target_range,
+	norm_ref_group,
+	coalesce(norm_ref_group, '') as norm_ref_group_str
+from
+	clin.v_test_results
+where
+	pk_test_type = %(pk_type)s
+"""
+		args = {'pk_type': self._payload[self._idx['pk_test_type']]}
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}])
+		return rows
+
+	def _set_reference_ranges(self, val):
+		raise AttributeError('[%s]: reference ranges not settable') % self.__class__.__name__
+
+	reference_ranges = property(_get_reference_ranges, _set_reference_ranges)
+	#--------------------------------------------------------
+	def set_review(self, technically_abnormal=None, clinically_relevant=None, comment=None, make_me_responsible=False):
+
+		if comment is not None:
+			comment = comment.strip()
+
+		if ((technically_abnormal is None) and
+			(clinically_relevant is None) and
+			(comment is None) and
+			(make_me_responsible is False)):
+				return True
+
+		# FIXME: this is not concurrency safe
+		if self._payload[self._idx['reviewed']]:
+			self.__change_existing_review (
+				technically_abnormal = technically_abnormal,
+				clinically_relevant = clinically_relevant,
+				comment = comment
+			)
+		else:
+			self.__set_new_review (
+				technically_abnormal = technically_abnormal,
+				clinically_relevant = clinically_relevant,
+				comment = comment
+			)
+
+		if make_me_responsible is True:
+			cmd = u"select pk from dem.staff where db_user = current_user"
+			rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd}])
+			self['pk_intended_reviewer'] = rows[0][0]
+			self.save_payload()
+		else:
+			self.refetch_payload()
+	#--------------------------------------------------------
+	# internal API
+	#--------------------------------------------------------
+	def __set_new_review(self, technically_abnormal=None, clinically_relevant=None, comment=None):
+		"""Add a review to a row.
+
+			- if technically abnormal is not provided/None it will be set
+			  to True if the lab's indicator has a meaningful value
+			- if clinically relevant is not provided/None it is set to
+			  whatever technically abnormal is
+		"""
+		if technically_abnormal is None:
+			technically_abnormal = False
+			if self._payload[self._idx['abnormality_indicator']] is not None:
+				if self._payload[self._idx['abnormality_indicator']].strip() != u'':
+					technically_abnormal = True
+
+		if clinically_relevant is None:
+			clinically_relevant = technically_abnormal
+
+		cmd = u"""
+insert into clin.reviewed_test_results (
+	fk_reviewed_row,
+	is_technically_abnormal,
+	clinically_relevant,
+	comment
+) values (
+	%(pk)s,
+	%(abnormal)s,
+	%(relevant)s,
+	%(cmt)s
+)"""
+		args = {
+			'pk': self._payload[self._idx['pk_test_result']],
+			'abnormal': technically_abnormal,
+			'relevant': clinically_relevant,
+			'cmt': comment
+		}
+
+		gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
+	#--------------------------------------------------------
+	def __change_existing_review(self, technically_abnormal=None, clinically_relevant=None, comment=None):
+		"""Change a review on a row.
+
+			- if technically abnormal/clinically relevant/comment are
+			  None (or empty) they are not set
+		"""
+		args = {
+			'pk_row': self._payload[self._idx['pk_test_result']],
+			'abnormal': technically_abnormal,
+			'relevant': clinically_relevant
+		}
+
+		set_parts = []
+
+		if technically_abnormal is not None:
+			set_parts.append(u'is_technically_abnormal = %(abnormal)s')
+
+		if clinically_relevant is not None:
+			set_parts.append(u'clinically_relevant= %(relevant)s')
+
+		if comment is not None:
+			set_parts.append('comment = %(cmt)s')
+			args['cmt'] = comment
+
+		cmd = u"""
+update clin.reviewed_test_results set
+	fk_reviewer = (select pk from dem.staff where db_user = current_user),
+	%s
+where
+	fk_reviewed_row = %%(pk_row)s
+""" % u',\n	'.join(set_parts)
+
+		gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
+#------------------------------------------------------------
+def delete_test_result(result=None):
+
+	try:
+		pk = int(result)
+	except (TypeError, AttributeError):
+		pk = result['pk_test_result']
+
+	cmd = u'delete from clin.test_result where pk = %(pk)s'
+	gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': {'pk': pk}}])
+#------------------------------------------------------------
+def create_test_result(encounter=None, episode=None, type=None, intended_reviewer=None, val_num=None, val_alpha=None, unit=None):
+
+	cmd1 = u"""
+insert into clin.test_result (
+	fk_encounter,
+	fk_episode,
+	fk_type,
+	fk_intended_reviewer,
+	val_num,
+	val_alpha,
+	val_unit
+) values (
+	%(enc)s,
+	%(epi)s,
+	%(type)s,
+	%(rev)s,
+	%(v_num)s,
+	%(v_alpha)s,
+	%(unit)s
+)"""
+
+	cmd2 = u"""
+select *
+from
+	clin.v_test_results
+where
+	pk_test_result = currval(pg_get_serial_sequence('clin.test_result', 'pk'))"""
+
+	args = {
+		u'enc': encounter,
+		u'epi': episode,
+		u'type': type,
+		u'rev': intended_reviewer,
+		u'v_num': val_num,
+		u'v_alpha': val_alpha,
+		u'unit': unit
+	}
+
+	rows, idx = gmPG2.run_rw_queries (
+		queries = [
+			{'cmd': cmd1, 'args': args},
+			{'cmd': cmd2}
+		],
+		return_data = True,
+		get_col_idx = True
+	)
+
+	tr = cTestResult(row = {
+		'pk_field': 'pk_test_result',
+		'idx': idx,
+		'data': rows[0]
+	})
+
+	return tr
 #============================================================
 class cLabResult(gmBusinessDBObject.cBusinessDBObject):
 	"""Represents one lab result."""
@@ -185,7 +614,7 @@ class cLabRequest(gmBusinessDBObject.cBusinessDBObject):
 				aPK_obj['req_id']
 				aPK_obj['lab']
 			except:
-				_log.LogException('[%s:??]: faulty <aPK_obj> structure: [%s]' % (self.__class__.__name__, aPK_obj), sys.exc_info())
+				_log.exception('[%s:??]: faulty <aPK_obj> structure: [%s]' % (self.__class__.__name__, aPK_obj), sys.exc_info())
 				raise gmExceptions.ConstructorError, '[%s:??]: cannot derive PK from [%s]' % (self.__class__.__name__, aPK_obj)
 			# generate query
 			where_snippets = []
@@ -217,192 +646,15 @@ class cLabRequest(gmBusinessDBObject.cBusinessDBObject):
 				vbp.pk_identity=vpi.pk_patient"""
 		pat = gmPG.run_ro_query('historica', cmd, None, self._payload[self._idx['pk_item']])
 		if pat is None:
-			_log.Log(gmLog.lErr, 'cannot get patient for lab request [%s]' % self._payload[self._idx['pk_item']])
+			_log.error('cannot get patient for lab request [%s]' % self._payload[self._idx['pk_item']])
 			return None
 		if len(pat) == 0:
-			_log.Log(gmLog.lErr, 'no patient associated with lab request [%s]' % self._payload[self._idx['pk_item']])
+			_log.error('no patient associated with lab request [%s]' % self._payload[self._idx['pk_item']])
 			return None
 		return pat[0]
 #============================================================
-class cTestType(gmBusinessDBObject.cBusinessDBObject):
-	"""Represents one test result type."""
-
-	_cmd_fetch_payload = """select *, xmin from test_type where pk=%s"""
-	_cmds_lock_rows_for_update = [
-		"""select 1 from test_type where pk=%(pk)s and xmin=%(xmin)s for update"""
-	]
-	_cmds_store_payload = [
-		"""update test_type set
-				fk_test_org=%(fk_test_org)s,
-				code=%(code)s,
-				coding_system=%(coding_system)s,
-				name=%(name)s,
-				comment=%(comment)s,
-				conversion_unit=%(conversion_unit)s
-			where pk=%(pk)s""",
-		"""select xmin from test_type where pk=%(pk)"""
-	]
-	_updatable_fields = [
-		'fk_test_org',
-		'code',
-		'coding_system',
-		'name',
-		'comment',
-		'conversion_unit'
-	]
-	#--------------------------------------------------------
-	def __init__(self, aPK_obj=None, row=None):
-		"""Instantiate lab request.
-
-		The aPK_obj can be either a dict with the keys "lab",
-		"code" and "name" or a simple primary key.
-		"""
-		# instantiate from row data ?
-		if aPK_obj is None:
-			gmBusinessDBObject.cBusinessDBObject.__init__(self, row=row)
-			return
-		pk = aPK_obj
-		# instantiate from lab/code/name ?
-		if type(aPK_obj) == types.DictType:
-			# sanity checks
-			try:
-				aPK_obj['lab']
-			except KeyError:
-				_log.LogException('[%s:??]: faulty <aPK_obj> structure: [%s]' % (self.__class__.__name__, aPK_obj), sys.exc_info())
-				raise gmExceptions.ConstructorError, '[%s:??]: cannot derive PK from [%s]' % (self.__class__.__name__, aPK_obj)
-			try:
-				aPK_obj['code']
-			except KeyError:
-				aPK_obj['code'] = None
-			try:
-				aPK_obj['name']
-			except KeyError:
-				if aPK_obj['code'] is None:
-					_log.LogException('[%s:??]: faulty <aPK_obj> structure: [%s]' % (self.__class__.__name__, aPK_obj), sys.exc_info())
-					raise gmExceptions.ConstructorError, '[%s:??]: must have <code> and/or <name>' % self.__class__.__name__
-				aPK_obj['name'] = None
-
-			# generate query
-			where_snippets = []
-			vals = {}
-			if type(aPK_obj['lab']) == types.IntType:
-				where_snippets.append('fk_test_org=%(lab)s')
-			else:
-				where_snippets.append('fk_test_org=(select pk from test_org where internal_name=%(lab)s)')
-			if aPK_obj['code'] is not None:
-				where_snippets.append('code=%(code)s')
-			if aPK_obj['name'] is not None:
-				where_snippets.append('name=%(name)s')
-			where_clause = ' and '.join(where_snippets)
-			cmd = "select pk from test_type where %s" % where_clause
-			# get pk
-			data = gmPG.run_ro_query('historica', cmd, None, aPK_obj)
-			if data is None:
-				raise gmExceptions.ConstructorError, 'error getting test type for [%s:%s:%s]' % (lab, code, name)
-			if len(data) == 0:
-				raise gmExceptions.NoSuchClinItemError, 'no test type for [%s:%s:%s]' % (lab, code, name)
-			pk = data[0][0]
-		# instantiate class
-		gmBusinessDBObject.cBusinessDBObject.__init__(self, aPK_obj=pk)
-	#--------------------------------------------------------
-	def __setitem__(self, attribute, value):
-		# find fk_test_org from name
-		if attribute == 'fk_test_org':
-			if type(value) != types.IntType:
-				cmd = "select pk from test_org where internal_name=%s"
-				data = gmPG.run_ro_query('historica', cmd, None, str(value))
-				# error
-				if data is None:
-					raise ValueError, '[%s]: error finding test org for [%s], cannot set <%s>' % (self.__class__.__name__, value, attribute)
-				if len(data) == 0:
-					raise ValueError, '[%s]: no test org for [%s], cannot set <%s>' % (self.__class__.__name__, value, attribute)
-				value = data[0][0]
-		gmBusinessDBObject.cBusinessDBObject.__setitem__(self, attribute, value)
-#============================================================
 # convenience functions
 #------------------------------------------------------------
-def create_test_type(lab=None, code=None, unit=None, name=None):
-	"""Create or get test type.
-
-		returns tuple (status, value):
-			(True, test type instance)
-			(False, error message)
-			(None, housekeeping_todo primary key)
-	"""
-	ttype = None
-	try:
-		ttype = cTestType(lab=lab, code=code)
-	except gmExceptions.NoSuchClinItemError:
-		_log.Log(gmLog.lInfo, 'will try to create test type')
-	except gmExceptions.ConstructorError, msg:
-		_log.LogException(str(msg), sys.exc_info(), verbose=0)
-		return (False, msg)
-	# found ?
-	if ttype is not None:
-		if name is None:
-			return (True, ttype)
-		db_lname = ttype['name']
-		# yes but ambigous
-		if name != db_lname:
-			_log.Log(gmLog.lErr, 'test type found for [%s:%s] but long name mismatch: expected [%s], in DB [%s]' % (lab, code, name, db_lname))
-			me = '$RCSfile: gmPathLab.py,v $ $Revision: 1.56 $'
-			to = 'user'
-			prob = _('The test type already exists but the long name is different. '
-					'The test facility may have changed the descriptive name of this test.')
-			sol = _('Verify with facility and change the old descriptive name to the new one.')
-			ctxt = _('lab [%s], code [%s], expected long name [%s], existing long name [%s], unit [%s]') % (lab, code, name, db_lname, unit)
-			cat = 'lab'
-			status, data = gmPG.add_housekeeping_todo(me, to, prob, sol, ctxt, cat)
-			return (None, data)
-		return (True, ttype)
-	# not found, so create it
-	if unit is None:
-		_log.Log(gmLog.lErr, 'need <unit> to create test type: %s:%s:%s:%s' % (lab, code, name, unit))
-		return (False, 'argument error: %s:%s:%s%s' % (lab, code, name, unit))
-	# make query
-	cols = []
-	val_snippets = []
-	vals = {}
-	# lab
-	cols.append('fk_test_org')
-	if type(lab) is types.IntType:
-		val_snippets.append('%(lab)s')
-		vals['lab'] = lab
-	else:
-		val_snippets.append('(select pk from test_org where internal_name=%(lab)s)')
-		vals['lab'] = str(lab)
-	# code
-	cols.append('code')
-	val_snippets.append('%(code)s')
-	vals['code'] = code
-	# unit
-	cols.append('conversion_unit')
-	val_snippets.append('%(unit)s')
-	vals['unit'] = unit
-	# name
-	if name is not None:
-		cols.append('name')
-		val_snippets.append('%(name)s')
-		vals['name'] = name
-	# join query parts
-	col_clause = ','.join(cols)
-	val_clause = ','.join(val_snippets)
-	queries = []
-	cmd = "insert into test_type(%s) values (%s)" % (col_clause, val_clause)
-	queries.append((cmd, [vals]))
-	cmd = "select currval('test_type_pk_seq')"
-	queries.append((cmd, []))
-	# insert new
-	result, err = gmPG.run_commit('historica', queries, True)
-	if result is None:
-		return (False, err)
-	try:
-		ttype = cTestType(aPK_obj=result[0][0])
-	except gmExceptions.ConstructorError, msg:
-		_log.LogException(str(msg), sys.exc_info(), verbose=0)
-		return (False, msg)
-	return (True, ttype)
-#------------------------------------------------------------
 def create_lab_request(lab=None, req_id=None, pat_id=None, encounter_id=None, episode_id=None):
 	"""Create or get lab request.
 
@@ -419,20 +671,20 @@ def create_lab_request(lab=None, req_id=None, pat_id=None, encounter_id=None, ep
 	try:
 		req = cLabRequest (aPK_obj)
 	except gmExceptions.NoSuchClinItemError, msg:
-		_log.Log(gmLog.lInfo, '%s: will try to create lab request' % str(msg))
+		_log.info('%s: will try to create lab request' % str(msg))
 	except gmExceptions.ConstructorError, msg:
-		_log.LogException(str(msg), sys.exc_info(), verbose=0)
+		_log.exception(str(msg), sys.exc_info(), verbose=0)
 		return (False, msg)
 	# found
 	if req is not None:
 		db_pat = req.get_patient()
 		if db_pat is None:
-			_log.Log(gmLog.lErr, 'cannot cross-check patient on lab request')
+			_log.error('cannot cross-check patient on lab request')
 			return (None, '')
 		# yes but ambigous
 		if pat_id != db_pat[0]:
-			_log.Log(gmLog.lErr, 'lab request found for [%s:%s] but patient mismatch: expected [%s], in DB [%s]' % (lab, req_id, pat_id, db_pat))
-			me = '$RCSfile: gmPathLab.py,v $ $Revision: 1.56 $'
+			_log.error('lab request found for [%s:%s] but patient mismatch: expected [%s], in DB [%s]' % (lab, req_id, pat_id, db_pat))
+			me = '$RCSfile: gmPathLab.py,v $ $Revision: 1.64 $'
 			to = 'user'
 			prob = _('The lab request already exists but belongs to a different patient.')
 			sol = _('Verify which patient this lab request really belongs to.')
@@ -457,7 +709,7 @@ def create_lab_request(lab=None, req_id=None, pat_id=None, encounter_id=None, ep
 	try:
 		req = cLabRequest(aPK_obj=result[0][0])
 	except gmExceptions.ConstructorError, msg:
-		_log.LogException(str(msg), sys.exc_info(), verbose=0)
+		_log.exception(str(msg), sys.exc_info(), verbose=0)
 		return (False, msg)
 	return (True, req)
 #------------------------------------------------------------
@@ -475,13 +727,13 @@ def create_lab_result(patient_id=None, when_field=None, when=None, test_type=Non
 	try:
 		tres = cLabResult(aPK_obj=data)
 		# exists already, so fail
-		_log.Log(gmLog.lErr, 'will not overwrite existing test result')
-		_log.Log(gmLog.lData, str(tres))
+		_log.error('will not overwrite existing test result')
+		_log.debug(str(tres))
 		return (None, tres)
 	except gmExceptions.NoSuchClinItemError:
-		_log.Log(gmLog.lData, 'test result not found - as expected, will create it')
+		_log.debug('test result not found - as expected, will create it')
 	except gmExceptions.ConstructorError, msg:
-		_log.LogException(str(msg), sys.exc_info(), verbose=0)
+		_log.exception(str(msg), sys.exc_info(), verbose=0)
 		return (False, msg)
 	if request is None:
 		return (False, _('need lab request when inserting lab result'))
@@ -502,7 +754,7 @@ def create_lab_result(patient_id=None, when_field=None, when=None, test_type=Non
 	try:
 		tres = cLabResult(aPK_obj=result[0][0])
 	except gmExceptions.ConstructorError, msg:
-		_log.LogException(str(msg), sys.exc_info(), verbose=0)
+		_log.exception(str(msg), sys.exc_info(), verbose=0)
 		return (False, msg)
 	return (True, tres)
 #------------------------------------------------------------
@@ -520,7 +772,7 @@ def get_unreviewed_results(limit=50):
 		limit %s""" % lim
 	rows = gmPG.run_ro_query('historica', cmd)
 	if rows is None:
-		_log.Log(gmLog.lErr, 'error retrieving unreviewed lab results')
+		_log.error('error retrieving unreviewed lab results')
 		return (None, _('error retrieving unreviewed lab results'))
 	if len(rows) == 0:
 		return (False, [])
@@ -536,7 +788,7 @@ def get_unreviewed_results(limit=50):
 		try:
 			results.append(cLabResult(aPK_obj=row[0]))
 		except gmExceptions.ConstructorError:
-			_log.LogException('skipping unreviewed lab result [%s]' % row[0], sys.exc_info(), verbose=0)
+			_log.exception('skipping unreviewed lab result [%s]' % row[0], sys.exc_info(), verbose=0)
 	return (more_avail, results)
 #------------------------------------------------------------
 def get_pending_requests(limit=250):
@@ -544,7 +796,7 @@ def get_pending_requests(limit=250):
 	cmd = "select pk from lab_request where is_pending is true limit %s" % lim
 	rows = gmPG.run_ro_query('historica', cmd)
 	if rows is None:
-		_log.Log(gmLog.lErr, 'error retrieving pending lab requests')
+		_log.error('error retrieving pending lab requests')
 		return (None, None)
 	if len(rows) == 0:
 		return (False, [])
@@ -561,7 +813,7 @@ def get_pending_requests(limit=250):
 		try:
 			requests.append(cLabRequest(aPK_obj=row[0]))
 		except gmExceptions.ConstructorError:
-			_log.LogException('skipping pending lab request [%s]' % row[0], sys.exc_info(), verbose=0)
+			_log.exception('skipping pending lab request [%s]' % row[0], sys.exc_info(), verbose=0)
 	return (too_many, requests)
 #------------------------------------------------------------
 def get_next_request_ID(lab=None, incrementor_func=None):
@@ -587,7 +839,7 @@ def get_next_request_ID(lab=None, incrementor_func=None):
 		)""" % lab_snippet
 	rows = gmPG.run_ro_query('historica', cmd, None, lab)
 	if rows is None:
-		_log.Log(gmLog.lWarn, 'error getting most recently used request ID for lab [%s]' % lab)
+		_log.warning('error getting most recently used request ID for lab [%s]' % lab)
 		return ''
 	if len(rows) == 0:
 		return ''
@@ -597,7 +849,7 @@ def get_next_request_ID(lab=None, incrementor_func=None):
 		try:
 			next = incrementor_func(most_recent)
 		except TypeError:
-			_log.Log(gmLog.lErr, 'cannot call incrementor function [%s]' % str(incrementor_func))
+			_log.error('cannot call incrementor function [%s]' % str(incrementor_func))
 			return most_recent
 		return next
 	# try to be smart ourselves
@@ -616,7 +868,30 @@ def get_next_request_ID(lab=None, incrementor_func=None):
 if __name__ == '__main__':
 	import time
 
+	#------------------------------------------
+	def test_create_test_result():
+		tr = create_test_result (
+			encounter = 1,
+			episode = 1,
+			type = 1,
+			intended_reviewer = 1,
+			val_num = '12',
+			val_alpha=None,
+			unit = 'mg/dl'
+		)
+		print tr
+		return tr
+	#------------------------------------------
+	def test_delete_test_result():
+		tr = test_create_test_result()
+		delete_test_result(tr)
+	#------------------------------------------
 	def test_result():
+		r = cTestResult(aPK_obj=1)
+		print r
+		print r.reference_ranges
+	#------------------------------------------
+	def test_lab_result():
 		print "test_result()"
 #		lab_result = cLabResult(aPK_obj=4)
 		data = {
@@ -637,7 +912,7 @@ if __name__ == '__main__':
 		print time.time()
 		print lab_result.get_patient()
 		print time.time()
-	#--------------------------------------------------------
+	#------------------------------------------
 	def test_request():
 		print "test_request()"
 		try:
@@ -660,11 +935,6 @@ if __name__ == '__main__':
 		print lab_req.get_patient()
 		print time.time()
 	#--------------------------------------------------------
-	def test_create_result():
-#		data = create_test_result(patient_id=12, when_field='lab_rxd_when', when='2000-09-17 15:40', test_type=6, val_num=9.5, unit='Gpt/l')
-		print data[0]
-		print data[1]
-	#--------------------------------------------------------
 	def test_unreviewed():
 		data = get_unreviewed_results()
 		for result in data:
@@ -675,18 +945,55 @@ if __name__ == '__main__':
 		for result in data:
 			print result
 	#--------------------------------------------------------
-	_log.SetAllLogLevels(gmLog.lData)
-	from Gnumed.pycommon import gmPG
+	def test_create_test_type():
+		print create_test_type (
+			lab = None,
+			code = u'tBZ2',
+			unit = u'mg%',
+			name = 'BZ (test 2)'
+		)
+	#--------------------------------------------------------
+	if (len(sys.argv) > 1) and (sys.argv[1] == 'test'):
 
-	test_result()
-	test_request()
-#	test_create_result()
-	test_unreviewed()
-	test_pending()
+		#test_result()
+		#test_create_test_result()
+		test_delete_test_result()
+		#test_create_test_type()
+		#test_lab_result()
+		#test_request()
+		#test_create_result()
+		#test_unreviewed()
+		#test_pending()
 
-	gmPG.ConnectionPool().StopListeners()
 #============================================================
 # $Log: gmPathLab.py,v $
+# Revision 1.64  2008/07/14 13:45:08  ncq
+# - add .format to test results
+#
+# Revision 1.63  2008/06/24 13:54:47  ncq
+# - delete_test_result and test
+#
+# Revision 1.62  2008/06/23 21:49:19  ncq
+# - pimp cTestType, find/create_test_type
+# - some tests
+#
+# Revision 1.61  2008/06/19 15:24:47  ncq
+# - fix updating cTestResult
+#
+# Revision 1.60  2008/06/16 15:01:53  ncq
+# - create_test_result
+# - test suite cleanup
+# - reference_ranges property on cTestResult
+#
+# Revision 1.59  2008/04/22 21:15:16  ncq
+# - cTestResult
+#
+# Revision 1.58  2008/02/25 17:31:41  ncq
+# - logging cleanup
+#
+# Revision 1.57  2008/01/30 13:34:50  ncq
+# - switch to std lib logging
+#
 # Revision 1.56  2007/07/17 11:13:25  ncq
 # - no more gmClinItem
 #
