@@ -1,9 +1,9 @@
 """GNUmed Surgery related middleware."""
 #============================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/business/gmSurgery.py,v $
-# $Id: gmSurgery.py,v 1.7 2007/10/23 21:20:24 ncq Exp $
+# $Id: gmSurgery.py,v 1.9 2008/07/16 10:32:50 ncq Exp $
 __license__ = "GPL"
-__version__ = "$Revision: 1.7 $"
+__version__ = "$Revision: 1.9 $"
 __author__ = "K.Hilbert <Karsten.Hilbert@gmx.net>"
 
 
@@ -12,8 +12,9 @@ import sys, os
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmPG2, gmTools, gmBorg, gmCLI, gmCfg
+from Gnumed.pycommon import gmPG2, gmTools, gmBorg, gmCfg2
 
+_cfg = gmCfg2.gmCfgData()
 #============================================================
 class gmCurrentPractice(gmBorg.cBorg):
 
@@ -39,30 +40,20 @@ class gmCurrentPractice(gmBorg.cBorg):
 		if self.__helpdesk is not None:
 			return self.__helpdesk
 
-		candidates = []
-		if gmCLI.has_arg('--conf-file'):
-			candidates.append(gmCLI.arg['--conf-file'])
-		paths = gmTools.gmPaths()
-		candidates.extend ([
-			os.path.join(paths.working_dir, 'gnumed.conf'),
-			os.path.join(paths.local_base_dir, 'gnumed.conf'),
-			os.path.join(paths.user_config_dir, 'gnumed.conf'),
-			os.path.join(paths.system_config_dir, 'gnumed-client.conf')
-		])
-
-		self.__helpdesk = None
-		for candidate in candidates:
-			try:
-				cfg = gmCfg.cCfgFile(aFile = candidate)
-			except IOError:
-				continue
-			tmp = cfg.get('workplace', 'help desk')
-			if tmp is not None:
-				self.__helpdesk = tmp
-				break
-
-		if self.__helpdesk is None:
-			self.__helpdesk = 'http://wiki.gnumed.de'
+		self.__helpdesk = gmTools.coalesce (
+			_cfg.get (
+				group = u'workplace',
+				option = u'help desk',
+				source_order = [
+					('explicit', 'return'),
+					('workbase', 'return'),
+					('local', 'return'),
+					('user', 'return'),
+					('system', 'return')
+				]
+			),
+			u'http://wiki.gnumed.de'
+		)
 
 		return self.__helpdesk
 
@@ -94,41 +85,25 @@ class gmCurrentPractice(gmBorg.cBorg):
 	def _get_workplace(self):
 		"""Return the current workplace (client profile) definition.
 
-		- from the command line config file
-		- from the config file in the working directory
-		- from the config file in the binary install directory
-		- from the user config file
-		- from the system-wide config file
-
 		The first occurrence counts.
 		"""
 		if self.__active_workplace is not None:
 			return self.__active_workplace
 
-		candidates = []
-		if gmCLI.has_arg('--conf-file'):
-			candidates.append(gmCLI.arg['--conf-file'])
-		paths = gmTools.gmPaths()
-		candidates.extend ([
-			os.path.join(paths.working_dir, 'gnumed.conf'),
-			os.path.join(paths.local_base_dir, 'gnumed.conf'),
-			os.path.join(paths.user_config_dir, 'gnumed.conf'),
-			os.path.join(paths.system_config_dir, 'gnumed-client.conf')
-		])
-
-		self.__active_workplace = None
-		for candidate in candidates:
-			try:
-				cfg = gmCfg.cCfgFile(aFile = candidate)
-			except IOError:
-				continue
-			tmp = cfg.get('workplace', 'name')
-			if tmp is not None:
-				self.__active_workplace = tmp
-				break
-
-		if self.__active_workplace is None:
-			self.__active_workplace = 'GNUmed Default'
+		self.__active_workplace = gmTools.coalesce (
+			_cfg.get (
+				group = u'workplace',
+				option = u'name',
+				source_order = [
+					('explicit', 'return'),
+					('workbase', 'return'),
+					('local', 'return'),
+					('user', 'return'),
+					('system', 'return'),
+				]
+			),
+			u'GNUmed Default'
+		)
 
 		return self.__active_workplace
 
@@ -142,6 +117,32 @@ class gmCurrentPractice(gmBorg.cBorg):
 		return [ r[0] for r in rows ]
 
 	workplaces = property(_get_workplaces, _set_workplaces)
+	#--------------------------------------------------------
+	def _get_user_email(self):
+		# FIXME: get this from the current users staff record in the database
+		return _cfg.get (
+			group = u'preferences',
+			option = u'user email',
+			source_order = [
+				('explicit', 'return'),
+				('user', 'return'),
+				('local', 'return'),
+				('workbase', 'return'),
+				('system', 'return')
+			]
+		)
+
+	def _set_user_email(self, val):
+		prefs_file = _cfg.get(option = 'user_preferences_file')
+		gmCfg2.set_option_in_INI_file (
+			filename = prefs_file,
+			group = u'preferences',
+			option = u'user email',
+			value = val
+		)
+		_cfg.reload_file_source(file = prefs_file)
+
+	user_email = property(_get_user_email, _set_user_email)
 #============================================================
 class cSurgery(object):
 
@@ -197,6 +198,12 @@ if __name__ == '__main__':
 
 #============================================================
 # $Log: gmSurgery.py,v $
+# Revision 1.9  2008/07/16 10:32:50  ncq
+# - add .user_email property
+#
+# Revision 1.8  2007/12/23 11:55:49  ncq
+# - cleanup, use gmCfg2
+#
 # Revision 1.7  2007/10/23 21:20:24  ncq
 # - cleanup
 #
