@@ -10,25 +10,26 @@ TODO:
 """
 #============================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/exporters/gmPatientExporter.py,v $
-# $Id: gmPatientExporter.py,v 1.109.2.1 2008/01/08 22:37:37 ncq Exp $
-__version__ = "$Revision: 1.109.2.1 $"
+# $Id: gmPatientExporter.py,v 1.127 2008/07/28 15:42:30 ncq Exp $
+__version__ = "$Revision: 1.127 $"
 __author__ = "Carlos Moro"
 __license__ = 'GPL'
 
-import os.path, sys, traceback, string, types, time, codecs, datetime as pyDT, locale
+import os.path, sys, types, time, codecs, datetime as pyDT, logging, shutil
+
 
 import mx.DateTime.Parser as mxParser
 import mx.DateTime as mxDT
 
+
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
+from Gnumed.pycommon import gmI18N, gmExceptions, gmNull, gmPG2, gmTools
+from Gnumed.business import gmClinicalRecord, gmPerson, gmAllergy, gmMedDoc, gmDemographicRecord, gmClinNarrative
 
-from Gnumed.pycommon import gmLog, gmI18N, gmCLI, gmCfg, gmExceptions, gmNull, gmPG2, gmTools
-from Gnumed.business import gmClinicalRecord, gmPerson, gmAllergy, gmMedDoc, gmDemographicRecord
 
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lInfo, __version__)
-_cfg = gmCfg.gmDefCfgFile
+_log = logging.getLogger('gm.export')
+_log.info(__version__)
 #============================================================
 class cEmrExport:
 
@@ -87,7 +88,7 @@ class cEmrExport:
             patient - Patient whose data are to be dumped
         """
         if patient is None:
-            _log.Log(gmLog.lErr, "can't set None patient for exporter")
+            _log.error("can't set None patient for exporter")
             return
         self.__patient = patient
     #--------------------------------------------------------
@@ -384,7 +385,7 @@ class cEmrExport:
         """
             Retrieve patient clinical items filtered by multiple constraints
         """
-        if not self.__patient.is_connected():
+        if not self.__patient.connected:
             return False
         emr = self.__patient.get_emr()
         filtered_items = []
@@ -402,7 +403,7 @@ class cEmrExport:
      #               episodes=self.__constraints['episodes'],
       #              issues=self.__constraints['issues']))
        # except:
-        #        _log.Error("vaccination error? outside regime")
+        #        _log.error("vaccination error? outside regime")
 
 #        filtered_items.extend(emr.get_lab_results(
  #           since=self.__constraints['since'],
@@ -471,7 +472,6 @@ class cEmrExport:
         """
         #TODO , caching eliminates tree update time, so don't really need this
         self._traverse_health_issues( emr_tree, self._update_health_issue_branch)
-	
     #--------------------------------------------------------             
     def get_historical_tree( self, emr_tree):
         self._traverse_health_issues( emr_tree, self._add_health_issue_branch)
@@ -498,13 +498,14 @@ class cEmrExport:
         # unlinked episodes
         if len(unlinked_episodes) > 0:
             h_issues.insert(0, {
-                'description': _('free-standing episodes'),
+                'description': _('Unattributed episodes'),
                 'pk': None
             })
         # existing issues        
         for a_health_issue in h_issues:
-		health_issue_action( emr_tree, a_health_issue)
+            health_issue_action( emr_tree, a_health_issue)
 
+        emr_tree.SortChildren(emr_tree.GetRootItem())
     #--------------------------------------------------------             
     def _add_health_issue_branch( self, emr_tree, a_health_issue):
             """appends to a wx emr_tree  , building wx treenodes from the health_issue  make this reusable for non-collapsing tree updates"""
@@ -515,6 +516,7 @@ class cEmrExport:
             episodes = emr.get_episodes(id_list=self.__constraints['episodes'], issues = [a_health_issue['pk']])
             for an_episode in episodes:
                 self._add_episode_to_tree( emr, emr_tree, issue_node,a_health_issue,  an_episode)
+            emr_tree.SortChildren(issue_node)
     #--------------------------------------------------------             
     def _add_episode_to_tree( self, emr , emr_tree, issue_node, a_health_issue, an_episode):
         episode_node =  emr_tree.AppendItem(issue_node, an_episode['description'])
@@ -524,13 +526,14 @@ class cEmrExport:
 
         encounters = self._get_encounters( an_episode, emr )
         self._add_encounters_to_tree( encounters,  emr_tree, episode_node )
+        emr_tree.SortChildren(episode_node)
         return episode_node
     #--------------------------------------------------------             
     def _add_encounters_to_tree( self, encounters, emr_tree, episode_node):
-               for an_encounter in encounters:
-                    label = '%s:%s' % (an_encounter['l10n_type'], an_encounter['started'].strftime('%Y-%m-%d'))
-                    encounter_node = emr_tree.AppendItem(episode_node, label)
-                    emr_tree.SetPyData(encounter_node, an_encounter)
+        for an_encounter in encounters:
+            label = '%s: %s' % (an_encounter['started'].strftime('%Y-%m-%d'), an_encounter['l10n_type'])
+            encounter_node = emr_tree.AppendItem(episode_node, label)
+            emr_tree.SetPyData(encounter_node, an_encounter)
     #--------------------------------------------------------
     def _get_encounters ( self, an_episode, emr ):
                encounters = emr.get_encounters (
@@ -539,64 +542,57 @@ class cEmrExport:
                return encounters
     #--------------------------------------------------------             
     def  _update_health_issue_branch(self, emr_tree, a_health_issue):
-            emr = self.__patient.get_emr()
-            root_node = emr_tree.GetRootItem()
-	    id, cookie = emr_tree.GetFirstChild( root_node)
-	    found = False
-	    while id.IsOk():
-	        #try:
-	    	#  _log.Log(gmLog.lErr, "emr_tree.GetItemText(id)  a_health_issue['description']\n\t\t: %s %s\n" % ( emr_tree.GetItemText(id)  , a_health_issue['description'] ) )
-		#except:
-		#  print sys.exc_info()[0], sys.exc_info()[1]
-		#  traceback.print_tb( sys.exc_info()[2])
+		emr = self.__patient.get_emr()
+		root_node = emr_tree.GetRootItem()
+		id, cookie = emr_tree.GetFirstChild(root_node)
+		found = False
+		while id.IsOk():
+			if emr_tree.GetItemText(id)  ==  a_health_issue['description']:
+				found = True
+				break
+			id,cookie = emr_tree.GetNextChild( root_node, cookie)
 
-	    	if emr_tree.GetItemText(id)  ==  a_health_issue['description'] :
-			found = True
-			break
-		id,cookie = emr_tree.GetNextChild( root_node, cookie)
-	    
-	    if not found:
-	    	_log.Log(gmLog.lErr, "health issue %s should exist in tree already" %  a_health_issue['description'] )
-		return
-	    issue_node= id
-	    episodes = emr.get_episodes(id_list=self.__constraints['episodes'], issues = [a_health_issue['pk']])
-	    
-	    #check for removed episode and update tree
-	    tree_episodes = {} 
-	    id_episode, cookie = emr_tree.GetFirstChild(issue_node)
-	    while id_episode.IsOk():
-		    	tree_episodes[ emr_tree.GetPyData(id_episode)['pk_episode'] ]= id_episode
+		if not found:
+			_log.error("health issue %s should exist in tree already", a_health_issue['description'] )
+			return
+		issue_node = id
+		episodes = emr.get_episodes(id_list=self.__constraints['episodes'], issues = [a_health_issue['pk']])
+
+		#check for removed episode and update tree
+		tree_episodes = {} 
+		id_episode, cookie = emr_tree.GetFirstChild(issue_node)
+		while id_episode.IsOk():
+			tree_episodes[ emr_tree.GetPyData(id_episode)['pk_episode'] ]= id_episode
 			id_episode,cookie = emr_tree.GetNextChild( issue_node, cookie)
-	
-	    existing_episode_pk = [ e['pk_episode'] for e in episodes]
-	    missing_tree_pk = [ pk for pk in tree_episodes.keys() if pk not in existing_episode_pk]
-	    for pk in missing_tree_pk:
-		emr_tree.Remove( tree_episodes[pk] )
-	
-	    added_episode_pk = [pk for pk in existing_episode_pk if pk not in tree_episodes.keys()]
-	    add_episodes = [ e for e in episodes if e['pk_episode'] in added_episode_pk]
-	   
-	    #check for added episodes and update tree
-	    for an_episode in add_episodes:
+
+		existing_episode_pk = [ e['pk_episode'] for e in episodes]
+		missing_tree_pk = [ pk for pk in tree_episodes.keys() if pk not in existing_episode_pk]
+		for pk in missing_tree_pk:
+			emr_tree.Remove( tree_episodes[pk] )
+
+		added_episode_pk = [pk for pk in existing_episode_pk if pk not in tree_episodes.keys()]
+		add_episodes = [ e for e in episodes if e['pk_episode'] in added_episode_pk]
+
+		#check for added episodes and update tree
+		for an_episode in add_episodes:
 			node = self._add_episode_to_tree( emr, emr_tree, issue_node, a_health_issue, an_episode)
 			tree_episodes[an_episode['pk_episode']] = node
-			
-			
-            for an_episode in episodes:
+
+		for an_episode in episodes:
 			# found episode, check for encounter change
 			try:
-			  #print "getting id_episode of ", an_episode['pk_episode']
-			  id_episode = tree_episodes[an_episode['pk_episode']]	
+				#print "getting id_episode of ", an_episode['pk_episode']
+				id_episode = tree_episodes[an_episode['pk_episode']]	
 			except:
-			  import pdb
-			  pdb.set_trace()
+				import pdb
+				pdb.set_trace()
 			# get a map of encounters in the tree by pk_encounter as key
 			tree_enc = {}
 			id_encounter, cookie = emr_tree.GetFirstChild(id_episode)
 			while id_encounter.IsOk():
 				tree_enc[ emr_tree.GetPyData(id_encounter)['pk_encounter'] ] = id_encounter
 				id_encounter,cookie = emr_tree.GetNextChild(id_episode, cookie)
-				
+
 			# remove encounters in tree not in existing encounters in episode
 #			encounters = self._get_encounters( a_health_issue, an_episode, emr )
 			encounters = self._get_encounters( an_episode, emr )
@@ -604,18 +600,14 @@ class cEmrExport:
 			missing_enc_pk = [ pk  for pk in tree_enc.keys() if pk not in existing_enc_pk]
 			for pk in missing_enc_pk:
 				emr_tree.Remove( tree_enc[pk] )
-					
+
 			# check for added encounter
 			added_enc_pk = [ pk for pk in existing_enc_pk if pk not in tree_enc.keys() ]
 			add_encounters = [ enc for enc in encounters if enc['pk_encounter'] in added_enc_pk]
 			if add_encounters != []:
 				#print "DEBUG found encounters to add"
 				self._add_encounters_to_tree( add_encounters, emr_tree, id_episode)
-				
-
-				
-    	
-    #--------------------------------------------------------  
+    #--------------------------------------------------------
     def get_summary_info(self, left_margin = 0):
         """
         Dumps patient EMR summary
@@ -623,7 +615,7 @@ class cEmrExport:
         txt = ''
         for an_item in self.__filtered_items:
             txt += self.get_item_summary(an_item, left_margin)
-        return txt                      
+        return txt
     #--------------------------------------------------------
     def get_issue_info(self, issue=None, left_margin=0):
         """Dumps health issue specific data
@@ -638,11 +630,13 @@ class cEmrExport:
             status = _('Active health issue')
         else:
             status = _('Inactive health issue')
-        txt = _('%s "%s" noted at age %s\n') % (
+        txt = _('%s "%s"\n noted at age %s\n') % (
             status,
             issue['description'],
-            issue['age_noted']
+#            issue['age_noted']
+            issue.age_noted_human_readable()
         )
+#        print issue.age_noted_human_readable()
         if issue['clinically_relevant']:
             txt += _('clinically relevant: yes\n')
         else:
@@ -714,13 +708,14 @@ class cEmrExport:
         """
         emr = self.__patient.get_emr()
         # general
-        txt = (' ' * left_margin) + '%s - %s: %s' % (
-            encounter['started'].strftime('%Y-%m-%d  %H:%M'),
-            encounter['last_affirmed'].strftime('%H:%M'),
+        txt = (' ' * left_margin) + '#%s: %s - %s   %s' % (
+            encounter['pk_encounter'],
+            encounter['started'].strftime('%Y-%m-%d %H:%M'),
+            encounter['last_affirmed'].strftime('%H:%M (%Z)'),
             encounter['l10n_type']
         )
         if (encounter['assessment_of_encounter'] is not None) and (len(encounter['assessment_of_encounter']) > 0):
-            txt += ' (%s)' % encounter['assessment_of_encounter']
+            txt += ' "%s"' % encounter['assessment_of_encounter']
         txt += '\n\n'
 
         # rfe/aoe
@@ -750,7 +745,7 @@ class cEmrExport:
             for soap_entry in soap_cat_narratives:
                 txt += gmTools.wrap (
                     '%s %.8s: %s\n' % (
-                        soap_entry['date'].strftime('%d.%m %H:%M'),
+                        soap_entry['date'].strftime('%d.%m. %H:%M'),
                         soap_entry['provider'],
                         soap_entry['narrative']
                     ), 75
@@ -768,14 +763,13 @@ class cEmrExport:
         for an_item in self.__filtered_items:
             if an_item['pk_encounter'] == encounter['pk_encounter']:
                 txt += self.get_item_output(an_item, left_margin)
-        return txt      
+        return txt
     #--------------------------------------------------------  
     def dump_historical_tree(self):
         """Dumps patient's historical in form of a tree of health issues
                                                         -> episodes
                                                            -> encounters
                                                               -> clinical items
-                                                              
         """
 
         # fecth all values
@@ -792,7 +786,7 @@ class cEmrExport:
         # unlinked episodes
         unlinked_episodes = emr.get_episodes(issues = [None])
         if len(unlinked_episodes) > 0:
-            h_issues.insert(0, {'description':_('free-standing episodes'), 'pk':None})        
+            h_issues.insert(0, {'description':_('Unattributed episodes'), 'pk':None})        
         for a_health_issue in h_issues:
             self.__target.write('\n' + 3*' ' + 'Health Issue: ' + a_health_issue['description'] + '\n')
             episodes = emr.get_episodes(id_list=self.__constraints['episodes'], issues = [a_health_issue['pk']])
@@ -830,7 +824,7 @@ class cEmrExport:
         """
         emr = self.__patient.get_emr()
         if emr is None:
-            _log.Log(gmLog.lErr, 'cannot get EMR text dump')
+            _log.error('cannot get EMR text dump')
             print(_(
                 'An error occurred while retrieving a text\n'
                 'dump of the EMR for the active patient.\n\n'
@@ -883,7 +877,7 @@ class cEmrExport:
             Dumps in ASCII format some basic patient's demographic data
         """
         if self.__patient is None:
-            _log.Log(gmLog.lErr, 'cannot get Demographic export')
+            _log.error('cannot get Demographic export')
             print(_(
                 'An error occurred while Demographic record export\n'
                 'Please check the log file for details.'
@@ -952,26 +946,18 @@ class cEMRJournalExporter:
 	"""
 	def __init__(self):
 		self.__part_len = 72
-		self.__tx_soap = {
-			u's': _('S'),
-			u'o': _('O'),
-			u'a': _('A'),
-			u'p': _('P'),
-			None: u' ',
-		}
 	#--------------------------------------------------------
 	# external API
 	#--------------------------------------------------------
 	def export_to_file(self, filename=None, patient=None):
-		"""
-		Export medical record into a file.
+		"""Export medical record into a file.
 
 		@type filename: None (creates filename by itself) or string
 		@type patient: None (use currently active patient) or <gmPerson.cIdentity> instance
 		"""
 		if patient is None:
 			patient = gmPerson.gmCurrentPatient()
-			if not patient.is_connected():
+			if not patient.connected:
 				raise ValueError('[%s].export_to_file(): no active patient' % self.__class__.__name__)
 
 		if filename is None:
@@ -999,7 +985,7 @@ class cEMRJournalExporter:
 		"""
 		if patient is None:
 			patient = gmPerson.gmCurrentPatient()
-			if not patient.is_connected():
+			if not patient.connected:
 				raise ValueError('[%s].export(): no active patient' % self.__class__.__name__)
 
 		# write header
@@ -1010,8 +996,9 @@ class cEMRJournalExporter:
 		target.write(_('Patient: %s (%s), No: %s\n') % (patient['description'], patient['gender'], patient['pk_identity']))
 		target.write(_('Born   : %s, age: %s\n\n') % (patient['dob'].strftime('%Y-%m-%d'), patient.get_medical_age()))
 		target.write(u'.-%10.10s---%9.9s-------%72.72s\n' % (u'-' * 10, u'-' * 9, u'-' * self.__part_len))
-		target.write(u'| %10.10s | %9.9s |   | %s\n' % (_('Date'), _('Doc'), _('Narrative')))
+		target.write(u'| %10.10s | %9.9s |     | %s\n' % (_('Entered'), _('Doc'), _('Narrative')))
 		target.write(u'|-%10.10s---%9.9s-------%72.72s\n' % (u'-' * 10, u'-' * 9, u'-' * self.__part_len))
+
 		# get data
 		cmd = u"""
 select
@@ -1019,46 +1006,64 @@ select
 	vemrj.*,
 	(select rank from clin.soap_cat_ranks where soap_cat=vemrj.soap_cat) as scr
 from clin.v_emr_journal vemrj
-where pk_patient=%s order by date, pk_episode, scr"""
+where pk_patient = %s
+order by date, pk_episode, scr, src_table"""
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': [patient['pk_identity']]}], get_col_idx = True)
+
 		# write data
 		prev_date = u''
 		prev_doc = u''
+		prev_soap = u''
 		for row in rows:
 			# narrative
-			if row[idx['narrative']] is None:
+			if row['narrative'] is None:
 				continue
-			txt = row[idx['narrative']].replace(u'\n', u' ').replace(u'\r', u' ')
-			no_parts = (len(txt) / self.__part_len) + 1
-			# doc
-			curr_doc = row[idx['modified_by']]
+
+			txt = gmTools.wrap (
+				text = row['narrative'].replace(u'\r', u''),
+				width = self.__part_len
+			).split('\n')
+
+			# same provider ?
+			curr_doc = row['modified_by']
 			if curr_doc != prev_doc:
 				prev_doc = curr_doc
 			else:
 				curr_doc = u''
-			# date
-			curr_date = row[idx['date']]
+
+			# same soap category ?
+			curr_soap = row['soap_cat']
+			if curr_soap != prev_soap:
+				prev_soap = curr_soap
+
+			# same date ?
+			curr_date = row['date']
 			if curr_date != prev_date:
 				prev_date = curr_date
-				curr_doc = row[idx['modified_by']]
+				curr_doc = row['modified_by']
+				prev_doc = curr_doc
+				curr_soap = row['soap_cat']
+				prev_soap = curr_soap
 			else:
 				curr_date = u''
+
 			# display first part
-			target.write(u'| %10.10s | %9.9s | %s | %s\n' % (
+			target.write(u'| %10.10s | %9.9s | %3.3s | %s\n' % (
 				curr_date,
 				curr_doc,
-				self.__tx_soap[row[idx['soap_cat']]],
-				txt[0:self.__part_len]
+				gmClinNarrative.soap_cat2l10n[curr_soap],
+				txt[0]
 			))
-			# more parts ?
-			if no_parts == 1:
+
+			# only one part ?
+			if len(txt) == 1:
 				continue
-			template = u'| %10.10s | %9.9s | %s | %s\n'
-			for part in range(1, no_parts):
-				soap_cat = self.__tx_soap[row[idx['soap_cat']]]
-				msg = txt[(part * self.__part_len):((part+1) * self.__part_len)]
-				line = template % (u'', u'', soap_cat, msg)
+
+			template = u'| %10.10s | %9.9s | %3.3s | %s\n'
+			for part in txt[1:]:
+				line = template % (u'', u'', u' ', part)
 				target.write(line)
+
 		# write footer
 		target.write(u'`-%10.10s---%9.9s-------%72.72s\n\n' % (u'-' * 10, u'-' * 9, u'-' * self.__part_len))
 		target.write(_('Exported: %s\n') % pyDT.datetime.now().strftime('%c').decode(gmI18N.get_encoding()))
@@ -1074,18 +1079,13 @@ class cMedistarSOAPExporter:
 			if not isinstance(patient, gmPerson.cIdentity):
 				raise gmExceptions.ConstructorError, '<patient> argument must be instance of <cIdentity>, but is: %s' % type(patient)
 			self.__pat = patient
-		self.__tx_soap = {
-			's': 'A',
-			'o': 'B',
-			'a': 'D',
-			'p': 'T'
-		}
 	#--------------------------------------------------------
 	# external API
 	#--------------------------------------------------------
-	def export_to_file(self, filename=None):
-		if not self.__pat.is_connected():
+	def export_to_file(self, filename=None, encounter=None, soap_cats=u'soap', export_to_import_file=False):
+		if not self.__pat.connected:
 			return (False, 'no active patient')
+
 		if filename is None:
 			path = os.path.abspath(os.path.expanduser('~/gnumed/export'))
 			filename = '%s-%s-%s-%s-%s.txt' % (
@@ -1095,31 +1095,52 @@ class cMedistarSOAPExporter:
 				self.__pat['firstnames'].replace(' ', '_'),
 				self.__pat['dob'].strftime('%Y-%m-%d')
 			)
-		f = codecs.open(filename = filename, mode = 'w+b', encoding = 'utf8')
-		status = self.__export(target = f)
+
+		f = codecs.open(filename = filename, mode = 'w+b', encoding = 'cp437', errors='replace')
+		status = self.__export(target = f, encounter = encounter, soap_cats = soap_cats)
 		f.close()
+
+		if export_to_import_file:
+			# detect "LW:\medistar\inst\soap.txt"
+			medistar_found = False
+			for drive in u'cdefghijklmnopqrstuvwxyz':
+				path = drive + ':\\medistar\\inst'
+				if not os.path.isdir(path):
+					continue
+				try:
+					import_fname = path + '\\soap.txt'
+					open(import_fname, mode = 'w+b').close()
+					_log.debug('exporting narrative to [%s] for Medistar import', import_fname)
+					shutil.copyfile(filename, import_fname)
+					medistar_found = True
+				except IOError:
+					continue
+
+			if not medistar_found:
+				_log.debug('no Medistar installation found (no <LW>:\\medistar\\inst\\)')
+
 		return (status, filename)
 	#--------------------------------------------------------
-	def export(self, target):
-		return self.__export(target)
+	def export(self, target, encounter=None, soap_cats=u'soap'):
+		return self.__export(target, encounter = encounter, soap_cats = soap_cats)
 	#--------------------------------------------------------
 	# interal API
 	#--------------------------------------------------------
-	def __export(self, target = None, encounter = None):
-		if not self.__pat.is_connected():
-			return False
-		emr = self.__pat.get_emr()
-		if encounter is None:
-			encounter = emr.get_active_encounter()
+	def __export(self, target=None, encounter=None, soap_cats=u'soap'):
 		# get data
 		cmd = u"select narrative from clin.v_emr_journal where pk_patient=%s and pk_encounter=%s and soap_cat=%s"
-		for soap_cat in 'soap':
+		for soap_cat in soap_cats:
 			rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': (self.__pat['pk_identity'], encounter['pk_encounter'], soap_cat)}])
-			target.write('*MD%s*\n' % self.__tx_soap[soap_cat])
+			target.write('*MD%s*\r\n' % gmClinNarrative.soap_cat2l10n[soap_cat])
 			for row in rows:
 				text = row[0]
-				if text is not None:
-					target.write('%s\n' % gmTools.wrap(text, 64))
+				if text is None:
+					continue
+				target.write('%s\r\n' % gmTools.wrap (
+					text = text,
+					width = 64,
+					eol = u'\r\n'
+				))
 		return True
 #============================================================
 # main
@@ -1131,54 +1152,10 @@ def usage():
     print 'usage: python gmPatientExporter [--fileout=<outputfilename>] [--conf-file=<file>] [--text-domain=<textdomain>]'
     sys.exit(0)
 #------------------------------------------------------------
-def parse_constraints():
-    """
-        Obtains, parses and normalizes config file options
-    """
-    if isinstance(_cfg, gmNull.cNull):
-        usage()
-
-    # Retrieve options
-    cfg_group = 'constraints'
-    constraints = {
-        'since': _cfg.get(cfg_group, 'since'),
-        'until': _cfg.get(cfg_group, 'until'),
-        'encounters': _cfg.get(cfg_group, 'encounters'),
-        'episodes': _cfg.get(cfg_group, 'episodes'),
-        'issues': _cfg.get(cfg_group, 'issues')
-    }
-
-    # Normalize null constraints (None is interpreted as non existing constraint along all methods)
-    for a_constraint in constraints.keys():
-        if len(constraints[a_constraint]) == 0:
-            constraints[a_constraint] = None
-
-    # Cast existing constraints to correct type
-    if not constraints['since'] is None:
-        constraints['since'] = mxParser.DateFromString(constraints['since'], formats= ['iso'])
-    if not constraints['until'] is None:
-        constraints['until'] = mxParser.DateFromString(constraints['until'], formats= ['iso'])
-    if not constraints['encounters'] is None:
-        constraints['encounters'] = map(lambda encounter: int(encounter), constraints['encounters'])
-    if not constraints['episodes'] is None:
-        constraints['episodes'] = map(lambda episode: int(episode), constraints['episodes'])
-    if not constraints['issues'] is None:
-        constraints['issues'] = map(lambda issue: int(issue), constraints['issues'])
-    return constraints
-#------------------------------------------------------------                 
 def run():
     """
         Main module application execution loop.
     """
-    # Check that output file name is defined and create an instance of exporter
-#    if gmCLI.has_arg('--fileout'):
-#        outFile = open(gmCLI.arg['--fileout'], 'wb')
-#    else:
-#        usage()
-#    export_tool = cEmrExport(parse_constraints(), outFile)
-
-#    export_tool.get_episode_summary()
-
     # More variable initializations
     patient = None
     patient_id = None
@@ -1216,8 +1193,6 @@ if __name__ == "__main__":
 	gmI18N.activate_locale()
 	gmI18N.install_domain()
 
-	gmLog.gmDefLog.SetAllLogLevels(gmLog.lData)
-
 	#--------------------------------------------------------
 	def export_journal():
 
@@ -1241,16 +1216,67 @@ if __name__ == "__main__":
 	print "\n\nGNUmed ASCII EMR Export"
 	print     "======================="
 
-	if gmCLI.has_arg('--help'):
-		usage()
-
 	# run main loop
 	export_journal()
 
 #============================================================
 # $Log: gmPatientExporter.py,v $
-# Revision 1.109.2.1  2008/01/08 22:37:37  ncq
-# - 'first' / 'last' -> 'firstnames' / 'lastnames'
+# Revision 1.127  2008/07/28 15:42:30  ncq
+# - cleanup
+# - enhance medistar exporter
+#
+# Revision 1.126  2008/07/12 15:20:55  ncq
+# - comment out print
+#
+# Revision 1.125  2008/07/07 13:39:22  ncq
+# - properly sort tree
+# - current patient .connected
+#
+# Revision 1.124  2008/06/24 13:55:14  ncq
+# - change encounter node label
+#
+# Revision 1.123  2008/06/23 09:59:57  ncq
+# - much improved journal layout
+#
+# Revision 1.122  2008/06/15 20:16:02  ncq
+# - add a space
+#
+# Revision 1.121  2008/05/19 15:44:16  ncq
+# - just silly cleanup
+#
+# Revision 1.120  2008/05/07 15:16:01  ncq
+# - use centralized soap category translations from gmClinNarrative
+#
+# Revision 1.119  2008/04/11 12:21:11  ncq
+# - some cleanup
+#
+# Revision 1.118  2008/04/02 10:15:54  ncq
+# - show local time zone in encounter summary
+#
+# Revision 1.117  2008/03/06 18:24:45  ncq
+# - indentation fix
+#
+# Revision 1.116  2008/03/05 22:25:09  ncq
+# - no more gmLog
+#
+# Revision 1.115  2008/01/30 13:46:17  ncq
+# - cleanup
+#
+# Revision 1.114  2008/01/22 11:52:24  ncq
+# - Unattributed
+# - improved Journal formatting as per list
+#
+# Revision 1.113  2008/01/13 01:13:58  ncq
+# - use issue.age_noted_human_readable()
+#
+# Revision 1.112  2008/01/11 16:10:00  ncq
+# - first/last -> first-/lastnames
+#
+# Revision 1.111  2007/12/26 22:26:04  shilbert
+# - indentation error fixed
+#
+# Revision 1.110  2007/12/23 11:56:38  ncq
+# - improve output, cleanup
 #
 # Revision 1.109  2007/11/28 11:52:13  ncq
 # - get_all_names() -> get_names()
