@@ -5,19 +5,25 @@ notifications from the database backend.
 """
 #=====================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/pycommon/gmBackendListener.py,v $
-__version__ = "$Revision: 1.12 $"
+__version__ = "$Revision: 1.18 $"
 __author__ = "H. Herb <hherb@gnumed.net>, K.Hilbert <karsten.hilbert@gmx.net>"
 
-import sys, time, threading, select
+import sys, time, threading, select, logging
 
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmDispatcher, gmLog, gmExceptions, gmBorg
+from Gnumed.pycommon import gmDispatcher, gmExceptions, gmBorg
 
 
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lInfo, __version__)
+_log = logging.getLogger('gm.database')
+_log.info(__version__)
+
+
+static_signals = [
+	u'db_maintenance_warning',		# warns of impending maintenance and asks for disconnect
+	u'db_maintenance_disconnect'	# announces a forced disconnect and disconnects
+]
 #=====================================================================
 class gmBackendListener(gmBorg.cBorg):
 
@@ -29,7 +35,7 @@ class gmBackendListener(gmBorg.cBorg):
 		except AttributeError:
 			pass
 
-		_log.Log(gmLog.lInfo, 'starting backend notifications listener thread')
+		_log.info('starting backend notifications listener thread')
 
 		# the listener thread will regularly try to acquire
 		# this lock, when it succeeds it will quit
@@ -37,12 +43,9 @@ class gmBackendListener(gmBorg.cBorg):
 		# take the lock now so it cannot be taken by the worker
 		# thread until it is released in stop_thread()
 		if not self._quit_lock.acquire(0):
-			_log.Log(gmLog.lErr, 'cannot acquire thread quit lock !?! aborting')
+			_log.error('cannot acquire thread quit lock !?! aborting')
 			raise gmExceptions.ConstructorError, "cannot acquire thread quit-lock"
 
-		# remember what signals we are listening for; no need to listen twice to the same signal
-#		self._intercepted_notifications = []
-
 		self._conn = conn
 		self._conn.set_isolation_level(0)
 		self._cursor = self._conn.cursor()
@@ -50,7 +53,7 @@ class gmBackendListener(gmBorg.cBorg):
 
 		self.curr_patient_pk = None
 		if patient is not None:
-			if patient.is_connected():
+			if patient.connected:
 				self.curr_patient_pk = patient.ID
 		self.__register_interests()
 
@@ -61,38 +64,27 @@ class gmBackendListener(gmBorg.cBorg):
 
 		self.already_inited = True
 	#-------------------------------
-#	def __del__(self):
-#		if self._listener_thread is None:
-#			return 1
-#		try:
-#			self.__unlisten_notification()
-#		except: pass
-#		try:
-#			self._quit_lock.release()
-#			# give the thread time to terminate
-#			self._listener_thread.join(self._poll_interval+2.0)
-#		except: pass
-#		try:
-#			if self._listener_thread.isAlive():
-#				print '__del__(): listener thread still alive after join()'
-#				print '__del__(): active threads:', threading.enumerate()
-#		except:	pass
-	#-------------------------------
 	# public API
 	#-------------------------------
 	def stop_thread(self):
 		if self._listener_thread is None:
 			return
-		_log.Log(gmLog.lInfo, 'stopping backend notifications listener thread')
+
+		_log.info('stopping backend notifications listener thread')
 		self._quit_lock.release()
-		# give the worker thread time to terminate
-		self._listener_thread.join(self._poll_interval+2.0)
 		try:
-			if self._listener_thread.isAlive():
-				_log.Log(gmLog.lErr, 'listener thread still alive after join()')
-				_log.Log(gmLog.lData, 'active threads: %s' % threading.enumerate())
+			# give the worker thread time to terminate
+			self._listener_thread.join(self._poll_interval+2.0)
+			try:
+				if self._listener_thread.isAlive():
+					_log.error('listener thread still alive after join()')
+					_log.debug('active threads: %s' % threading.enumerate())
+			except:
+				pass
 		except:
-			pass
+			print sys.exc_info()
+
+		self._listener_thread = None
 
 		self.__unregister_patient_notifications()
 		self.__unregister_unspecific_notifications()
@@ -120,8 +112,8 @@ class gmBackendListener(gmBorg.cBorg):
 		self._conn_lock.release()
 		rows = self._cursor.fetchall()
 		self.patient_specific_notifications = [ '%s_mod_db' % row[0] for row in rows ]
-		_log.Log(gmLog.lInfo, 'configured patient specific notifications:')
-		_log.Log(gmLog.lInfo, '%s' % self.patient_specific_notifications)
+		_log.info('configured patient specific notifications:')
+		_log.info('%s' % self.patient_specific_notifications)
 		gmDispatcher.known_signals.extend(self.patient_specific_notifications)
 
 		# determine unspecific notifications
@@ -131,8 +123,9 @@ class gmBackendListener(gmBorg.cBorg):
 		self._conn_lock.release()
 		rows = self._cursor.fetchall()
 		self.unspecific_notifications = [ '%s_mod_db' % row[0] for row in rows ]
-		_log.Log(gmLog.lInfo, 'configured unspecific notifications:')
-		_log.Log(gmLog.lInfo, '%s' % self.unspecific_notifications)
+		self.unspecific_notifications.extend(static_signals)
+		_log.info('configured unspecific notifications:')
+		_log.info('%s' % self.unspecific_notifications)
 		gmDispatcher.known_signals.extend(self.unspecific_notifications)
 
 		# determine our backend PID
@@ -142,7 +135,7 @@ class gmBackendListener(gmBorg.cBorg):
 		self._conn_lock.release()
 		rows = self._cursor.fetchall()
 		self.backend_pid = rows[0][0]
-		_log.Log(gmLog.lInfo, 'listener process PID: [%s]' % self.backend_pid)
+		_log.info('listener process PID: [%s]' % self.backend_pid)
 
 		# listen to patient changes inside the local client
 		# so we can re-register patient specific notifications
@@ -162,7 +155,7 @@ class gmBackendListener(gmBorg.cBorg):
 			return
 		for notification in self.patient_specific_notifications:
 			notification = '%s:%s' % (notification, self.curr_patient_pk)
-			_log.Log(gmLog.lData, 'starting to listen for [%s]' % notification)
+			_log.debug('starting to listen for [%s]' % notification)
 			cmd = 'LISTEN "%s"' % notification
 			self._conn_lock.acquire(1)
 			self._cursor.execute(cmd)
@@ -173,7 +166,7 @@ class gmBackendListener(gmBorg.cBorg):
 			return
 		for notification in self.patient_specific_notifications:
 			notification = '%s:%s' % (notification, self.curr_patient_pk)
-			_log.Log(gmLog.lData, 'stopping to listen for [%s]' % notification)
+			_log.debug('stopping to listen for [%s]' % notification)
 			cmd = 'UNLISTEN "%s"' % notification
 			self._conn_lock.acquire(1)
 			self._cursor.execute(cmd)
@@ -182,7 +175,7 @@ class gmBackendListener(gmBorg.cBorg):
 	def __register_unspecific_notifications(self):
 		for sig in self.unspecific_notifications:
 			sig = '%s:' % sig
-			_log.Log(gmLog.lInfo, 'starting to listen for [%s]' % sig)
+			_log.info('starting to listen for [%s]' % sig)
 			cmd = 'LISTEN "%s"' % sig
 			self._conn_lock.acquire(1)
 			self._cursor.execute(cmd)
@@ -191,7 +184,7 @@ class gmBackendListener(gmBorg.cBorg):
 	def __unregister_unspecific_notifications(self):
 		for sig in self.unspecific_notifications:
 			sig = '%s:' % sig
-			_log.Log(gmLog.lInfo, 'stopping to listen for [%s]' % sig)
+			_log.info('stopping to listen for [%s]' % sig)
 			cmd = 'UNLISTEN "%s"' % sig
 			self._conn_lock.acquire(1)
 			self._cursor.execute(cmd)
@@ -206,7 +199,7 @@ class gmBackendListener(gmBorg.cBorg):
 			name = self.__class__.__name__
 		)
 		self._listener_thread.setDaemon(True)
-		_log.Log(gmLog.lInfo, 'starting listener thread')
+		_log.info('starting listener thread')
 		self._listener_thread.start()
 	#-------------------------------
 	# the actual thread code
@@ -261,7 +254,8 @@ class gmBackendListener(gmBorg.cBorg):
 					_have_quit_lock = 1
 					break
 
-		self._listener_thread = None
+		# exit thread activity
+		return
 #=====================================================================
 # main
 #=====================================================================
@@ -269,10 +263,6 @@ if __name__ == "__main__":
 
 	notifies = 0
 
-	_log.SetAllLogLevels(gmLog.lData)
-
-	import time
-
 	from Gnumed.pycommon import gmPG2, gmI18N
 	from Gnumed.business import gmPerson
 
@@ -312,7 +302,7 @@ if __name__ == "__main__":
 
 		# now try with listener to measure impact
 		print "Now in a new shell connect psql to the"
-		print "database <gnumed_v8> on localhost, return"
+		print "database <gnumed_v9> on localhost, return"
 		print "here and hit <enter> to continue."
 		raw_input('hit <enter> when done starting psql')
 		print "You now have about 30 seconds to go"
@@ -357,10 +347,6 @@ if __name__ == "__main__":
 				print '==> got notification from database "%s":' % kwargs['signal']
 			except KeyError:
 				print '==> received signal from client: "%s"' % kwargs['signal']
-#			if kwargs['signal'].endswith('_db'):
-#				print '==> got notification from database "%s":' % kwargs['signal']
-#			else:
-#				print '==> received signal from client: "%s"' % kwargs['signal']
 			del kwargs['signal']
 			for key in kwargs.keys():
 				print '    [%s]: %s' % (key, kwargs[key])
@@ -398,6 +384,25 @@ if __name__ == "__main__":
 
 #=====================================================================
 # $Log: gmBackendListener.py,v $
+# Revision 1.18  2008/07/07 13:39:47  ncq
+# - current patient .connected
+#
+# Revision 1.17  2008/06/15 20:17:17  ncq
+# - be even more careful rejoining worker threads
+#
+# Revision 1.16  2008/04/28 13:31:16  ncq
+# - now static signals for database maintenance
+#
+# Revision 1.15  2008/01/07 19:48:22  ncq
+# - bump db version
+#
+# Revision 1.14  2007/12/12 16:17:15  ncq
+# - better logger names
+#
+# Revision 1.13  2007/12/11 14:16:29  ncq
+# - cleanup
+# - use logging
+#
 # Revision 1.12  2007/10/30 12:48:17  ncq
 # - attach_identity_pk -> carries_identity_pk
 #
