@@ -54,20 +54,22 @@ Concurrency handling
 GnuMed connections always run transactions in isolation level
 "serializable". This prevents transactions happening at the
 *very same time* to overwrite each other's data. All but one
-of them will abort with a concurrency error (eg if a transaction
-runs a select-for-update later than another one it will hang
-until the first transaction ends. Then it will suceed or fail
-depending on what the first transaction did).
+of them will abort with a concurrency error (eg if a
+transaction runs a select-for-update later than another one
+it will hang until the first transaction ends. Then it will
+succeed or fail depending on what the first transaction
+did). This is standard transactional behaviour.
 
-However, another transaction may have updated our row between
-the time we first fetched the data and the time we start the
-update transaction. This is noticed by getting the XMIN system
-column for the row when initially fetching the data and using
-that value as a where condition value when locking the row for
-update. If the row had been updated (xmin changed) or deleted
-(primary key disappeared) in the meantime then getting the row
-lock will touch zero rows even if the query itself formally
-succeeds.
+However, another transaction may have updated our row
+between the time we first fetched the data and the time we
+start the update transaction. This is noticed by getting the
+XMIN system column for the row when initially fetching the
+data and using that value as a where condition value when
+updating the row later. If the row had been updated (xmin
+changed) or deleted (primary key disappeared) in the
+meantime the update will touch zero rows (as no row with
+both PK and XMIN matching is found) even if the query itself
+syntactically succeeds.
 
 When detecting a change in a row due to XMIN being different
 one needs to be careful how to represent that to the user.
@@ -84,7 +86,8 @@ self.original_payload
 
 self.modified_payload
 - contains the modified payload just before the last
-  failure of save_payload()
+  failure of save_payload() - IOW what is currently
+  in the database
 
 self._payload
 - contains the currently active payload which may or
@@ -92,25 +95,52 @@ self._payload
 
 For discussion on this see the thread starting at:
 
-http://archives.postgresql.org/pgsql-general/2004-10/msg01352.php
+	http://archives.postgresql.org/pgsql-general/2004-10/msg01352.php
+
+and here
+
+	http://groups.google.com/group/pgsql.general/browse_thread/thread/e3566ba76173d0bf/6cf3c243a86d9233
+	(google for "XMIN semantic at peril")
+
+Problem cases with XMIN:
+
+1) not unlikely
+- a very old row is read with XMIN
+- vacuum comes along and sets XMIN to FrozenTransactionId
+  - now XMIN changed but the row actually didn't !
+- an update with "... where xmin = old_xmin ..." fails
+  although there is no need to fail
+
+2) quite unlikely
+- a row is read with XMIN
+- a long time passes
+- the original XMIN gets frozen to FrozenTransactionId
+- another writer comes along and changes the row
+- incidentally the exact same old row gets the old XMIN *again*
+  - now XMIN is (again) the same but the data changed !
+- a later update fails to detect the concurrent change !!
+
+TODO:
+The solution is to use our own column for optimistic locking
+which gets updated by an AFTER UPDATE trigger.
 """
 #============================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/pycommon/gmBusinessDBObject.py,v $
-# $Id: gmBusinessDBObject.py,v 1.42 2007/05/21 14:47:22 ncq Exp $
-__version__ = "$Revision: 1.42 $"
+# $Id: gmBusinessDBObject.py,v 1.48.2.2 2008/10/16 08:27:58 ncq Exp $
+__version__ = "$Revision: 1.48.2.2 $"
 __author__ = "K.Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL"
 
-import sys, copy, types, inspect
+import sys, copy, types, inspect, logging
 
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmExceptions, gmLog, gmPG2
+from Gnumed.pycommon import gmExceptions, gmPG2
 
 
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lInfo, __version__)
+_log = logging.getLogger('gm.database')
+_log.info(__version__)
 #============================================================
 class cBusinessDBObject:
 	"""Represents business objects in the database.
@@ -151,13 +181,14 @@ class cBusinessDBObject:
 		"""Init business object.
 		"""
 		self._is_modified = False
+		self._payload = []		# the cache for backend object values (mainly table fields)
+		self._ext_cache = {}	# the cache for extended method's results
+		self._idx = {}
+		self.pk_obj = '<?>'
 		# check descendants
 		self.__class__._cmd_fetch_payload
 		self.__class__._cmds_store_payload
 		self.__class__._updatable_fields
-		self._payload = []		# the cache for backend object values (mainly table fields)
-		self._ext_cache = {}	# the cache for extended method's results
-		self._idx = {}
 
 		if aPK_obj is not None:
 			self.__init_from_pk(aPK_obj=aPK_obj)
@@ -203,11 +234,11 @@ class cBusinessDBObject:
 			self._payload = row['data']
 			self.pk_obj = self._payload[self._idx[row['pk_field']]]
 		except:
-			_log.LogException('faulty <row> argument structure: %s' % row, sys.exc_info())
+			_log.exception('faulty <row> argument structure: %s' % row)
 			raise gmExceptions.ConstructorError, "[%s:??]: error loading instance from row data" % self.__class__.__name__
 		if len(self._idx.keys()) != len(self._payload):
-			_log.Log(gmLog.lPanic, 'field index vs. payload length mismatch: %s field names vs. %s fields' % (len(self._idx.keys()), len(self._payload)))
-			_log.LogException('faulty <row> argument structure: %s' % row, sys.exc_info())
+			_log.critical('field index vs. payload length mismatch: %s field names vs. %s fields' % (len(self._idx.keys()), len(self._payload)))
+			_log.critical('faulty <row> argument structure: %s' % row)
 			raise gmExceptions.ConstructorError, "[%s:??]: error loading instance from row data" % self.__class__.__name__
 		self.original_payload = {}
 		for field in self._idx.keys():
@@ -216,14 +247,17 @@ class cBusinessDBObject:
 	def __del__(self):
 		if self.__dict__.has_key('_is_modified'):
 			if self._is_modified:
-				_log.Log(gmLog.lPanic, '[%s:%s]: loosing payload changes' % (self.__class__.__name__, self.pk_obj))
-				_log.Log(gmLog.lData, 'original: %s' % self.original_payload)
-				_log.Log(gmLog.lData, 'modified: %s' % self._payload)
+				_log.critical('[%s:%s]: loosing payload changes' % (self.__class__.__name__, self.pk_obj))
+				_log.debug('original: %s' % self.original_payload)
+				_log.debug('modified: %s' % self._payload)
 	#--------------------------------------------------------
 	def __str__(self):
 		tmp = []
-		[tmp.append('%s: %s' % (attr, self._payload[self._idx[attr]])) for attr in self._idx.keys()]
-		return '[%s:%s]: %s' % (self.__class__.__name__, self.pk_obj, str(tmp))
+		try:
+			[ tmp.append('%s: %s' % (attr, self._payload[self._idx[attr]])) for attr in self._idx.keys() ]
+			return '[%s:%s]: %s' % (self.__class__.__name__, self.pk_obj, str(tmp))
+		except:
+			return 'nascent [%s @ %s], cannot show payload and primary key' %(self.__class__.__name__, id(self))
 	#--------------------------------------------------------
 	def __getitem__(self, attribute):
 		# use try: except: as it is faster and we want this as fast as possible
@@ -237,11 +271,11 @@ class cBusinessDBObject:
 		# 2) extension method results ...
 		getter = getattr(self, 'get_%s' % attribute, None)
 		if not callable(getter):
-			_log.Log(gmLog.lWarn, '[%s]: no attribute [%s]' % (self.__class__.__name__, attribute))
-			_log.Log(gmLog.lWarn, '[%s]: valid attributes: %s' % (self.__class__.__name__, str(self._idx.keys())))
-			_log.Log(gmLog.lWarn, '[%s]: no getter method [get_%s]' % (self.__class__.__name__, attribute))
+			_log.warning('[%s]: no attribute [%s]' % (self.__class__.__name__, attribute))
+			_log.warning('[%s]: valid attributes: %s' % (self.__class__.__name__, str(self._idx.keys())))
+			_log.warning('[%s]: no getter method [get_%s]' % (self.__class__.__name__, attribute))
 			methods = filter(lambda x: x[0].startswith('get_'), inspect.getmembers(self, inspect.ismethod))
-			_log.Log(gmLog.lWarn, '[%s]: valid getter methods: %s' % (self.__class__.__name__, str(methods)))
+			_log.warning('[%s]: valid getter methods: %s' % (self.__class__.__name__, str(methods)))
 			raise gmExceptions.NoSuchBusinessObjectAttributeError, '[%s]: cannot access [%s]' % (self.__class__.__name__, attribute)
 
 		self._ext_cache[attribute] = getter()
@@ -257,8 +291,8 @@ class cBusinessDBObject:
 					self._is_modified = True
 				return
 			except KeyError:
-				_log.Log(gmLog.lWarn, '[%s]: cannot set attribute <%s> despite marked settable' % (self.__class__.__name__, attribute))
-				_log.Log(gmLog.lWarn, '[%s]: supposedly settable attributes: %s' % (self.__class__.__name__, str(self.__class__._updatable_fields)))
+				_log.warning('[%s]: cannot set attribute <%s> despite marked settable' % (self.__class__.__name__, attribute))
+				_log.warning('[%s]: supposedly settable attributes: %s' % (self.__class__.__name__, str(self.__class__._updatable_fields)))
 				raise gmExceptions.NoSuchBusinessObjectAttributeError, '[%s]: cannot access [%s]' % (self.__class__.__name__, attribute)
 
 		# 2) setters providing extensions
@@ -280,10 +314,10 @@ class cBusinessDBObject:
 				return
 
 		# 3) don't know what to do with <attribute>
-		_log.Log(gmLog.lErr, '[%s]: cannot find attribute <%s> or setter method [set_%s]' % (self.__class__.__name__, attribute, attribute))
-		_log.Log(gmLog.lWarn, '[%s]: settable attributes: %s' % (self.__class__.__name__, str(self.__class__._updatable_fields)))
+		_log.error('[%s]: cannot find attribute <%s> or setter method [set_%s]' % (self.__class__.__name__, attribute, attribute))
+		_log.warning('[%s]: settable attributes: %s' % (self.__class__.__name__, str(self.__class__._updatable_fields)))
 		methods = filter(lambda x: x[0].startswith('set_'), inspect.getmembers(self, inspect.ismethod))
-		_log.Log(gmLog.lWarn, '[%s]: valid setter methods: %s' % (self.__class__.__name__, str(methods)))
+		_log.warning('[%s]: valid setter methods: %s' % (self.__class__.__name__, str(methods)))
 		raise gmExceptions.BusinessObjectAttributeNotSettableError, '[%s]: cannot set [%s]' % (self.__class__.__name__, attribute)
 	#--------------------------------------------------------
 	# external API
@@ -298,14 +332,14 @@ class cBusinessDBObject:
 		return self.__class__._updatable_fields
 	#--------------------------------------------------------
 	def get_patient(self):
-		_log.Log(gmLog.lErr, '[%s:%s]: forgot to override get_patient()' % (self.__class__.__name__, self.pk_obj))
+		_log.error('[%s:%s]: forgot to override get_patient()' % (self.__class__.__name__, self.pk_obj))
 		return None
 	#--------------------------------------------------------
 	def refetch_payload(self):
 		"""Fetch field values from backend.
 		"""
 		if self._is_modified:
-			_log.Log(gmLog.lPanic, '[%s:%s]: cannot reload, payload changed' % (self.__class__.__name__, self.pk_obj))
+			_log.critical('[%s:%s]: cannot reload, payload changed' % (self.__class__.__name__, self.pk_obj))
 			return False
 		if type(self.pk_obj) == types.DictType:
 			arg = self.pk_obj
@@ -316,7 +350,7 @@ class cBusinessDBObject:
 			get_col_idx = True
 		)
 		if len(rows) == 0:
-			_log.Log(gmLog.lErr, '[%s:%s]: no such instance' % (self.__class__.__name__, self.pk_obj))
+			_log.error('[%s:%s]: no such instance' % (self.__class__.__name__, self.pk_obj))
 			return False
 		self._payload = rows[0]
 		return True
@@ -365,12 +399,11 @@ class cBusinessDBObject:
 			except KeyError:
 				conn.rollback()
 				close_conn()
-				_log.Log(gmLog.lErr, '[%s:%s]: cannot update instance, XMIN refetch key mismatch on [%s]' % (self.__class__.__name__, self.pk_obj, key))
-				_log.Log(gmLog.lErr, 'payload keys: %s' % str(self._idx))
-				_log.Log(gmLog.lErr, 'XMIN refetch keys: %s' % str(idx))
-				_log.Log(gmLog.lErr, args)
-				# FIXME: turn into proper exception
-				return (False, None)
+				_log.error('[%s:%s]: cannot update instance, XMIN refetch key mismatch on [%s]' % (self.__class__.__name__, self.pk_obj, key))
+				_log.error('payload keys: %s' % str(self._idx))
+				_log.error('XMIN refetch keys: %s' % str(idx))
+				_log.error(args)
+				raise
 
 		conn.commit()
 		close_conn()
@@ -384,11 +417,6 @@ class cBusinessDBObject:
 		return (True, None)
 #============================================================
 if __name__ == '__main__':
-	from Gnumed.pycommon import gmI18N
-	gmI18N.activate_locale()
-	gmI18N.install_domain()
-
-	_log.SetAllLogLevels(gmLog.lData)
 	#--------------------------------------------------------
 	class cTestObj(cBusinessDBObject):
 		_cmd_fetch_payload = None
@@ -401,17 +429,47 @@ if __name__ == '__main__':
 		def set_something(self):
 			pass
 	#--------------------------------------------------------
-	data = {
-		'pk_field': 'bogus_pk',
-		'idx': {'bogus_pk': 0, 'bogus_field': 1},
-		'data': [-1, 'bogus_data']
-	}
-	obj = cTestObj(row=data)
-#	print obj['wrong_field']
-	obj['wrong_field'] = 1
+	if len(sys.argv) > 1 and sys.argv[1] == u'test':
+
+		from Gnumed.pycommon import gmI18N
+		gmI18N.activate_locale()
+		gmI18N.install_domain()
+
+		data = {
+			'pk_field': 'bogus_pk',
+			'idx': {'bogus_pk': 0, 'bogus_field': 1},
+			'data': [-1, 'bogus_data']
+		}
+		obj = cTestObj(row=data)
+		#print obj['wrong_field']
+		obj['wrong_field'] = 1
 
 #============================================================
 # $Log: gmBusinessDBObject.py,v $
+# Revision 1.48.2.2  2008/10/16 08:27:58  ncq
+# - make __str__ robust in the face of very early failure
+#
+# Revision 1.48.2.1  2008/10/14 11:03:55  ncq
+# - init instance variables needed by __str__ when failing early
+#
+# Revision 1.48  2007/12/12 16:17:15  ncq
+# - better logger names
+#
+# Revision 1.47  2007/12/11 14:17:18  ncq
+# - use stdlib logging
+#
+# Revision 1.46  2007/11/28 13:58:32  ncq
+# - hide one less exception
+#
+# Revision 1.45  2007/10/19 12:49:39  ncq
+# - much improved XMIN docs, TODO for XMIN removal
+#
+# Revision 1.44  2007/10/12 07:26:25  ncq
+# - somewhat improved docs
+#
+# Revision 1.43  2007/08/13 21:55:10  ncq
+# - fix logging statement
+#
 # Revision 1.42  2007/05/21 14:47:22  ncq
 # - no caching of get_*()ers anymore, but don't deprecate them eiter
 #
