@@ -1,11 +1,10 @@
-#===========================================================================
 __doc__ = """
 GNUmed date/time handling.
 
 This modules provides access to date/time handling
 and offers an fuzzy timestamp implementation
 
-It utilitzes
+It utilizes
 
 	- Python time
 	- Python datetime
@@ -15,7 +14,7 @@ Note that if you want locale-aware formatting you need to call
 
 	locale.setlocale(locale.LC_ALL, '')
 
-somehwere before importing this script.
+somewhere before importing this script.
 
 Note regarding UTC offsets
 --------------------------
@@ -35,14 +34,14 @@ This is useful in fields such as medicine where only partial
 timestamps may be known for certain events.
 """
 #===========================================================================
-# $Id: gmDateTime.py,v 1.14 2007/09/04 23:28:06 ncq Exp $
+# $Id: gmDateTime.py,v 1.21.2.3 2008/11/20 18:24:42 ncq Exp $
 # $Source: /sources/gnumed/gnumed/gnumed/client/pycommon/gmDateTime.py,v $
-__version__ = "$Revision: 1.14 $"
+__version__ = "$Revision: 1.21.2.3 $"
 __author__ = "K. Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL (details at http://www.gnu.org)"
 
 # stdlib
-import sys, datetime as pyDT, time, os, re as regex, locale
+import sys, datetime as pyDT, time, os, re as regex, locale, logging
 
 
 # 3rd party
@@ -50,23 +49,28 @@ import mx.DateTime as mxDT
 import psycopg2						# this will go once datetime has timezone classes
 
 
-# GNUmed libs
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmLog, gmI18N
-
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lInfo, __version__)
+from Gnumed.pycommon import gmI18N
 
 
+_log = logging.getLogger('gm.datetime')
+_log.info(__version__)
+
+
+dst_locally_in_use = None
 dst_currently_in_effect = None
-current_utc_offset = None
-current_timezone_interval = None
-current_iso_timezone_string = None
+
+current_local_utc_offset_in_seconds = None
+current_local_timezone_interval = None
+current_local_iso_numeric_timezone_string = None
+current_local_timezone_name = None
+
 cLocalTimezone = psycopg2.tz.LocalTimezone					# remove as soon as datetime supports timezone classes
 cFixedOffsetTimezone = psycopg2.tz.FixedOffsetTimezone		# remove as soon as datetime supports timezone classes
 gmCurrentLocalTimezone = 'gmCurrentLocalTimezone not initialized'
 
+
 (	acc_years,
 	acc_months,
 	acc_days,
@@ -104,61 +108,77 @@ month_length = {
 #===========================================================================
 def init():
 
-	_log.Log(gmLog.lData, 'mx.DateTime.now(): [%s]' % mxDT.now())
-	_log.Log(gmLog.lData, 'datetime.now()   : [%s]' % pyDT.datetime.now())
-	_log.Log(gmLog.lData, 'time.localtime() : [%s]' % str(time.localtime()))
-	_log.Log(gmLog.lData, 'time.gmtime()    : [%s]' % str(time.gmtime()))
+	_log.debug('mx.DateTime.now(): [%s]' % mxDT.now())
+	_log.debug('datetime.now()   : [%s]' % pyDT.datetime.now())
+	_log.debug('time.localtime() : [%s]' % str(time.localtime()))
+	_log.debug('time.gmtime()    : [%s]' % str(time.gmtime()))
 
 	try:
-		_log.Log(gmLog.lData, '$TZ: [%s]' % os.environ['TZ'])
+		_log.debug('$TZ: [%s]' % os.environ['TZ'])
 	except KeyError:
-		_log.Log(gmLog.lData, '$TZ not defined')
+		_log.debug('$TZ not defined')
 
-	_log.Log(gmLog.lData, 'time.daylight: [%s] (whether or not DST is locally used at all)' % time.daylight)
-	_log.Log(gmLog.lData, 'time.timezone: [%s] seconds' % time.timezone)
-	_log.Log(gmLog.lData, 'time.altzone : [%s] seconds' % time.altzone)
-	_log.Log(gmLog.lData, 'time.tzname  : [%s / %s] (non-DST / DST)' % time.tzname)
-	_log.Log(gmLog.lData, 'mx.DateTime.now().gmtoffset(): [%s]' % mxDT.now().gmtoffset())
+	_log.debug('time.daylight: [%s] (whether or not DST is locally used at all)' % time.daylight)
+	_log.debug('time.timezone: [%s] seconds' % time.timezone)
+	_log.debug('time.altzone : [%s] seconds' % time.altzone)
+	_log.debug('time.tzname  : [%s / %s] (non-DST / DST)' % time.tzname)
+	_log.debug('mx.DateTime.now().gmtoffset(): [%s]' % mxDT.now().gmtoffset())
+
+	global dst_locally_in_use
+	dst_locally_in_use = (time.daylight != 0)
 
 	global dst_currently_in_effect
 	dst_currently_in_effect = bool(time.localtime()[8])
-	_log.Log(gmLog.lData, 'DST currently in effect: [%s]' % dst_currently_in_effect)
+	_log.debug('DST currently in effect: [%s]' % dst_currently_in_effect)
 
-	global current_utc_offset
-	msg = 'DST currently%sin effect, using UTC offset of [%s] seconds instead of [%s] seconds'
+	if (not dst_locally_in_use) and dst_currently_in_effect:
+		_log.error('system inconsistency: DST not in use - but DST currently in effect ?')
+
+	global current_local_utc_offset_in_seconds
+	msg = 'DST currently%sin effect: using UTC offset of [%s] seconds instead of [%s] seconds'
 	if dst_currently_in_effect:
-		current_utc_offset = time.altzone * -1
-		_log.Log(gmLog.lData, msg % (' ', time.altzone * -1, time.timezone * -1))
+		current_local_utc_offset_in_seconds = time.altzone * -1
+		_log.debug(msg % (' ', time.altzone * -1, time.timezone * -1))
 	else:
-		current_utc_offset = time.timezone * -1
-		_log.Log(gmLog.lData, msg % (' not ', time.timezone * -1, time.altzone * -1))
+		current_local_utc_offset_in_seconds = time.timezone * -1
+		_log.debug(msg % (' not ', time.timezone * -1, time.altzone * -1))
 
-	if current_utc_offset > 0:
-		_log.Log(gmLog.lData, 'UTC offset is positive, assuming EAST of Greenwich ("clock is ahead")')
-	elif current_utc_offset < 0:
-		_log.Log(gmLog.lData, 'UTC offset is negative, assuming WEST of Greenwich ("clock is behind")')
+	if current_local_utc_offset_in_seconds > 0:
+		_log.debug('UTC offset is positive, assuming EAST of Greenwich ("clock is ahead")')
+	elif current_local_utc_offset_in_seconds < 0:
+		_log.debug('UTC offset is negative, assuming WEST of Greenwich ("clock is behind")')
 	else:
-		_log.Log(gmLog.lData, 'UTC offset is ZERO, assuming Greenwich Time')
+		_log.debug('UTC offset is ZERO, assuming Greenwich Time')
 
-	global current_timezone_interval
-	current_timezone_interval = mxDT.now().gmtoffset()
-	_log.Log(gmLog.lData, 'ISO timezone: [%s] (taken from mx.DateTime.now().gmtoffset())' % current_timezone_interval)
-	global current_iso_timezone_string
-	current_iso_timezone_string = str(current_timezone_interval).replace(',', '.')
+	global current_local_timezone_interval
+	current_local_timezone_interval = mxDT.now().gmtoffset()
+	_log.debug('ISO timezone: [%s] (taken from mx.DateTime.now().gmtoffset())' % current_local_timezone_interval)
+
+	global current_local_iso_numeric_timezone_string
+	current_local_iso_numeric_timezone_string = str(current_local_timezone_interval).replace(',', '.')
+
+	global current_local_timezone_name
+	try:
+		current_local_timezone_name = os.environ['TZ'].decode(gmI18N.get_encoding(), 'replace')
+	except KeyError:
+		if dst_currently_in_effect:
+			current_local_timezone_name = time.tzname[1]
+		else:
+			current_local_timezone_name = time.tzname[0]
 
 	# do some magic to convert Python's timezone to a valid ISO timezone
 	# is this safe or will it return things like 13.5 hours ?
 	#_default_client_timezone = "%+.1f" % (-tz / 3600.0)
-	#_log.Log(gmLog.lInfo, 'assuming default client time zone of [%s]' % _default_client_timezone)
+	#_log.info('assuming default client time zone of [%s]' % _default_client_timezone)
 
 	global gmCurrentLocalTimezone
 	gmCurrentLocalTimezone = cFixedOffsetTimezone (
-		offset = (current_utc_offset / 60),
-		name = current_iso_timezone_string
+		offset = (current_local_utc_offset_in_seconds / 60),
+		name = current_local_iso_numeric_timezone_string
 	)
 
 #===========================================================================
-def __explicit_offset(str2parse, offset_chars='hdwmy'):
+def __explicit_offset(str2parse, offset_chars=None):
 	"""
 			Default is 'hdwm':
 			h - hours
@@ -169,6 +189,9 @@ def __explicit_offset(str2parse, offset_chars='hdwmy'):
 
 		This also defines the significance of the order of the characters.
 	"""
+	if offset_chars is None:
+		offset_chars = _('hdwmy (single character date offset triggers)')[:5].lower()
+
 	# "+/-XXd/w/m/t"
 	if not regex.match(u"^(\s|\t)*(\+|-)?(\s|\t)*\d{1,2}(\s|\t)*[%s](\s|\t)*$" % offset_chars, str2parse, flags = regex.LOCALE | regex.UNICODE):
 		return []
@@ -239,7 +262,7 @@ def __explicit_offset(str2parse, offset_chars='hdwmy'):
 	}
 	return [tmp]
 #---------------------------------------------------------------------------
-def __single_char(str2parse, trigger_chars='ndmy'):
+def __single_char(str2parse, trigger_chars=None):
 	"""This matches on single characters.
 
 	Spaces and tabs are discarded.
@@ -252,6 +275,9 @@ def __single_char(str2parse, trigger_chars='ndmy'):
 
 	This also defines the significance of the order of the characters.
 	"""
+	if trigger_chars is None:
+		trigger_chars = _('ndmy (single character date triggers)')[:4].lower()
+
 	if not regex.match(u'^(\s|\t)*[%s]{1}(\s|\t)*$' % trigger_chars, str2parse, flags = regex.LOCALE | regex.UNICODE):
 		return []
 	val = str2parse.strip().lower()
@@ -666,7 +692,7 @@ def __single_dot(str2parse):
 
 	return matches
 #---------------------------------------------------------------------------
-def str2fuzzy_timestamp_matches(str2parse=None, default_time=None):
+def str2fuzzy_timestamp_matches(str2parse=None, default_time=None, patterns=None):
 	"""
 	Turn a string into candidate fuzzy timestamps and auto-completions the user is likely to type.
 
@@ -674,9 +700,12 @@ def str2fuzzy_timestamp_matches(str2parse=None, default_time=None):
 	somewhere in your code previously.
 
 	@param default_time: if you want to force the time part of the time
-		stamp to a given value and the user doesn't type and time part
+		stamp to a given value and the user doesn't type any time part
 		this value will be used
 	@type default_time: an mx.DateTime.DateTimeDelta instance
+
+	@param patterns: list of [time.strptime compatible date/time pattern, accuracy]
+	@type patterns: list
 	"""
 	matches = __single_dot(str2parse)
 	matches.extend(__numbers_only(str2parse))
@@ -693,8 +722,8 @@ def str2fuzzy_timestamp_matches(str2parse=None, default_time=None):
 				formats = ('euro', 'iso', 'us', 'altus', 'altiso', 'lit', 'altlit', 'eurlit')
 			)
 			# time, too ?
-			time = mxDT.Parser.TimeFromString(text = str2parse)
-			datetime = date_only + time
+			time_part = mxDT.Parser.TimeFromString(text = str2parse)
+			datetime = date_only + time_part
 			if datetime == date_only:
 				accuracy = acc_days
 				if isinstance(default_time, mxDT.DateTimeDeltaType):
@@ -713,6 +742,30 @@ def str2fuzzy_timestamp_matches(str2parse=None, default_time=None):
 		except (ValueError, mxDT.RangeError):
 			pass
 
+	if patterns is None:
+		patterns = []
+
+	patterns.append(['%Y.%m.%d', acc_days])
+	patterns.append(['%Y/%m/%d', acc_days])
+
+	for pattern in patterns:
+		try:
+			fts = cFuzzyTimestamp (
+				timestamp = pyDT.datetime.fromtimestamp(time.mktime(time.strptime(str2parse, pattern[0]))),
+				accuracy = pattern[1]
+			)
+			matches.append ({
+				'data': fts,
+				'label': fts.format_accurately()
+			})
+		except AttributeError:
+			# strptime() only available starting with Python 2.5
+			break
+		except ValueError:
+			continue
+		except OverflowError:
+			continue
+
 	return matches
 
 #===========================================================================
@@ -848,14 +901,12 @@ class cFuzzyTimestamp:
 #---------------------------------------------------------------------------
 if __name__ == '__main__':
 
-	_log.SetAllLogLevels(gmLog.lData)
-
 	#-------------------------------------------------
 	def test_date_time():
 		print "DST currently in effect:", dst_currently_in_effect
-		print "current UTC offset:", current_utc_offset, "seconds"
-		print "current timezone (interval):", current_timezone_interval
-		print "current timezone (ISO conformant string):", current_iso_timezone_string
+		print "current UTC offset:", current_local_utc_offset_in_seconds, "seconds"
+		print "current timezone (interval):", current_local_timezone_interval
+		print "current timezone (ISO conformant numeric string):", current_local_iso_numeric_timezone_string
 		print "local timezone class:", cLocalTimezone
 		print ""
 		tz = cLocalTimezone()
@@ -917,19 +968,53 @@ if __name__ == '__main__':
 		print "fts           :", fts
 		print "fts.get_pydt():", fts.get_pydt()
 	#-------------------------------------------------
-	if len(sys.argv) > 0 and sys.argv[1] == "test":
+	if len(sys.argv) > 1 and sys.argv[1] == "test":
+
+		# GNUmed libs
 		gmI18N.activate_locale()
 		gmI18N.install_domain('gnumed')
 
 		init()
 
 		#test_date_time()
-		#test_str2fuzzy_timestamp_matches()
+		test_str2fuzzy_timestamp_matches()
 		#test_cFuzzyTimeStamp()
-		test_get_pydt()
+		#test_get_pydt()
 
 #===========================================================================
 # $Log: gmDateTime.py,v $
+# Revision 1.21.2.3  2008/11/20 18:24:42  ncq
+# - fix spelling
+#
+# Revision 1.21.2.2  2008/11/11 21:01:08  ncq
+# - need to decode $TZ
+#
+# Revision 1.21.2.1  2008/10/14 10:54:20  ncq
+# - handle OverflowError in time.mktime
+#
+# Revision 1.21  2008/06/18 15:28:32  ncq
+# - properly i18n trigger chars in str 2 timestamp conversions
+# - document "patterns" arg for str 2 timestamp conversion
+#
+# Revision 1.20  2008/05/19 15:45:26  ncq
+# - re-adjust timezone handling code
+# - remember timezone *name* for PG
+#
+# Revision 1.19  2008/04/12 22:30:46  ncq
+# - support more date/time patterns
+#
+# Revision 1.18  2008/01/13 01:14:26  ncq
+# - does need gmI18N
+#
+# Revision 1.17  2008/01/05 16:37:47  ncq
+# - typo fix
+#
+# Revision 1.16  2007/12/12 16:17:15  ncq
+# - better logger names
+#
+# Revision 1.15  2007/12/11 14:18:20  ncq
+# - stdlib logging
+#
 # Revision 1.14  2007/09/04 23:28:06  ncq
 # - document what's happening
 #
