@@ -1,4 +1,4 @@
-"""GnuMed client internal signal handling.
+"""GNUmed client internal signal handling.
 
 # this code has been written by Patrick O'Brien <pobrien@orbtech.com>
 # downloaded from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/87056
@@ -8,6 +8,26 @@ import types
 import sys
 import weakref
 import traceback
+import logging
+
+
+wx_core_PyDeadObjectError = None
+
+
+known_signals = [
+	u'pre_patient_selection',
+	u'post_patient_selection',
+	u'patient_locked',
+	u'patient_unlocked',
+	u'import_document_from_file',
+	u'statustext',					# args: msg=message, beep=whether to beep or not
+	u'display_widget',				# args: name=name of widget, other=widget specific (see receivers)
+	u'application_closing',
+	u'request_user_attention',
+	u'register_pre_exit_callback'	# args: callback = function to call
+]
+
+_log = logging.getLogger('gm.messaging')
 
 connections = {}
 senders = {}
@@ -18,6 +38,8 @@ class _Any:
 	pass
 
 Any = _Any()
+
+known_signals.append(Any)
 #=====================================================================
 class DispatcherError(exceptions.Exception):
 	def __init__(self, args=None):
@@ -25,7 +47,7 @@ class DispatcherError(exceptions.Exception):
 #=====================================================================
 # external API
 #---------------------------------------------------------------------
-def connect(receiver, signal=Any, sender=Any, weak=1):
+def connect(receiver=None, signal=Any, sender=Any, weak=1):
 	"""Connect receiver to sender for signal.
 	
 	If sender is Any, receiver will receive signal from any sender.
@@ -59,11 +81,18 @@ def connect(receiver, signal=Any, sender=Any, weak=1):
 		Especially if the widget can get a reference to updated data through
 		a global reference, such as via gmCurrentPatient.
 """
-	if signal is None:
-		raise DispatcherError, 'signal cannot be None'
+	if receiver is None:
+		raise ValueError('gmDispatcher.connect(): must define <receiver>')
+
+	if signal not in known_signals:
+		_log.error('unknown signal [%(sig)s]', {'sig': signal})
+		print "DISPATCHER ERROR: connect(): unknown signal [%s]" % signal
+
 	if signal is not Any:
 		signal = str(signal)
-	if weak: receiver = safeRef(receiver)
+
+	if weak:
+		receiver = safeRef(receiver)
 	senderkey = id(sender)
 	signals = {}
 	if connections.has_key(senderkey):
@@ -95,8 +124,10 @@ def disconnect(receiver, signal=Any, sender=Any, weak=1):
 	
 	Disconnecting is not required. The use of disconnect is the same as for
 	connect, only in reverse. Think of it as undoing a previous connection."""
-	if signal is None:
-		raise DispatcherError, 'signal cannot be None'
+	if signal not in known_signals:
+		_log.error('unknown signal [%(sig)s]', {'sig': signal})
+		print "DISPATCHER ERROR: disconnect(): unknown signal [%s]" % signal
+
 	if signal is not Any:
 		signal = str(signal)
 	if weak: receiver = safeRef(receiver)
@@ -104,19 +135,26 @@ def disconnect(receiver, signal=Any, sender=Any, weak=1):
 	try:
 		receivers = connections[senderkey][signal]
 	except KeyError:
+		_log.error('no receivers for signal %(sig)s from sender %(sender)s', {'sig': repr(signal), 'sender': sender})
 		print 'DISPATCHER ERROR: no receivers for signal %s from sender %s' % (repr(signal), sender)
 		return
 	try:
 		receivers.remove(receiver)
 	except ValueError:
+		_log.error('receiver [%(rx)s] not connected to signal [%(sig)s] from [%(sender)s]', {'rx': receiver, 'sig': repr(signal), 'sender': sender})
 		print "DISPATCHER ERROR: receiver [%s] not connected to signal [%s] from [%s]" % (receiver, repr(signal), sender)
 	_cleanupConnections(senderkey, signal)
 #---------------------------------------------------------------------
-def send(signal, sender=None, **kwds):
+def send(signal=None, sender=None, **kwds):
 	"""Send signal from sender to all connected receivers.
 	
 	Return a list of tuple pairs [(receiver, response), ... ].
-	If sender is None, signal is sent anonymously."""
+	If sender is None, signal is sent anonymously.
+	"""
+	if signal not in known_signals:
+		_log.error('unknown signal [%(sig)s]', {'sig': signal})
+		print "DISPATCHER ERROR: send(): unknown signal [%s]" % signal
+
 	signal = str(signal)
 	senderkey = id(sender)
 	anykey = id(Any)
@@ -149,8 +187,7 @@ def send(signal, sender=None, **kwds):
 	# Return a list of tuple pairs [(receiver, response), ... ].
 	responses = []
 	for receiver in receivers:
-		if type(receiver) is weakref.ReferenceType \
-		or isinstance(receiver, BoundMethodWeakref):
+		if (type(receiver) is weakref.ReferenceType) or (isinstance(receiver, BoundMethodWeakref)):
 			# Dereference the weak reference.
 			receiver = receiver()
 			if receiver is None:
@@ -160,12 +197,11 @@ def send(signal, sender=None, **kwds):
 			response = _call(receiver, signal=signal, sender=sender, **kwds)
 			responses += [(receiver, response)]
 		except:
-			# this seems such a fundamental error that it appears reasonable
-			# to print directly to the console instead of making the whole
-			# module depend on gmLog.py
+			# this seems such a fundamental error that it appears
+			# reasonable to print directly to the console
 			typ, val, tb = sys.exc_info()
-			print 'DISPATCHER ERROR: %s <%s>' % (typ, val)
-			print 'DISPATCHER ERROR: calling <%s> failed' % str(receiver)
+			_log.critical('%(t)s, <%(v)s>', {'t': typ, 'v': val})
+			_log.critical('calling <%(rx)s> failed', {'rx': str(receiver)})
 			traceback.print_tb(tb)
 	return responses
 #---------------------------------------------------------------------
@@ -204,12 +240,22 @@ class BoundMethodWeakref:
 	#------------------------------------------------------------------
 	def __call__(self):
 		"""Return a strong reference to the bound method."""
+
+		global wx_core_PyDeadObjectError
+		if wx_core_PyDeadObjectError is None:
+			from wx._core import PyDeadObjectError as wx_core_PyDeadObjectError
+
 		if self.isDead:
 			return None
 		else:
 			object = self.weakSelf()
 			method = self.weakFunc().__name__
-			return getattr(object, method)
+			try:
+				return getattr(object, method)
+			except wx_core_PyDeadObjectError:
+				self.isDead = 1
+				_removeReceiver(receiver=self)
+				return None
 #=====================================================================
 # internal API
 #---------------------------------------------------------------------
@@ -222,18 +268,19 @@ def _call(receiver, **kwds):
 	if hasattr(receiver, 'im_func'):
 		# receiver is a method. Drop the first argument, usually 'self'.
 		fc = receiver.im_func.func_code
-		acceptable = fc.co_varnames[1:fc.co_argcount]
+		acceptable_args = fc.co_varnames[1:fc.co_argcount]
 	elif hasattr(receiver, 'func_code'):
 		# receiver is a function.
 		fc = receiver.func_code
-		acceptable = fc.co_varnames[0:fc.co_argcount]
+		acceptable_args = fc.co_varnames[0:fc.co_argcount]
 	else:
-		print 'DISPATCHER ERROR: <%s> must be instance, method or function' % str(receiver)
+		_log.error('<%(rx)s> must be instance, method or function', {'rx': str(receiver)})
+		print 'DISPATCHER ERROR: _call(): <%s> must be instance, method or function' % str(receiver)
 	if not (fc.co_flags & 8):
 		# fc does not have a **kwds type parameter, therefore 
 		# remove unacceptable arguments.
 		for arg in kwds.keys():
-			if arg not in acceptable:
+			if arg not in acceptable_args:
 				del kwds[arg]
 	return receiver(**kwds)
 #---------------------------------------------------------------------
@@ -267,6 +314,44 @@ def _removeSender(senderkey):
 
 #=====================================================================
 # $Log: gmDispatcher.py,v $
+# Revision 1.16.2.4  2008/11/26 12:17:49  ncq
+# - properly import dead object exception
+#
+# Revision 1.16.2.3  2008/11/25 12:10:46  ncq
+# - import PyDeadObjectError from wx._core (uh oh !)
+#
+# Revision 1.16.2.2  2008/10/15 14:44:19  ncq
+# - PyDeadObjectError is in wx._core
+#
+# Revision 1.16.2.1  2008/09/09 17:23:23  ncq
+# - robustify against dead python objects in weak ref calls
+#
+# Revision 1.16  2008/08/08 13:29:56  ncq
+# - add register_pre_exit_callback signal
+#
+# Revision 1.15  2008/06/28 22:33:57  ncq
+# - remove obsolete signal
+#
+# Revision 1.14  2007/12/12 16:17:15  ncq
+# - better logger names
+#
+# Revision 1.13  2007/12/11 15:35:46  ncq
+# - log, don't print, but critical
+#
+# Revision 1.12  2007/12/11 14:19:27  ncq
+# - stdlib logging
+#
+# Revision 1.11  2007/11/02 13:52:52  ncq
+# - add two signals
+#
+# Revision 1.10  2007/10/25 12:19:18  ncq
+# - allergy_updated is no more
+# - by default know signal "Any"
+#
+# Revision 1.9  2007/08/11 23:55:07  ncq
+# - register more signals
+# - report unknown signals but still pass them on
+#
 # Revision 1.8  2006/09/06 10:26:52  shilbert
 # - removed some weird EOL via dos2unix
 #
