@@ -4,21 +4,23 @@
 """
 #=======================================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/pycommon/gmMimeLib.py,v $
-# $Id: gmMimeLib.py,v 1.14 2007/10/12 14:19:18 ncq Exp $
-__version__ = "$Revision: 1.14 $"
+# $Id: gmMimeLib.py,v 1.22 2008/07/22 13:54:25 ncq Exp $
+__version__ = "$Revision: 1.22 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL"
 
-import os, mailcap, sys, mimetypes, shutil
+# stdlib
+import os, mailcap, sys, mimetypes, shutil, logging
 
 
+# GNUmed
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-import gmLog, gmShellAPI, gmTools, gmCfg
+import gmShellAPI, gmTools, gmCfg2
 
 
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lInfo, __version__)
+_log = logging.getLogger('gm.docs')
+_log.info(__version__)
 #=======================================================================================
 def guess_mimetype(aFileName = None):
 	"""Guess mime type of arbitrary file.
@@ -28,7 +30,15 @@ def guess_mimetype(aFileName = None):
 	worst_case = "application/octet-stream"
 
 	# 1) use Python libextractor
-	# - but we don't have docs for that
+	try:
+		import extractor
+		xtract = extractor.Extractor()
+		props = xtract.extract(filename = aFileName)
+		for prop, val in props:
+			if (prop == 'mimetype') and (val != worst_case):
+				return val
+	except ImportError:
+		_log.exception('Python wrapper for libextractor not installed.')
 
 	ret_code = -1
 
@@ -40,50 +50,50 @@ def guess_mimetype(aFileName = None):
 	# it might work on Cygwin installations
 	aPipe = os.popen(mime_guesser_cmd.encode(sys.getfilesystemencoding()), 'r')
 	if aPipe is None:
-		_log.Log(gmLog.lData, "cannot open pipe to [%s]" % mime_guesser_cmd)
+		_log.debug("cannot open pipe to [%s]" % mime_guesser_cmd)
 	else:
 		pipe_output = aPipe.readline().replace('\n', '').strip()
 		ret_code = aPipe.close()
 		if ret_code is None:
-			_log.Log(gmLog.lData, '[%s]: <%s>' % (mime_guesser_cmd, pipe_output))
+			_log.debug('[%s]: <%s>' % (mime_guesser_cmd, pipe_output))
 			if pipe_output not in [u'', worst_case]:
 				return pipe_output
 		else:
-			_log.Log(gmLog.lErr, '[%s] on %s (%s): failed with exit(%s)' % (mime_guesser_cmd, os.name, sys.platform, ret_code))
+			_log.error('[%s] on %s (%s): failed with exit(%s)' % (mime_guesser_cmd, os.name, sys.platform, ret_code))
 
 	# 3) use "extract" shell level libextractor wrapper
 	mime_guesser_cmd = 'extract -p mimetype "%s"' % aFileName
 	aPipe = os.popen(mime_guesser_cmd.encode(sys.getfilesystemencoding()), 'r')
 	if aPipe is None:
-		_log.Log(gmLog.lData, "cannot open pipe to [%s]" % mime_guesser_cmd)
+		_log.debug("cannot open pipe to [%s]" % mime_guesser_cmd)
 	else:
 		pipe_output = aPipe.readline()[11:].replace('\n', '').strip()
 		ret_code = aPipe.close()
 		if ret_code is None:
-			_log.Log(gmLog.lData, '[%s]: <%s>' % (mime_guesser_cmd, pipe_output))
+			_log.debug('[%s]: <%s>' % (mime_guesser_cmd, pipe_output))
 			if pipe_output not in [u'', worst_case]:
 				return pipe_output
 		else:
-			_log.Log(gmLog.lErr, '[%s] on %s (%s): failed with exit(%s)' % (mime_guesser_cmd, os.name, sys.platform, ret_code))
+			_log.error('[%s] on %s (%s): failed with exit(%s)' % (mime_guesser_cmd, os.name, sys.platform, ret_code))
 
-	# if we and up here we either have an insufficient systemwide
+	# If we and up here we either have an insufficient systemwide
 	# magic number file or we suffer from a deficient operating system
-	# altogether, it can't get much worse if we try ourselves
+	# alltogether. It can't get much worse if we try ourselves.
 
-	_log.Log(gmLog.lInfo, "OS level mime detection failed, falling back to built-in magic")
+	_log.info("OS level mime detection failed, falling back to built-in magic")
 
 	import gmMimeMagic
 	mime_type = gmTools.coalesce(gmMimeMagic.file(aFileName), worst_case)
 	del gmMimeMagic
 
-	_log.Log(gmLog.lData, '"%s" -> <%s>' % (aFileName, mime_type))
+	_log.debug('"%s" -> <%s>' % (aFileName, mime_type))
 	return mime_type
 #-----------------------------------------------------------------------------------
 def get_viewer_cmd(aMimeType = None, aFileName = None, aToken = None):
 	"""Return command for viewer for this mime type complete with this file"""
 
 	if aFileName is None:
-		_log.Log(gmLog.lErr, "You should specify a file name for the replacement of %s.")
+		_log.error("You should specify a file name for the replacement of %s.")
 		# last resort: if no file name given replace %s in original with literal '%s'
 		# and hope for the best - we certainly don't want the module default "/dev/null"
 		aFileName = """%s"""
@@ -92,40 +102,34 @@ def get_viewer_cmd(aMimeType = None, aFileName = None, aToken = None):
 	(viewer, junk) = mailcap.findmatch(mailcaps, aMimeType, key = 'view', filename = '%s' % aFileName)
 	# FIXME: we should check for "x-token" flags
 
-	_log.Log(gmLog.lData, "<%s> viewer: [%s]" % (aMimeType, viewer))
+	_log.debug("<%s> viewer: [%s]" % (aMimeType, viewer))
 
 	return viewer
 #-----------------------------------------------------------------------------------
 def guess_ext_by_mimetype(mimetype=''):
 	"""Return file extension based on what the OS thinks a file of this mimetype should end in."""
 
-	# FIXME: does this screw up with scope of binding vs. scope of import ?
+	# ask system first
 	ext = mimetypes.guess_extension(mimetype)
 	if ext is not None:
-		_log.Log(gmLog.lData, '<%s>: *.%s' % (mimetype, ext))
+		_log.debug('<%s>: *.%s' % (mimetype, ext))
 		return ext
 
-	_log.Log(gmLog.lErr, "<%s>: no suitable file extension known to the OS" % mimetype)
+	_log.error("<%s>: no suitable file extension known to the OS" % mimetype)
 
 	# try to help the OS a bit
-	fname = u'mime_type2file_extension.conf'
-	paths = gmTools.gmPaths()
-	candidates = [
-		os.path.join(paths.user_config_dir, fname),
-		os.path.join(paths.system_config_dir, fname)
-	]
+	cfg = gmCfg2.gmCfgData()
+	ext = cfg.get (
+		group = u'extensions',
+		option = mimetype,
+		source_order = [('user-mime', 'return'), ('system-mime', 'return')]
+	)
 
-	for candidate in candidates:
-		try:
-			cfg = gmCfg.cCfgFile(aFile = candidate)
-		except IOError:
-			continue
-		ext = cfg.get('extensions', mimetype)
-		if ext is not None:
-			_log.Log(gmLog.lData, '<%s>: *.%s (%s)' % (mimetype, ext, candidate))
-			return ext
+	if ext is not None:
+		_log.debug('<%s>: *.%s (%s)' % (mimetype, ext, candidate))
+		return ext
 
-	_log.Log(gmLog.lErr, "<%s>: no suitable file extension found in [%s]" % (mimetype, ', '.join(candidates)))
+	_log.error("<%s>: no suitable file extension found in config files" % mimetype)
 
 	return ext
 #-----------------------------------------------------------------------------------
@@ -141,37 +145,44 @@ def guess_ext_for_file(aFile=None):
 	mime_type = guess_mimetype(aFile)
 	f_ext = guess_ext_by_mimetype(mime_type)
 	if f_ext is None:
-		_log.Log(gmLog.lErr, 'unable to guess file extension for mime type [%s]' % mime_type)
+		_log.error('unable to guess file extension for mime type [%s]' % mime_type)
 		return None
 
 	return f_ext
 #-----------------------------------------------------------------------------------
-kfmclient = None
+_system_startfile_cmd = None
 
-def __detect_kfmclient():
-	global kfmclient
+def _get_system_startfile_cmd(filename):
 
-	if kfmclient is not None:
-		return
+	global _system_startfile_cmd
 
-	kfmclient = ''
-	cmd = 'which kfmclient'
-	pipe = os.popen(cmd.encode(sys.getfilesystemencoding()), 'r')
-	tmp = pipe.readline()
-	ret_code = pipe.close()
-	_log.Log(gmLog.lData, 'which kfmclient: [%s]' % tmp)
+	if _system_startfile_cmd == u'':
+		return False, None
 
-	if tmp == '':
-		_log.Log(gmLog.lData, 'kfmclient not found')
-		return
+	if _system_startfile_cmd is not None:
+		return True, _system_startfile_cmd % filename
 
-	tmp = tmp.split('\n')[0]
-	if not (os.path.isfile(tmp) and os.access(tmp, os.X_OK)):
-		_log.Log(gmLog.lData, 'kfmclient not detectable')
-		return
+	open_cmds = {
+		'xdg-open': 'xdg-open %s',				# nascent standard on Linux
+		'kfmclient': 'kfmclient exec %s',		# KDE
+		'gnome-open': 'gnome-open %s',			# GNOME
+		'exo-open': 'exo-open %s',
+		'op': 'op %s',
+		'open': 'open %s'						# MacOSX
+		#'run-mailcap'
+		#'explorer'
+	}
 
-	kfmclient = tmp
-	return
+	for candidate in open_cmds.keys():
+		found, binary = gmShellAPI.detect_external_binary(binary = candidate)
+		if not found:
+			continue
+		_system_startfile_cmd = open_cmds[candidate]
+		_log.info('detected local startfile cmd: [%s]', _system_startfile_cmd)
+		return True, _system_startfile_cmd % filename
+
+	_system_startfile_cmd = u''
+	return False, None
 #-----------------------------------------------------------------------------------
 def call_viewer_on_file(aFile = None, block=None):
 	"""Try to find an appropriate viewer with all tricks and call it.
@@ -180,17 +191,30 @@ def call_viewer_on_file(aFile = None, block=None):
 	"""
 	# does this file exist, actually ?
 	if not (os.path.isfile(aFile) and os.access(aFile, os.R_OK)):
-		msg = '[%s] is not a readable file' % aFile
-		_log.Log(gmLog.lErr, msg)
-		raise IOError(msg)
+		msg = '[%s] is not a readable file'
+		_log.error(msg, aFile)
+		raise IOError(msg % aFile)
 
-	# well, maybe we are on KDE, so try to detect kfmclient
-	__detect_kfmclient()
-	if kfmclient != '':
-		cmd = 'kfmclient exec %s' % aFile
-		if gmShellAPI.run_command_in_shell(command = cmd, blocking = block):
+	# try to detect any of the UNIX openers
+	found, startfile_cmd = _get_system_startfile_cmd(aFile)
+	if found:
+		if gmShellAPI.run_command_in_shell(command = startfile_cmd, blocking = block):
 			return True, ''
 
+#	# well, maybe we are on KDE, so try to detect kfmclient
+#	__detect_kfmclient()
+#	if kfmclient != u'':
+#		cmd = 'kfmclient exec %s' % aFile
+#		if gmShellAPI.run_command_in_shell(command = cmd, blocking = block):
+#			return True, ''
+
+#	# or else, are we on GNOME ? try to detect gnome-open
+#	__detect_gnome_open()
+#	if gnome_open != u'':
+#		cmd = 'gnome-open %s' % aFile
+#		if gmShellAPI.run_command_in_shell(command = cmd, blocking = block):
+#			return True, ''
+
 	mime_type = guess_mimetype(aFile)
 	viewer_cmd = get_viewer_cmd(mime_type, aFile)
 
@@ -198,18 +222,11 @@ def call_viewer_on_file(aFile = None, block=None):
 		if gmShellAPI.run_command_in_shell(command=viewer_cmd, blocking=block):
 			return True, ''
 
-	_log.Log(gmLog.lWarn, "no viewer found via standard mailcap system")
+	_log.warning("no viewer found via standard mailcap system")
 	if os.name == "posix":
-		_log.Log(gmLog.lWarn, "You should add a viewer for this mime type to your mailcap file.")
-		msg = _("Unable to display the file:\n\n"
-				" [%s]\n\n"
-				"Your system does not seem to have a (working)\n"
-				"viewer registered for the file type\n"
-				" [%s]"
-		) % (aFile, mime_type)
-		return False, msg
+		_log.warning("you should add a viewer for this mime type to your mailcap file")
+	_log.info("let's see what the OS can do about that")
 
-	_log.Log(gmLog.lInfo, "let's see what the OS can do about that")
 	# does the file already have an extension ?
 	(path_name, f_ext) = os.path.splitext(aFile)
 	# no
@@ -217,7 +234,7 @@ def call_viewer_on_file(aFile = None, block=None):
 		# try to guess one
 		f_ext = guess_ext_by_mimetype(mime_type)
 		if f_ext is None:
-			_log.Log(gmLog.lWarn, "no suitable file extension found, trying anyway")
+			_log.warning("no suitable file extension found, trying anyway")
 			file_to_display = aFile
 			f_ext = '?unknown?'
 		else:
@@ -227,17 +244,20 @@ def call_viewer_on_file(aFile = None, block=None):
 	else:
 		file_to_display = aFile
 
-	#xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
-	# FIXME: this needs more experimentation on Windows
 	file_to_display = os.path.normpath(file_to_display)
-	#xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
+	_log.debug("file %s <type %s> (ext %s) -> file %s" % (aFile, mime_type, f_ext, file_to_display))
 
-	_log.Log(gmLog.lData, "file %s <type %s> (ext %s) -> file %s" % (aFile, mime_type, f_ext, file_to_display))
 	try:
 		os.startfile(file_to_display)
 	except:
 		_log.LogException('os.startfile(%s) failed' % file_to_display)
-		return False, _("Unable to start viewer on file [%s].") % file_to_display
+		msg = _("Unable to display the file:\n\n"
+				" [%s]\n\n"
+				"Your system does not seem to have a (working)\n"
+				"viewer registered for the file type\n"
+				" [%s]"
+		) % (file_to_display, mime_type)
+		return False, msg
 
 	# don't kill the file from under the (possibly async) viewer
 #	if file_to_display != aFile:
@@ -246,15 +266,42 @@ def call_viewer_on_file(aFile = None, block=None):
 	return True, ''
 #=======================================================================================
 if __name__ == "__main__":
-	_log.SetAllLogLevels(gmLog.lData)
-	filename = sys.argv[1]
 
-	print guess_mimetype(filename)
-	print get_viewer_cmd(guess_mimetype(filename), filename)
-#	print guess_ext_by_mimetype(mimetype=filename)
+	if len(sys.argv) > 1 and sys.argv[1] == u'test':
+		filename = sys.argv[2]
+
+		print guess_mimetype(filename)
+		print get_viewer_cmd(guess_mimetype(filename), filename)
+		#print guess_ext_by_mimetype(mimetype=filename)
 
 #=======================================================================================
 # $Log: gmMimeLib.py,v $
+# Revision 1.22  2008/07/22 13:54:25  ncq
+# - xdg-open is intended to become the standard so look for that first
+# - some cleanup
+#
+# Revision 1.21  2008/03/11 16:58:11  ncq
+# - much improved detection of startfile cmd under UNIX
+#
+# Revision 1.20  2008/01/14 20:28:21  ncq
+# - use detect_external_binary()
+#
+# Revision 1.19  2008/01/11 16:11:40  ncq
+# - support gnome-open just like kfmclient
+#
+# Revision 1.18  2008/01/05 16:38:56  ncq
+# - eventually use python libextractor module if available
+# - do not assume every POSIX system knows mailcap, MacOSX doesn't
+#
+# Revision 1.17  2007/12/23 11:58:50  ncq
+# - use gmCfg2
+#
+# Revision 1.16  2007/12/12 16:17:15  ncq
+# - better logger names
+#
+# Revision 1.15  2007/12/11 14:31:12  ncq
+# - use std logging
+#
 # Revision 1.14  2007/10/12 14:19:18  ncq
 # - if file ext is ".tmp" and we were unable to run a viewer on that
 #   file - try to replace the extension based on the mime type
