@@ -1,22 +1,17 @@
-"""GnuMed scripting listener.
+"""GNUmed scripting listener.
 
 This module implements threaded listening for scripting.
 """
 #=====================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/pycommon/gmScriptingListener.py,v $
-__version__ = "$Revision: 1.3 $"
+__version__ = "$Revision: 1.6 $"
 __author__ = "K.Hilbert <karsten.hilbert@gmx.net>"
 
-import sys, time, threading, SimpleXMLRPCServer, select
+import sys, time, threading, SimpleXMLRPCServer, select, logging
 
 
-if __name__ == '__main__':
-	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmDispatcher, gmLog, gmExceptions
-
-
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lInfo, __version__)
+_log = logging.getLogger('gm.scripting')
+_log.info(__version__)
 #=====================================================================
 class cScriptingListener:
 
@@ -35,8 +30,9 @@ class cScriptingListener:
 		# this lock, when it succeeds it will quit
 		self._quit_lock = threading.Lock()
 		if not self._quit_lock.acquire(0):
-			_log.Log(gmLog.lErr, 'cannot acquire thread quit lock !?! aborting')
-			raise gmExceptions.ConstructorError, "cannot acquire thread quit-lock"
+			_log.error('cannot acquire thread quit lock !?! aborting')
+			import thread
+			raise thread.error("cannot acquire thread quit-lock")
 
 		# check for data every 'poll_interval' seconds
 		self._poll_interval = poll_interval
@@ -49,31 +45,49 @@ class cScriptingListener:
 		self._server.register_instance(self._macro_executor)
 		self._server.allow_reuse_address = True
 
-		self._thread = threading.Thread(target = self._process_RPCs)
+		self._thread = threading.Thread (
+			target = self._process_RPCs,
+			name = self.__class__.__name__
+		)
+		self._thread.setDaemon(True)
 		self._thread.start()
 
-		_log.Log(gmLog.lInfo, 'scripting listener started on [%s:%s]' % (self._listener_address, self._port))
-		_log.Log(gmLog.lInfo, 'macro executor is [%s]' % self._macro_executor)
+		_log.info('scripting listener started on [%s:%s]' % (self._listener_address, self._port))
+		_log.info('macro executor is [%s]' % self._macro_executor)
 	#-------------------------------
 	# public API
 	#-------------------------------
 	def shutdown(self):
 		"""Cleanly shut down. Complement to __init__()."""
 
-		# allow listener thread to acquire quit lock and give it time to terminate
+		if self._thread is None:
+			return
+
+		_log.info('stopping frontend scripting listener thread')
 		self._quit_lock.release()
-		if self._thread is not None:
+		try:
+			# give the worker thread time to terminate
 			self._thread.join(self._poll_interval+5)
+			try:
+				if self._thread.isAlive():
+					_log.error('listener thread still alive after join()')
+					_log.debug('active threads: %s' % threading.enumerate())
+			except:
+				pass
+		except:
+			print sys.exc_info()
+
+		self._thread = None
 
 		try:
 			self._server.socket.shutdown(2)
 		except:
-			_log.LogException('cannot cleanly shutdown(5) scripting listener socket')
+			_log.exception('cannot cleanly shutdown(5) scripting listener socket')
 
 		try:
 			self._server.socket.close()
 		except:
-			_log.LogException('cannot cleanly close() scripting listener socket')
+			_log.exception('cannot cleanly close() scripting listener socket')
 	#-------------------------------
 	# internal helpers
 	#-------------------------------
@@ -105,14 +119,13 @@ class cScriptingListener:
 				if self._quit_lock.acquire(0):
 					break
 
-		self._thread = None
+		# exit thread activity
+		return
 #=====================================================================
 # main
 #=====================================================================
 if __name__ == "__main__":
 
-	_log.SetAllLogLevels(gmLog.lData)
-
 	#-------------------------------
 	class runner:
 		def tell_time(self):
@@ -124,7 +137,7 @@ if __name__ == "__main__":
 		try:
 			listener = cScriptingListener(macro_executor=runner(), port=9999)
 		except:
-			_log.LogException('cannot instantiate scripting listener')
+			_log.exception('cannot instantiate scripting listener')
 			sys.exit(1)
 
 		s = xmlrpclib.ServerProxy('http://localhost:9999')
@@ -132,11 +145,20 @@ if __name__ == "__main__":
 			t = s.tell_time()
 			print t
 		except:
-			_log.LogException('cannot interact with server')
+			_log.exception('cannot interact with server')
 
 		listener.shutdown()
 #=====================================================================
 # $Log: gmScriptingListener.py,v $
+# Revision 1.6  2008/06/23 21:49:58  ncq
+# - clean up thread handling
+#
+# Revision 1.5  2007/12/12 16:17:16  ncq
+# - better logger names
+#
+# Revision 1.4  2007/12/11 15:39:01  ncq
+# - use std lib logging
+#
 # Revision 1.3  2007/12/03 20:43:53  ncq
 # - lots of cleanup
 # - fix/enhance test suite
