@@ -2,30 +2,33 @@
 __doc__ = """GNUmed general tools."""
 
 #===========================================================================
-# $Id: gmTools.py,v 1.43 2007/11/28 13:59:23 ncq Exp $
+# $Id: gmTools.py,v 1.65.2.2 2008/11/20 18:25:26 ncq Exp $
 # $Source: /sources/gnumed/gnumed/gnumed/client/pycommon/gmTools.py,v $
-__version__ = "$Revision: 1.43 $"
+__version__ = "$Revision: 1.65.2.2 $"
 __author__ = "K. Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL (details at http://www.gnu.org)"
 
 # std libs
-import datetime as pydt, re as regex, sys, os, os.path, csv, tempfile
+import datetime as pydt, re as regex, sys, os, os.path, csv, tempfile, logging, codecs, urllib2 as wget
 
 
 # GNUmed libs
 if __name__ == '__main__':
+	# for testing:
+	logging.basicConfig(level = logging.DEBUG)
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmLog, gmBorg
+	from Gnumed.pycommon import gmI18N
+	gmI18N.activate_locale()
+	gmI18N.install_domain()
+
+from Gnumed.pycommon import gmBorg
 
 
-_ = lambda x:x
-
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lInfo, __version__)
+_log = logging.getLogger('gm.tools')
+_log.info(__version__)
 
 # CAPitalization modes:
-(
-	CAPS_NONE,					# don't touch it
+(	CAPS_NONE,					# don't touch it
 	CAPS_FIRST,					# CAP first char, leave rest as is
 	CAPS_ALLCAPS,				# CAP all chars
 	CAPS_WORDS,					# CAP first char of every word
@@ -38,6 +41,115 @@ default_mail_receiver = u'gnumed-devel@gnu.org'
 default_mail_server = u'mail.gmx.net'
 
 
+u_registered_trademark = u'\u00ae'
+u_ellipsis = u'\u2026'
+u_diameter = u'\u2300'
+u_checkmark_crossed_out = u'\u237B'
+u_checkmark_thin = u'\u2713'
+u_checkmark_thick = u'\u2714'
+u_writing_hand = u'\u270d'
+u_pencil_1 = u'\u270e'
+u_pencil_2 = u'\u270f'
+u_pencil_3 = u'\u2710'
+
+#===========================================================================
+def check_for_update(url=None, current_branch=None, current_version=None, consider_latest_branch=False):
+	"""Check for new releases at <url>.
+
+	Returns (bool, text).
+	True: new release available
+	False: up to date
+	None: don't know
+	"""
+	try:
+		remote_file = wget.urlopen(url)
+	except (wget.URLError, ValueError, OSError):
+		_log.exception("cannot retrieve version file from [%s]", url)
+		return (None, _('Cannot retrieve version information.'))
+
+	_log.debug('retrieving version information from [%s]', url)
+
+	from Gnumed.pycommon import gmCfg2
+	cfg = gmCfg2.gmCfgData()
+	cfg.add_stream_source(source = 'gm-versions', stream = remote_file)
+	remote_file.close()
+
+	latest_branch = cfg.get('latest branch', 'branch', source_order = [('gm-versions', 'return')])
+	latest_release_on_latest_branch = cfg.get('latest branch', 'latest release', source_order = [('gm-versions', 'return')])
+	latest_release_on_current_branch = cfg.get('branch %s' % current_branch, 'latest release', source_order = [('gm-versions', 'return')])
+
+	_log.info('current release: %s', current_version)
+	_log.info('current branch: %s', current_branch)
+	_log.info('latest release on current branch: %s', latest_release_on_current_branch)
+	_log.info('latest branch: %s', latest_branch)
+	_log.info('latest release on latest branch: %s', latest_release_on_latest_branch)
+
+	# anything known ?
+	no_release_information_available = (
+		(
+			(latest_release_on_current_branch is None) and
+			(latest_release_on_latest_branch is None)
+		) or (
+			not consider_latest_branch and
+			(latest_release_on_current_branch is None)
+		)
+	)
+	if no_release_information_available:
+		msg = _('There is no version information available from:\n\n %s') % url
+		return (None, msg)
+
+	# up to date ?
+	if not consider_latest_branch:
+		if current_version >= latest_release_on_current_branch:
+			return (False, None)
+	else:
+		if current_version >= latest_release_on_latest_branch:
+			return (False, None)
+		if latest_release_on_latest_branch is None:
+			if current_version >= latest_release_on_current_branch:
+				return (False, None)
+
+	current_branch_release_available = (
+		(latest_release_on_current_branch is not None) and
+		(latest_release_on_current_branch > current_version)
+	)
+
+	latest_branch_release_available = (
+		(latest_branch is not None)
+			and
+		(
+			(latest_branch > current_branch) or (
+				(latest_branch == current_branch) and
+				(latest_release_on_latest_branch > current_version)
+			)
+		)
+	)
+
+	if not (current_branch_release_available or latest_branch_release_available):
+		return (False, None)
+
+	# not up to date
+	msg = _('A new version of GNUmed is available.\n\n')
+	msg += _(' Your version: "%s"\n') % current_version
+	if consider_latest_branch:
+		if current_branch_release_available:
+			msg += _(' New version: "%s"\n') % latest_release_on_current_branch
+			msg += _(' - bug fixes only\n')
+			msg += _(' - no database upgrade needed\n')
+		if latest_branch_release_available:
+			if latest_release_on_latest_branch != latest_release_on_current_branch:
+				msg += _(' New version: "%s"\n') % latest_release_on_latest_branch
+				msg += _(' - bug fixes and new features\n')
+				msg += _(' - database upgrade required\n')
+	else:
+		msg += _(' New version: "%s"\n') % latest_release_on_current_branch
+		msg += _(' - bug fixes only\n')
+		msg += _(' - no database upgrade needed\n')
+
+	msg += '\n\n'
+	msg += _('Version information loaded from:\n [%s]') % url
+
+	return (True, msg)
 #===========================================================================
 def utf_8_encoder(unicode_csv_data):
 	for line in unicode_csv_data:
@@ -51,21 +163,23 @@ def unicode_csv_reader(unicode_csv_data, dialect=csv.excel, **kwargs):
 		# decode UTF-8 back to Unicode, cell by cell:
 		yield [unicode(cell, 'utf-8') for cell in row]
 #===========================================================================
-def handle_uncaught_exception(t, v, tb):
+def handle_uncaught_exception_console(t, v, tb):
 
 	print ",========================================================"
 	print "| Unhandled exception caught !"
 	print "| Type :", t
 	print "| Value:", v
 	print "`========================================================"
-	_log.LogException('unhandled exception caught', (t,v,tb), verbose=True)
-	# FIXME: allow user to mail report to developers from here
+	_log.critical('unhandled exception caught', exc_info = (t,v,tb))
 	sys.__excepthook__(t,v,tb)
-
 #===========================================================================
 class gmPaths(gmBorg.cBorg):
 
 	def __init__(self, app_name=None, wx=None):
+		"""Setup pathes.
+
+		<app_name> will default to (name of the script - .py)
+		"""
 		try:
 			self.already_inited
 			return
@@ -81,6 +195,9 @@ class gmPaths(gmBorg.cBorg):
 
 		if app_name is None:
 			app_name, ext = os.path.splitext(os.path.basename(sys.argv[0]))
+			_log.info('app name detected as [%s]', app_name)
+		else:
+			_log.info('app name passed in as [%s]', app_name)
 
 		self.local_base_dir = os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]), '..'))
 		self.working_dir = os.path.abspath(os.curdir)
@@ -101,13 +218,13 @@ class gmPaths(gmBorg.cBorg):
 			self.system_app_data_dir = self.local_base_dir
 
 		if wx is None:
-			_log.Log(gmLog.lData, 'wxPython not available')
+			_log.debug('wxPython not available')
 			self.__log_paths()
 			return True
 
-		#--------------------
 		# retry with wxPython
 		std_paths = wx.StandardPaths.Get()
+		_log.info('wxPython app name set to [%s]', wx.GetApp().GetAppName())
 
 		try:
 			self.user_config_dir = os.path.join(std_paths.GetUserConfigDir(), '.%s' % app_name)
@@ -116,8 +233,10 @@ class gmPaths(gmBorg.cBorg):
 			self.user_config_dir = os.path.join(std_paths.GetUserConfigDir(), '.%s' % app_name)
 
 		try:
-			#self.system_config_dir = os.path.join(std_paths.GetConfigDir(), app_name)
-			self.system_config_dir = std_paths.GetConfigDir()
+			tmp = std_paths.GetConfigDir()
+			if not tmp.endswith(app_name):
+				tmp = os.path.join(tmp, app_name)
+			self.system_config_dir = tmp
 		except ValueError:
 			pass
 
@@ -125,7 +244,7 @@ class gmPaths(gmBorg.cBorg):
 			# Robin attests that the following doesn't give
 			# sane values on Windows, so IFDEF it
 			if 'wxMSW' in wx.PlatformInfo:
-				_log.Log(gmLog.lWarn, 'this platform returns a broken value for the system-wide application data dir')
+				_log.warning('this platform (wxMSW) returns a broken value for the system-wide application data dir')
 				self.system_app_data_dir = self.local_base_dir
 			else:
 				self.system_app_data_dir = std_paths.GetDataDir()
@@ -136,19 +255,19 @@ class gmPaths(gmBorg.cBorg):
 		return True
 	#--------------------------------------
 	def __log_paths(self):
-		_log.Log(gmLog.lData, 'local application base dir: %s' % self.local_base_dir)
-		_log.Log(gmLog.lData, 'current working dir: %s' % self.working_dir)
-		_log.Log(gmLog.lData, 'user home dir: %s' % os.path.expanduser('~'))
-		_log.Log(gmLog.lData, 'user-specific config dir: %s' % self.user_config_dir)
-		_log.Log(gmLog.lData, 'system-wide config dir: %s' % self.system_config_dir)
-		_log.Log(gmLog.lData, 'system-wide application data dir: %s' % self.system_app_data_dir)
+		_log.debug('local application base dir: %s', self.local_base_dir)
+		_log.debug('current working dir: %s', self.working_dir)
+		_log.debug('user home dir: %s', os.path.expanduser('~'))
+		_log.debug('user-specific config dir: %s', self.user_config_dir)
+		_log.debug('system-wide config dir: %s', self.system_config_dir)
+		_log.debug('system-wide application data dir: %s', self.system_app_data_dir)
 	#--------------------------------------
 	# properties
 	#--------------------------------------
 	def _set_user_config_dir(self, path):
 		if not (os.access(path, os.R_OK) and os.access(path, os.X_OK)):
 			msg = '[%s:user_config_dir]: invalid path [%s]' % (self.__class__.__name__, path)
-			_log.Log(gmLog.lErr, msg)
+			_log.error(msg)
 			raise ValueError(msg)
 		self.__user_config_dir = path
 
@@ -160,7 +279,7 @@ class gmPaths(gmBorg.cBorg):
 	def _set_system_config_dir(self, path):
 		if not (os.access(path, os.R_OK) and os.access(path, os.X_OK)):
 			msg = '[%s:system_config_dir]: invalid path [%s]' % (self.__class__.__name__, path)
-			_log.Log(gmLog.lErr, msg)
+			_log.error(msg)
 			raise ValueError(msg)
 		self.__system_config_dir = path
 
@@ -172,7 +291,7 @@ class gmPaths(gmBorg.cBorg):
 	def _set_system_app_data_dir(self, path):
 		if not (os.access(path, os.R_OK) and os.access(path, os.X_OK)):
 			msg = '[%s:system_app_data_dir]: invalid path [%s]' % (self.__class__.__name__, path)
-			_log.Log(gmLog.lErr, msg)
+			_log.error(msg)
 			raise ValueError(msg)
 		self.__system_app_data_dir = path
 
@@ -209,7 +328,7 @@ To: %s
 Subject: %s
 
 %s
-""" % (sender, u', '.join(receiver), subject, message)
+""" % (sender, u', '.join(receiver), subject[:50].replace('\r', '/').replace('\n', '/'), message)
 
 	import smtplib
 	session = smtplib.SMTP(server)
@@ -219,7 +338,7 @@ Subject: %s
 	refused = session.sendmail(sender, receiver, body.encode(encoding))
 	session.quit()
 	if len(refused) != 0:
-		_log.Log(gmLog.lErr, "refused recipients: %s" % refused)
+		_log.error("refused recipients: %s" % refused)
 		return False
 
 	return True
@@ -262,7 +381,7 @@ def import_module_from_directory(module_path=None, module_name=None):
 	"""Import a module from any location."""
 
 	if module_path not in sys.path:
-		_log.Log(gmLog.lInfo, 'appending to sys.path: [%s]' % module_path)
+		_log.info('appending to sys.path: [%s]' % module_path)
 		sys.path.append(module_path)
 		remove_path = True
 	else:
@@ -273,9 +392,9 @@ def import_module_from_directory(module_path=None, module_name=None):
 
 	try:
 		module = __import__(module_name)
-		_log.Log(gmLog.lInfo, 'imported module [%s] as [%s]' % (module_name, module))
+		_log.info('imported module [%s] as [%s]' % (module_name, module))
 	except StandardError:
-		_log.LogException('cannot __import__() module [%s] from [%s]' % (module_name, module_path), verbose=0)
+		_log.exception('cannot __import__() module [%s] from [%s]' % (module_name, module_path))
 		if remove_path:
 			sys.path.remove(module_path)
 		raise
@@ -293,12 +412,22 @@ days_per_week = 7
 #---------------------------------------------------------------------------
 def str2interval(str_interval=None):
 
-	# "(~)35(yYjJaA)"	- at age 35 years
-	if regex.match(u'^(\s|\t)*~*(\s|\t)*\d+(y|Y|j|J|a|A|\s|\t)*$', str_interval, flags = regex.LOCALE | regex.UNICODE):
+	unit_keys = {
+		'year': _('yYaA_keys_year'),
+		'month': _('mM_keys_month'),
+		'week': _('wW_keys_week'),
+		'day': _('dD_keys_day'),
+		'hour': _('hH_keys_hour')
+	}
+
+	# "(~)35(yY)"	- at age 35 years
+	keys = '|'.join(list(unit_keys['year'].replace('_keys_year', u'')))
+	if regex.match(u'^(\s|\t)*~*(\s|\t)*\d+(%s|\s|\t)*$' % keys, str_interval, flags = regex.LOCALE | regex.UNICODE):
 		return pydt.timedelta(days = (int(regex.findall(u'\d+', str_interval, flags = regex.LOCALE | regex.UNICODE)[0]) * days_per_year))
 
 	# "(~)12mM" - at age 12 months
-	if regex.match(u'^(\s|\t)*~*(\s|\t)*\d+(\s|\t)*(m|M)+(\s|\t)*$', str_interval, flags = regex.LOCALE | regex.UNICODE):
+	keys = '|'.join(list(unit_keys['month'].replace('_keys_month', u'')))
+	if regex.match(u'^(\s|\t)*~*(\s|\t)*\d+(\s|\t)*(%s)+(\s|\t)*$' % keys, str_interval, flags = regex.LOCALE | regex.UNICODE):
 		years, months = divmod (
 			int(regex.findall(u'\d+', str_interval, flags = regex.LOCALE | regex.UNICODE)[0]),
 			12
@@ -306,15 +435,18 @@ def str2interval(str_interval=None):
 		return pydt.timedelta(days = ((years * days_per_year) + (months * days_per_month)))
 
 	# weeks
-	if regex.match(u'^(\s|\t)*~*(\s|\t)*\d+(\s|\t)*(w|W)+(\s|\t)*$', str_interval, flags = regex.LOCALE | regex.UNICODE):
+	keys = '|'.join(list(unit_keys['week'].replace('_keys_week', u'')))
+	if regex.match(u'^(\s|\t)*~*(\s|\t)*\d+(\s|\t)*(%s)+(\s|\t)*$' % keys, str_interval, flags = regex.LOCALE | regex.UNICODE):
 		return pydt.timedelta(weeks = int(regex.findall(u'\d+', str_interval, flags = regex.LOCALE | regex.UNICODE)[0]))
 
 	# days
-	if regex.match(u'^(\s|\t)*~*(\s|\t)*\d+(\s|\t)*(d|D|t|T)+(\s|\t)*$', str_interval, flags = regex.LOCALE | regex.UNICODE):
+	keys = '|'.join(list(unit_keys['day'].replace('_keys_day', u'')))
+	if regex.match(u'^(\s|\t)*~*(\s|\t)*\d+(\s|\t)*(%s)+(\s|\t)*$' % keys, str_interval, flags = regex.LOCALE | regex.UNICODE):
 		return pydt.timedelta(days = int(regex.findall(u'\d+', str_interval, flags = regex.LOCALE | regex.UNICODE)[0]))
 
 	# hours
-	if regex.match(u'^(\s|\t)*~*(\s|\t)*\d+(\s|\t)*(h|H)+(\s|\t)*$', str_interval, flags = regex.LOCALE | regex.UNICODE):
+	keys = '|'.join(list(unit_keys['hour'].replace('_keys_hour', u'')))
+	if regex.match(u'^(\s|\t)*~*(\s|\t)*\d+(\s|\t)*(%s)+(\s|\t)*$' % keys, str_interval, flags = regex.LOCALE | regex.UNICODE):
 		return pydt.timedelta(hours = int(regex.findall(u'\d+', str_interval, flags = regex.LOCALE | regex.UNICODE)[0]))
 
 	# x/12 - months
@@ -343,12 +475,23 @@ def str2interval(str_interval=None):
 		return pydt.timedelta(minutes = int(regex.findall(u'\d+', str_interval, flags = regex.LOCALE | regex.UNICODE)[0]))
 
 	# nYnM - years, months
-	if regex.match(u'^(\s|\t)*~*(\s|\t)*\d+(y|Y|j|J|a|A|\s|\t)+\d+(\s|\t)*(m|M)+(\s|\t)*$', str_interval, flags = regex.LOCALE | regex.UNICODE):
+	keys_year = '|'.join(list(unit_keys['year'].replace('_keys_year', u'')))
+	keys_month = '|'.join(list(unit_keys['month'].replace('_keys_month', u'')))
+	if regex.match(u'^(\s|\t)*~*(\s|\t)*\d+(%s|\s|\t)+\d+(\s|\t)*(%s)+(\s|\t)*$' % (keys_year, keys_month), str_interval, flags = regex.LOCALE | regex.UNICODE):
 		parts = regex.findall(u'\d+', str_interval, flags = regex.LOCALE | regex.UNICODE)
 		years, months = divmod(int(parts[1]), 12)
 		years += int(parts[0])
 		return pydt.timedelta(days = ((years * days_per_year) + (months * days_per_month)))
 
+	# nMnW - months, weeks
+	keys_month = '|'.join(list(unit_keys['month'].replace('_keys_month', u'')))
+	keys_week = '|'.join(list(unit_keys['week'].replace('_keys_week', u'')))
+	if regex.match(u'^(\s|\t)*~*(\s|\t)*\d+(%s|\s|\t)+\d+(\s|\t)*(%s)+(\s|\t)*$' % (keys_month, keys_week), str_interval, flags = regex.LOCALE | regex.UNICODE):
+		parts = regex.findall(u'\d+', str_interval, flags = regex.LOCALE | regex.UNICODE)
+		months, weeks = divmod(int(parts[1]), 4)
+		months += int(parts[0])
+		return pydt.timedelta(days = ((months * days_per_month) + (weeks * days_per_week)))
+
 	return None
 #===========================================================================
 # text related tools
@@ -374,10 +517,21 @@ def size2str(size=0, template='%s'):
 		return template % u'%.1f TB' % (float(size) / _TB)
 	return template % u'%.1f PB' % (float(size) / _PB)
 #---------------------------------------------------------------------------
-def bool2str(bool=None, true_str='True', false_str='False'):
-	if bool is True:
-		return true_str
-	return false_str
+def bool2subst(boolean=None, true_return=True, false_return=False, none_return=None):
+	if boolean is None:
+		return none_return
+	if boolean is True:
+		return true_return
+	if boolean is False:
+		return false_return
+	raise ValueError('bool2subst(): <boolean> arg must be either of True, False, None')
+#---------------------------------------------------------------------------
+def bool2str(boolean=None, true_str='True', false_str='False'):
+	return bool2subst (
+		boolean = bool(boolean),
+		true_return = true_str,
+		false_return = false_str
+	)
 #---------------------------------------------------------------------------
 def none_if(value=None, none_equivalent=None):
 	"""Modelled after the SQL NULLIF function."""
@@ -410,12 +564,19 @@ def coalesce(initial=None, instead=None, template_initial=None, template_instead
 		- list of insteads: initial, [instead, template], [instead, template], [instead, template], template_initial, ...
 	"""
 	if initial is None:
+
 		if template_instead is None:
 			return instead
+
 		return template_instead % instead
+
 	if template_initial is None:
 		return initial
-	return template_initial % initial
+
+	try:
+		return template_initial % initial
+	except TypeError:
+		return template_initial
 #---------------------------------------------------------------------------
 def __cap_name(match_obj=None):
 	val = match_obj.group(0).lower()
@@ -438,12 +599,16 @@ def capitalize(text=None, mode=CAPS_NAMES):
 		return text
 
 	if mode == CAPS_FIRST:
+		if len(text) == 1:
+			return text[0].upper()
 		return text[0].upper() + text[1:]
 
 	if mode == CAPS_ALLCAPS:
 		return text.upper()
 
 	if mode == CAPS_FIRST_ONLY:
+		if len(text) == 1:
+			return text[0].upper()
 		return text[0].upper() + text[1:].lower()
 
 	if mode == CAPS_WORDS:
@@ -456,34 +621,32 @@ def capitalize(text=None, mode=CAPS_NAMES):
 	print "ERROR: invalid capitalization mode: [%s], leaving input as is" % mode
 	return text
 #---------------------------------------------------------------------------
-def wrap(text, width):
-	"""
-	A word-wrap function that preserves existing line breaks
+def wrap(text=None, width=None, initial_indent=u'', subsequent_indent=u'', eol=u'\n'):
+	"""A word-wrap function that preserves existing line breaks
 	and most spaces in the text. Expects that existing line
 	breaks are posix newlines (\n).
-
-	FIXME: add initial/subsequent indent etc
 	"""
-	return reduce (
+	wrapped = initial_indent + reduce (
 		lambda line, word, width=width: '%s%s%s' % (
 			line,
-			' \n'[(
-				len(line)
-				- line.rfind('\n')
-				- 1
-				+ len(word.split('\n',1)[0])
-				>= width
-			)],
-			word),
+			' \n'[(len(line) - line.rfind('\n') - 1 + len(word.split('\n',1)[0]) >= width)],
+			word
+		),
 		text.split(' ')
 	)
+
+	if subsequent_indent != u'':
+		wrapped = (u'\n%s' % subsequent_indent).join(wrapped.split('\n'))
+
+	if eol != u'\n':
+		wrapped = wrapped.replace('\n', eol)
+
+	return wrapped
 #===========================================================================
 # main
 #---------------------------------------------------------------------------
 if __name__ == '__main__':
 
-	_log.SetAllLogLevels(gmLog.lData)
-
 	#-----------------------------------------------------------------------
 	def test_str2interval():
 		print "testing str2interval()"
@@ -498,11 +661,13 @@ if __name__ == '__main__':
 			'12h', '17 h', '12	H', '17H', '	~ 	17	h	 ', ' ~ 36	/ 24', '7/24', '0/24',
 			' ~ 36	/ 60', '7/60', '190/60', '0/60',
 			'12a1m', '12 a 1  M', '12	a17m', '12j		12m', '12J7m', '12y7m', '12Y7M', '	~ 	12	a	 37 m	', '~0a0m',
+			'10m1w',
 			'invalid interval input'
 		]
 
 		for str_interval in str_intervals:
-			print "interval:", str2interval(str_interval=str_interval), "; original: <%s>" % str_interval
+			print "input: <%s>" % str_interval
+			print "  ==>", str2interval(str_interval=str_interval)
 
 		return True
 	#-----------------------------------------------------------------------
@@ -511,7 +676,11 @@ if __name__ == '__main__':
 		print "------------------"
 		tests = [
 			[None, 'something other than <None>', None, None, 'something other than <None>'],
-			['Captain', 'Mr.', '%s.'[:4], 'Mr.', 'Capt.']
+			['Captain', 'Mr.', '%s.'[:4], 'Mr.', 'Capt.'],
+			['value to test', 'test 3 failed', 'template with "%s" included', None, 'template with "value to test" included'],
+			['value to test', 'test 4 failed', 'template with value not included', None, 'template with value not included'],
+			[None, 'initial value was None', 'template_initial: %s', None, 'initial value was None'],
+			[None, 'initial value was None', 'template_initial: %%(abc)s', None, 'initial value was None']
 		]
 		passed = True
 		for test in tests:
@@ -640,7 +809,12 @@ This is a test mail from the gmTools.py module.
 				print 'ERROR: bool2str(%s, %s, %s) returned [%s], expected [%s]' % (test[0], test[1], test[2], bool2str(test[0], test[1], test[2]), test[3])
 
 		return True
+	#-----------------------------------------------------------------------
+	def test_bool2subst():
 
+		print bool2subst(True, 'True', 'False', 'is None')
+		print bool2subst(False, 'True', 'False', 'is None')
+		print bool2subst(None, 'True', 'False', 'is None')
 	#-----------------------------------------------------------------------
 	def test_get_unique_filename():
 		print get_unique_filename()
@@ -656,9 +830,62 @@ This is a test mail from the gmTools.py module.
 		for test in tests:
 			print size2str(test)
 	#-----------------------------------------------------------------------
+	def test_wrap():
+		test = 'line 1\nline 2\nline 3'
+
+		print "wrap 5-6-7 initial 0, subsequent 0"
+		print wrap(test, 5)
+		print
+		print wrap(test, 6)
+		print
+		print wrap(test, 7)
+		print "-------"
+		raw_input()
+		print "wrap 5 initial 1-1-3, subsequent 1-3-1"
+		print wrap(test, 5, u' ', u' ')
+		print
+		print wrap(test, 5, u' ', u'   ')
+		print
+		print wrap(test, 5, u'   ', u' ')
+		print "-------"
+		raw_input()
+		print "wrap 6 initial 1-1-3, subsequent 1-3-1"
+		print wrap(test, 6, u' ', u' ')
+		print
+		print wrap(test, 6, u' ', u'   ')
+		print
+		print wrap(test, 6, u'   ', u' ')
+		print "-------"
+		raw_input()
+		print "wrap 7 initial 1-1-3, subsequent 1-3-1"
+		print wrap(test, 7, u' ', u' ')
+		print
+		print wrap(test, 7, u' ', u'   ')
+		print
+		print wrap(test, 7, u'   ', u' ')
+	#-----------------------------------------------------------------------
+	def test_check_for_update():
+
+		test_data = [
+			('http://www.gnumed.de/downloads/gnumed-versions.txt', None, None, False),
+			('file:///home/ncq/gm-versions.txt', None, None, False),
+			('file:///home/ncq/gm-versions.txt', '0.2', '0.2.8.1', False),
+			('file:///home/ncq/gm-versions.txt', '0.2', '0.2.8.1', True),
+			('file:///home/ncq/gm-versions.txt', '0.2', '0.2.8.5', True)
+		]
+
+		for test in test_data:
+			print "arguments:", test
+			found, msg = check_for_update(test[0], test[1], test[2], test[3])
+			print msg
+
+		return
+	#-----------------------------------------------------------------------
 	if len(sys.argv) > 1 and sys.argv[1] == 'test':
+
+		#test_check_for_update()
 		#test_str2interval()
-		#test_coalesce()
+		test_coalesce()
 		#test_capitalize()
 		#test_import_module()
 		#test_mkdir()
@@ -666,11 +893,88 @@ This is a test mail from the gmTools.py module.
 		#test_gmPaths()
 		#test_none_if()
 		#test_bool2str()
+		#test_bool2subst()
 		#test_get_unique_filename()
-		test_size2str()
+		#test_size2str()
+		#test_wrap()
 
 #===========================================================================
 # $Log: gmTools.py,v $
+# Revision 1.65.2.2  2008/11/20 18:25:26  ncq
+# - do not announce new version twice
+#
+# Revision 1.65.2.1  2008/10/28 13:58:18  ncq
+# - fix logic reversal when detecting new client version
+#
+# Revision 1.65  2008/08/20 13:53:57  ncq
+# - add some coalesce tests
+#
+# Revision 1.64  2008/07/28 15:43:35  ncq
+# - teach wrap() about target EOL
+#
+# Revision 1.63  2008/07/12 15:30:56  ncq
+# - improved coalesce test
+#
+# Revision 1.62  2008/07/12 15:24:37  ncq
+# - impove coalesce to allow template_initial to be returned *instead* of
+#   initial substituted into the template by not including a substitution
+#
+# Revision 1.61  2008/07/10 20:51:38  ncq
+# - better logging
+#
+# Revision 1.60  2008/07/10 19:59:09  ncq
+# - better logging
+# - check whether sys config dir ends in "gnumed"
+#
+# Revision 1.59  2008/07/07 11:34:41  ncq
+# - robustify capsify on single character strings
+#
+# Revision 1.58  2008/06/28 18:25:01  ncq
+# - add unicode Registered TM symbol
+#
+# Revision 1.57  2008/05/31 16:32:42  ncq
+# - a couple of unicode shortcuts
+#
+# Revision 1.56  2008/05/26 12:05:50  ncq
+# - improved wording of update message
+# - better handling of CVS tip
+#
+# Revision 1.55  2008/05/21 15:51:45  ncq
+# - if cannot open update URL may throw OSError, so deal with that
+#
+# Revision 1.54  2008/05/21 14:01:32  ncq
+# - add check_for_update and tests
+#
+# Revision 1.53  2008/05/13 14:09:36  ncq
+# - str2interval: support xMxW syntax
+#
+# Revision 1.52  2008/05/07 15:18:01  ncq
+# - i18n str2interval
+#
+# Revision 1.51  2008/04/16 20:34:43  ncq
+# - add bool2subst tests
+#
+# Revision 1.50  2008/04/11 12:24:39  ncq
+# - add initial_indent/subsequent_indent and tests to wrap()
+#
+# Revision 1.49  2008/03/20 15:29:51  ncq
+# - bool2subst() supporting None, make bool2str() use it
+#
+# Revision 1.48  2008/03/02 15:10:32  ncq
+# - truncate exception comment to 50 chars when used as subject
+#
+# Revision 1.47  2008/01/16 19:42:24  ncq
+# - whitespace sync
+#
+# Revision 1.46  2007/12/23 11:59:40  ncq
+# - improved docs
+#
+# Revision 1.45  2007/12/12 16:24:09  ncq
+# - general cleanup
+#
+# Revision 1.44  2007/12/11 14:33:48  ncq
+# - use standard logging module
+#
 # Revision 1.43  2007/11/28 13:59:23  ncq
 # - test improved
 #
