@@ -3,11 +3,11 @@
 """
 ############################################################################
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmAllergyWidgets.py,v $
-__version__ = "$Revision: 1.28.2.1 $"
+__version__ = "$Revision: 1.32 $"
 __author__  = "R.Terry <rterry@gnumed.net>, H.Herb <hherb@gnumed.net>, K.Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = 'GPL (details at http://www.gnu.org)'
 
-import sys, time, datetime as pyDT
+import sys, time, datetime as pyDT, logging
 
 
 import wx
@@ -15,13 +15,13 @@ import wx
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmLog, gmDispatcher, gmSignals, gmI18N, gmDateTime, gmTools, gmMatchProvider
+from Gnumed.pycommon import gmDispatcher, gmI18N, gmDateTime, gmTools, gmMatchProvider
 from Gnumed.wxpython import gmDateTimeInput, gmTerryGuiParts, gmRegetMixin
 from Gnumed.business import gmPerson, gmAllergy
 from Gnumed.wxGladeWidgets import wxgAllergyEditAreaPnl, wxgAllergyEditAreaDlg, wxgAllergyManagerDlg
 
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lInfo, __version__)
+_log = logging.getLogger('gm.ui')
+_log.info(__version__)
 
 #======================================================================
 class cAllergyEditAreaPnl(wxgAllergyEditAreaPnl.wxgAllergyEditAreaPnl):
@@ -213,6 +213,10 @@ class cAllergyManagerDlg(wxgAllergyManagerDlg.wxgAllergyManagerDlg):
 		wxgAllergyManagerDlg.wxgAllergyManagerDlg.__init__(self, *args, **kwargs)
 
 		self.__set_columns()
+		# MacOSX crashes on this with:
+		#  exception type : wx._core.PyAssertionError
+		#  exception value: C++ assertion "i" failed at /BUILD/wxPython-src-2.8.3.0/src/common/wincmn.cpp(2634) in DoMoveInTabOrder(): MoveBefore/AfterInTabOrder(): win is not a sibling
+		# while Win/Linux work just fine
 		#self._PNL_edit_area._ChBOX_definite.MoveAfterInTabOrder(self._BTN_save)
 		self.__refresh_ui()
 	#--------------------------------------------------------
@@ -268,7 +272,6 @@ class cAllergyManagerDlg(wxgAllergyManagerDlg.wxgAllergyManagerDlg):
 	# event handlers
 	#--------------------------------------------------------
 	def _on_dismiss_button_pressed(self, evt):
-#		wx.CallAfter(gmDispatcher.send, signal = gmSignals.allergy_updated())
 		if self.IsModal():
 			self.EndModal(wx.ID_OK)
 		else:
@@ -385,15 +388,15 @@ class cAllergyPanel(wx.Panel, gmRegetMixin.cRegetOnPaintMixin):
 		wx.EVT_LIST_ITEM_ACTIVATED(self, ID_ALLERGY_LIST, self._on_allergy_activated)
 
 		# client internal signals
-		gmDispatcher.connect(signal=gmSignals.post_patient_selection(), receiver=self._schedule_data_reget)
-		gmDispatcher.connect(signal=gmSignals.vaccinations_updated(), receiver=self._schedule_data_reget)
+		gmDispatcher.connect(signal = u'post_patient_selection', receiver=self._schedule_data_reget)
+#		gmDispatcher.connect(signal = u'vaccinations_updated', receiver=self._schedule_data_reget)
 	#-----------------------------------------------
 	def __reset_ui_content(self):
 		self.editarea.set_data()
 		self.LCTRL_allergies.DeleteAllItems()
 	#-----------------------------------------------
 	def _populate_with_data(self):
-		if not self.__pat.is_connected():
+		if not self.__pat.connected:
 			return False
 
 		self.LCTRL_allergies.DeleteAllItems()
@@ -467,8 +470,18 @@ if __name__ == "__main__":
 #		app.MainLoop()
 #======================================================================
 # $Log: gmAllergyWidgets.py,v $
-# Revision 1.28.2.1  2008/01/16 15:16:16  ncq
-# - MacOSX doesn't like Move*InTabOrder across sizers
+# Revision 1.32  2008/07/07 13:43:16  ncq
+# - current patient .connected
+#
+# Revision 1.31  2008/03/06 18:29:29  ncq
+# - standard lib logging only
+#
+# Revision 1.30  2008/01/30 14:03:41  ncq
+# - use signal names directly
+# - switch to std lib logging
+#
+# Revision 1.29  2008/01/16 19:38:15  ncq
+# - wxMAC doesn't like some Move*InTabOrder()
 #
 # Revision 1.28  2007/10/25 12:19:53  ncq
 # - no more useless allergy update
