@@ -5,14 +5,14 @@ functions for authenticating users.
 """
 #================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmAuthWidgets.py,v $
-# $Id: gmAuthWidgets.py,v 1.2.2.3 2008/07/10 09:03:50 ncq Exp $
-__version__ = "$Revision: 1.2.2.3 $"
+# $Id: gmAuthWidgets.py,v 1.25.2.3 2008/11/07 11:08:06 ncq Exp $
+__version__ = "$Revision: 1.25.2.3 $"
 __author__ = "karsten.hilbert@gmx.net, H.Herb, H.Berger, R.Terry"
 __license__ = "GPL (details at http://www.gnu.org)"
 
 
 # stdlib
-import sys, os.path, cPickle, zlib
+import sys, os.path, cPickle, zlib, logging
 
 
 # 3rd party
@@ -22,14 +22,14 @@ import wx
 # GNUmed
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmGuiBroker, gmLoginInfo, gmLog, gmI18N, gmPG2, gmBackendListener, gmCfg, gmTools, gmCLI
+from Gnumed.pycommon import gmLoginInfo, gmPG2, gmBackendListener, gmTools, gmCfg2, gmI18N
 from Gnumed.business import gmSurgery
-from Gnumed.wxpython import gmGuiHelpers
+from Gnumed.wxpython import gmGuiHelpers, gmExceptionHandlingWidgets
 
 
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lInfo, __version__)
-_cfg = gmCfg.gmDefCfgFile
+_log = logging.getLogger('gm.ui')
+_log.info(__version__)
+_cfg = gmCfg2.gmCfgData()
 
 try:
 	_('do-not-translate-but-make-epydoc-happy')
@@ -41,8 +41,9 @@ GNUmed database version mismatch.
 
 This database version cannot be used with this client:
 
- version detected: %s
- version needed: %s
+ client version: %s
+ database version detected: %s
+ database version needed: %s
 
 Currently connected to database:
 
@@ -51,11 +52,34 @@ Currently connected to database:
  user: %s
 """)
 
+msg_time_skew_fail = _("""\
+The server and client clocks are off
+by more than %s minutes !
+
+You must fix the time settings before
+you can use this database with this
+client.
+
+You may have to contact your
+administrator for help.""")
+
+msg_time_skew_warn = _("""\
+The server and client clocks are off
+by more than %s minutes !
+
+You should fix the time settings.
+Otherwise clinical data may appear to
+have been entered at the wrong time.
+
+You may have to contact your
+administrator for help.""")
+
 msg_insanity = _("""
 There is a serious problem with the database settings:
 
 %s
-""")
+
+You may have to contact your administrator for help.""")
 
 msg_fail = _("""
 You must connect to a different database in order
@@ -70,11 +94,11 @@ There may be schema related errors. Please report and/or
 fix them. Do not rely on this database to work properly
 in all cases !""")
 
-curr_db = 'gnumed_v8'
+current_db_name = 'gnumed_v9'
 #================================================================
 # convenience functions
 #----------------------------------------------------------------
-def connect_to_database(max_attempts=3, expected_version=None, require_version=True):
+def connect_to_database(max_attempts=3, expected_version=None, require_version=True, client_version=u'*** unknown ***'):
 	"""Display the login dialog and try to log into the backend.
 
 	- up to max_attempts times
@@ -85,7 +109,7 @@ def connect_to_database(max_attempts=3, expected_version=None, require_version=T
 
 	attempt = 0
 
-	dlg = cLoginDialog(None, -1)
+	dlg = cLoginDialog(None, -1, client_version = client_version)
 	dlg.Centre(wx.BOTH)
 
 	while attempt < max_attempts:
@@ -95,7 +119,7 @@ def connect_to_database(max_attempts=3, expected_version=None, require_version=T
 		dlg.ShowModal()
 		login = dlg.panel.GetLoginInfo()
 		if login is None:
-			_log.Log(gmLog.lInfo, "user cancelled login dialog")
+			_log.info("user cancelled login dialog")
 			break
 
 		# try getting a connection to verify the DSN works
@@ -107,25 +131,31 @@ def connect_to_database(max_attempts=3, expected_version=None, require_version=T
 			password = login.password
 		)
 		try:
-			conn = gmPG2.get_raw_connection(dsn = dsn, verbose = True)
+			conn = gmPG2.get_raw_connection(dsn = dsn, verbose = True, readonly = True)
 			connected = True
 		except gmPG2.cAuthenticationError, e:
 			attempt += 1
-			_log.LogException(u"login attempt %s/%s failed" % (attempt, max_attempts), verbose=0)
+			_log.error(u"login attempt %s/%s failed: %s", attempt, max_attempts, e)
 			if attempt < max_attempts:
 				gmGuiHelpers.gm_show_error (_(
-						"Unable to connect to database:\n\n"
-						"%s\n\n"
-						"Please retry or cancel !"
+					"Unable to connect to database:\n\n"
+					"%s\n\n"
+					"Please retry with proper\n"
+					"credentials or cancel !"
 					) % e,
-					_('Connecting to backend'),
-					gmLog.lErr
+					_('Connecting to backend')
 				)
-			_log.LogException(u"login attempt %s/%s failed, may retry" % (attempt, max_attempts), verbose=0)
 			continue
-		except StandardError:
-			_log.LogException(u"login attempt %s/%s failed, useless to retry" % (attempt+1, max_attempts), verbose=0)
-			break
+		except gmPG2.dbapi.OperationalError, e:
+			_log.error(u"login attempt %s/%s failed: %s", attempt+1, max_attempts, e)
+			gmGuiHelpers.gm_show_error (_(
+				"Unable to connect to database:\n\n"
+				"%s\n\n"
+				"Please retry with another profile !\n"
+				) % e,
+				_('Connecting to backend')
+			)
+			continue
 
 		# connect was successful:
 		gmPG2.set_default_login(login = login)
@@ -137,16 +167,31 @@ def connect_to_database(max_attempts=3, expected_version=None, require_version=T
 
 		if not compatible:
 			connected_db_version = gmPG2.get_schema_version()
-			msg = msg_generic % (connected_db_version, expected_version, login.host, login.database, login.user)
+			msg = msg_generic % (client_version, connected_db_version, expected_version, login.host, login.database, login.user)
 			if require_version:
-				gmGuiHelpers.gm_show_error(msg + msg_fail, _('Verifying database version'), None)
+				gmGuiHelpers.gm_show_error(msg + msg_fail, _('Verifying database version'))
 				continue
-			gmGuiHelpers.gm_show_info(msg + msg_override, _('Verifying database version'), None)
+			gmGuiHelpers.gm_show_info(msg + msg_override, _('Verifying database version'))
 
-		sanity = gmPG2.sanity_check_database_settings()
-		if sanity is not True:
-			gmGuiHelpers.gm_show_error((msg_insanity % sanity) + msg_fail, _('Verifying database settings'), None)
-			continue
+		# FIXME: make configurable
+		max_skew = 1		# minutes
+		if _cfg.get(option = 'debug'):
+			max_skew = 10
+		if not gmPG2.sanity_check_time_skew(tolerance = (max_skew * 60)):
+			if _cfg.get(option = 'debug'):
+				gmGuiHelpers.gm_show_warning(msg_time_skew_warn % max_skew, _('Verifying database settings'))
+			else:
+				gmGuiHelpers.gm_show_error(msg_time_skew_fail % max_skew, _('Verifying database settings'))
+				continue
+
+		sanity_level, message = gmPG2.sanity_check_database_settings()
+		if sanity_level != 0:
+			gmGuiHelpers.gm_show_error((msg_insanity % message), _('Verifying database settings'))
+			if sanity_level == 2:
+				continue
+
+		gmExceptionHandlingWidgets.set_is_public_database(login.public_db)
+		gmExceptionHandlingWidgets.set_helpdesk(login.helpdesk)
 
 		listener = gmBackendListener.gmBackendListener(conn=conn)
 		break
@@ -182,11 +227,10 @@ Please enter the password for <gm-dbo>:""") % procedure,
 	try:
 		conn = gmPG2.get_connection(dsn=dsn, readonly=False, verbose=True, pooled=False)
 	except:
-		_log.LogException('cannot connect')
+		_log.exception('cannot connect')
 		gmGuiHelpers.gm_show_error (
 			aMessage = _('Cannot connect as the GNUmed database owner <gm-dbo>.'),
-			aTitle = procedure,
-			aLogLevel = None
+			aTitle = procedure
 		)
 		return None
 
@@ -206,10 +250,10 @@ class cLoginDialog(wx.Dialog):
 \xa6\x01\xbbt9\xceR\xc8\x81e_$\x98\xb9\x9c\xa9\x8d,y\xa9t\xc8\xcf\x152\xe0x\
 \xe9$\xf5\x07\x95\x0cD\x95t:\xb1\x92\xae\x9cI\xa8~\x84\x1f\xe0\xa3ec'
 
-	def __init__(self, parent, id, title=_("Welcome to the")):
+	def __init__(self, parent, id, title=_("Welcome to the"), client_version=u'*** unknown ***'):
 		wx.Dialog.__init__(self, parent, id, title)
-		self.panel = cLoginPanel(self, -1, isDialog=1)
-		self.Fit () # needed for Windoze.
+		self.panel = cLoginPanel(self, -1, isDialog=1, client_version = client_version)
+		self.Fit() # needed for Windoze.
 		self.Centre()
 
 		# set window icon
@@ -225,11 +269,8 @@ class cLoginPanel(wx.Panel):
 		previously entered settings.
 	"""
 	def __init__(self, parent, id,
-		pos = wx.DefaultPosition,
-		size = wx.DefaultSize,
-		style = wx.TAB_TRAVERSAL,
-		isDialog = 0
-		):
+					pos = wx.DefaultPosition, size = wx.DefaultSize, style = wx.TAB_TRAVERSAL,
+					isDialog = 0, client_version = u'*** unknown ***'):
 		"""Create login panel.
 
 		isDialog:	if this panel is the main panel of a dialog, the panel will
@@ -239,24 +280,6 @@ class cLoginPanel(wx.Panel):
 		wx.Panel.__init__(self, parent, id, pos, size, style)
 		self.parent = parent
 
-		paths = gmTools.gmPaths(app_name = 'gnumed', wx=wx)
-		fnames = []
-		if gmCLI.has_arg('--conf-file'):
-			fnames.append(gmCLI.arg['--conf-file'])
-		fnames.append(os.path.join(paths.user_config_dir, 'gnumed.conf'))
-		fnames.append(os.path.join(paths.local_base_dir, 'gnumed.conf'))
-		fnames.append(os.path.join(paths.working_dir, 'gnumed.conf'))
-		self.user_preferences_file = None
-		for fname in fnames:
-			try:
-				open(fname, 'r+b')
-				self.user_preferences_file = fname
-				break
-			except IOError:
-				continue
-
-		self.gb = gmGuiBroker.GuiBroker()
-
 		#True if dialog was cancelled by user 
 		#if the dialog is closed manually, login should be cancelled
 		self.cancelled = True
@@ -267,6 +290,7 @@ class cLoginPanel(wx.Panel):
 		self.topsizer = wx.BoxSizer(wx.VERTICAL)
 
 		# find bitmap
+		paths = gmTools.gmPaths(app_name = u'gnumed', wx = wx)
 		bitmap = os.path.join(paths.system_app_data_dir, 'bitmaps', 'gnumedlogo.png')
 		try:
 			png = wx.Image(bitmap, wx.BITMAP_TYPE_PNG).ConvertToBitmap()
@@ -290,11 +314,7 @@ class cLoginPanel(wx.Panel):
 				border = 10
 			)
 
-#		if self.gb.has_key('main.slave_mode') and self.gb['main.slave_mode']:
-		if gmCLI.has_arg('--slave'):
-			paramsbox_caption = _("Slave Login - %s" % gmSurgery.gmCurrentPractice().active_workplace)
-		else:
-			paramsbox_caption = _("Login - %s" % gmSurgery.gmCurrentPractice().active_workplace)
+		paramsbox_caption = _('"%s" (version %s)') % (gmSurgery.gmCurrentPractice().active_workplace, client_version)
 
 		# FIXME: why doesn't this align in the centre ?
 		self.paramsbox = wx.StaticBox( self, -1, paramsbox_caption, style = wx.ALIGN_CENTRE_HORIZONTAL)
@@ -355,7 +375,7 @@ class cLoginPanel(wx.Panel):
 		label = wx.StaticText(self, -1, _('Options'), wx.DefaultPosition, wx.DefaultSize, 0)
 		label.SetForegroundColour(wx.Colour(35, 35, 142))
 		self.pboxgrid.Add(label, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)
-		self._CHBOX_debug = wx.CheckBox(self, -1, _('debug mode'))
+		self._CHBOX_debug = wx.CheckBox(self, -1, _('&Debug mode'))
 		self._CHBOX_debug.SetToolTipString(_('Check this to run GNUmed client in debugging mode.'))
 		self.pboxgrid.Add(self._CHBOX_debug, 0, wx.GROW | wx.ALIGN_CENTER_VERTICAL | wx.ALL, 5)
 
@@ -363,7 +383,7 @@ class cLoginPanel(wx.Panel):
 		label = wx.StaticText(self, -1, '', wx.DefaultPosition, wx.DefaultSize, 0)
 		label.SetForegroundColour(wx.Colour(35, 35, 142))
 		self.pboxgrid.Add(label, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)
-		self._CHBOX_slave = wx.CheckBox(self, -1, _('enable remote control'))
+		self._CHBOX_slave = wx.CheckBox(self, -1, _('Enable &remote control'))
 		self._CHBOX_slave.SetToolTipString(_('Check this to run GNUmed client in slave mode for remote control.'))
 		self.pboxgrid.Add(self._CHBOX_slave, 0, wx.GROW | wx.ALIGN_CENTER_VERTICAL | wx.ALL, 5)
 
@@ -428,36 +448,18 @@ class cLoginPanel(wx.Panel):
 	#----------------------------------------------------------
 	def __get_previously_used_accounts(self):
 
-		accounts = []
-
-		if gmCLI.has_arg('--conf-file'):
-			fnames = [gmCLI.arg['--conf-file']]
-		else:
-			paths = gmTools.gmPaths(app_name = 'gnumed', wx = wx)
-			fnames = [
-				os.path.join(paths.user_config_dir, 'gnumed.conf'),
-				os.path.join(paths.working_dir, 'gnumed.conf')
-			]
-
-		for fname in fnames:
-			try:
-				cfg_file = gmCfg.cCfgFile (
-					aFile = fname,
-					flags = gmCfg.cfg_IGNORE_CMD_LINE
-				)
-			except IOError:
-				continue
-
-			accounts = gmTools.coalesce (
-				cfg_file.get('backend', 'logins'),
-				[]
-			)
-
-			if len(accounts) > 0:
-				return accounts
-
-		if len(accounts) == 0:
-			return ['any-doc']
+		accounts = gmTools.coalesce (
+			_cfg.get (
+				group = u'backend',
+				option = u'logins',
+				source_order = [
+					(u'explicit', u'extend'),
+					(u'user', u'extend'),
+					(u'workbase', u'extend')
+				]
+			),
+			['any-doc']
+		)
 
 		return accounts
 	#----------------------------------------------------
@@ -471,101 +473,105 @@ class cLoginPanel(wx.Panel):
 		as a profile in the system file will override the
 		system file.
 		"""
+		src_order = [
+			(u'explicit', u'extend'),
+			(u'system', u'extend'),
+			(u'user', u'extend'),
+			(u'workbase', u'extend')
+		]
+
+		profile_names = gmTools.coalesce (
+			_cfg.get(group = u'backend', option = u'profiles', source_order = src_order),
+			[]
+		)
+
+		src_order = [
+			(u'explicit', u'return'),
+			(u'workbase', u'return'),
+			(u'user', u'return'),
+			(u'system', u'return')
+		]
+
 		profiles = {}
 
-		if gmCLI.has_arg('--conf-file'):
-			fnames = [gmCLI.arg['--conf-file']]
-		else:
-			paths = gmTools.gmPaths()
-			fnames = [
-				os.path.join(paths.system_config_dir, 'gnumed-client.conf'),
-				os.path.join(paths.user_config_dir, 'gnumed.conf'),
-				os.path.join(paths.working_dir, 'gnumed.conf')
-			]
-
-		for fname in fnames:
+		for profile_name in profile_names:
+			# FIXME: once the profile has been found always use the corresponding source !
+			profile = cBackendProfile()
+			profile.name = profile_name
+			profile_section = 'profile %s' % profile_name
+			profile.host = gmTools.coalesce(_cfg.get(profile_section, u'host', src_order), u'').strip()
+			port = gmTools.coalesce(_cfg.get(profile_section, u'port', src_order), 5432)
 			try:
-				cfg_file = gmCfg.cCfgFile (
-					aFile = fname,
-					flags = gmCfg.cfg_IGNORE_CMD_LINE
-				)
-			except IOError:
+				profile.port = int(port)
+				if profile.port < 1024:
+					raise ValueError('refusing to use priviledged port (< 1024)')
+			except ValueError:
+				_log.warning('invalid port definition: [%s], skipping profile [%s]', port, profile_name)
 				continue
-
-			profile_names = gmTools.coalesce (
-				cfg_file.get('backend', 'profiles'),
-				[]
-			)
-
-			for profile_name in profile_names:
-				profile = cBackendProfile()
-				profile.name = profile_name
-				profile_section = 'profile %s' % profile_name
-				profile.host = gmTools.coalesce(cfg_file.get(profile_section, 'host'), '').strip()
-				try:
-					profile.port = int (gmTools.coalesce (
-						cfg_file.get(profile_section, 'port'),
-						5432
-					))
-					if profile.port < 1024:
-						raise ValueError('refusing to use priviledged port (< 1024)')
-				except ValueError:
-					_log.LogException('invalid port definition: [%s], skipping profile [%s] in [%s]' % (cfg_file.get(profile_section, 'port'), profile_name, fname))
-					continue
-				profile.database = gmTools.coalesce(cfg_file.get(profile_section, 'database'), '').strip()
-				if profile.database == '':
-					_log.Log(gmLog.lErr, 'database name not specified, skipping profile [%s] in [%s]' % (profile_name, fname))
-					continue
-				profile.encoding = gmTools.coalesce (
-					cfg_file.get(profile_section, 'encoding'),
-					'UTF8'
-				)
-				profiles[profile_name] = profile
+			profile.database = gmTools.coalesce(_cfg.get(profile_section, u'database', src_order), u'').strip()
+			if profile.database == u'':
+				_log.warning('database name not specified, skipping profile [%s]', profile_name)
+				continue
+			profile.encoding = gmTools.coalesce(_cfg.get(profile_section, u'encoding', src_order), u'UTF8')
+			profile.public_db = bool(_cfg.get(profile_section, u'public/open access', src_order))
+			profile.helpdesk = _cfg.get(profile_section, u'help desk', src_order)
+			profiles[profile_name] = profile
 
 		if len(profiles) == 0:
-			host = 'salaam.homeunix.com'
-			label = 'public GNUmed database (%s@%s)' % (curr_db, host)
+			host = u'salaam.homeunix.com'
+			label = u'public GNUmed database (%s@%s)' % (current_db_name, host)
 			profiles[label] = cBackendProfile()
 			profiles[label].name = label
 			profiles[label].host = host
 			profiles[label].port = 5432
-			profiles[label].database = curr_db
-			profiles[label].encoding = 'UTF8'
-			
+			profiles[label].database = current_db_name
+			profiles[label].encoding = u'UTF8'
+			profiles[label].public_db = True
+			profiles[label].helpdesk = u'http://wiki.gnumed.de'
 		return profiles
 	#----------------------------------------------------------
 	def __load_state(self):
 
-		prefs = gmCfg.cCfgFile (
-			aFile = self.user_preferences_file,
-			flags = gmCfg.cfg_IGNORE_CMD_LINE
-		)
+		src_order = [
+			(u'explicit', u'return'),
+			(u'workbase', u'return'),
+			(u'user', u'return'),
+		]
+
 		self._CBOX_user.SetValue (
 			gmTools.coalesce (
-				prefs.get('preferences', 'login'),
+				_cfg.get(u'preferences', u'login', src_order),
 				self.__previously_used_accounts[0]
 			)
 		)
 
-		prefs_profile = prefs.get('preferences', 'profile')
+		prefs_profile = _cfg.get(u'preferences', u'profile', src_order)
 		try:
 			self._CBOX_profile.SetValue(self.__backend_profiles[prefs_profile].name)
 		except KeyError:
 			self._CBOX_profile.SetValue(self.__backend_profiles[self.__backend_profiles.keys()[0]].name)
 
-		self._CHBOX_debug.SetValue(gmCLI.has_arg('--debug'))
-		self._CHBOX_slave.SetValue(gmCLI.has_arg('--slave'))
+		self._CHBOX_debug.SetValue(_cfg.get(option = 'debug'))
+		self._CHBOX_slave.SetValue(_cfg.get(option = 'slave'))
 	#----------------------------------------------------
 	def save_state(self):
 		"""Save parameter settings to standard configuration file"""
+		prefs_name = _cfg.get(option = 'user_preferences_file')
+		_log.debug(u'saving login choices in [%s]', prefs_name)
 
-		prefs = gmCfg.cCfgFile (
-			aFile = self.user_preferences_file,
-			flags = gmCfg.cfg_IGNORE_CMD_LINE
+		gmCfg2.set_option_in_INI_file (
+			filename = prefs_name,
+			group = 'preferences',
+			option = 'login',
+			value = self._CBOX_user.GetValue()
+		)
+
+		gmCfg2.set_option_in_INI_file (
+			filename = prefs_name,
+			group = 'preferences',
+			option = 'profile',
+			value = self._CBOX_profile.GetValue()
 		)
-		prefs.set('preferences', 'login', self._CBOX_user.GetValue())
-		prefs.set('preferences', 'profile', self._CBOX_profile.GetValue())
-		prefs.store()
 	#############################################################################
 	# Retrieve current settings from user interface widgets
 	#############################################################################
@@ -581,6 +587,8 @@ class cLoginPanel(wx.Panel):
 				database = profile.database,
 				port = profile.port
 			)
+			login.public_db = profile.public_db
+			login.helpdesk = profile.helpdesk
 			return login
 
 		return None
@@ -588,16 +596,12 @@ class cLoginPanel(wx.Panel):
 	# event handlers
 	#----------------------------
 	def OnHelp(self, event):
-		tmp = _cfg.get('workplace', 'help desk')
-		if tmp is None:
-			print "You need to set the option [workplace] -> <help desk> in the config file !"
-			tmp = "http://www.gnumed.org"
-
+		praxis = gmSurgery.gmCurrentPractice()
 		wx.MessageBox(_(
-"""GnuMed main login screen
+"""GNUmed main login screen
 
 USER:
- name of the GnuMed user
+ name of the GNUmed user
 PASSWORD
  password for this user
 
@@ -606,35 +610,35 @@ button OK:
 button OPTIONS:
  set advanced options
 button CANCEL:
- abort login and quit GnuMed client
+ abort login and quit GNUmed client
 button HELP:
  this help screen
 
-For assistance on using GnuMed please contact:
- %s""") % tmp)
+For assistance on using GNUmed please contact:
+ %s""") % praxis.helpdesk)
 
 	#----------------------------
 	def __on_login_button_pressed(self, event):
 
+		root_logger = logging.getLogger()
 		if self._CHBOX_debug.GetValue():
-			_log.SetAllLogLevels(gmLog.lData)
-			gmCLI._cli_args['--debug'] = True
-			_log.Log(gmLog.lInfo, 'debug mode enabled')
+			_log.info('debug mode enabled')
+			_cfg.set_option(option = 'debug', value = True)
+			root_logger.setLevel(logging.DEBUG)
 		else:
-			_log.Log(gmLog.lInfo, 'debug mode disabled')
-			gmCLI._cli_args['--debug'] = False
-			del gmCLI._cli_args['--debug']
-			if gmCLI.has_arg("--quiet"):
-				_log.SetAllLogLevels(gmLog.lErr)
+			_log.info('debug mode disabled')
+			_cfg.set_option(option = 'debug', value = False)
+			if _cfg.get(option = '--quiet', source_order = [('cli', 'return')]):
+				root_logger.setLevel(logging.ERROR)
 			else:
-				_log.SetAllLogLevels(gmLog.lInfo)
+				root_logger.setLevel(logging.WARNING)
 
-		if self._CHBOX_slave.GetValue():	
-			_log.Log(gmLog.lInfo, 'slave mode enabled')
-			self.gb['main.slave_mode'] = True
+		if self._CHBOX_slave.GetValue():
+			_log.info('slave mode enabled')
+			_cfg.set_option(option = 'slave', value = True)
 		else:
-			_log.Log(gmLog.lInfo, 'slave mode disabled')
-			self.gb['main.slave_mode'] = False
+			_log.info('slave mode disabled')
+			_cfg.set_option(option = 'slave', value = False)
 
 		self.backend_profile = self.__backend_profiles[self._CBOX_profile.GetValue().encode('latin1').strip()]
 #		self.user = self._CBOX_user.GetValue().strip()
@@ -650,7 +654,11 @@ For assistance on using GnuMed please contact:
 # main
 #----------------------------------------------------------------
 if __name__ == "__main__":
-	_log.SetAllLogLevels(gmLog.lData)
+
+	from Gnumed.pycommon import gmI18N
+
+	logging.basicConfig(level = logging.DEBUG)
+
 	gmI18N.activate_locale()
 	gmI18N.install_domain(domain='gnumed')
 	#-----------------------------------------------
@@ -676,14 +684,90 @@ if __name__ == "__main__":
 
 #================================================================
 # $Log: gmAuthWidgets.py,v $
-# Revision 1.2.2.3  2008/07/10 09:03:50  ncq
-# - be more generous when looking for a writable user prefs file
+# Revision 1.25.2.3  2008/11/07 11:08:06  ncq
+# - need to set help desk on default profile, too
 #
-# Revision 1.2.2.2  2008/06/25 08:59:06  ncq
-# - don't try printing utf8 to console
+# Revision 1.25.2.2  2008/09/09 17:26:21  ncq
+# - default to any-doc for previously used accounts
 #
-# Revision 1.2.2.1  2008/01/30 11:19:54  ncq
-# - don't fail on faulty profile name in [preferences]
+# Revision 1.25.2.1  2008/08/31 22:15:48  ncq
+# - properly set src_order and return policy on retrieving backend profile data
+#
+# Revision 1.25  2008/07/30 12:50:34  ncq
+# - cleanup
+#
+# Revision 1.24  2008/07/16 11:11:10  ncq
+# - set_helpdesk (no more _database_)
+#
+# Revision 1.23  2008/07/10 20:52:21  ncq
+# - better to call path detection with app_name and wx
+#
+# Revision 1.22  2008/07/07 11:35:19  ncq
+# - keyboard shortcuts for check boxes
+#
+# Revision 1.21  2008/05/28 21:26:40  ncq
+# - gracefully handle OperationalErrors such as "server not available"
+#
+# Revision 1.20  2008/05/13 14:10:35  ncq
+# - be more permissive in time skew check
+# - handle per-profile public-db/helpdesk options
+#
+# Revision 1.19  2008/04/16 20:39:39  ncq
+# - working versions of the wxGlade code and use it, too
+# - show client version in login dialog
+#
+# Revision 1.18  2008/03/20 15:30:21  ncq
+# - use gmPG2.sanity_check_time_skew()
+#
+# Revision 1.17  2008/03/11 17:00:49  ncq
+# - explicitely request readonly raw connection
+#
+# Revision 1.16  2008/03/06 18:29:29  ncq
+# - standard lib logging only
+#
+# Revision 1.15  2008/03/05 22:26:52  ncq
+# - spelling
+#
+# Revision 1.14  2008/02/25 17:33:16  ncq
+# - use improved db sanity checks
+#
+# Revision 1.13  2008/01/30 14:07:02  ncq
+# - protect against faulty backend profile in preferences
+#
+# Revision 1.12  2008/01/27 21:13:17  ncq
+# - no more gmCfg
+#
+# Revision 1.11  2008/01/14 20:33:06  ncq
+# - cleanup
+# - properly detect connection errors
+#
+# Revision 1.10  2008/01/13 01:16:52  ncq
+# - log DSN on any connect errors
+#
+# Revision 1.9  2008/01/07 19:51:54  ncq
+# - bump db version
+# - we still need gmCfg
+#
+# Revision 1.8  2007/12/26 23:22:27  ncq
+# - fix invalid variable access
+#
+# Revision 1.7  2007/12/26 22:44:31  ncq
+# - missing import of logging
+# - use std lib logger
+# - cleanup
+#
+# Revision 1.6  2007/12/26 22:01:25  shilbert
+# - cleanup
+#
+# Revision 1.5  2007/12/23 22:02:56  ncq
+# - no more gmCLI
+#
+# Revision 1.4  2007/12/23 20:26:31  ncq
+# - cleanup
+#
+# Revision 1.3  2007/12/23 12:07:40  ncq
+# - cleanup
+# - use gmCfg2
 #
 # Revision 1.2  2007/12/04 16:14:52  ncq
 # - get_dbowner_connection()
