@@ -2,26 +2,26 @@
 """
 #================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmEMRBrowser.py,v $
-# $Id: gmEMRBrowser.py,v 1.81 2007/09/07 10:56:57 ncq Exp $
-__version__ = "$Revision: 1.81 $"
+# $Id: gmEMRBrowser.py,v 1.93.2.2 2008/10/12 17:02:59 ncq Exp $
+__version__ = "$Revision: 1.93.2.2 $"
 __author__ = "cfmoro1976@yahoo.es, sjtan@swiftdsl.com.au, Karsten.Hilbert@gmx.net"
 __license__ = "GPL"
 
 # std lib
-import sys, types, os.path, StringIO, codecs
+import sys, types, os.path, StringIO, codecs, logging
 
 # 3rd party
 import wx
 
 # GNUmed libs
-from Gnumed.pycommon import gmLog, gmI18N, gmDispatcher, gmSignals, gmExceptions, gmTools
+from Gnumed.pycommon import gmI18N, gmDispatcher, gmExceptions, gmTools
 from Gnumed.exporters import gmPatientExporter
 from Gnumed.business import gmEMRStructItems, gmPerson, gmSOAPimporter
 from Gnumed.wxpython import gmGuiHelpers, gmEMRStructWidgets, gmSOAPWidgets, gmAllergyWidgets, gmNarrativeWidgets
 from Gnumed.wxGladeWidgets import wxgScrolledEMRTreePnl, wxgSplittedEMRTreeBrowserPnl
 
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lInfo, __version__)
+_log = logging.getLogger('gm.ui')
+_log.info(__version__)
 
 #============================================================
 def export_emr_to_ascii(parent=None):
@@ -32,10 +32,10 @@ def export_emr_to_ascii(parent=None):
 	"""
 	# sanity checks
 	if parent is None:
-		raise TypeError('[export_emr_to_ascii]: expected wx.Window instance as parent, got <None>')
+		raise TypeError('expected wx.Window instance as parent, got <None>')
 
 	pat = gmPerson.gmCurrentPatient()
-	if not pat.is_connected():
+	if not pat.connected:
 		gmDispatcher.send(signal='statustext', msg=_('Cannot export EMR. No active patient.'))
 		return False
 
@@ -58,7 +58,7 @@ def export_emr_to_ascii(parent=None):
 	if choice != wx.ID_OK:
 		return None
 
-	_log.Log(gmLog.lData, 'exporting EMR to [%s]' % fname)
+	_log.debug('exporting EMR to [%s]', fname)
 
 #	output_file = open(fname, 'wb')
 	output_file = codecs.open(fname, 'wb', encoding='utf8', errors='replace')
@@ -91,6 +91,7 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 		except KeyError:
 			self.__narr_display = None
 		self.__pat = gmPerson.gmCurrentPatient()
+		self.__curr_node = None
 		self.__exporter = gmPatientExporter.cEmrExport(patient = self.__pat)
 
 		self.__make_popup_menus()
@@ -99,7 +100,7 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 	# external API
 	#--------------------------------------------------------
 	def refresh(self):
-		if not self.__pat.is_connected():
+		if not self.__pat.connected:
 			gmDispatcher.send(signal='statustext', msg=_('Cannot load clinical narrative. No active patient.'),)
 			return False
 
@@ -117,6 +118,10 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 		"""Configures enabled event signals."""
 		wx.EVT_TREE_SEL_CHANGED (self, self.GetId(), self._on_tree_item_selected)
 		wx.EVT_TREE_ITEM_RIGHT_CLICK (self, self.GetId(), self._on_tree_item_right_clicked)
+
+		gmDispatcher.connect(signal = 'narrative_mod_db', receiver = self._on_narrative_mod_db)
+		gmDispatcher.connect(signal = 'episode_mod_db', receiver = self._on_episode_mod_db)
+		gmDispatcher.connect(signal = 'health_issue_mod_db', receiver = self._on_issue_mod_db)
 	#--------------------------------------------------------
 	def __populate_tree(self):
 		"""Updates EMR browser data."""
@@ -126,35 +131,59 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 		wx.BeginBusyCursor()
 
 		self.snapshot_expansion()
-		self.DeleteAllItems()
 
 		# init new tree
-		root_item = self.AddRoot(_('%s EMR') % self.__pat['description'])
+		self.DeleteAllItems()
+		root_item = self.AddRoot(_('EMR of %s') % self.__pat['description'])
 		self.SetPyData(root_item, None)
 		self.SetItemHasChildren(root_item, True)
 
 		# have the tree filled by the exporter
 		self.__exporter.get_historical_tree(self)
+
 		self.SelectItem(root_item)
-
-		# and uncollapse
 		self.Expand(root_item)
-
-		self.SortChildren(root_item)
-
-		# display patient summary info
-		label = _('Summary')
-		underline = '=' * len(label)
-		if self.__narr_display is not None:
-			self.__narr_display.Clear()
-			self.__narr_display.WriteText('%s\n%s\n\n' % (label, underline))
-			self.__narr_display.WriteText(self.__exporter.get_summary_info(0))
+		self.__curr_node = root_item
+		self.__update_text_for_selected_node()
 
 		self.restore_expansion()
 
 		wx.EndBusyCursor()
 		return True
 	#--------------------------------------------------------
+	def __update_text_for_selected_node(self):
+		"""Displays information for the selected tree node."""
+
+		if self.__narr_display is None:
+			return
+
+		if self.__curr_node is None:
+			return
+
+		node_data = self.GetPyData(self.__curr_node)
+
+		# update displayed text
+		if isinstance(node_data, (gmEMRStructItems.cHealthIssue, types.DictType)):
+			# FIXME: turn into real dummy issue
+			if node_data['pk'] is None:
+				txt = _('Pool of unassociated episodes:\n\n  "%s"') % node_data['description']
+			else:
+				txt = node_data.format(left_margin=1, patient = self.__pat)
+
+		elif isinstance(node_data, gmEMRStructItems.cEpisode):
+			txt = node_data.format(left_margin = 1, patient = self.__pat)
+
+		elif isinstance(node_data, gmEMRStructItems.cEncounter):
+			epi = self.GetPyData(self.GetItemParent(self.__curr_node))
+			txt = node_data.format(episode = epi, with_soap = True, left_margin = 1, patient = self.__pat)
+
+		else:
+			emr = self.__pat.get_emr()
+			txt = emr.format_summary()
+
+		self.__narr_display.Clear()
+		self.__narr_display.WriteText(txt)
+	#--------------------------------------------------------
 	def __make_popup_menus(self):
 
 		# - episodes
@@ -180,17 +209,28 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 		self.__enc_context_popup.AppendItem(wx.MenuItem(self.__enc_context_popup, menu_id, _('Edit details')))
 		wx.EVT_MENU(self.__enc_context_popup, menu_id, self.__edit_consultation_details)
 
+		item = self.__enc_context_popup.Append(-1, _('Export for Medistar'))
+		self.Bind(wx.EVT_MENU, self.__export_encounter_for_medistar, item)
+
 		# - health issues
 		self.__issue_context_popup = wx.Menu(title = _('Health Issue Menu'))
-		# edit
+
 		menu_id = wx.NewId()
 		self.__issue_context_popup.AppendItem(wx.MenuItem(self.__issue_context_popup, menu_id, _('Edit details')))
 		wx.EVT_MENU(self.__issue_context_popup, menu_id, self.__edit_issue)
-		# delete
+
 		menu_id = wx.NewId()
 		self.__issue_context_popup.AppendItem(wx.MenuItem(self.__issue_context_popup, menu_id, _('Delete')))
 		wx.EVT_MENU(self.__issue_context_popup, menu_id, self.__delete_issue)
 
+		self.__issue_context_popup.AppendSeparator()
+
+		menu_id = wx.NewId()
+		self.__issue_context_popup.AppendItem(wx.MenuItem(self.__issue_context_popup, menu_id, _('Open to encounter level')))
+		wx.EVT_MENU(self.__issue_context_popup, menu_id, self.__expand_issue_to_encounter_level)
+		# print " attach issue to another patient"
+		# print " move all episodes to another issue"
+
 		# - root node
 		self.__root_context_popup = wx.Menu(title = _('EMR Menu'))
 		# add health issue
@@ -201,19 +241,43 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 		menu_id = wx.NewId()
 		self.__root_context_popup.AppendItem(wx.MenuItem(self.__root_context_popup, menu_id, _('manage allergies')))
 		wx.EVT_MENU(self.__root_context_popup, menu_id, self.__document_allergy)
-		# print " attach issue to another patient"
-		# print " move all episodes to another issue"
+
+		self.__root_context_popup.AppendSeparator()
+
+		# expand tree
+		expand_menu = wx.Menu()
+		self.__root_context_popup.AppendMenu(wx.NewId(), _('Open EMR to ...'), expand_menu)
+
+		menu_id = wx.NewId()
+		expand_menu.AppendItem(wx.MenuItem(expand_menu, menu_id, _('... issue level')))
+		wx.EVT_MENU(expand_menu, menu_id, self.__expand_to_issue_level)
+
+		menu_id = wx.NewId()
+		expand_menu.AppendItem(wx.MenuItem(expand_menu, menu_id, _('... episode level')))
+		wx.EVT_MENU(expand_menu, menu_id, self.__expand_to_episode_level)
+
+		menu_id = wx.NewId()
+		expand_menu.AppendItem(wx.MenuItem(expand_menu, menu_id, _('... encounter level')))
+		wx.EVT_MENU(expand_menu, menu_id, self.__expand_to_encounter_level)
+	#--------------------------------------------------------
+	def __handle_root_context(self, pos=wx.DefaultPosition):
+		self.PopupMenu(self.__root_context_popup, pos)
+	#--------------------------------------------------------
+	def __handle_issue_context(self, pos=wx.DefaultPosition):
+#		self.__issue_context_popup.SetTitle(_('Episode %s') % episode['description'])
+		self.PopupMenu(self.__issue_context_popup, pos)
 	#--------------------------------------------------------
-	# episodes
 	def __handle_episode_context(self, pos=wx.DefaultPosition):
 		self.__epi_context_popup.SetTitle(_('Episode %s') % self.__curr_node_data['description'])
 		self.PopupMenu(self.__epi_context_popup, pos)
 	#--------------------------------------------------------
+	def __handle_encounter_context(self, pos=wx.DefaultPosition):
+		self.PopupMenu(self.__enc_context_popup, pos)
+	#--------------------------------------------------------
+	#--------------------------------------------------------
 	def __edit_episode(self, event):
 		dlg = gmEMRStructWidgets.cEpisodeEditAreaDlg(parent=self, episode=self.__curr_node_data)
-		result = dlg.ShowModal()
-		if result == wx.ID_OK:
-			self.__populate_tree()
+		dlg.ShowModal()
 	#--------------------------------------------------------
 	def __delete_episode(self, event):
 		dlg = gmGuiHelpers.c2ButtonQuestionDlg (
@@ -239,17 +303,12 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 		except gmExceptions.DatabaseObjectInUseError:
 			gmDispatcher.send(signal = 'statustext', msg = _('Cannot delete episode. There is still clinical data recorded for it.'))
 			return
-
-		self.__populate_tree()
-	#--------------------------------------------------------
-	# encounters
-	def __handle_encounter_context(self, pos=wx.DefaultPosition):
-		self.PopupMenu(self.__enc_context_popup, pos)
 	#--------------------------------------------------------
 	def __edit_consultation_details(self, event):
 		node_data = self.GetPyData(self.__curr_node)
 		dlg = gmEMRStructWidgets.cEncounterEditAreaDlg(parent=self, encounter=node_data)
 		dlg.ShowModal()
+		dlg.Destroy()
 		self.__populate_tree()
 	#--------------------------------------------------------
 	def __relink_encounter_data2episode(self, event):
@@ -270,16 +329,10 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 		if result == wx.ID_YES:
 			self.__populate_tree()
 	#--------------------------------------------------------
-	# health issues
-	def __handle_issue_context(self, pos=wx.DefaultPosition):
-#		self.__issue_context_popup.SetTitle(_('Episode %s') % episode['description'])
-		self.PopupMenu(self.__issue_context_popup, pos)
-	#--------------------------------------------------------
 	def __edit_issue(self, event):
-		ea = gmEMRStructWidgets.cHealthIssueEditAreaDlg(parent=self, id=-1, issue=self.__curr_node_data)
-		if ea.ShowModal() == wx.ID_OK:
-			self.__populate_tree()
-		return
+		dlg = gmEMRStructWidgets.cHealthIssueEditAreaDlg(parent=self, id=-1, issue=self.__curr_node_data)
+		dlg.ShowModal()
+		dlg.Destroy()
 	#--------------------------------------------------------
 	def __delete_issue(self, event):
 		dlg = gmGuiHelpers.c2ButtonQuestionDlg (
@@ -298,62 +351,123 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 		)
 		result = dlg.ShowModal()
 		if result != wx.ID_YES:
+			dlg.Destroy()
 			return
 
+		dlg.Destroy()
+
 		try:
 			gmEMRStructItems.delete_health_issue(health_issue = self.__curr_node_data)
 		except gmExceptions.DatabaseObjectInUseError:
 			gmDispatcher.send(signal = 'statustext', msg = _('Cannot delete health issue. There is still clinical data recorded for it.'))
+	#--------------------------------------------------------
+	def __expand_issue_to_encounter_level(self, evt):
+
+		if not self.__curr_node.IsOk():
 			return
 
-		self.__populate_tree()
-	#--------------------------------------------------------
-	# root
-	def __handle_root_context(self, pos=wx.DefaultPosition):
-		self.PopupMenu(self.__root_context_popup, pos)
+		self.Expand(self.__curr_node)
+
+		epi, epi_cookie = self.GetFirstChild(self.__curr_node)
+		while epi.IsOk():
+			self.Expand(epi)
+			epi, epi_cookie = self.GetNextChild(self.__curr_node, epi_cookie)
 	#--------------------------------------------------------
 	def __create_issue(self, event):
 		ea = gmEMRStructWidgets.cHealthIssueEditAreaDlg(parent=self, id=-1)
-		if ea.ShowModal() == wx.ID_OK:
-			self.__populate_tree()
-		return
+		ea.ShowModal()
 	#--------------------------------------------------------
 	def __document_allergy(self, event):
 		dlg = gmAllergyWidgets.cAllergyManagerDlg(parent=self, id=-1)
+		# FIXME: use signal and use node level update
 		if dlg.ShowModal() == wx.ID_OK:
 			self.__populate_tree()
+		dlg.Destroy()
 		return
 	#--------------------------------------------------------
+	def __expand_to_issue_level(self, evt):
+
+		root_item = self.GetRootItem()
+
+		if not root_item.IsOk():
+			return
+
+		self.Expand(root_item)
+
+		# collapse episodes and issues
+		issue, issue_cookie = self.GetFirstChild(root_item)
+		while issue.IsOk():
+			self.Collapse(issue)
+			epi, epi_cookie = self.GetFirstChild(issue)
+			while epi.IsOk():
+				self.Collapse(epi)
+				epi, epi_cookie = self.GetNextChild(issue, epi_cookie)
+			issue, issue_cookie = self.GetNextChild(root_item, issue_cookie)
+	#--------------------------------------------------------
+	def __expand_to_episode_level(self, evt):
+
+		root_item = self.GetRootItem()
+
+		if not root_item.IsOk():
+			return
+
+		self.Expand(root_item)
+
+		# collapse episodes, expand issues
+		issue, issue_cookie = self.GetFirstChild(root_item)
+		while issue.IsOk():
+			self.Expand(issue)
+			epi, epi_cookie = self.GetFirstChild(issue)
+			while epi.IsOk():
+				self.Collapse(epi)
+				epi, epi_cookie = self.GetNextChild(issue, epi_cookie)
+			issue, issue_cookie = self.GetNextChild(root_item, issue_cookie)
+	#--------------------------------------------------------
+	def __expand_to_encounter_level(self, evt):
+
+		root_item = self.GetRootItem()
+
+		if not root_item.IsOk():
+			return
+
+		self.Expand(root_item)
+
+		# collapse episodes, expand issues
+		issue, issue_cookie = self.GetFirstChild(root_item)
+		while issue.IsOk():
+			self.Expand(issue)
+			epi, epi_cookie = self.GetFirstChild(issue)
+			while epi.IsOk():
+				self.Expand(epi)
+				epi, epi_cookie = self.GetNextChild(issue, epi_cookie)
+			issue, issue_cookie = self.GetNextChild(root_item, issue_cookie)
+	#--------------------------------------------------------
+	def __export_encounter_for_medistar(self, evt):
+		gmNarrativeWidgets.export_narrative_for_medistar_import (
+			parent = self,
+			soap_cats = u'soap',
+			encounter = self.__curr_node_data
+		)
+	#--------------------------------------------------------
 	# event handlers
 	#--------------------------------------------------------
+	def _on_narrative_mod_db(self, *args, **kwargs):
+		wx.CallAfter(self.__update_text_for_selected_node)
+	#--------------------------------------------------------
+	def _on_episode_mod_db(self, *args, **kwargs):
+		wx.CallAfter(self.__populate_tree)
+	#--------------------------------------------------------
+	def _on_issue_mod_db(self, *args, **kwargs):
+		wx.CallAfter(self.__populate_tree)
+	#--------------------------------------------------------
 	def _on_tree_item_selected(self, event):
-		"""Displays information for a selected tree node."""
-		# retrieve the selected EMR element
 		sel_item = event.GetItem()
-		node_data = self.GetPyData(sel_item)
 		self.__curr_node = sel_item
-
-		# update displayed text
-		if isinstance(node_data, (gmEMRStructItems.cHealthIssue, types.DictType)):
-			txt = self.__exporter.get_issue_info(issue=node_data)
-		elif isinstance(node_data, gmEMRStructItems.cEpisode):
-			txt = self.__exporter.get_episode_summary(episode=node_data)
-		elif isinstance(node_data, gmEMRStructItems.cEncounter):
-			epi = self.GetPyData(self.GetItemParent(sel_item))
-			txt = self.__exporter.get_encounter_info(episode=epi, encounter=node_data)
-		else:
-			txt = _('Summary') + '\n=======\n\n' + self.__exporter.get_summary_info(0)
-
-		if self.__narr_display is not None:
-			self.__narr_display.Clear()
-			self.__narr_display.WriteText(txt)
-
+		self.__update_text_for_selected_node()
 		return True
 	#--------------------------------------------------------
 	def _on_tree_item_right_clicked(self, event):
 		"""Right button clicked: display the popup for the tree"""
-		# FIXME: should get the list item at the current position
-		# FIXME: should then update the context
 
 		node = event.GetItem()
 		self.SelectItem(node)
@@ -369,6 +483,9 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 			self.__handle_encounter_context(pos=pos)
 		elif node == self.GetRootItem():
 			self.__handle_root_context()
+		elif type(self.__curr_node_data) == type({}):
+			# ignore pseudo node "free-standing episodes"
+			pass
 		else:
 			print "error: unknown node type, no popup menu"
 		event.Skip()
@@ -385,6 +502,15 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 		item1 = self.GetPyData(node1)
 		item2 = self.GetPyData(node2)
 
+		# encounters: reverse chron
+		if isinstance(item1, gmEMRStructItems.cEncounter):
+			if item1['started'] == item2['started']:
+				return 0
+			if item1['started'] > item2['started']:
+				return -1
+			return 1
+
+		# episodes: chron
 		if isinstance(item1, gmEMRStructItems.cEpisode):
 			start1 = item1.get_access_range()[0]
 			start2 = item2.get_access_range()[0]
@@ -394,13 +520,7 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 				return -1
 			return 1
 
-		if isinstance(item1, gmEMRStructItems.cEncounter):
-			if item1['started'] == item2['started']:
-				return 0
-			if item1['started'] < item2['started']:
-				return -1
-			return 1
-
+		# issues: alpha
 		if isinstance(item1, gmEMRStructItems.cHealthIssue):
 			if item1['description'].lower() == item2['description'].lower():
 				return 0
@@ -408,6 +528,10 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 				return 1
 			return -1
 
+		# dummy health issue always on top
+		if isinstance(item1, type({})):
+			return -1
+
 		return 0
 #================================================================
 class cScrolledEMRTreePnl(wxgScrolledEMRTreePnl.wxgScrolledEMRTreePnl):
@@ -422,7 +546,7 @@ class cScrolledEMRTreePnl(wxgScrolledEMRTreePnl.wxgScrolledEMRTreePnl):
 #		self.__register_interests()
 	#--------------------------------------------------------
 #	def __register_interests(self):
-#		gmDispatcher.connect(signal=gmSignals.post_patient_selection(), receiver=self.repopulate_ui)
+#		gmDispatcher.connect(signal= u'post_patient_selection', receiver=self.repopulate_ui)
 	#--------------------------------------------------------
 	def repopulate_ui(self):
 		self._emr_tree.refresh()
@@ -442,7 +566,7 @@ class cSplittedEMRTreeBrowserPnl(wxgSplittedEMRTreeBrowserPnl.wxgSplittedEMRTree
 		self.__register_events()
 	#--------------------------------------------------------
 	def __register_events(self):
-		gmDispatcher.connect(signal = gmSignals.post_patient_selection(), receiver = self._on_post_patient_selection)
+		gmDispatcher.connect(signal = u'post_patient_selection', receiver = self._on_post_patient_selection)
 		return True
 	#--------------------------------------------------------
 	def _on_post_patient_selection(self):
@@ -461,9 +585,7 @@ class cEMRJournalPanel(wx.Panel):
 		wx.Panel.__init__(self, *args, **kwargs)
 
 		self.__do_layout()
-
-		if not self.__register_events():
-			raise gmExceptions.ConstructorError, 'cannot register interests'
+		self.__register_events()
 	#--------------------------------------------------------
 	def __do_layout(self):
 		self.__journal = wx.TextCtrl (
@@ -484,8 +606,7 @@ class cEMRJournalPanel(wx.Panel):
 		self.Layout()
 	#--------------------------------------------------------
 	def __register_events(self):
-		gmDispatcher.connect(signal = gmSignals.post_patient_selection(), receiver = self._on_post_patient_selection)
-		return True
+		gmDispatcher.connect(signal = u'post_patient_selection', receiver = self._on_post_patient_selection)
 	#--------------------------------------------------------
 	def _on_post_patient_selection(self):
 		"""Expects to be in a Notebook."""
@@ -503,29 +624,22 @@ class cEMRJournalPanel(wx.Panel):
 		try:
 			exporter.export(txt)
 			self.__journal.SetValue(txt.getvalue())
-			txt.close()
 		except ValueError:
-			_log.LogException('cannot get EMR journal')
+			_log.exception('cannot get EMR journal')
 			self.__journal.SetValue (_(
 				'An error occurred while retrieving the EMR\n'
 				'in journal form for the active patient.\n\n'
 				'Please check the log file for details.'
 			))
+		txt.close()
+		self.__journal.ShowPosition(self.__journal.GetLastPosition())
 		return True
 #================================================================
 # MAIN
 #----------------------------------------------------------------
 if __name__ == '__main__':
 
-	from Gnumed.pycommon import gmCfg
-
-	_log.SetAllLogLevels(gmLog.lData)
-	_log.Log (gmLog.lInfo, "starting emr browser...")
-
-	_cfg = gmCfg.gmDefCfgFile	 
-	if _cfg is None:
-		_log.Log(gmLog.lErr, "Cannot run without config file.")
-		sys.exit("Cannot run without config file.")
+	_log.info("starting emr browser...")
 
 	try:
 		# obtain patient
@@ -550,14 +664,62 @@ if __name__ == '__main__':
 			except:
 				print "error cleaning up patient"
 	except StandardError:
-		_log.LogException("unhandled exception caught !", sys.exc_info(), 1)
+		_log.exception("unhandled exception caught !")
 		# but re-raise them
 		raise
 
-	_log.Log (gmLog.lInfo, "closing emr browser...")
+	_log.info("closing emr browser...")
 
 #================================================================
 # $Log: gmEMRBrowser.py,v $
+# Revision 1.93.2.2  2008/10/12 17:02:59  ncq
+# - wording of EMR browser root node as per Jim
+#
+# Revision 1.93.2.1  2008/08/31 22:17:01  ncq
+# - don't crash on getting narrative change signal before tree has been populated
+#
+# Revision 1.93  2008/08/15 15:56:38  ncq
+# - properly handle context click on pseudo-issue
+#
+# Revision 1.92  2008/07/28 15:44:39  ncq
+# - context menu based Medistar export for any encounter
+#
+# Revision 1.91  2008/07/14 13:46:11  ncq
+# - better naming of dummy health issue
+#
+# Revision 1.90  2008/07/12 15:31:23  ncq
+# - improved formatting of issue info
+#
+# Revision 1.89  2008/07/07 13:44:33  ncq
+# - current patient .connected
+# - properly sort tree, encounters: most recent on top as per user request
+#
+# Revision 1.88  2008/06/28 18:25:58  ncq
+# - add expand to ... level popup menu items in EMR tree
+#
+# Revision 1.87  2008/05/19 16:23:33  ncq
+# - let EMR format its summary itself
+#
+# Revision 1.86  2008/04/11 12:27:45  ncq
+# - listen to issue/episode/narrative change signals thereby
+#   reducing direct repopulate calls
+# - factor out __update_text_for_selected_node() and
+#   call format() on nodes that have it
+# - rearrange code layout
+#
+# Revision 1.85  2008/03/05 22:30:14  ncq
+# - new style logging
+#
+# Revision 1.84  2008/01/30 14:07:24  ncq
+# - do not use old cfg file support anymore
+#
+# Revision 1.83  2008/01/22 12:20:53  ncq
+# - dummy health issue always on top
+# - auto-scroll to bottom of journal
+#
+# Revision 1.82  2007/12/11 12:49:25  ncq
+# - explicit signal handling
+#
 # Revision 1.81  2007/09/07 10:56:57  ncq
 # - cleanup
 #
