@@ -8,13 +8,13 @@
 """
 #================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmEMRStructWidgets.py,v $
-# $Id: gmEMRStructWidgets.py,v 1.71.2.2 2008/05/13 11:46:09 ncq Exp $
-__version__ = "$Revision: 1.71.2.2 $"
+# $Id: gmEMRStructWidgets.py,v 1.79.2.1 2008/10/15 14:45:07 ncq Exp $
+__version__ = "$Revision: 1.79.2.1 $"
 __author__ = "cfmoro1976@yahoo.es, karsten.hilbert@gmx.net"
 __license__ = "GPL"
 
 # stdlib
-import sys, re, datetime as pydt
+import sys, re, datetime as pydt, logging
 
 
 # 3rd party
@@ -24,18 +24,18 @@ import wx
 # GNUmed
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmLog, gmI18N, gmMatchProvider, gmDispatcher, gmSignals, gmTools, gmDateTime, gmCfg
+from Gnumed.pycommon import gmI18N, gmMatchProvider, gmDispatcher, gmTools, gmDateTime, gmCfg
 from Gnumed.business import gmEMRStructItems, gmPerson, gmSOAPimporter, gmSurgery
-from Gnumed.wxpython import gmPhraseWheel, gmGuiHelpers, gmListWidgets
+from Gnumed.wxpython import gmPhraseWheel, gmGuiHelpers, gmListWidgets, gmEditArea
 from Gnumed.wxGladeWidgets import wxgIssueSelectionDlg, wxgMoveNarrativeDlg
 from Gnumed.wxGladeWidgets import wxgHealthIssueEditAreaPnl, wxgHealthIssueEditAreaDlg
 from Gnumed.wxGladeWidgets import wxgEncounterEditAreaPnl, wxgEncounterEditAreaDlg
+from Gnumed.wxGladeWidgets import wxgEncounterTypeEditAreaPnl
 from Gnumed.wxGladeWidgets import wxgEpisodeEditAreaPnl, wxgEpisodeEditAreaDlg
 
 
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lInfo, __version__)
-	
+_log = logging.getLogger('gm.ui')
+_log.info(__version__)
 #================================================================
 # encounter related widgets/functions
 #----------------------------------------------------------------
@@ -56,6 +56,54 @@ def ask_for_encounter_continuation(msg=None, caption=None):
 		return True
 	return False
 #----------------------------------------------------------------
+def manage_encounter_types(parent=None):
+
+	if parent is None:
+		parent = wx.GetApp().GetTopWindow()
+
+	def edit(enc_type=None):
+		return edit_encounter_type(parent = parent, encounter_type = enc_type)
+
+	def delete(enc_type=None):
+		if gmEMRStructItems.delete_encounter_type(description = enc_type['description']):
+			return True
+		gmDispatcher.send (
+			signal = u'statustext',
+			msg = _('Cannot delete encounter type [%s]. It is in use.') % enc_type['l10n_description'],
+			beep = True
+		)
+		return False
+
+	def refresh(lctrl):
+		enc_types = gmEMRStructItems.get_encounter_types()
+		lctrl.set_string_items(items = enc_types)
+
+	enc_types = gmEMRStructItems.get_encounter_types()
+
+	gmListWidgets.get_choices_from_list (
+		parent = parent,
+		msg = _('\nSelect the encounter type you want to edit !\n'),
+		caption = _('Editing encounter types ...'),
+		choices = enc_types,
+		data = None,
+		columns = [_('Name'), _('System type')],
+		single_selection = True,
+		edit_callback = edit,
+		new_callback = edit,
+		delete_callback = delete,
+		refresh_callback = refresh
+	)
+#----------------------------------------------------------------
+def edit_encounter_type(parent=None, encounter_type=None):
+	ea = cEncounterTypeEditAreaPnl(parent = parent, id = -1)
+	ea.data = encounter_type
+	ea.mode = gmTools.coalesce(encounter_type, 'new', 'edit')
+	dlg = gmEditArea.cGenericEditAreaDlg2(parent = parent, id = -1, edit_area = ea)
+	dlg.SetTitle(gmTools.coalesce(encounter_type, _('Adding new encounter type'), _('Editing encounter type')))
+	if dlg.ShowModal() == wx.ID_OK:
+		return True
+	return False
+#----------------------------------------------------------------
 class cEncounterTypePhraseWheel(gmPhraseWheel.cPhraseWheel):
 	"""Phrasewheel to allow selection of encounter type.
 
@@ -101,6 +149,91 @@ select pk, l10n_description from (
 		self.selection_only = True
 		self.picklist_delay = 50
 #----------------------------------------------------------------
+class cEncounterTypeEditAreaPnl(wxgEncounterTypeEditAreaPnl.wxgEncounterTypeEditAreaPnl, gmEditArea.cGenericEditAreaMixin):
+
+	def __init__(self, *args, **kwargs):
+
+		wxgEncounterTypeEditAreaPnl.wxgEncounterTypeEditAreaPnl.__init__(self, *args, **kwargs)
+		gmEditArea.cGenericEditAreaMixin.__init__(self)
+
+		self.__register_interests()
+	#-------------------------------------------------------
+	# generic edit area API
+	#-------------------------------------------------------
+	def _valid_for_save(self):
+		if self.mode == 'edit':
+			if self._TCTRL_l10n_name.GetValue().strip() == u'':
+				self.display_tctrl_as_valid(tctrl = self._TCTRL_l10n_name, valid = False)
+				return False
+			self.display_tctrl_as_valid(tctrl = self._TCTRL_l10n_name, valid = True)
+			return True
+
+		no_errors = True
+
+		if self._TCTRL_l10n_name.GetValue().strip() == u'':
+			if self._TCTRL_name.GetValue().strip() == u'':
+				self.display_tctrl_as_valid(tctrl = self._TCTRL_l10n_name, valid = False)
+				no_errors = False
+			else:
+				self.display_tctrl_as_valid(tctrl = self._TCTRL_l10n_name, valid = True)
+		else:
+			self.display_tctrl_as_valid(tctrl = self._TCTRL_l10n_name, valid = True)
+
+		if self._TCTRL_name.GetValue().strip() == u'':
+			if self._TCTRL_l10n_name.GetValue().strip() == u'':
+				self.display_tctrl_as_valid(tctrl = self._TCTRL_name, valid = False)
+				no_errors = False
+			else:
+				self.display_tctrl_as_valid(tctrl = self._TCTRL_name, valid = True)
+		else:
+			self.display_tctrl_as_valid(tctrl = self._TCTRL_name, valid = True)
+
+		return no_errors
+	#-------------------------------------------------------
+	def _save_as_new(self):
+		enc_type = gmEMRStructItems.create_encounter_type (
+			description = gmTools.none_if(self._TCTRL_name.GetValue().strip(), u''),
+			l10n_description = gmTools.coalesce (
+				gmTools.none_if(self._TCTRL_l10n_name.GetValue().strip(), u''),
+				self._TCTRL_name.GetValue().strip()
+			)
+		)
+		if enc_type is None:
+			return False
+		self.data = enc_type
+		return True
+	#-------------------------------------------------------
+	def _save_as_update(self):
+		enc_type = gmEMRStructItems.update_encounter_type (
+			description = self._TCTRL_name.GetValue().strip(),
+			l10n_description = self._TCTRL_l10n_name.GetValue().strip()
+		)
+		if enc_type is None:
+			return False
+		self.data = enc_type
+		return True
+	#-------------------------------------------------------
+	def _refresh_as_new(self):
+		self._TCTRL_l10n_name.SetValue(u'')
+		self._TCTRL_name.SetValue(u'')
+		self._TCTRL_name.Enable(True)
+	#-------------------------------------------------------
+	def _refresh_from_existing(self):
+		self._TCTRL_l10n_name.SetValue(self.data['l10n_description'])
+		self._TCTRL_name.SetValue(self.data['description'])
+		# disallow changing type on all encounters by editing system name
+		self._TCTRL_name.Enable(False)
+	#-------------------------------------------------------
+	def _refresh_as_new_from_existing(self):
+		self._TCTRL_l10n_name.SetValue(self.data['l10n_description'])
+		self._TCTRL_name.SetValue(self.data['description'])
+		self._TCTRL_name.Enable(True)
+	#-------------------------------------------------------
+	# internal API
+	#-------------------------------------------------------
+	def __register_interests(self):
+		return
+#----------------------------------------------------------------
 class cEncounterEditAreaPnl(wxgEncounterEditAreaPnl.wxgEncounterEditAreaPnl):
 
 	def __init__(self, *args, **kwargs):
@@ -166,7 +299,7 @@ class cEncounterEditAreaPnl(wxgEncounterEditAreaPnl.wxgEncounterEditAreaPnl):
 		else:
 			self._TCTRL_aoe.SetFocus()
 
-		self._LBL_patient.SetLabel(pat.get_description())
+		self._TCTRL_patient.SetLabel(pat.get_description())
 
 		return True
 	#--------------------------------------------------------
@@ -218,8 +351,6 @@ class cEncounterEditAreaDlg(wxgEncounterEditAreaDlg.wxgEncounterEditAreaDlg):
 
 		self._PNL_edit_area.refresh(encounter=encounter)
 
-#		self.Refresh()		# needed ?
-
 		self.Fit()
 	#--------------------------------------------------------
 	def _on_save_button_pressed(self, evt):
@@ -398,41 +529,44 @@ class cEpisodeSelectionPhraseWheel(gmPhraseWheel.cPhraseWheel):
 	"""
 	def __init__(self, *args, **kwargs):
 
-		ctxt = {'ctxt_pat': {'where_part': u'pk_patient=%(pat)s', 'placeholder': u'pat'}}
+		ctxt = {'ctxt_pat': {'where_part': u'and pk_patient = %(pat)s', 'placeholder': u'pat'}}
 
 		mp = gmMatchProvider.cMatchProvider_SQL2 (
 			queries = [
-u"""
-select * from (
-	(select
-		pk_episode,
-		case when health_issue is Null
-			then description
-			else description || ' - ' || health_issue
-		end as description,
-		1 as weight
-	 from
-	  	clin.v_pat_episodes
-	 where
-		episode_open is true and
-		description %(fragment_condition)s and
-		%(ctxt_pat)s
-	) union all (
-	 select
-		pk_episode,
-		case when health_issue is Null
-			then description || _(' (closed)')
-			else description || _(' (closed)') || ' - ' || health_issue
-		end as description,
-		1 as weight
-	 from
-		clin.v_pat_episodes
-	 where
-		description %(fragment_condition)s and
-		episode_open is false and
-		%(ctxt_pat)s
-	)) as union_result
-order by weight, description
+u"""(
+
+select
+	pk_episode,
+	coalesce (
+		description || ' - ' || health_issue,
+		description
+	) as description,
+	1 as rank
+from
+  	clin.v_pat_episodes
+where
+	episode_open is true and
+	description %(fragment_condition)s
+	%(ctxt_pat)s
+
+) union all (
+
+select
+	pk_episode,
+	coalesce (
+		description || _(' (closed)') || ' - ' || health_issue,
+		description || _(' (closed)')
+	) as description,
+	1 as rank
+from
+	clin.v_pat_episodes
+where
+	description %(fragment_condition)s and
+	episode_open is false
+	%(ctxt_pat)s
+
+)
+order by rank, description
 limit 30"""
 ],
 			context = ctxt
@@ -447,7 +581,7 @@ limit 30"""
 			self.use_current_patient = True
 			self.__register_patient_change_signals()
 			pat = gmPerson.gmCurrentPatient()
-			if pat.is_connected():
+			if pat.connected:
 				mp.set_context('pat', pat.ID)
 		else:
 			self.use_current_patient = False
@@ -494,8 +628,8 @@ limit 30"""
 	# internal API
 	#--------------------------------------------------------
 	def __register_patient_change_signals(self):
-		gmDispatcher.connect(self._pre_patient_selection, gmSignals.pre_patient_selection())
-		gmDispatcher.connect(self._post_patient_selection, gmSignals.post_patient_selection())
+		gmDispatcher.connect(self._pre_patient_selection, u'pre_patient_selection')
+		gmDispatcher.connect(self._post_patient_selection, u'post_patient_selection')
 	#--------------------------------------------------------
 	def _pre_patient_selection(self):
 		return True
@@ -666,7 +800,7 @@ union
 			self.use_current_patient = True
 			self.__register_patient_change_signals()
 			pat = gmPerson.gmCurrentPatient()
-			if pat.is_connected():
+			if pat.connected:
 				mp.set_context('pat', pat.ID)
 		else:
 			self.use_current_patient = False
@@ -713,8 +847,8 @@ union
 	# internal API
 	#--------------------------------------------------------
 	def __register_patient_change_signals(self):
-		gmDispatcher.connect(self._pre_patient_selection, gmSignals.pre_patient_selection())
-		gmDispatcher.connect(self._post_patient_selection, gmSignals.post_patient_selection())
+		gmDispatcher.connect(self._pre_patient_selection, u'pre_patient_selection')
+		gmDispatcher.connect(self._post_patient_selection, u'post_patient_selection')
 	#--------------------------------------------------------
 	def _pre_patient_selection(self):
 		return True
@@ -751,8 +885,6 @@ class cIssueSelectionDlg(wxgIssueSelectionDlg.wxgIssueSelectionDlg):
 class cHealthIssueEditAreaPnl(wxgHealthIssueEditAreaPnl.wxgHealthIssueEditAreaPnl):
 	"""Panel encapsulating health issue edit area functionality."""
 
-	# FIXME: add on_lose_focus handling for year_diagnosed
-
 	def __init__(self, *args, **kwargs):
 		wxgHealthIssueEditAreaPnl.wxgHealthIssueEditAreaPnl.__init__(self, *args, **kwargs)
 
@@ -780,10 +912,13 @@ class cHealthIssueEditAreaPnl(wxgHealthIssueEditAreaPnl.wxgHealthIssueEditAreaPn
 	#--------------------------------------------------------
 	def _on_leave_age_noted(self, *args, **kwargs):
 
+		if not self._PRW_age_noted.IsModified():
+			return True
+
 		str_age = self._PRW_age_noted.GetValue().strip()
 
-		if str_age == '':
-			wx.CallAfter(self._PRW_year_noted.SetText, '')
+		if str_age == u'':
+			wx.CallAfter(self._PRW_year_noted.SetText, u'', None, True)
 			return True
 
 		age = gmTools.str2interval(str_interval = str_age)
@@ -794,14 +929,19 @@ class cHealthIssueEditAreaPnl(wxgHealthIssueEditAreaPnl.wxgHealthIssueEditAreaPn
 			gmDispatcher.send(signal='statustext', msg=_('Cannot parse [%s] into valid interval.') % str_age)
 			self._PRW_age_noted.SetBackgroundColour('pink')
 			self._PRW_age_noted.Refresh()
-			wx.CallAfter(self._PRW_year_noted.SetText, '')
+			wx.CallAfter(self._PRW_year_noted.SetText, u'', None, True)
 			return True
 
 		if age >= max_age:
-			gmDispatcher.send(signal='statustext', msg=_('Patient is only %s old. Cannot accept age [%s].') % (pat.get_medical_age(), age))
+			gmDispatcher.send (
+				signal = 'statustext',
+				msg = _(
+					'Foundational health issue cannot have been noted at age %s. Patient is only %s old.'
+				) % (age, pat.get_medical_age())
+			)
 			self._PRW_age_noted.SetBackgroundColour('pink')
 			self._PRW_age_noted.Refresh()
-			wx.CallAfter(self._PRW_year_noted.SetText, '')
+			wx.CallAfter(self._PRW_year_noted.SetText, u'', None, True)
 			return True
 
 		self._PRW_age_noted.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
@@ -822,15 +962,18 @@ class cHealthIssueEditAreaPnl(wxgHealthIssueEditAreaPnl.wxgHealthIssueEditAreaPn
 	#--------------------------------------------------------
 	def _on_leave_year_noted(self, *args, **kwargs):
 
+		if not self._PRW_year_noted.IsModified():
+			return True
+
 		year_noted = self._PRW_year_noted.GetData()
 
 		if year_noted is None:
-			if self._PRW_year_noted.GetValue().strip() == '':
-				wx.CallAfter(self._PRW_age_noted.SetText, '')
+			if self._PRW_year_noted.GetValue().strip() == u'':
+				wx.CallAfter(self._PRW_age_noted.SetText, u'', None, True)
 				return True
 			self._PRW_year_noted.SetBackgroundColour('pink')
 			self._PRW_year_noted.Refresh()
-			wx.CallAfter(self._PRW_age_noted.SetText, u'', None)
+			wx.CallAfter(self._PRW_age_noted.SetText, u'', None, True)
 			return True
 
 		year_noted = year_noted.get_pydt()
@@ -839,7 +982,7 @@ class cHealthIssueEditAreaPnl(wxgHealthIssueEditAreaPnl.wxgHealthIssueEditAreaPn
 			gmDispatcher.send(signal='statustext', msg=_('Condition diagnosed in the future.'))
 			self._PRW_year_noted.SetBackgroundColour('pink')
 			self._PRW_year_noted.Refresh()
-			wx.CallAfter(self._PRW_age_noted.SetText, u'')
+			wx.CallAfter(self._PRW_age_noted.SetText, u'', None, True)
 			return True
 
 		self._PRW_year_noted.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
@@ -853,17 +996,11 @@ class cHealthIssueEditAreaPnl(wxgHealthIssueEditAreaPnl.wxgHealthIssueEditAreaPn
 		return True
 	#--------------------------------------------------------
 	def _on_modified_age_noted(self, *args, **kwargs):
-		self._PRW_year_noted.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
-		#self._PRW_year_noted.Refresh()
-		self._PRW_year_noted.suppress_text_update_smarts = True
-		wx.CallAfter(self._PRW_year_noted.SetText)
+		wx.CallAfter(self._PRW_year_noted.SetText, u'', None, True)
 		return True
 	#--------------------------------------------------------
 	def _on_modified_year_noted(self, *args, **kwargs):
-		self._PRW_age_noted.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
-		#self._PRW_age_noted.Refresh()
-		self._PRW_age_noted.suppress_text_update_smarts = True
-		wx.CallAfter(self._PRW_age_noted.SetText)
+		wx.CallAfter(self._PRW_age_noted.SetText, u'', None, True)
 		return True
 	#--------------------------------------------------------
 	# external API
@@ -964,10 +1101,8 @@ class cHealthIssueEditAreaPnl(wxgHealthIssueEditAreaPnl.wxgHealthIssueEditAreaPn
 				gmDispatcher.send(signal='statustext', msg=_('Creating new health issue not allowed.'))
 				return False
 			pat = gmPerson.gmCurrentPatient()
-			success, self.__issue = gmEMRStructItems.create_health_issue (
-				patient_id = pat.ID,
-				description = desc
-			)
+			emr = pat.get_emr()
+			self.__issue = emr.add_health_issue(issue_name = desc)
 		else:
 			self.__issue['description'] = desc
 
@@ -993,7 +1128,7 @@ class cHealthIssueEditAreaPnl(wxgHealthIssueEditAreaPnl.wxgHealthIssueEditAreaPn
 		if narr != '':
 			pat = gmPerson.gmCurrentPatient()
 			emr = pat.get_emr()
-			epi = emr.add_episode(episode_name = _('past medical history'), pk_health_issue = self.__issue['pk'], is_open=None)
+			epi = emr.add_episode(episode_name = _('inception notes'), pk_health_issue = self.__issue['pk'], is_open=None)
 			if epi is not None:
 				epi['episode_open'] = False
 				epi.save_payload()			# FIXME: error handling
@@ -1036,12 +1171,6 @@ class cHealthIssueEditAreaDlg(wxgHealthIssueEditAreaDlg.wxgHealthIssueEditAreaDl
 # MAIN
 #----------------------------------------------------------------
 if __name__ == '__main__':
-
-	_log.SetAllLogLevels(gmLog.lData)
-
-	gmI18N.activate_locale()
-	gmI18N.install_domain()
-	gmDateTime.init()
 	
 	#================================================================	
 	class testapp (wx.App):
@@ -1143,37 +1272,65 @@ if __name__ == '__main__':
 		app.MainLoop()
 	#================================================================
 
-	# obtain patient
-	pat = gmPerson.ask_for_patient()
-	if pat is None:
-		print "No patient. Exiting gracefully..."
-		sys.exit(0)
-	gmPerson.set_active_patient(patient=pat)
+	if (len(sys.argv) > 1) and (sys.argv[1] == 'test'):
+
+		gmI18N.activate_locale()
+		gmI18N.install_domain()
+		gmDateTime.init()
+
+		# obtain patient
+		pat = gmPerson.ask_for_patient()
+		if pat is None:
+			print "No patient. Exiting gracefully..."
+			sys.exit(0)
+		gmPerson.set_active_patient(patient=pat)
 
 #	try:
 		# lauch emr dialogs test application
 #		app = testapp(0)
 #		app.MainLoop()
 #	except StandardError:
-#		_log.LogException("unhandled exception caught !", sys.exc_info(), 1)
+#		_log.exception("unhandled exception caught !")
 		# but re-raise them
 #		raise
 
-	#test_encounter_edit_area_panel()
-	#test_encounter_edit_area_dialog()
-	#test_epsiode_edit_area_pnl()
-	#test_episode_edit_area_dialog()
-	#test_health_issue_edit_area_dlg()
-	test_episode_selection_prw()
+		#test_encounter_edit_area_panel()
+		#test_encounter_edit_area_dialog()
+		#test_epsiode_edit_area_pnl()
+		#test_episode_edit_area_dialog()
+		#test_health_issue_edit_area_dlg()
+		test_episode_selection_prw()
 
 #================================================================
 # $Log: gmEMRStructWidgets.py,v $
-# Revision 1.71.2.2  2008/05/13 11:46:09  ncq
-# - properly handle age is None in pHX edit area
+# Revision 1.79.2.1  2008/10/15 14:45:07  ncq
+# - rename pseudo episode for health issue
 #
-# Revision 1.71.2.1  2008/05/07 15:12:50  ncq
-# - do not crash when leaving an invalidly filled year noted
-#   field in the health issue edit area
+# Revision 1.79  2008/07/24 13:58:40  ncq
+# - manage encounter types
+#
+# Revision 1.78  2008/07/07 13:43:16  ncq
+# - current patient .connected
+#
+# Revision 1.77  2008/06/09 15:33:59  ncq
+# - improved episode selector SQL
+#
+# Revision 1.76  2008/05/13 14:11:53  ncq
+# - properly handle age=None in pHX ea
+#
+# Revision 1.75  2008/05/07 15:21:10  ncq
+# - move health issue EA behaviour close to Richard's specs
+#
+# Revision 1.74  2008/03/05 22:37:45  ncq
+# - new style logging
+# - new health issue adding
+#
+# Revision 1.73  2008/01/30 14:03:41  ncq
+# - use signal names directly
+# - switch to std lib logging
+#
+# Revision 1.72  2008/01/22 12:21:27  ncq
+# - better encounter editor
 #
 # Revision 1.71  2007/10/07 12:32:41  ncq
 # - workplace property now on gmSurgery.gmCurrentPractice() borg
