@@ -11,316 +11,20 @@ to anybody else.
 """
 # ========================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmGuiHelpers.py,v $
-# $Id: gmGuiHelpers.py,v 1.76.2.1 2007/12/24 23:03:23 shilbert Exp $
-__version__ = "$Revision: 1.76.2.1 $"
+# $Id: gmGuiHelpers.py,v 1.96 2008/08/08 13:30:12 ncq Exp $
+__version__ = "$Revision: 1.96 $"
 __author__  = "K. Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL (details at http://www.gnu.org)"
 
-import sys, os, shutil, datetime as pyDT, traceback, exceptions, re as regex, codecs
-if __name__ == '__main__':
-	sys.exit("This is not intended to be run standalone !")
+import os
 
 
 import wx
 
 
 from Gnumed.business import gmSurgery
-from Gnumed.pycommon import gmLog, gmGuiBroker, gmPG2, gmLoginInfo, gmDispatcher, gmSignals, gmTools, gmCfg, gmI18N
-from Gnumed.wxGladeWidgets import wxg3ButtonQuestionDlg, wxg2ButtonQuestionDlg, wxgUnhandledExceptionDlg, wxgGreetingEditorDlg
+from Gnumed.wxGladeWidgets import wxg3ButtonQuestionDlg, wxg2ButtonQuestionDlg, wxgGreetingEditorDlg
 
-
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lData, __version__)
-
-_prev_excepthook = None
-
-# ========================================================================
-def handle_uncaught_exception_wx(t, v, tb):
-
-	_log.LogException('unhandled exception caught', (t,v,tb), verbose=True)
-
-	# careful: MSW does reference counting on Begin/End* :-(
-	try: wx.EndBusyCursor()
-	except: pass
-
-	if t == exceptions.ImportError:
-		gm_show_error (
-			aTitle = _('Missing GNUmed module'),
-			aMessage = _(
-				'GNUmed detected that parts of it are not\n'
-				'properly insalled. The following message\n'
-				'names the missing part:\n'
-				'\n'
-				' "%s"\n'
-				'\n'
-				'Please make sure to get the missing\n'
-				'parts installed. Otherwise some of the\n'
-				'functionality will not be accessible.'
-			) % v
-		)
-	else:
-		for target in _log.get_targets():
-			if not isinstance(target, gmLog.cLogTargetFile):
-				continue
-			original_name = target.ID
-			name = os.path.basename(target.ID)
-			name, ext = os.path.splitext(name)
-			new_name = os.path.expanduser(os.path.join (
-				'~',
-				'gnumed',
-				'logs',
-				'%s_%s%s' % (name, pyDT.datetime.now().strftime('%Y-%m-%d_%H-%M-%S'), ext)
-			))
-			_log.Log(gmLog.lWarn, 'syncing log file for backup to [%s]' % new_name)
-			_log.flush()
-			shutil.copy2(original_name, new_name)
-
-		dlg = cUnhandledExceptionDlg(parent = None, id = -1, exception = (t, v, tb), logfile = new_name)
-		dlg.ShowModal()
-		comment = dlg._TCTRL_comment.GetValue()
-		if comment is not None:
-			_log.Log(gmLog.lErr, u'user comment: %s' % comment.strip())
-			_log.flush()
-		dlg.Destroy()
-# ------------------------------------------------------------------------
-def install_wx_exception_handler():
-	office = gmSurgery.gmCurrentPractice()
-	global helpdesk
-	helpdesk = office.helpdesk
-
-	global _prev_excepthook
-	_prev_excepthook = sys.excepthook
-	sys.excepthook = handle_uncaught_exception_wx
-
-	return True
-# ------------------------------------------------------------------------
-def uninstall_wx_exception_handler():
-	if _prev_excepthook is None:
-		sys.excepthook = sys.__excepthook__
-		return True
-	sys.excepthook = _prev_excepthook
-	return True
-# ========================================================================
-class cUnhandledExceptionDlg(wxgUnhandledExceptionDlg.wxgUnhandledExceptionDlg):
-
-	def __init__(self, *args, **kwargs):
-
-		exception = kwargs['exception']
-		del kwargs['exception']
-		self.logfile = kwargs['logfile']
-		del kwargs['logfile']
-
-		wxgUnhandledExceptionDlg.wxgUnhandledExceptionDlg.__init__(self, *args, **kwargs)
-
-		self._TCTRL_helpdesk.SetValue(helpdesk)
-		self._TCTRL_logfile.SetValue(self.logfile)
-		t, v, tb = exception
-		self._TCTRL_exc_type.SetValue(str(t))
-		self._TCTRL_exc_value.SetValue(str(v))
-		self._TCTRL_traceback.SetValue(''.join(traceback.format_tb(tb)))
-
-		self.Fit()
-	#------------------------------------------
-	def _on_close_gnumed_button_pressed(self, evt):
-		top_win = wx.GetApp().GetTopWindow()
-		wx.CallAfter(top_win.Close)
-		evt.Skip()
-	#------------------------------------------
-	def _on_mail_button_pressed(self, evt):
-		receivers = regex.findall (
-			'[\S]+@[\S]+',
-			self._TCTRL_helpdesk.GetValue().strip(),
-			flags = regex.UNICODE | regex.LOCALE
-		)
-		if len(receivers) == 0:
-			receivers = [u'gnumed-devel@gnu.org']
-
-		receiver_string = wx.GetTextFromUser (
-			message = _(
-				'Edit the list of email addresses to send the\n'
-				'bug report to (separate addresses by spaces).\n'
-				'\n'
-				'Note that <gnumed-devel@gnu.org> refers to\n'
-				'the public GNUmed mailing list.'
-			),
-			caption = _('Sending bug report'),
-			default_value = ','.join(receivers),
-			parent = self
-		)
-		if receiver_string.strip() == u'':
-			evt.Skip()
-			return
-
-		receivers = regex.findall (
-			'[\S]+@[\S]+',
-			receiver_string,
-			flags = regex.UNICODE | regex.LOCALE
-		)
-
-		dlg = c2ButtonQuestionDlg (
-			self,
-			-1,
-			caption = _('Sending bug report'),
-			question = _(
-				'Your bug report will be sent to:\n'
-				'\n'
-				'%s\n'
-				'\n'
-				'Make sure you have reviewed the log file for potentially\n'
-				'sensitive information before sending out the bug report.\n'
-				'\n'
-				'Note that emailing the report may take a while depending\n'
-				'on the speed of your internet connection.\n'
-			) % u'\n'.join(receivers),
-			button_defs = [
-				{'label': _('Send report'), 'tooltip': _('Yes, send the bug report.')},
-				{'label': _('Cancel'), 'tooltip': _('No, do not send the bug report.')}
-			],
-			show_checkbox = True,
-			checkbox_msg = _('include log file in bug report')
-		)
-		dlg._CHBOX_dont_ask_again.SetValue(True)
-		go_ahead = dlg.ShowModal()
-		if go_ahead == wx.ID_NO:
-			dlg.Destroy()
-			evt.Skip()
-			return
-
-		comment = self._TCTRL_comment.GetValue().strip()
-		if comment == u'':
-			comment = wx.GetTextFromUser (
-				message = _(
-					'Please enter a short note on what you\n'
-					'were about to do in GNUmed:'
-				),
-				caption = _('Sending bug report'),
-				parent = self
-			)
-			if comment.strip() == u'':
-				dlg.Destroy()
-				evt.Skip()
-				return
-
-		msg = u"Report sent via GNUmed's handler for unexpected exceptions:\n\n %s\n\n" % comment
-		if dlg._CHBOX_dont_ask_again.GetValue():
-			for line in codecs.open(self.logfile, 'rU', 'latin1', 'replace'):
-				msg = msg + line
-
-		dlg.Destroy()
-
-		wx.BeginBusyCursor()
-		gmTools.send_mail (
-			sender = gmTools.default_mail_sender,
-			receiver = receivers,
-			subject = u'<bug>: %s' % comment,
-			message = msg,
-			encoding = gmI18N.get_encoding(),
-			server = gmTools.default_mail_server,
-			auth = {'user': gmTools.default_mail_sender, 'password': u'gnumed-at-gmx-net'}
-		)
-		wx.EndBusyCursor()
-
-		evt.Skip()
-	#------------------------------------------
-	def _on_view_log_button_pressed(self, evt):
-		from Gnumed.pycommon import gmMimeLib
-		gmMimeLib.call_viewer_on_file(self.logfile, block = False)
-		evt.Skip()
-# ========================================================================
-def configure_string_option(parent=None, message=None, option=None, bias=u'user', default_value=u'', validator=None):
-
-	dbcfg = gmCfg.cCfgSQL()
-
-	current_value = dbcfg.get2 (
-		option = option,
-		workplace = gmSurgery.gmCurrentPractice().active_workplace,
-		bias = bias,
-		default = default_value
-	)
-
-	if parent is None:
-		parent = wx.GetApp().GetTopWindow()
-
-	while True:
-		dlg = wx.TextEntryDialog (
-			parent = parent,
-			message = message,
-			caption = _('Configuration'),
-			defaultValue = u'%s' % current_value,
-			style = wx.OK | wx.CANCEL | wx.CENTRE
-		)
-		result = dlg.ShowModal()
-		if result == wx.ID_CANCEL:
-			dlg.Destroy()
-			return
-
-		user_val = dlg.GetValue().strip()
-		dlg.Destroy()
-
-		if user_val == current_value:
-			return
-
-		validated, user_val = validator(user_val)
-		if validated:
-			break
-
-		gmDispatcher.send (
-			signal = u'statustext',
-			msg = _('Value [%s] not valid for option <%s>.') % (user_val, option),
-			beep = True
-		)
-
-	dbcfg = gmCfg.cCfgSQL()
-	dbcfg.set (
-		workplace = gmSurgery.gmCurrentPractice().active_workplace,
-		option = option,
-		value = user_val
-	)
-
-	return
-# ========================================================================
-def configure_boolean_option(parent=None, question=None, option=None, button_tooltips=None):
-
-	if parent is None:
-		parent = wx.GetApp().GetTopWindow()
-
-	tooltips = [
-		_('Set "%s" to <True>.') % option,
-		_('Set "%s" to <False>.') % option,
-		_('Abort the dialog and do not change the current setting.')
-	]
-	if button_tooltips is not None:
-		for idx in range(len(button_tooltips)):
-			tooltips[idx] = button_tooltips[idx]
-
-	dlg = c3ButtonQuestionDlg (
-		parent,
-		-1,
-		caption = _('Configuration'),
-		question = question,
-		button_defs = [
-			{'label': _('Yes'), 'tooltip': tooltips[0]},
-			{'label': _('No'), 'tooltip': tooltips[1]},
-			{'label': _('Cancel'), 'tooltip': tooltips[2], 'default': True}
-		]
-	)
-
-	decision = dlg.ShowModal()
-	dbcfg = gmCfg.cCfgSQL()
-	if decision == wx.ID_YES:
-		dbcfg.set (
-			workplace = gmSurgery.gmCurrentPractice().active_workplace,
-			option = option,
-			value = True
-		)
-	elif decision == wx.ID_NO:
-		dbcfg.set (
-			workplace = gmSurgery.gmCurrentPractice().active_workplace,
-			option = option,
-			value = False
-		)
-
-	return
 # ========================================================================
 class c2ButtonQuestionDlg(wxg2ButtonQuestionDlg.wxg2ButtonQuestionDlg):
 
@@ -329,6 +33,10 @@ class c2ButtonQuestionDlg(wxg2ButtonQuestionDlg.wxg2ButtonQuestionDlg):
 		caption = kwargs['caption']
 		question = kwargs['question']
 		button_defs = kwargs['button_defs'][:2]
+		del kwargs['caption']
+		del kwargs['question']
+		del kwargs['button_defs']
+
 		try:
 			show_checkbox = kwargs['show_checkbox']
 			del kwargs['show_checkbox']
@@ -341,9 +49,11 @@ class c2ButtonQuestionDlg(wxg2ButtonQuestionDlg.wxg2ButtonQuestionDlg):
 		except KeyError:
 			checkbox_msg = None
 
-		del kwargs['caption']
-		del kwargs['question']
-		del kwargs['button_defs']
+		try:
+			checkbox_tooltip = kwargs['checkbox_tooltip']
+			del kwargs['checkbox_tooltip']
+		except KeyError:
+			checkbox_tooltip = None
 
 		wxg2ButtonQuestionDlg.wxg2ButtonQuestionDlg.__init__(self, *args, **kwargs)
 
@@ -355,14 +65,17 @@ class c2ButtonQuestionDlg(wxg2ButtonQuestionDlg.wxg2ButtonQuestionDlg):
 		else:
 			if checkbox_msg is not None:
 				self._CHBOX_dont_ask_again.SetLabel(checkbox_msg)
+			if checkbox_tooltip is not None:
+				self._CHBOX_dont_ask_again.SetToolTipString(checkbox_tooltip)
 
 		buttons = [self._BTN_1, self._BTN_2]
 		for idx in range(len(button_defs)):
 			buttons[idx].SetLabel(label = button_defs[idx]['label'])
 			buttons[idx].SetToolTipString(button_defs[idx]['tooltip'])
 			try:
-				if button_defs[idx]['default']:
+				if button_defs[idx]['default'] is True:
 					buttons[idx].SetDefault()
+					buttons[idx].SetFocus()
 			except KeyError:
 				pass
 
@@ -404,8 +117,9 @@ class c3ButtonQuestionDlg(wxg3ButtonQuestionDlg.wxg3ButtonQuestionDlg):
 			buttons[idx].SetLabel(label = button_defs[idx]['label'])
 			buttons[idx].SetToolTipString(button_defs[idx]['tooltip'])
 			try:
-				if button_defs[idx]['default']:
+				if button_defs[idx]['default'] is True:
 					buttons[idx].SetDefault()
+					buttons[idx].SetFocus()
 			except KeyError:
 				pass
 
@@ -424,7 +138,6 @@ class c3ButtonQuestionDlg(wxg3ButtonQuestionDlg.wxg3ButtonQuestionDlg):
 			self.EndModal(wx.ID_NO)
 		else:
 			self.Close()
-
 # ========================================================================
 class cStartupProgressBar(wx.ProgressDialog):
 	def __init__(self, nr_actions):
@@ -445,7 +158,8 @@ class cStartupProgressBar(wx.ProgressDialog):
 		try:
 			icon.LoadFile(png_fname, wx.BITMAP_TYPE_PNG)
 		except:
-			_log.Log(gmLog.lWarn, 'wx.Icon.LoadFile() not supported')
+			pass
+#			_log2.warning('wx.Icon.LoadFile() not supported')
 		self.SetIcon(icon)
 		self.idx = 0
 #		self.nr_plugins = nr_plugins
@@ -595,14 +309,10 @@ class cFileDropTarget(wx.FileDropTarget):
 	def OnDropFiles(self, x, y, filenames):
 		self.target.add_filenames(filenames)
 # ========================================================================
-def gm_show_error(aMessage = None, aTitle = None, aLogLevel = None):
+def gm_show_error(aMessage = None, aTitle = None):
 	if aMessage is None:
 		aMessage = _('programmer forgot to specify error message')
 
-	if aLogLevel is not None:
-		log_msg = aMessage.replace('\015', ' ').replace('\012', ' ')
-		_log.Log(aLogLevel, log_msg)
-
 	aMessage += _("\n\nPlease consult the error log for all the gory details !")
 
 	if aTitle is None:
@@ -618,14 +328,10 @@ def gm_show_error(aMessage = None, aTitle = None, aLogLevel = None):
 	dlg.Destroy()
 	return True
 #-------------------------------------------------------------------------
-def gm_show_info(aMessage = None, aTitle = None, aLogLevel = None):
+def gm_show_info(aMessage = None, aTitle = None):
 	if aMessage is None:
 		aMessage = _('programmer forgot to specify info message')
 
-	if aLogLevel is not None:
-		log_msg = aMessage.replace('\015', ' ').replace('\012', ' ')
-		_log.Log(aLogLevel, log_msg)
-
 	if aTitle is None:
 		aTitle = _('generic info message')
 
@@ -639,14 +345,10 @@ def gm_show_info(aMessage = None, aTitle = None, aLogLevel = None):
 	dlg.Destroy()
 	return True
 #-------------------------------------------------------------------------
-def gm_show_warning(aMessage = None, aTitle = None, aLogLevel = None):
+def gm_show_warning(aMessage=None, aTitle=None):
 	if aMessage is None:
 		aMessage = _('programmer forgot to specify warning')
 
-	if aLogLevel is not None:
-		log_msg = aMessage.replace('\015', ' ').replace('\012', ' ')
-		_log.Log(aLogLevel, log_msg)
-
 	if aTitle is None:
 		aTitle = _('generic warning message')
 
@@ -660,7 +362,7 @@ def gm_show_warning(aMessage = None, aTitle = None, aLogLevel = None):
 	dlg.Destroy()
 	return True
 #-------------------------------------------------------------------------
-def gm_show_question(aMessage = 'programmer forgot to specify question', aTitle = 'generic user question dialog', cancel_button=False):
+def gm_show_question(aMessage='programmer forgot to specify question', aTitle='generic user question dialog', cancel_button=False):
 	if cancel_button:
 		style = wx.YES_NO | wx.CANCEL | wx.ICON_QUESTION | wx.STAY_ON_TOP
 	else:
@@ -806,8 +508,72 @@ class cTextWidgetValidator(wx.PyValidator):
 
 # ========================================================================
 # $Log: gmGuiHelpers.py,v $
-# Revision 1.76.2.1  2007/12/24 23:03:23  shilbert
-# - removed the dlg.setfocus stuff
+# Revision 1.96  2008/08/08 13:30:12  ncq
+# - needs gmSurgery
+#
+# Revision 1.95  2008/08/06 13:21:42  ncq
+# - add checkbox tooltip support to 2 button question dialog
+#
+# Revision 1.94  2008/07/07 11:36:44  ncq
+# - just cleanup
+#
+# Revision 1.93  2008/06/24 13:59:18  ncq
+# - properly handle default buttons, SetFocus, too, in 2/3ButtonDlg
+#
+# Revision 1.92  2008/05/13 14:12:33  ncq
+# - factor out exception handling
+#
+# Revision 1.91  2008/04/12 19:18:48  ncq
+# - listen to application_closing and ignore
+#   PyDeadObjectError when closing down
+#
+# Revision 1.90  2008/03/06 18:33:12  ncq
+# - properly log exception information for unhandled exceptions
+#
+# Revision 1.89  2008/03/02 15:11:55  ncq
+# - support sender email in bug reporting
+#
+# Revision 1.88  2008/02/26 16:27:20  ncq
+# - cleanup exception handler
+# - actually log exception :-(
+#
+# Revision 1.87  2008/02/25 17:34:39  ncq
+# - use new logging
+# - auto-enable debug mode on first unhandled exception
+#
+# Revision 1.86  2008/01/22 12:22:18  ncq
+# - better layout of bug report email
+#
+# Revision 1.85  2008/01/16 19:38:43  ncq
+# - configure_*() factored out
+#
+# Revision 1.84  2008/01/13 01:17:50  ncq
+# - use log_stack_trace()
+# - annouce completed bug report emailing
+#
+# Revision 1.83  2008/01/11 16:14:05  ncq
+# - cleanup
+# - use staff name/system account in log mailing
+#
+# Revision 1.82  2008/01/07 19:52:40  ncq
+# - proper use of flush()
+#
+# Revision 1.81  2008/01/06 08:12:29  ncq
+# - auto-switch to --debug on detecting an unhandled exception
+# - always save user comment if there is any
+# - always backup the log file with comment for later perusal
+#
+# Revision 1.80  2008/01/05 16:41:27  ncq
+# - remove logging from gm_show_*()
+#
+# Revision 1.79  2007/12/24 23:31:24  shilbert
+# - remove some dlg.SetFocus statements
+#
+# Revision 1.78  2007/12/23 12:10:49  ncq
+# - cleanup
+#
+# Revision 1.77  2007/12/11 12:49:26  ncq
+# - explicit signal handling
 #
 # Revision 1.76  2007/12/04 17:32:33  ncq
 # - improved wording
