@@ -1,9 +1,20 @@
 """GNUmed list controls and widgets.
+
+TODO:
+
+	From: Rob McMullen <rob.mcmullen@gmail.com>
+	To: wxPython-users@lists.wxwidgets.org
+	Subject: Re: [wxPython-users] ANN: ColumnSizer mixin for ListCtrl
+
+	Thanks for all the suggestions, on and off line.  There's an update
+	with a new name (ColumnAutoSizeMixin) and better sizing algorithm at:
+
+	http://trac.flipturn.org/browser/trunk/peppy/lib/column_autosize.py
 """
 #================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmListWidgets.py,v $
-# $Id: gmListWidgets.py,v 1.21 2007/11/28 22:37:00 ncq Exp $
-__version__ = "$Revision: 1.21 $"
+# $Id: gmListWidgets.py,v 1.25 2008/08/06 13:22:14 ncq Exp $
+__version__ = "$Revision: 1.25 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL"
 
@@ -59,6 +70,7 @@ def get_choices_from_list(parent=None, msg=None, caption=None, choices=None, sel
 	return None
 #----------------------------------------------------------------
 class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDlg):
+	"""A dialog holding a list and a few buttons to act on the items."""
 
 	def __init__(self, *args, **kwargs):
 
@@ -119,6 +131,7 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 		if self.refresh_callback is None:
 			return
 		self.refresh_callback(lctrl = self._LCTRL_items)
+		self._LCTRL_items.set_column_widths()
 	#------------------------------------------------------------
 	def _on_edit_button_pressed(self, event):
 		# if the edit button *can* be pressed there are *supposed*
@@ -128,6 +141,7 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 		if self.refresh_callback is None:
 			return
 		self.refresh_callback(lctrl = self._LCTRL_items)
+		self._LCTRL_items.set_column_widths()
 	#------------------------------------------------------------
 	def _on_delete_button_pressed(self, event):
 		# if the delete button *can* be pressed there are *supposed*
@@ -137,6 +151,7 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 		if self.refresh_callback is None:
 			return
 		self.refresh_callback(lctrl = self._LCTRL_items)
+		self._LCTRL_items.set_column_widths()
 	#------------------------------------------------------------
 	# properties
 	#------------------------------------------------------------
@@ -150,6 +165,7 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 	new_callback = property(_get_noop, _set_new_callback)
 #================================================================
 class cGenericListManagerPnl(wxgGenericListManagerPnl.wxgGenericListManagerPnl):
+	"""A panel holding a generic multi-column list and action buttions."""
 
 	def __init__(self, *args, **kwargs):
 
@@ -270,8 +286,12 @@ class cReportListCtrl(wx.ListCtrl, listmixins.ListCtrlAutoWidthMixin):
 	#------------------------------------------------------------
 	def set_column_widths(self, widths=None):
 		if widths is None:
+			if self.GetItemCount() == 0:
+				width_type = wx.LIST_AUTOSIZE_USEHEADER
+			else:
+				width_type = wx.LIST_AUTOSIZE
 			for idx in range(self.GetColumnCount()):
-				self.SetColumnWidth(col = idx, width = wx.LIST_AUTOSIZE)
+				self.SetColumnWidth(col = idx, width = width_type)
 			return
 
 		for idx in range(len(widths)):
@@ -287,11 +307,21 @@ class cReportListCtrl(wx.ListCtrl, listmixins.ListCtrlAutoWidthMixin):
 			return
 
 		for item in items:
-			if type(item) == types.ListType:
-				# cannot use errors='replace' since then None/ints/unicode strings fails to get encoded
+			try:
+				item[0]
+				if not isinstance(item, basestring):
+					is_numerically_iterable = True
+				else:
+					is_numerically_iterable = False
+			except TypeError:
+				is_numerically_iterable = False
+
+			if is_numerically_iterable:
+				# cannot use errors='replace' since then
+				# None/ints/unicode strings fail to get encoded
 				col_val = unicode(item[0])
 				row_num = self.InsertStringItem(index = sys.maxint, label = col_val)
-				for col_idx in range(1, len(item)):
+				for col_idx in range(1, min(self.GetColumnCount(), len(item))):
 					col_val = unicode(item[col_idx])
 					self.SetStringItem(index = row_num, col = col_idx, label = col_val)
 			else:
@@ -405,6 +435,21 @@ if __name__ == '__main__':
 
 #================================================================
 # $Log: gmListWidgets.py,v $
+# Revision 1.25  2008/08/06 13:22:14  ncq
+# - fix detection of item list type
+#
+# Revision 1.24  2008/07/24 14:00:18  ncq
+# - better comments
+# - resize columns after list refreshing in generic list selector
+# - differentiate between iterables and non-iterables by means of
+#   an exception rather than checking for type.ListType in set_string_items
+#
+# Revision 1.23  2008/05/31 16:33:07  ncq
+# - add TODO with URL
+#
+# Revision 1.22  2008/02/26 16:28:04  ncq
+# - when auto-setting col widths in lists w/o items use header as width ;-)
+#
 # Revision 1.21  2007/11/28 22:37:00  ncq
 # - robustify in the absence of selected values
 #
