@@ -2,11 +2,11 @@
 """
 #================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmMedDocWidgets.py,v $
-# $Id: gmMedDocWidgets.py,v 1.150.2.2 2008/02/25 15:05:35 ncq Exp $
-__version__ = "$Revision: 1.150.2.2 $"
+# $Id: gmMedDocWidgets.py,v 1.166.2.4 2008/11/21 16:11:31 ncq Exp $
+__version__ = "$Revision: 1.166.2.4 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 
-import os.path, sys, re as regex
+import os.path, sys, re as regex, logging
 
 
 import wx
@@ -14,15 +14,17 @@ import wx
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmLog, gmI18N, gmCfg, gmPG2, gmMimeLib, gmExceptions, gmMatchProvider, gmDispatcher, gmSignals, gmDateTime, gmTools
+from Gnumed.pycommon import gmI18N, gmCfg, gmPG2, gmMimeLib, gmExceptions, gmMatchProvider, gmDispatcher, gmDateTime, gmTools, gmShellAPI, gmHooks
 from Gnumed.business import gmPerson, gmMedDoc, gmEMRStructItems, gmSurgery
 from Gnumed.wxpython import gmGuiHelpers, gmRegetMixin, gmPhraseWheel, gmPlugin, gmEMRStructWidgets, gmListWidgets
 from Gnumed.wxGladeWidgets import wxgScanIdxPnl, wxgReviewDocPartDlg, wxgSelectablySortedDocTreePnl, wxgEditDocumentTypesPnl, wxgEditDocumentTypesDlg
 
 
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lInfo, __version__)
+_log = logging.getLogger('gm.ui')
+_log.info(__version__)
 
+
+default_chunksize = 1 * 1024 * 1024		# 1 MB
 #============================================================
 def _save_file_as_new_document(**kwargs):
 	wx.CallAfter(save_file_as_new_document, **kwargs)
@@ -30,7 +32,7 @@ def _save_file_as_new_document(**kwargs):
 def save_file_as_new_document(parent=None, filename=None, document_type=None, unlock_patient=False, **kwargs):
 
 	pat = gmPerson.gmCurrentPatient()
-	if not pat.is_connected():
+	if not pat.connected:
 		return
 
 	emr = pat.get_emr()
@@ -129,14 +131,18 @@ class cEditDocumentTypesPnl(wxgEditDocumentTypesPnl.wxgEditDocumentTypesPnl):
 	def __init__(self, *args, **kwargs):
 		wxgEditDocumentTypesPnl.wxgEditDocumentTypesPnl.__init__(self, *args, **kwargs)
 		self.__init_ui()
+		self.__register_interests()
 		self.repopulate_ui()
 	#--------------------------------------------------------
 	def __init_ui(self):
-		# add column headers
-		self._LCTRL_doc_type.InsertColumn(0, _('type'))
-		self._LCTRL_doc_type.InsertColumn(1, _('description'))
-		self._LCTRL_doc_type.InsertColumn(2, _('user defined'))
-		self._LCTRL_doc_type.InsertColumn(3, _('in use'))
+		self._LCTRL_doc_type.set_columns([_('Type'), _('Translation'), _('User defined'), _('In use')])
+		self._LCTRL_doc_type.set_column_widths()
+	#--------------------------------------------------------
+	def __register_interests(self):
+		gmDispatcher.connect(signal = u'doc_type_mod_db', receiver = self._on_doc_type_mod_db)
+	#--------------------------------------------------------
+	def _on_doc_type_mod_db(self):
+		wx.CallAfter(self.repopulate_ui)
 	#--------------------------------------------------------
 	def repopulate_ui(self):
 
@@ -159,12 +165,14 @@ class cEditDocumentTypesPnl(wxgEditDocumentTypesPnl.wxgEditDocumentTypesPnl):
 			self._LCTRL_doc_type.SetColumnWidth(col=2, width=wx.LIST_AUTOSIZE_USEHEADER)
 			self._LCTRL_doc_type.SetColumnWidth(col=3, width=wx.LIST_AUTOSIZE_USEHEADER)
 
-		self._BTN_set_translation.Enable(False)
-		self._BTN_delete.Enable(False)
-
 		self._TCTRL_type.SetValue('')
 		self._TCTRL_l10n_type.SetValue('')
 
+		self._BTN_set_translation.Enable(False)
+		self._BTN_delete.Enable(False)
+		self._BTN_add.Enable(False)
+		self._BTN_reassign.Enable(False)
+
 		self._LCTRL_doc_type.SetFocus()
 	#--------------------------------------------------------
 	# event handlers
@@ -178,13 +186,23 @@ class cEditDocumentTypesPnl(wxgEditDocumentTypesPnl.wxgEditDocumentTypesPnl):
 		self._BTN_set_translation.Enable(True)
 		self._BTN_delete.Enable(not bool(doc_type['is_in_use']))
 		self._BTN_add.Enable(False)
+		self._BTN_reassign.Enable(True)
+
+		return
+	#--------------------------------------------------------
+	def _on_type_modified(self, event):
+		self._BTN_set_translation.Enable(False)
+		self._BTN_delete.Enable(False)
+		self._BTN_reassign.Enable(False)
+
+		self._BTN_add.Enable(True)
+#		self._LCTRL_doc_type.deselect_selected_item()
 		return
 	#--------------------------------------------------------
 	def _on_set_translation_button_pressed(self, event):
 		doc_type = self._LCTRL_doc_type.get_selected_item_data()
 		doc_type.set_translation(translation = self._TCTRL_l10n_type.GetValue().strip())
 
-		self.repopulate_ui()
 		return
 	#--------------------------------------------------------
 	def _on_delete_button_pressed(self, event):
@@ -202,7 +220,6 @@ class cEditDocumentTypesPnl(wxgEditDocumentTypesPnl.wxgEditDocumentTypesPnl):
 
 		gmMedDoc.delete_document_type(document_type = doc_type)
 
-		self.repopulate_ui()
 		return
 	#--------------------------------------------------------
 	def _on_add_button_pressed(self, event):
@@ -212,15 +229,39 @@ class cEditDocumentTypesPnl(wxgEditDocumentTypesPnl.wxgEditDocumentTypesPnl):
 			l10n_desc = self._TCTRL_l10n_type.GetValue().strip()
 			if (l10n_desc != '') and (l10n_desc != doc_type['l10n_type']):
 				doc_type.set_translation(translation = l10n_desc)
-			self.repopulate_ui()
 
 		return
 	#--------------------------------------------------------
-	def _on_type_modified(self, event):
-		self._BTN_set_translation.Enable(False)
-		self._BTN_delete.Enable(False)
-		self._BTN_add.Enable(True)
-#		self._LCTRL_doc_type.deselect_selected_item()
+	def _on_reassign_button_pressed(self, event):
+
+		orig_type = self._LCTRL_doc_type.get_selected_item_data()
+		doc_types = gmMedDoc.get_document_types()
+
+		new_type = gmListWidgets.get_choices_from_list (
+			parent = self,
+			msg = _(
+				'From the list below select the document type you want\n'
+				'all documents currently classified as:\n\n'
+				' "%s"\n\n'
+				'to be changed to.\n\n'
+				'Be aware that this change will be applied to ALL such documents. If there\n'
+				'are many documents to change it can take quite a while.\n\n'
+				'Make sure this is what you want to happen !\n'
+			) % orig_type['l10n_type'],
+			caption = _('Reassigning document type'),
+			choices = [ [gmTools.bool2subst(dt['is_user_defined'], u'X', u''), dt['type'], dt['l10n_type']] for dt in doc_types ],
+			columns = [_('User defined'), _('Type'), _('Translation')],
+			data = doc_types,
+			single_selection = True
+		)
+
+		if new_type is None:
+			return
+
+		wx.BeginBusyCursor()
+		gmMedDoc.reclassify_documents_by_type(original_type = orig_type, target_type = new_type)
+		wx.EndBusyCursor()
+
 		return
 #============================================================
 class cDocumentTypeSelectionPhraseWheel(gmPhraseWheel.cPhraseWheel):
@@ -281,10 +322,14 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 		# associated episode (add " " to avoid popping up pick list)
 		self._PhWheel_episode.SetText('%s ' % self.__part['episode'], self.__part['pk_episode'])
 		self._PhWheel_doc_type.SetText(value = self.__part['l10n_type'], data = self.__part['pk_type'])
+		self._PhWheel_doc_type.add_callback_on_set_focus(self._on_doc_type_gets_focus)
 		self._PhWheel_doc_type.add_callback_on_lose_focus(self._on_doc_type_loses_focus)
 
-		self._PRW_doc_comment.SetText(gmTools.coalesce(self.__part['doc_comment'], ''))
-		self._PRW_doc_comment.set_context(context = 'pk_doc_type', val = self.__part['pk_type'])
+		if self.__reviewing_doc:
+			self._PRW_doc_comment.SetText(gmTools.coalesce(self.__part['doc_comment'], ''))
+			self._PRW_doc_comment.set_context(context = 'pk_doc_type', val = self.__part['pk_type'])
+		else:
+			self._PRW_doc_comment.SetText(gmTools.coalesce(self.__part['obj_comment'], ''))
 
 		fts = gmDateTime.cFuzzyTimestamp(timestamp = self.__part['date_generated'])
 		self._PhWheel_doc_date.SetText(fts.strftime('%Y-%m-%d'), fts)
@@ -393,7 +438,8 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 		# to it so we don't check patient change
 		self.__doc['pk_episode'] = pk_episode
 		self.__doc['pk_type'] = doc_type
-		self.__doc['comment'] = self._PRW_doc_comment.GetValue().strip()
+		if self.__reviewing_doc:
+			self.__doc['comment'] = self._PRW_doc_comment.GetValue().strip()
 		self.__doc['date'] = self._PhWheel_doc_date.GetData().get_pydt()
 		self.__doc['ext_ref'] = self._TCTRL_reference.GetValue().strip()
 
@@ -419,21 +465,22 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 						msg = _('Error setting responsible clinician for this document.')
 			else:								# - just on this page
 				if not self.__part.set_reviewed(technically_abnormal = abnormal, clinically_relevant = relevant):
-					msg = _('Error setting "reviewed" status of this page.')
+					msg = _('Error setting "reviewed" status of this part.')
 				if self._ChBOX_responsible.GetValue():
 					self.__part['pk_intended_reviewer'] = provider['pk_staff']
 			if msg is not None:
 				gmGuiHelpers.gm_show_error(msg, _('editing document properties'))
 				return False
 
-		# 3) handle page specific parts
+		# 3) handle "page" specific parts
 		if not self.__reviewing_doc:
 			self.__part['filename'] = gmTools.none_if(self._TCTRL_filename.GetValue().strip(), u'')
 			self.__part['seq_idx'] = gmTools.none_if(self._SPINCTRL_seq_idx.GetValue(), 0)
+			self.__part['obj_comment'] = self._PRW_doc_comment.GetValue().strip()
 			success, data = self.__part.save_payload()
 			if not success:
 				gmGuiHelpers.gm_show_error (
-					_('Error saving page properties.'),
+					_('Error saving part properties.'),
 					_('editing document properties')
 				)
 				return False
@@ -446,6 +493,12 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 		self._ChBOX_relevant.Enable(enable = state)
 		self._ChBOX_responsible.Enable(enable = state)
 	#--------------------------------------------------------
+	def _on_doc_type_gets_focus(self):
+		"""Per Jim: Changing the doc type happens a lot more often
+		   then correcting spelling, hence select-all on getting focus.
+		"""
+		self._PhWheel_doc_type.SetSelection(-1, -1)
+	#--------------------------------------------------------
 	def _on_doc_type_loses_focus(self):
 		pk_doc_type = self._PhWheel_doc_type.GetData()
 		if pk_doc_type is None:
@@ -483,7 +536,7 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 	#--------------------------------------------------------
 	def add_filenames(self, filenames):
 		pat = gmPerson.gmCurrentPatient()
-		if not pat.is_connected():
+		if not pat.connected:
 			gmDispatcher.send(signal='statustext', msg=_('Cannot accept new documents. No active patient.'))
 			return
 
@@ -553,17 +606,32 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 		if len(self.acquired_pages) > 0:
 			for i in range(len(self.acquired_pages)):
 				fname = self.acquired_pages[i]
-				self._LBOX_doc_pages.Append(_('page %s: %s' % (i+1, fname)), fname)
+				self._LBOX_doc_pages.Append(_('part %s: %s' % (i+1, fname)), fname)
 	#--------------------------------------------------------
 	def __valid_for_save(self):
 		title = _('saving document')
 
 		if self.acquired_pages is None or len(self.acquired_pages) == 0:
-			gmGuiHelpers.gm_show_error (
-				aMessage = _('No pages to save. Aquire some pages first.'),
-				aTitle = title
-			)
-			return False
+			dbcfg = gmCfg.cCfgSQL()
+			allow_empty = bool(dbcfg.get2 (
+				option =  u'horstspace.scan_index.allow_partless_documents',
+				workplace = gmSurgery.gmCurrentPractice().active_workplace,
+				bias = 'user',
+				default = False
+			))
+			if allow_empty:
+				save_empty = gmGuiHelpers.gm_show_question (
+					aMessage = _('No parts to save. Really save an empty document as a reference ?'),
+					aTitle = title
+				)
+				if not save_empty:
+					return False
+			else:
+				gmGuiHelpers.gm_show_error (
+					aMessage = _('No parts to save. Aquire some parts first.'),
+					aTitle = title
+				)
+				return False
 
 		doc_type_pk = self._PhWheel_doc_type.GetData(can_create = True)
 		if doc_type_pk is None:
@@ -589,7 +657,7 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 
 		if self._PhWheel_reviewer.GetData() is None:
 			gmGuiHelpers.gm_show_error (
-				aMessage = _('You need to select the doctor who must review the document from the list of staff members.'),
+				aMessage = _('You need to select from the list of staff members the doctor who is intended to sign the document.'),
 				aTitle = title
 			)
 			return False
@@ -614,7 +682,7 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 		try:
 			devices = self.scan_module.get_devices()
 		except:
-			_log.LogException('cannot retrieve list of image sources')
+			_log.exception('cannot retrieve list of image sources')
 			gmDispatcher.send(signal = 'statustext', msg = _('There is no scanner support installed on this machine.'))
 			return None
 
@@ -711,8 +779,8 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 		page_idx = self._LBOX_doc_pages.GetSelection()
 		if page_idx == -1:
 			gmGuiHelpers.gm_show_info (
-				aMessage = _('You must select a page before you can view it.'),
-				aTitle = _('displaying page')
+				aMessage = _('You must select a part before you can view it.'),
+				aTitle = _('displaying part')
 			)
 			return None
 		# now, which file was that again ?
@@ -721,7 +789,7 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 		if not result:
 			gmGuiHelpers.gm_show_warning (
 				aMessage = _('Cannot display document part:\n%s') % msg,
-				aTitle = _('displaying page')
+				aTitle = _('displaying part')
 			)
 			return None
 		return 1
@@ -730,8 +798,8 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 		page_idx = self._LBOX_doc_pages.GetSelection()
 		if page_idx == -1:
 			gmGuiHelpers.gm_show_info (
-				aMessage = _('You must select a page before you can delete it.'),
-				aTitle = _('deleting page')
+				aMessage = _('You must select a part before you can delete it.'),
+				aTitle = _('deleting part')
 			)
 			return None
 		page_fname = self._LBOX_doc_pages.GetClientData(page_idx)
@@ -757,16 +825,16 @@ not want to lose the file.
 
 Pressing [YES] will permanently remove the file
 from your computer.""") % page_fname,
-			_('deleting page')
+			_('deleting part')
 		)
 		if do_delete:
 			try:
 				os.remove(page_fname)
 			except:
-				_log.LogException('Error deleting file.')
+				_log.exception('Error deleting file.')
 				gmGuiHelpers.gm_show_error (
-					aMessage = _('Cannot delete page in file [%s].\n\nYou may not have write access to it.') % page_fname,
-					aTitle = _('deleting page')
+					aMessage = _('Cannot delete part in file [%s].\n\nYou may not have write access to it.') % page_fname,
+					aTitle = _('deleting part')
 				)
 
 		return 1
@@ -945,9 +1013,10 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		gmRegetMixin.cRegetOnPaintMixin.__init__(self)
 
 		tmp = _('available documents (%s)')
+		unsigned = _('unsigned (%s) on top') % u'\u270D'
 		cDocTree._root_node_labels = {
 			'age': tmp % _('most recent on top'),
-			'review': tmp % _('unreviewed on top'),
+			'review': tmp % unsigned,
 			'episode': tmp % _('sorted by episode'),
 			'type': tmp % _('sorted by type')
 		}
@@ -955,6 +1024,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		self.root = None
 		self.__sort_mode = 'age'
 
+		self.__build_context_menus()
 		self.__register_interests()
 		self._schedule_data_reget()
 	#--------------------------------------------------------
@@ -989,7 +1059,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		self.__sort_mode = mode
 
 		curr_pat = gmPerson.gmCurrentPatient()
-		if not curr_pat.is_connected():
+		if not curr_pat.connected:
 			return
 
 		self._schedule_data_reget()
@@ -1000,7 +1070,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 	#--------------------------------------------------------
 	def _populate_with_data(self):
 		curr_pat = gmPerson.gmCurrentPatient()
-		if not curr_pat.is_connected():
+		if not curr_pat.connected:
 			gmDispatcher.send(signal = 'statustext', msg = _('Cannot load documents. No active patient.'))
 			return False
 
@@ -1023,6 +1093,74 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		gmDispatcher.connect(signal = u'doc_mod_db', receiver = self._on_doc_mod_db)
 		gmDispatcher.connect(signal = u'doc_page_mod_db', receiver = self._on_doc_page_mod_db)
 	#--------------------------------------------------------
+	def __build_context_menus(self):
+
+		# --- part context menu ---
+		self.__part_context_menu = wx.Menu(title = _('part menu'))
+
+		ID = wx.NewId()
+		self.__part_context_menu.Append(ID, _('Display part'))
+		wx.EVT_MENU(self.__part_context_menu, ID, self.__display_curr_part)
+
+		ID = wx.NewId()
+		self.__part_context_menu.Append(ID, _('%s Sign/Edit properties') % u'\u270D')
+		wx.EVT_MENU(self.__part_context_menu, ID, self.__review_curr_part)
+
+		self.__part_context_menu.AppendSeparator()
+
+		ID = wx.NewId()
+		self.__part_context_menu.Append(ID, _('Print part'))
+		wx.EVT_MENU(self.__part_context_menu, ID, self.__print_part)
+
+		ID = wx.NewId()
+		self.__part_context_menu.Append(ID, _('Fax part'))
+		wx.EVT_MENU(self.__part_context_menu, ID, self.__fax_part)
+
+		ID = wx.NewId()
+		self.__part_context_menu.Append(ID, _('Mail part'))
+		wx.EVT_MENU(self.__part_context_menu, ID, self.__mail_part)
+
+		self.__part_context_menu.AppendSeparator()
+
+
+		# --- doc context menu ---
+		self.__doc_context_menu = wx.Menu(title = _('document menu'))
+
+		ID = wx.NewId()
+		self.__doc_context_menu.Append(ID, _('%s Sign/Edit properties') % u'\u270D')
+		wx.EVT_MENU(self.__doc_context_menu, ID, self.__review_curr_part)
+
+		ID = wx.NewId()
+		self.__doc_context_menu.Append(ID, _('Export to disk'))
+		wx.EVT_MENU(self.__doc_context_menu, ID, self.__export_doc_to_disk)
+
+		ID = wx.NewId()
+		self.__doc_context_menu.Append(ID, _('Print parts'))
+		wx.EVT_MENU(self.__doc_context_menu, ID, self.__print_doc)
+
+		ID = wx.NewId()
+		self.__doc_context_menu.Append(ID, _('Fax parts'))
+		wx.EVT_MENU(self.__doc_context_menu, ID, self.__fax_doc)
+
+		ID = wx.NewId()
+		self.__doc_context_menu.Append(ID, _('Mail parts'))
+		wx.EVT_MENU(self.__doc_context_menu, ID, self.__mail_doc)
+
+		ID = wx.NewId()
+		self.__doc_context_menu.Append(ID, _('Access external original'))
+		wx.EVT_MENU(self.__doc_context_menu, ID, self.__access_external_original)
+
+		ID = wx.NewId()
+		self.__doc_context_menu.Append(ID, _('Edit corresponding consultation'))
+		wx.EVT_MENU(self.__doc_context_menu, ID, self.__edit_consultation_details)
+
+		ID = wx.NewId()
+		self.__doc_context_menu.Append(ID, _('Delete'))
+		wx.EVT_MENU(self.__doc_context_menu, ID, self.__delete_document)
+
+		self.__doc_context_menu.AppendSeparator()
+
+	#--------------------------------------------------------
 	def __populate_tree(self):
 
 		wx.BeginBusyCursor()
@@ -1064,14 +1202,14 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 
 			cmt = gmTools.coalesce(doc['comment'], _('no comment available'))
 			page_num = len(parts)
-			ref = gmTools.coalesce(initial = doc['ext_ref'], instead = _('no reference ID found'), template_initial = u'>%s<')
+			ref = gmTools.coalesce(initial = doc['ext_ref'], instead = _('no reference ID found'), template_initial = u'\u00BB%s\u00AB')
 
 			if doc.has_unreviewed_parts():
-				review = '!'
+				review = u'\u270D'
 			else:
-				review = ''
+				review = u''
 
-			label = _('%s%7s %s: %s (%s page(s), %s)') % (
+			label = _('%s%7s %s: %s (%s part(s), %s)') % (
 				review,
 				doc['date'].strftime('%m/%Y'),
 				doc['l10n_type'][:26],
@@ -1103,13 +1241,13 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 			# now add parts as child nodes
 			for part in parts:
 
-				pg = _('page %2s') % part['seq_idx']
+				pg = _('part %2s') % part['seq_idx']
 				cmt = gmTools.coalesce(part['obj_comment'], _("no comment available"))
 				sz = gmTools.size2str(part['size'])
 				rev = gmTools.bool2str (
-					bool = part['reviewed'] or part['reviewed_by_you'] or part['reviewed_by_intended_reviewer'],
+					boolean = part['reviewed'] or part['reviewed_by_you'] or part['reviewed_by_intended_reviewer'],
 					true_str = u'',
-					false_str = ' [%s]' % _('unreviewed')
+					false_str = u' \u270D'
 				)
 
 #				if part['clinically_relevant']:
@@ -1197,7 +1335,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 				return 1
 
 			else:
-				_log.Log(gmLog.lErr, 'unknown document sort mode [%s], reverse-sorting by age' % self.__sort_mode)
+				_log.error('unknown document sort mode [%s], reverse-sorting by age', self.__sort_mode)
 				# reverse sort by date
 				if item1[date_field] > item2[date_field]:
 					return -1
@@ -1300,29 +1438,6 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 	#--------------------------------------------------------
 	def __handle_doc_context(self):
 
-		menu = wx.Menu(title = _('document menu'))
-
-		# edit metadata
-		ID = wx.NewId()
-		menu.AppendItem(wx.MenuItem(menu, ID, _('Review/Edit properties')))
-		wx.EVT_MENU(menu, ID, self.__review_curr_part)
-
-		# export pages
-		ID = wx.NewId()
-		menu.AppendItem(wx.MenuItem(menu, ID, _('Export to disk')))
-		wx.EVT_MENU(menu, ID, self.__export_doc_to_disk)
-
-		# edit encounter
-		ID = wx.NewId()
-		menu.AppendItem(wx.MenuItem(menu, ID, _('Edit corresponding consultation')))
-		wx.EVT_MENU(menu, ID, self.__edit_consultation_details)
-
-		# delete for good
-		ID = wx.NewId()
-		menu.AppendItem(wx.MenuItem(menu, ID, _('Delete')))
-		wx.EVT_MENU(menu, ID, self.__delete_document)
-
-		# show descriptions
 		descriptions = self.__curr_node_data.get_descriptions()
 		desc_menu = wx.Menu()
 		for desc in descriptions:
@@ -1331,49 +1446,53 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 			tmp = regex.split('\r\n+|\r+|\n+|\s+|\t+', desc)
 			tmp = ' '.join(tmp)
 			# but only use first 30 characters
-			tmp = "%s ..." % tmp[:30]
-			desc_menu.AppendItem(wx.MenuItem(desc_menu, d_id, tmp))
+			tmp = "%s%s" % (tmp[:30], u'\u2026')
+			desc_menu.Append(d_id, tmp)
 			# connect handler
 			wx.EVT_MENU(desc_menu, d_id, self.__show_description)
+		ID = wx.NewId()
+		self.__doc_context_menu.AppendMenu(ID, _('descriptions ...'), desc_menu)
 
-		menu.AppendMenu(wx.NewId(), _('descriptions ...'), desc_menu)
+		self.PopupMenu(self.__doc_context_menu, wx.DefaultPosition)
 
-		# show menu
-		self.PopupMenu(menu, wx.DefaultPosition)
-		menu.Destroy()
+		self.__doc_context_menu.Delete(ID)
+		try:
+			desc_menu.Destroy()
+		# the C++ object has been observed to have been destroyed already
+		except wx._core.PyDeadObjectError:
+			pass
 	#--------------------------------------------------------
 	def __handle_part_context(self):
-		# build menu
-		menu = wx.Menu(title = _('page menu'))
-		# display file
-		ID = wx.NewId()
-		menu.AppendItem(wx.MenuItem(menu, ID, _('Display page')))
-		wx.EVT_MENU(menu, ID, self.__display_curr_part)
-		# edit metadata
-		ID = wx.NewId()
-		menu.AppendItem(wx.MenuItem(menu, ID, _('Review/Edit properties')))
-		wx.EVT_MENU(menu, ID, self.__review_curr_part)
+
 		# make active patient photograph
 		if self.__curr_node_data['type'] == 'patient photograph':
 			ID = wx.NewId()
-			menu.AppendItem(wx.MenuItem(menu, ID, _('Activate as current photo')))
-			wx.EVT_MENU(menu, ID, self.__activate_as_current_photo)
-		# show menu
-		self.PopupMenu(menu, wx.DefaultPosition)
-		menu.Destroy()
+			self.__part_context_menu.Append(ID, _('Activate as current photo'))
+			wx.EVT_MENU(self.__part_context_menu, ID, self.__activate_as_current_photo)
+		else:
+			ID = None
+
+		self.PopupMenu(self.__part_context_menu, wx.DefaultPosition)
+
+		if ID is not None:
+			self.__part_context_menu.Delete(ID)
+	#--------------------------------------------------------
+	# part level context menu handlers
 	#--------------------------------------------------------
 	def __display_part(self, part):
 		"""Display document part."""
 
 		# sanity check
 		if part['size'] == 0:
-			_log.Log(gmLog.lErr, 'cannot display part [%s] - 0 bytes' % part['pk_obj'])
+			_log.debug('cannot display part [%s] - 0 bytes', part['pk_obj'])
 			gmGuiHelpers.gm_show_error (
 				aMessage = _('Document part does not seem to exist in database !'),
 				aTitle = _('showing document')
 			)
 			return None
 
+		wx.BeginBusyCursor()
+
 		cfg = gmCfg.cCfgSQL()
 
 		# get export directory for temporary files
@@ -1383,16 +1502,16 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 				workplace = gmSurgery.gmCurrentPractice().active_workplace,
 				bias = 'workplace'
 			),
-			os.path.expanduser(os.path.join('~', '.gnumed', 'tmp', 'docs'))
+			os.path.expanduser(os.path.join('~', '.gnumed', 'tmp'))
 		)
-		_log.Log(gmLog.lData, "working into directory [%s]" % tmp_dir)
+		_log.debug("temporary directory [%s]", tmp_dir)
 
 		# determine database export chunk size
 		chunksize = int(cfg.get2 (
 			option = "horstspace.blob_export_chunk_size",
 			workplace = gmSurgery.gmCurrentPractice().active_workplace,
 			bias = 'workplace',
-			default = 1 * 1024 * 1024		# 1 MB
+			default = default_chunksize
 		))
 
 		# shall we force blocking during view ?
@@ -1409,6 +1528,9 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 			chunksize = chunksize,
 			block = block_during_view
 		)
+
+		wx.EndBusyCursor()
+
 		if not successful:
 			gmGuiHelpers.gm_show_error (
 				aMessage = _('Cannot display document part:\n%s') % msg,
@@ -1429,7 +1551,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		if review_after_display == 1:			# always review
 			self.__review_part(part=part)
 		elif review_after_display == 2:			# review if no review by me exists
-			review_by_me = filter(lambda rev: rev['is_review_by_you'], part.get_reviews())
+			review_by_me = filter(lambda rev: rev['is_your_review'], part.get_reviews())
 			if len(review_by_me) == 0:
 				self.__review_part(part=part)
 
@@ -1443,6 +1565,88 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		)
 		dlg.ShowModal()
 	#--------------------------------------------------------
+	def __process_part(self, action=None, l10n_action=None):
+
+		gmHooks.run_hook_script(hook = u'before_%s_doc_part' % action)
+
+		wx.BeginBusyCursor()
+
+		# detect wrapper
+		found, external_cmd = gmShellAPI.detect_external_binary(u'gm_%s_doc.sh' % action)
+		if not found:
+			found, external_cmd = gmShellAPI.detect_external_binary(u'gm_%s_doc.bat' % action)
+		if not found:
+			_log.error('neither of gm_%s_doc.sh or gm_%s_doc.bat found', action, action)
+			wx.EndBusyCursor()
+			gmGuiHelpers.gm_show_error (
+				_('Cannot %(l10n_action)s document part - %(l10n_action)s command not found.\n'
+				  '\n'
+				  'Either of gm_%(action)s_doc.sh or gm_%(action)s_doc.bat\n'
+				  'must be in the execution path. The command will\n'
+				  'be passed the filename to %(l10n_action)s.'
+				) % {'action': action, 'l10n_action': l10n_action},
+				_('Processing document part: %s') % l10n_action
+			)
+			return
+
+		cfg = gmCfg.cCfgSQL()
+
+		# get export directory for temporary files
+		tmp_dir = gmTools.coalesce (
+			cfg.get2 (
+				option = "horstspace.tmp_dir",
+				workplace = gmSurgery.gmCurrentPractice().active_workplace,
+				bias = 'workplace'
+			),
+			os.path.expanduser(os.path.join('~', '.gnumed', 'tmp'))
+		)
+		_log.debug("temporary directory [%s]", tmp_dir)
+
+		# determine database export chunk size
+		chunksize = int(cfg.get2 (
+			option = "horstspace.blob_export_chunk_size",
+			workplace = gmSurgery.gmCurrentPractice().active_workplace,
+			bias = 'workplace',
+			default = default_chunksize
+		))
+
+		part_file = self.__curr_node_data.export_to_file (
+			aTempDir = tmp_dir,
+			aChunkSize = chunksize
+		)
+
+		cmd = u'%s %s' % (external_cmd, part_file)
+		success = gmShellAPI.run_command_in_shell (
+			command = cmd,
+			blocking = False
+		)
+
+		wx.EndBusyCursor()
+
+		if not success:
+			_log.error('%s command failed: [%s]', action, cmd)
+			gmGuiHelpers.gm_show_error (
+				_('Cannot %(l10n_action)s document part - %(l10n_action)s command failed.\n'
+				  '\n'
+				  'You may need to check and fix either of\n'
+				  ' gm_%(action)s_doc.sh (Unix/Mac) or\n'
+				  ' gm_%(action)s_doc.bat (Windows)\n'
+				  '\n'
+				  'The command is passed the filename to %(l10n_action)s.'
+				) % {'action': action, 'l10n_action': l10n_action},
+				_('Processing document part: %s') % l10n_action
+			)
+	#--------------------------------------------------------
+	# FIXME: icons in the plugin toolbar
+	def __print_part(self, evt):
+		self.__process_part(action = u'print', l10n_action = _('print'))
+	#--------------------------------------------------------
+	def __fax_part(self, evt):
+		self.__process_part(action = u'fax', l10n_action = _('fax'))
+	#--------------------------------------------------------
+	def __mail_part(self, evt):
+		self.__process_part(action = u'mail', l10n_action = _('mail'))
+	#--------------------------------------------------------
 	# document level context menu handlers
 	#--------------------------------------------------------
 	def __edit_consultation_details(self, evt):
@@ -1450,6 +1654,135 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		dlg = gmEMRStructWidgets.cEncounterEditAreaDlg(parent=self, encounter=enc)
 		dlg.ShowModal()
 	#--------------------------------------------------------
+	def __process_doc(self, action=None, l10n_action=None):
+
+		gmHooks.run_hook_script(hook = u'before_%s_doc' % action)
+
+		wx.BeginBusyCursor()
+
+		# detect wrapper
+		found, external_cmd = gmShellAPI.detect_external_binary(u'gm_%s_doc.sh' % action)
+		if not found:
+			found, external_cmd = gmShellAPI.detect_external_binary(u'gm_%s_doc.bat' % action)
+		if not found:
+			_log.error('neither of gm_%s_doc.sh or gm_%s_doc.bat found', action, action)
+			wx.EndBusyCursor()
+			gmGuiHelpers.gm_show_error (
+				_('Cannot %(l10n_action)s document - %(l10n_action)s command not found.\n'
+				  '\n'
+				  'Either of gm_%(action)s_doc.sh or gm_%(action)s_doc.bat\n'
+				  'must be in the execution path. The command will\n'
+				  'be passed a list of filenames to %(l10n_action)s.'
+				) % {'action': action, 'l10n_action': l10n_action},
+				_('Processing document: %s') % l10n_action
+			)
+			return
+
+		cfg = gmCfg.cCfgSQL()
+
+		# get export directory for temporary files
+		tmp_dir = gmTools.coalesce (
+			cfg.get2 (
+				option = "horstspace.tmp_dir",
+				workplace = gmSurgery.gmCurrentPractice().active_workplace,
+				bias = 'workplace'
+			),
+			os.path.expanduser(os.path.join('~', '.gnumed', 'tmp'))
+		)
+		_log.debug("temporary directory [%s]", tmp_dir)
+
+		# determine database export chunk size
+		chunksize = int(cfg.get2 (
+			option = "horstspace.blob_export_chunk_size",
+			workplace = gmSurgery.gmCurrentPractice().active_workplace,
+			bias = 'workplace',
+			default = default_chunksize
+		))
+
+		part_files = self.__curr_node_data.export_parts_to_files (
+			export_dir = tmp_dir,
+			chunksize = chunksize
+		)
+
+		cmd = external_cmd + u' ' + u' '.join(part_files)
+		success = gmShellAPI.run_command_in_shell (
+			command = cmd,
+			blocking = False
+		)
+
+		wx.EndBusyCursor()
+
+		if not success:
+			_log.error('%s command failed: [%s]', action, cmd)
+			gmGuiHelpers.gm_show_error (
+				_('Cannot %(l10n_action)s document - %(l10n_action)s command failed.\n'
+				  '\n'
+				  'You may need to check and fix either of\n'
+				  ' gm_%(action)s_doc.sh (Unix/Mac) or\n'
+				  ' gm_%(action)s_doc.bat (Windows)\n'
+				  '\n'
+				  'The command is passed a list of filenames to %(l10n_action)s.'
+				) % {'action': action, 'l10n_action': l10n_action},
+				_('Processing document: %s') % l10n_action
+			)
+	#--------------------------------------------------------
+	# FIXME: icons in the plugin toolbar
+	def __print_doc(self, evt):
+		self.__process_doc(action = u'print', l10n_action = _('print'))
+	#--------------------------------------------------------
+	def __fax_doc(self, evt):
+		self.__process_doc(action = u'fax', l10n_action = _('fax'))
+	#--------------------------------------------------------
+	def __mail_doc(self, evt):
+		self.__process_doc(action = u'mail', l10n_action = _('mail'))
+	#--------------------------------------------------------
+	def __access_external_original(self, evt):
+
+		gmHooks.run_hook_script(hook = u'before_external_doc_access')
+
+		wx.BeginBusyCursor()
+
+		# detect wrapper
+		found, external_cmd = gmShellAPI.detect_external_binary(u'gm_access_external_doc.sh')
+		if not found:
+			found, external_cmd = gmShellAPI.detect_external_binary(u'gm_access_external_doc.bat')
+		if not found:
+			_log.error('neither of gm_access_external_doc.sh or .bat found')
+			wx.EndBusyCursor()
+			gmGuiHelpers.gm_show_error (
+				_('Cannot access external document - access command not found.\n'
+				  '\n'
+				  'Either of gm_access_external_doc.sh or *.bat must be\n'
+				  'in the execution path. The command will be passed the\n'
+				  'document type and the reference URL for processing.'
+				),
+				_('Accessing external document')
+			)
+			return
+
+		cmd = u'%s "%s" "%s"' % (external_cmd, self.__curr_node_data['type'], self.__curr_node_data['ext_ref'])
+		success = gmShellAPI.run_command_in_shell (
+			command = cmd,
+			blocking = False
+		)
+
+		wx.EndBusyCursor()
+
+		if not success:
+			_log.error('External access command failed: [%s]', cmd)
+			gmGuiHelpers.gm_show_error (
+				_('Cannot access external document - access command failed.\n'
+				  '\n'
+				  'You may need to check and fix either of\n'
+				  ' gm_access_external_doc.sh (Unix/Mac) or\n'
+				  ' gm_access_external_doc.bat (Windows)\n'
+				  '\n'
+				  'The command is passed the document type and the\n'
+				  'external reference URL on the command line.'
+				),
+				_('Accessing external document')
+			)
+	#--------------------------------------------------------
 	def __export_doc_to_disk(self, evt):
 		"""Export document into directory.
 
@@ -1479,23 +1812,40 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 			return True
 
 		wx.BeginBusyCursor()
-		fnames = self.__curr_node_data.export_parts_to_files(export_dir=dirname)
+
+		cfg = gmCfg.cCfgSQL()
+
+		# determine database export chunk size
+		chunksize = int(cfg.get2 (
+			option = "horstspace.blob_export_chunk_size",
+			workplace = gmSurgery.gmCurrentPractice().active_workplace,
+			bias = 'workplace',
+			default = default_chunksize
+		))
+
+		fnames = self.__curr_node_data.export_parts_to_files(export_dir = dirname, chunksize = chunksize)
+
 		wx.EndBusyCursor()
 
-		gmDispatcher.send(signal='statustext', msg=_('Successfully exported %s pages into the directory [%s].') % (len(fnames), dirname))
+		gmDispatcher.send(signal='statustext', msg=_('Successfully exported %s parts into the directory [%s].') % (len(fnames), dirname))
 
 		return True
 	#--------------------------------------------------------
 	def __delete_document(self, evt):
-		curr_pat = gmPerson.gmCurrentPatient()
-		emr = curr_pat.get_emr()
-		enc = emr.get_active_encounter()
-		gmMedDoc.delete_document(document_id = self.__curr_node_data['pk_doc'], encounter_id = enc['pk_encounter'])
+		result = gmGuiHelpers.gm_show_question (
+			aMessage = _('Are you sure you want to delete the document ?'),
+			aTitle = _('Deleting document')
+		)
+		if result is True:
+			curr_pat = gmPerson.gmCurrentPatient()
+			emr = curr_pat.get_emr()
+			enc = emr.get_active_encounter()
+			gmMedDoc.delete_document(document_id = self.__curr_node_data['pk_doc'], encounter_id = enc['pk_encounter'])
 #============================================================
 # main
 #------------------------------------------------------------
 if __name__ == '__main__':
-	_log.SetAllLogLevels(gmLog.lData)
+
 	gmI18N.activate_locale()
 	gmI18N.install_domain(domain = 'gnumed')
 
@@ -1507,11 +1857,75 @@ if __name__ == '__main__':
 
 #============================================================
 # $Log: gmMedDocWidgets.py,v $
-# Revision 1.150.2.2  2008/02/25 15:05:35  ncq
-# - do not crash on dropping files on listbox
+# Revision 1.166.2.4  2008/11/21 16:11:31  ncq
+# - apply the "comment" to the document or part when the review/details editor is
+#   invoked from a document or part, respectively
 #
-# Revision 1.150.2.1  2008/01/08 22:39:31  ncq
-# - 'first' / 'last' -> 'firstnames' / 'lastnames'
+# Revision 1.166.2.3  2008/11/20 21:57:48  ncq
+# - missing cfg =
+# - confirmation before deleting documents
+#
+# Revision 1.166.2.2  2008/10/15 14:46:22  ncq
+# - PyDeadObjectError is in wx._core
+#
+# Revision 1.166.2.1  2008/09/09 17:34:09  ncq
+# - fix faulty column access in review
+#
+# Revision 1.166  2008/08/20 14:54:46  ncq
+# - carefully work around wxMenu.Remove() doing something other
+#   than what the documentation suggests it does
+#
+# Revision 1.165  2008/07/10 20:00:34  ncq
+# - a few Begin/EndBusyCursor
+#
+# Revision 1.164  2008/07/07 13:43:17  ncq
+# - current patient .connected
+#
+# Revision 1.163  2008/05/31 16:38:57  ncq
+# - add permalink handling
+#
+# Revision 1.162  2008/05/29 15:31:46  ncq
+# - transition doc types editor to db change signal listener
+#
+# Revision 1.161  2008/05/29 13:29:42  ncq
+# - doc type handling: improve layout, support type change across all docs
+#
+# Revision 1.160  2008/04/12 19:21:19  ncq
+# - build doc/part context menus only once as far as
+#   possible in doc tree thereby preserving wx IDs
+# - handle print/fax/mail doc part
+# - properly call hook script
+#
+# Revision 1.159  2008/04/11 23:14:09  ncq
+# - centralize default_chunksize
+# - handle print/fax/mail document
+#
+# Revision 1.158  2008/04/11 12:28:55  ncq
+# - use signing hand again
+#
+# Revision 1.157  2008/04/02 10:21:25  ncq
+# - select-all on tabbing into doc type phrasewheel
+# - review -> sign
+# - use signing hand/not-checkmark unicode in some places
+#
+# Revision 1.156  2008/03/06 18:29:29  ncq
+# - standard lib logging only
+#
+# Revision 1.155  2008/02/25 17:38:05  ncq
+# - make parts listbox file drop target, too
+#
+# Revision 1.154  2008/01/27 21:16:45  ncq
+# - label changes per Jim
+# - allow partless docs
+#
+# Revision 1.153  2008/01/11 16:15:33  ncq
+# - first/last -> first-/lastnames
+#
+# Revision 1.152  2007/12/23 20:29:35  ncq
+# - store tmp docs in tmp/, not tmp/docs/
+#
+# Revision 1.151  2007/12/11 12:49:26  ncq
+# - explicit signal handling
 #
 # Revision 1.150  2007/11/05 11:41:46  ncq
 # - use blobs.delete_document()
