@@ -1,8 +1,8 @@
 #  coding: latin-1
-"""GNUmed quick patient search widget.
+"""GNUmed quick person search widgets.
 
-This widget allows to search for patients based on the
-critera name, date of birth and patient ID. It goes to
+This widget allows to search for persons based on the
+critera name, date of birth and person ID. It goes to
 considerable lengths to understand the user's intent from
 her input. For that to work well we need per-culture
 query generators. However, there's always the fallback
@@ -10,27 +10,30 @@ generator.
 """
 #============================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmPatSearchWidgets.py,v $
-# $Id: gmPatSearchWidgets.py,v 1.97.2.2 2008/05/07 16:04:11 ncq Exp $
-__version__ = "$Revision: 1.97.2.2 $"
+# $Id: gmPatSearchWidgets.py,v 1.110.2.2 2009/01/15 08:58:19 ncq Exp $
+__version__ = "$Revision: 1.110.2.2 $"
 __author__ = "K.Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = 'GPL (for details see http://www.gnu.org/)'
 
-import sys, os.path, time, glob, datetime as pyDT, re as regex
+import sys, os.path, glob, datetime as pyDT, re as regex, logging
 
 
 import wx
 
 
-from Gnumed.pycommon import gmLog, gmDispatcher, gmSignals, gmPG2, gmI18N, gmCfg, gmTools, gmDateTime, gmMatchProvider
+if __name__ == '__main__':
+	sys.path.insert(0, '../../')
+	from Gnumed.pycommon import gmLog2
+from Gnumed.pycommon import gmDispatcher, gmPG2, gmI18N, gmCfg, gmTools, gmDateTime, gmMatchProvider, gmCfg2
 from Gnumed.business import gmPerson, gmKVK, gmSurgery
 from Gnumed.wxpython import gmGuiHelpers, gmDemographicsWidgets
 from Gnumed.wxGladeWidgets import wxgSelectPersonFromListDlg, wxgSelectPersonDTOFromListDlg
 
 
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lInfo, __version__)
+_log = logging.getLogger('gm.person')
+_log.info(__version__)
 
-_cfg = gmCfg.gmDefCfgFile
+_cfg = gmCfg2.gmCfgData()
 
 ID_PatPickList = wx.NewId()
 ID_BTN_AddNew = wx.NewId()
@@ -79,7 +82,7 @@ class cSelectPersonFromListDlg(wxgSelectPersonFromListDlg.wxgSelectPersonFromLis
 			self._LCTRL_persons.SetStringItem(index = row_num, col = 6, label = label)
 			try: self._LCTRL_persons.SetStringItem(index = row_num, col = 7, label = person['match_type'])
 			except:
-				_log.LogException('cannot set match_type field')
+				_log.exception('cannot set match_type field')
 				self._LCTRL_persons.SetStringItem(index = row_num, col = 7, label = u'??')
 
 		for col in range(len(self.__cols)):
@@ -185,28 +188,64 @@ def load_persons_from_xdt():
 		bdt_files.append({'file': candidate, 'source': 'MCS/Isynet %s' % filename[-6:-4]})
 
 	# some need to be configured
-	xdt_profiles = _cfg.get('workplace', 'XDT profiles')
+	# aggregate sources
+	src_order = [
+		('explicit', 'return'),
+		('workbase', 'append'),
+		('local', 'append'),
+		('user', 'append'),
+		('system', 'append')
+	]
+	xdt_profiles = _cfg.get (
+		group = 'workplace',
+		option = 'XDT profiles',
+		source_order = src_order
+	)
 	if xdt_profiles is None:
 		return []
 
+	# first come first serve
+	src_order = [
+		('explicit', 'return'),
+		('workbase', 'return'),
+		('local', 'return'),
+		('user', 'return'),
+		('system', 'return')
+	]
 	for profile in xdt_profiles:
-		name = _cfg.get('XDT profile %s' % profile, 'filename')
+		name = _cfg.get (
+			group = 'XDT profile %s' % profile,
+			option = 'filename',
+			source_order = src_order
+		)
 		if name is None:
-			_log.Log(gmLog.lErr, 'XDT profile [%s] does not define a <filename>' % profile)
+			_log.error('XDT profile [%s] does not define a <filename>' % profile)
 			continue
-		encoding = _cfg.get('XDT profile %s' % profile, 'encoding')
+		encoding = _cfg.get (
+			group = 'XDT profile %s' % profile,
+			option = 'encoding',
+			source_order = src_order
+		)
 		if encoding is None:
-			_log.Log(gmLog.lWarn, 'xDT source profile [%s] does not specify an <encoding> for BDT file [%s]' % (profile, name))
-		source = _cfg.get('XDT profile %s' % profile, 'source')
-		dob_format = _cfg.get('XDT profile %s' % profile, 'DOB format')
+			_log.warning('xDT source profile [%s] does not specify an <encoding> for BDT file [%s]' % (profile, name))
+		source = _cfg.get (
+			group = 'XDT profile %s' % profile,
+			option = 'source',
+			source_order = src_order
+		)
+		dob_format = _cfg.get (
+			group = 'XDT profile %s' % profile,
+			option = 'DOB format',
+			source_order = src_order
+		)
 		if dob_format is None:
-			_log.Log(gmLog.lWarn, 'XDT profile [%s] does not define a date of birth format in <DOB format>' % profile)
+			_log.warning('XDT profile [%s] does not define a date of birth format in <DOB format>' % profile)
 		bdt_files.append({'file': name, 'source': source, 'encoding': encoding, 'dob_format': dob_format})
 
 	dtos = []
 	for bdt_file in bdt_files:
 		try:
-			# FIXME: potentially return several patients per file
+			# FIXME: potentially return several persons per file
 			dto = gmPerson.get_person_from_xdt (
 				filename = bdt_file['file'],
 				encoding = bdt_file['encoding'],
@@ -223,7 +262,7 @@ def load_persons_from_xdt():
 				) % bdt_file,
 				_('Activating xDT patient')
 			)
-			_log.LogException('cannot access xDT file [%s]' % bdt_file['file'])
+			_log.exception('cannot access xDT file [%s]' % bdt_file['file'])
 			continue
 		except:
 			gmGuiHelpers.gm_show_error (
@@ -232,9 +271,8 @@ def load_persons_from_xdt():
 				' [%s]'
 				) % bdt_file,
 				_('Activating xDT patient')
-			, aLogLevel = None
 			)
-			_log.LogException('cannot read patient from xDT file [%s]' % bdt_file['file'])
+			_log.exception('cannot read patient from xDT file [%s]' % bdt_file['file'])
 			continue
 
 		dtos.append({'dto': dto, 'source': gmTools.coalesce(bdt_file['source'], dto.source)})
@@ -255,15 +293,39 @@ def load_persons_from_pracsoft_au():
 		drive, filename = os.path.splitdrive(candidate)
 		pracsoft_files.append({'file': candidate, 'source': 'PracSoft (AU): drive %s' % drive})
 
+	src_order = [
+		('explicit', 'append'),
+		('workbase', 'append'),
+		('local', 'append'),
+		('user', 'append'),
+		('system', 'append')
+	]
 	# add configured one
-	fname = _cfg.get('AU PracSoft PATIENTS.IN', 'filename')
-	source = _cfg.get('AU PracSoft PATIENTS.IN', 'source')
-	if fname is not None and source is not None:
-		fname = os.path.abspath(os.path.expanduser(fname))
-		if os.access(fname, os.R_OK):
-			pracsoft_files.append({'file': os.path.expanduser(fname), 'source': source})
-		else:
-			_log.Log(gmLog.lErr, 'cannot read [%s] in AU PracSoft profile' % fname)
+	fnames = _cfg.get (
+		group = 'AU PracSoft PATIENTS.IN',
+		option = 'filename',
+		source_order = src_order
+	)
+	src_order = [
+		('explicit', 'return'),
+		('user', 'return'),
+		('system', 'return'),
+		('local', 'return'),
+		('workbase', 'return')
+	]
+	source = _cfg.get (
+		group = 'AU PracSoft PATIENTS.IN',
+		option = 'source',
+		source_order = src_order
+	)
+
+	if source is not None:
+		for fname in fnames:
+			fname = os.path.abspath(os.path.expanduser(fname))
+			if os.access(fname, os.R_OK):
+				pracsoft_files.append({'file': os.path.expanduser(fname), 'source': source})
+			else:
+				_log.error('cannot read [%s] in AU PracSoft profile' % fname)
 
 	# and parse them
 	dtos = []
@@ -271,7 +333,7 @@ def load_persons_from_pracsoft_au():
 		try:
 			tmp = gmPerson.get_persons_from_pracsoft_file(filename = pracsoft_file['file'])
 		except:
-			_log.LogException('cannot parse PracSoft file [%s]' % pracsoft_file['file'])
+			_log.exception('cannot parse PracSoft file [%s]' % pracsoft_file['file'])
 			continue
 		for dto in tmp:
 			dtos.append({'dto': dto, 'source': pracsoft_file['source']})
@@ -280,8 +342,8 @@ def load_persons_from_pracsoft_au():
 #============================================================
 def load_persons_from_kvks():
 
-	db_cfg = gmCfg.cCfgSQL()
-	kvk_dir = os.path.abspath(os.path.expanduser(db_cfg.get2 (
+	dbcfg = gmCfg.cCfgSQL()
+	kvk_dir = os.path.abspath(os.path.expanduser(dbcfg.get2 (
 		option = 'DE.KVK.spool_dir',
 		workplace = gmSurgery.gmCurrentPractice().active_workplace,
 		bias = 'workplace',
@@ -293,7 +355,7 @@ def load_persons_from_kvks():
 
 	return dtos
 #============================================================
-def load_patient_from_external_sources(parent=None, search_immediately=False):
+def get_person_from_external_sources(parent=None, search_immediately=False, activate_immediately=False):
 	"""Load patient from external source.
 
 	- scan external sources for candidates
@@ -313,31 +375,31 @@ def load_patient_from_external_sources(parent=None, search_immediately=False):
 	dtos.extend(load_persons_from_pracsoft_au())
 	dtos.extend(load_persons_from_kvks())
 
-	# no external patients
+	# no external persons
 	if len(dtos) == 0:
 		gmDispatcher.send(signal='statustext', msg=_('No patients found in external sources.'))
-		return True
+		return None
 
 	# one external patient with DOB - already active ?
 	if (len(dtos) == 1) and (dtos[0]['dto'].dob is not None):
 		dto = dtos[0]['dto']
 		# is it already the current patient ?
 		curr_pat = gmPerson.gmCurrentPatient()
-		if curr_pat.is_connected():
+		if curr_pat.connected:
 			key_dto = dto.firstnames + dto.lastnames + dto.dob.strftime('%Y-%m-%d') + dto.gender
 			names = curr_pat.get_active_name()
 			key_pat = names['firstnames'] + names['lastnames'] + curr_pat['dob'].strftime('%Y-%m-%d') + curr_pat['gender']
-			_log.Log(gmLog.lData, 'current patient: %s' % key_pat)
-			_log.Log(gmLog.lData, 'dto patient    : %s' % key_dto)
+			_log.debug('current patient: %s' % key_pat)
+			_log.debug('dto patient    : %s' % key_dto)
 			if key_dto == key_pat:
 				gmDispatcher.send(signal='statustext', msg=_('The only external patient is already active in GNUmed.'), beep=False)
-				return True
+				return None
 
-	# one external patient - activate immediately ?
+	# one external person - look for internal match immediately ?
 	if (len(dtos) == 1) and search_immediately:
 		dto = dtos[0]['dto']
 
-	# several external patients
+	# several external persons
 	else:
 		if parent is None:
 			parent = wx.GetApp().GetTopWindow()
@@ -345,15 +407,12 @@ def load_patient_from_external_sources(parent=None, search_immediately=False):
 		dlg.set_dtos(dtos=dtos)
 		result = dlg.ShowModal()
 		if result == wx.ID_CANCEL:
-			return True
+			return None
 		dto = dlg.get_selected_dto()['dto']
 		dlg.Destroy()
 
 	# search
-#	searcher = gmPerson.cPatientSearcher_SQL()
-#	idents = searcher.get_identities(dto = dto)
 	idents = dto.get_candidate_identities(can_create=True)
-
 	if idents is None:
 		gmGuiHelpers.gm_show_info (_(
 			'Cannot create new patient:\n\n'
@@ -361,25 +420,7 @@ def load_patient_from_external_sources(parent=None, search_immediately=False):
 			) % (dto.firstnames, dto.lastnames, dto.gender, dto.dob.strftime('%x').decode(gmI18N.get_encoding())),
 			_('Activating external patient')
 		)
-		return False
-
-#	ident = None
-#	if len(idents) == 0:
-#		ident = gmPerson.create_identity (
-#			firstnames = dto.firstnames,
-#			lastnames = dto.lastnames,
-#			gender = dto.gender,
-#			dob = dto.dob
-#		)
-#		if ident is None:
-#			gmGuiHelpers.gm_show_info (
-#				_(
-#				'Cannot create new patient:\n\n'
-#				' [%s %s (%s), %s]'
-#				) % (dto.firstnames, dto.lastnames, dto.gender, dto.dob.strftime('%x').decode(gmI18N.get_encoding())),
-#				_('Activating xDT patient')
-#			)
-#			return False
+		return None
 
 	if len(idents) == 1:
 		ident = idents[0]
@@ -391,42 +432,317 @@ def load_patient_from_external_sources(parent=None, search_immediately=False):
 		dlg.set_persons(persons=idents)
 		result = dlg.ShowModal()
 		if result == wx.ID_CANCEL:
-			return True
+			return None
 		ident = dlg.get_selected_person()
 		dlg.Destroy()
 
-	if not gmPerson.set_active_patient(patient = ident):
-		gmGuiHelpers.gm_show_info (
-			_(
-			'Cannot activate patient:\n\n'
-			'%s %s (%s)\n'
-			'%s'
-			) % (dto.firstnames, dto.lastnames, dto.gender, dto.dob.strftime('%x').decode(gmI18N.get_encoding())),
-			_('Activating external patient')
-		)
-		return False
+	if activate_immediately:
+		if not gmPerson.set_active_patient(patient = ident):
+			gmGuiHelpers.gm_show_info (
+				_(
+				'Cannot activate patient:\n\n'
+				'%s %s (%s)\n'
+				'%s'
+				) % (dto.firstnames, dto.lastnames, dto.gender, dto.dob.strftime('%x').decode(gmI18N.get_encoding())),
+				_('Activating external patient')
+			)
+			return None
 
 	dto.import_extra_data(identity = ident)
 	dto.delete_from_source()
 
-	return True
+	return ident
 #============================================================
-class cPatientSelector(wx.TextCtrl):
-	"""Widget for smart search for patients."""
-	def __init__ (self, parent, id = -1, pos = wx.DefaultPosition, size = wx.DefaultSize):
-		self.curr_pat = gmPerson.gmCurrentPatient()
+class cPersonSearchCtrl(wx.TextCtrl):
+	"""Widget for smart search for persons."""
+
+	def __init__(self, *args, **kwargs):
+
+		try:
+			kwargs['style'] = kwargs['style'] | wx.TE_PROCESS_ENTER
+		except KeyError:
+			kwargs['style'] = wx.TE_PROCESS_ENTER
 
 		# need to explicitely process ENTER events to avoid
 		# them being handed over to the next control
-		wx.TextCtrl.__init__(
-			self,
-			parent,
-			id,
-			'',
-			pos,
-			size,
-			style = wx.TE_PROCESS_ENTER
-		)
+		wx.TextCtrl.__init__(self, *args, **kwargs)
+
+		self.person = None
+
+		self.SetToolTipString (_(
+			'To search for a person type any of:                   \n'
+			'\n'
+			' - fragment of last or first name\n'
+			" - date of birth (can start with '$' or '*')\n"
+			" - GNUmed ID of person (can start with '#')\n"
+			' - exterenal ID of person\n'
+			'\n'
+			'and hit <ENTER>.\n'
+			'\n'
+			'Shortcuts:\n'
+			' <F2>\n'
+			'  - scan external sources for persons\n'
+			' <CURSOR-UP>\n'
+			'  - recall most recently used search term\n'
+			' <CURSOR-DOWN>\n'
+			'  - list 10 most recently found persons\n'
+		))
+
+		# FIXME: set query generator
+		self.__person_searcher = gmPerson.cPatientSearcher_SQL()
+
+		self._prev_search_term = None
+		self.__prev_idents = []
+		self._lclick_count = 0
+
+		self._display_name()
+
+		self.__register_events()
+	#--------------------------------------------------------
+	# utility methods
+	#--------------------------------------------------------
+	def _display_name(self):
+		name = u''
+
+		if self.person is not None:
+			name = self.person['description']
+
+		self.SetValue(name)
+	#--------------------------------------------------------
+	def _remember_ident(self, ident=None):
+
+		if not isinstance(ident, gmPerson.cIdentity):
+			return False
+
+		# only unique identities
+		for known_ident in self.__prev_idents:
+			if known_ident['pk_identity'] == ident['pk_identity']:
+				return True
+
+		self.__prev_idents.append(ident)
+
+		# and only 10 of them
+		if len(self.__prev_idents) > 10:
+			self.__prev_idents.pop(0)
+
+		return True
+	#--------------------------------------------------------
+	# event handling
+	#--------------------------------------------------------
+	def __register_events(self):
+		wx.EVT_CHAR(self, self.__on_char)
+		wx.EVT_SET_FOCUS(self, self._on_get_focus)
+		wx.EVT_KILL_FOCUS (self, self._on_loose_focus)
+		wx.EVT_TEXT_ENTER (self, self.GetId(), self.__on_enter)
+	#--------------------------------------------------------
+#	def _on_left_mousebutton_up(self, evt):
+#		"""upon left click release
+
+#		- select all text in the field so that the next
+#		  character typed will delete it
+		
+#		- or set cursor to text position in case more left
+#		  clicks follow
+#		"""
+		# unclicked, not highlighted
+#		if self._lclick_count == 0:
+#			self.SetSelection (-1,-1)			# highlight entire text
+#			self._lclick_count = 1
+#			evt.Skip()
+#			return None
+			
+		# has been clicked before - should be highlighted
+#		start, end = self.GetSelection()
+#		self.SetSelection(start, end)
+#		self._lclick_count = 0
+#		evt.Skip()
+#		return None
+	#--------------------------------------------------------
+	def _on_get_focus(self, evt):
+		"""upon tabbing in
+
+		- select all text in the field so that the next
+		  character typed will delete it
+		"""
+		wx.CallAfter(self.SetSelection, -1, -1)
+		evt.Skip()
+	#--------------------------------------------------------
+	def _on_loose_focus(self, evt):
+		# - redraw the currently active name upon losing focus
+
+		# if we use wx.EVT_KILL_FOCUS we will also receive this event
+		# when closing our application or loosing focus to another
+		# application which is NOT what we intend to achieve,
+		# however, this is the least ugly way of doing this due to
+		# certain vagaries of wxPython (see the Wiki)
+
+		# remember fragment
+#		curr_search_term = self.GetValue()
+#		if self.IsModified() and (curr_search_term.strip() != ''):
+#			self._prev_search_term = curr_search_term
+
+		# just for good measure
+		wx.CallAfter(self.SetSelection, 0, 0)
+#		self._lclick_count = 0
+
+		self._display_name()
+		self._remember_ident(self.person)
+
+		evt.Skip()
+	#--------------------------------------------------------
+	def __on_char(self, evt):
+		self._on_char(evt)
+
+	def _on_char(self, evt):
+		"""True: patient was selected.
+		   False: no patient was selected.
+		"""
+		keycode = evt.GetKeyCode()
+
+		# list of previously active patients
+		if keycode == wx.WXK_DOWN:
+			evt.Skip()
+			if len(self.__prev_idents) == 0:
+				return False
+
+			dlg = cSelectPersonFromListDlg(parent = wx.GetTopLevelParent(self), id = -1)
+			dlg.set_persons(persons = self.__prev_idents)
+			result = dlg.ShowModal()
+			if result == wx.ID_OK:
+				wx.BeginBusyCursor()
+				self.person = dlg.get_selected_person()
+				self._display_name()
+				dlg.Destroy()
+				wx.EndBusyCursor()
+				return True
+
+			dlg.Destroy()
+			return False
+
+		# recall previous search fragment
+		if keycode == wx.WXK_UP:
+			evt.Skip()
+			# FIXME: cycling through previous fragments
+			if self._prev_search_term is not None:
+				self.SetValue(self._prev_search_term)
+			return False
+
+		# invoke external patient sources
+		if keycode == wx.WXK_F2:
+			evt.Skip()
+			dbcfg = gmCfg.cCfgSQL()
+			search_immediately = bool(dbcfg.get2 (
+				option = 'patient_search.external_sources.immediately_search_if_single_source',
+				workplace = gmSurgery.gmCurrentPractice().active_workplace,
+				bias = 'user',
+				default = 0
+			))
+			p = get_person_from_external_sources (
+				parent = wx.GetTopLevelParent(self),
+				search_immediately = search_immediately
+			)
+			if p is not None:
+				self.person = p
+				self._display_name()
+				return True
+			return False
+
+		# FIXME: invoke add new person
+		# FIXME: add popup menu apart from system one
+
+		evt.Skip()
+	#--------------------------------------------------------
+	def __on_enter(self, evt):
+
+		# ENTER but no search term ?
+		curr_search_term = self.GetValue().strip()
+		if curr_search_term == '':
+			return None
+
+		# same person anywys ?
+		if self.person is not None:
+			if curr_search_term == self.person['description']:
+				return None
+
+		# remember search fragment
+		if self.IsModified():
+			self._prev_search_term = curr_search_term
+
+		self._on_enter(search_term = curr_search_term)
+	#--------------------------------------------------------
+	def _on_enter(self, search_term=None):
+
+		wx.BeginBusyCursor()
+
+		# get list of matching ids
+		idents = self.__person_searcher.get_identities(search_term)
+
+		if idents is None:
+			wx.EndBusyCursor()
+			gmGuiHelpers.gm_show_info (
+				_('Error searching for matching persons.\n\n'
+				  'Search term: "%s"'
+				) % search_term,
+				_('selecting person')
+			)
+			return None
+
+		_log.info("%s matching person(s) found", len(idents))
+
+		if len(idents) == 0:
+			wx.EndBusyCursor()
+
+			dlg = gmGuiHelpers.c2ButtonQuestionDlg (
+				wx.GetTopLevelParent(self),
+				-1,
+				caption = _('Selecting patient'),
+				question = _(
+					'Cannot find any matching patients for the search term\n\n'
+					' "%s"\n\n'
+					'You may want to try a shorter search term.\n'
+				) % search_term,
+				button_defs = [
+					{'label': _('Go back'), 'tooltip': _('Go back and search again.'), 'default': True},
+					{'label': _('Create new'), 'tooltip': _('Create new patient.')}
+				]
+			)
+			if dlg.ShowModal() == wx.ID_YES:
+				return
+
+			wiz = gmDemographicsWidgets.cNewPatientWizard(parent = self.GetParent())
+			self.person = wiz.RunWizard(activate = False)
+			self._display_name()
+			return None
+
+		# only one matching identity
+		if len(idents) == 1:
+			self.person = idents[0]
+			self._display_name()		# needed when the found patient is the same as the active one
+			wx.EndBusyCursor()
+			return None
+
+		# more than one matching identity: let user select from pick list
+		dlg = cSelectPersonFromListDlg(parent=wx.GetTopLevelParent(self), id=-1)
+		dlg.set_persons(persons=idents)
+		wx.EndBusyCursor()
+		result = dlg.ShowModal()
+		if result == wx.ID_CANCEL:
+			dlg.Destroy()
+			return None
+
+		wx.BeginBusyCursor()
+		self.person = dlg.get_selected_person()
+		dlg.Destroy()
+		self._display_name()		# needed when the found patient is the same as the active one
+		wx.EndBusyCursor()
+
+		return None
+#============================================================
+class cActivePatientSelector(cPersonSearchCtrl):
+
+	def __init__ (self, *args, **kwargs):
+
+		cPersonSearchCtrl.__init__(self, *args, **kwargs)
 
 		selector_tooltip = _(
 		'Patient search field.                             \n'
@@ -446,15 +762,6 @@ class cPatientSelector(wx.TextCtrl):
 		)
 		self.SetToolTip(wx.ToolTip(selector_tooltip))
 
-		self._display_name()
-
-		# FIXME: set query generator
-		self.__pat_searcher = gmPerson.cPatientSearcher_SQL()
-
-		self.__prev_search_term = None
-		self.__prev_idents = []
-		self._lclick_count = 0
-
 		# get configuration
 		cfg = gmCfg.cCfgSQL()
 
@@ -478,12 +785,26 @@ class cPatientSelector(wx.TextCtrl):
 
 		self.__register_events()
 	#--------------------------------------------------------
-	def SetActivePatient(self, pat):
+	# utility methods
+	#--------------------------------------------------------
+	def _display_name(self):
+		name = u''
+
+		curr_pat = gmPerson.gmCurrentPatient()
+		if curr_pat.connected:
+			name = curr_pat['description']
+			if curr_pat.locked:
+				name = _('%(name)s (locked)') % {'name': name}
+#			name = '%s%s' % (name, gmTools.coalesce(self._prev_search_term, u'', u' [%s]'))
+
+		self.SetValue(name)
+	#--------------------------------------------------------
+	def _set_person_as_active_patient(self, pat):
 		if not gmPerson.set_active_patient(patient=pat, forced_reload = self.__always_reload_after_search):
-			_log.Log (gmLog.lErr, 'cannot change active patient')
+			_log.error('cannot change active patient')
 			return None
 
-		self.__remember_ident(pat)
+		self._remember_ident(pat)
 
 		dbcfg = gmCfg.cCfgSQL()
 		dob_distance = dbcfg.get2 (
@@ -509,245 +830,52 @@ class cPatientSelector(wx.TextCtrl):
 
 		return True
 	#--------------------------------------------------------
-	# utility methods
-	#--------------------------------------------------------
-	def _display_name(self):
-		name = u''
-
-		if self.curr_pat.is_connected():
-			name = self.curr_pat['description']
-			if self.curr_pat.locked:
-				name = _('%(name)s (locked)') % {'name': name}
-
-		self.SetValue(name)
-	#--------------------------------------------------------
-	def __remember_ident(self, ident=None):
-
-		if not isinstance(ident, gmPerson.cIdentity):
-			return False
-
-		# only unique identities
-		for known_ident in self.__prev_idents:
-			if known_ident['pk_identity'] == ident['pk_identity']:
-				return True
-
-		self.__prev_idents.append(ident)
-
-		# and only 10 of them
-		if len(self.__prev_idents) > 10:
-			self.__prev_idents.pop(0)
-
-		return True
-	#--------------------------------------------------------
 	# event handling
 	#--------------------------------------------------------
 	def __register_events(self):
-		# - process some special chars
-		wx.EVT_CHAR(self, self._on_char)
-		# - select data in input field upon tabbing in
-		wx.EVT_SET_FOCUS(self, self._on_get_focus)
-		# - redraw the currently active name upon losing focus
-		#   (but see the caveat in the handler)
-		# FIXME: causes core dump in one version of wxPython -SJTAN
-		#wx.EVT_KILL_FOCUS (self, self._on_loose_focus)
-
-		wx.EVT_TEXT_ENTER (self, self.GetId(), self._on_enter)
-		wx.EVT_LEFT_UP (self, self._on_left_mousebutton_up)
-
 		# client internal signals
-		gmDispatcher.connect(signal=gmSignals.post_patient_selection(), receiver=self._on_post_patient_selection)
+		gmDispatcher.connect(signal = u'post_patient_selection', receiver = self._on_post_patient_selection)
 		gmDispatcher.connect(signal = 'patient_locked', receiver = self._on_post_patient_selection)
 		gmDispatcher.connect(signal = 'patient_unlocked', receiver = self._on_post_patient_selection)
-		#----------------------------------------------
+	#----------------------------------------------
 	def _on_post_patient_selection(self, **kwargs):
+		if gmPerson.gmCurrentPatient().connected:
+			self.person = gmPerson.gmCurrentPatient().patient
+		else:
+			self.person = None
 		wx.CallAfter(self._display_name)
-	#--------------------------------------------------------
-	def _on_left_mousebutton_up(self, evt):
-		"""upon left click release
-
-		- select all text in the field so that the next
-		  character typed will delete it
-		
-		- or set cursor to text position in case more left
-		  clicks follow
-		"""
-		# unclicked, not highlighted
-		if self._lclick_count == 0:
-			self.SetSelection (-1,-1)			# highlight entire text
-			self._lclick_count = 1
-			evt.Skip()
-			return None
-			
-		# has been clicked before - should be highlighted
-		start, end = self.GetSelection()
-		self.SetSelection(start, end)
-		self._lclick_count = 0
-		evt.Skip()
-		return None
-	#--------------------------------------------------------
-	def _on_get_focus(self, evt):
-		"""upon tabbing in
-
-		- select all text in the field so that the next
-		  character typed will delete it
-		"""
-		self.SetSelection (-1,-1)
-		evt.Skip()
-	#--------------------------------------------------------
-	def _on_loose_focus(self, evt):
-		# if we use wx.EVT_KILL_FOCUS we will also receive this event
-		# when closing our application or loosing focus to another
-		# application which is NOT what we intend to achieve,
-		# however, this is the least ugly way of doing this due to
-		# certain vagaries of wxPython (see the Wiki)
-
-		# remember fragment
-		curr_search_term = self.GetValue()
-		if self.IsModified() and (curr_search_term.strip() != ''):
-			self.__prev_search_term = curr_search_term
-
-		# and display currently active patient
-		self._display_name()
-		# unset highlighting
-		self.SetSelection (0,0)
-		# reset highlight counter
-		self._lclick_count = 0
-
-		evt.Skip()
-	#--------------------------------------------------------
-	def _on_char(self, evt):
-		keycode = evt.GetKeyCode()
-
-		# list of previously active patients
-		if keycode == wx.WXK_DOWN:
-			evt.Skip()
-			if len(self.__prev_idents) == 0:
-				return True
-
-			dlg = cSelectPersonFromListDlg(parent=wx.GetTopLevelParent(self), id=-1)
-			dlg.set_persons(persons=self.__prev_idents)
-			result = dlg.ShowModal()
-			if result == wx.ID_OK:
-				wx.BeginBusyCursor()
-				person = dlg.get_selected_person()
-				self.SetActivePatient(person)
-				wx.EndBusyCursor()
-
-			dlg.Destroy()
-			return True
-
-		# previous search fragment
-		if keycode == wx.WXK_UP:
-			evt.Skip()
-			# FIXME: cycling through previous fragments
-			if self.__prev_search_term is not None:
-				self.SetValue(self.__prev_search_term)
-			return True
-
-		# invoke external patient sources
-		if keycode == wx.WXK_F2:
-			evt.Skip()
-			dbcfg = gmCfg.cCfgSQL()
-			search_immediately = bool(dbcfg.get2 (
-				option = 'patient_search.external_sources.immediately_search_if_single_source',
-				workplace = gmSurgery.gmCurrentPractice().active_workplace,
-				bias = 'user',
-				default = 0
-			))
-			load_patient_from_external_sources(parent=wx.GetTopLevelParent(self), search_immediately=search_immediately)
-			return True
-
-		# FIXME: invoke add new patient
-		# FIXME: add popup menu apart from system one
-
-		evt.Skip()
-	#--------------------------------------------------------
-	def _on_enter(self, evt):
-
-		curr_search_term = self.GetValue().strip()
-		if curr_search_term == '':
-			return None
-
-		if self.curr_pat.is_connected():
-			if curr_search_term == self.curr_pat['description']:
-				return None
-
-		wx.BeginBusyCursor()
+	#----------------------------------------------
+	def _on_enter(self, search_term = None):
 
 		if self.__always_dismiss_after_search:
-			print "dismissing patient"
-			self.SetActivePatient(-1)
+			_log.warning("dismissing patient before patient search")
+			self._set_person_as_active_patient(-1)
 
-		# remember fragment
-		if self.IsModified():
-			self.__prev_search_term = curr_search_term
+		super(self.__class__, self)._on_enter(search_term=search_term)
 
-		# get list of matching ids
-		start = time.time()
-		idents = self.__pat_searcher.get_identities(curr_search_term)
-		duration = time.time() - start
+		if self.person is None:
+			return
 
-		if idents is None:
-			wx.EndBusyCursor()
-			gmGuiHelpers.gm_show_info (
-				_('Error searching for matching patients.\n\n'
-				  'Search term: "%s"'
-				) % curr_search_term,
-				_('selecting patient')
-			)
-			return None
+		self._set_person_as_active_patient(self.person)
+		self._display_name()
+	#----------------------------------------------
+	def _on_char(self, evt):
 
-		_log.Log (gmLog.lInfo, "%s person objects(s) fetched in %3.3f seconds" % (len(idents), duration))
-
-		if len(idents) == 0:
-			wx.EndBusyCursor()
-			gmGuiHelpers.gm_show_info (
-				_('Cannot find any matching patients.\n'
-				  'You may want to try a shorter search term.\n\n'
-				  'Search term: "%s"\n\n'
-				  'You will be taken to the "New Patient" wizard now.'
-				) % curr_search_term,
-				_('selecting patient')
-			)
-			wiz = gmDemographicsWidgets.cNewPatientWizard(parent=self.GetParent())
-			wiz.RunWizard(activate=True)
-			return None
-
-		# only one matching identity
-		if len(idents) == 1:
-			self.SetActivePatient(idents[0])
-			self._display_name()		# needed when the found patient is the same as the active one
-			wx.EndBusyCursor()
-			return None
-
-		# more than one matching identity: let user select from pick list
-		dlg = cSelectPersonFromListDlg(parent=wx.GetTopLevelParent(self), id=-1)
-		dlg.set_persons(persons=idents)
-		wx.EndBusyCursor()
-		result = dlg.ShowModal()
-		if result == wx.ID_CANCEL:
-			dlg.Destroy()
-			return None
-
-		wx.BeginBusyCursor()
-		ident = dlg.get_selected_person()
-		dlg.Destroy()
-		self.SetActivePatient(ident)
-		self._display_name()		# needed when the found patient is the same as the active one
-		wx.EndBusyCursor()
-
-		return None
+		success = super(self.__class__, self)._on_char(evt)
+		if success:
+			self._set_person_as_active_patient(self.person)
 #============================================================
 # main
 #------------------------------------------------------------
 if __name__ == "__main__":
-	_log.SetAllLogLevels(gmLog.lData)
 
 	gmI18N.activate_locale()
 	gmI18N.install_domain()
 
 	app = wx.PyWidgetTester(size = (200, 40))
-	app.SetWidget(cSelectPersonFromListDlg, -1)
+#	app.SetWidget(cSelectPersonFromListDlg, -1)
+#	app.SetWidget(cPersonSearchCtrl, -1)
+	app.SetWidget(cActivePatientSelector, -1)
 	app.MainLoop()
 
 #============================================================
@@ -856,11 +984,57 @@ if __name__ == "__main__":
 
 #============================================================
 # $Log: gmPatSearchWidgets.py,v $
-# Revision 1.97.2.2  2008/05/07 16:04:11  ncq
-# - MacOSX detected a faulty indentation for wx.EndBusyCursor()
+# Revision 1.110.2.2  2009/01/15 08:58:19  ncq
+# - do not hard code an initial patient plugin
 #
-# Revision 1.97.2.1  2008/01/08 22:39:31  ncq
-# - 'first' / 'last' -> 'firstnames' / 'lastnames'
+# Revision 1.110.2.1  2008/09/09 17:35:11  ncq
+# - don't crash on incomplete PracSoft profiles
+#
+# Revision 1.110  2008/07/28 20:27:20  ncq
+# - do not try to activate None person
+#
+# Revision 1.109  2008/07/07 13:43:17  ncq
+# - current patient .connected
+#
+# Revision 1.108  2008/05/13 14:13:57  ncq
+# - fix on-focus-select-all behaviour
+# - don't display search term after name - when a search failed this gets confusing
+#
+# Revision 1.107  2008/04/16 20:39:39  ncq
+# - working versions of the wxGlade code and use it, too
+# - show client version in login dialog
+#
+# Revision 1.106  2008/03/20 15:31:59  ncq
+# - missing \n added
+#
+# Revision 1.105  2008/03/09 20:18:22  ncq
+# - cleanup
+# - load_patient_* -> get_person_*
+# - make cPatientSelector() generic -> cPersonSearchCtrl()
+#
+# Revision 1.104  2008/02/25 17:40:18  ncq
+# - new style logging
+#
+# Revision 1.103  2008/01/30 14:09:39  ncq
+# - switch to new style cfg file support
+# - cleanup
+#
+# Revision 1.102  2008/01/27 21:17:49  ncq
+# - improve message on patient not found
+#
+# Revision 1.101  2008/01/22 12:24:55  ncq
+# - include search fragment into patient name display
+# - reenable on kill focus handler restoring patient name
+# - improved wording on patient not found
+#
+# Revision 1.100  2008/01/11 16:15:33  ncq
+# - first/last -> first-/lastnames
+#
+# Revision 1.99  2008/01/05 16:41:27  ncq
+# - remove logging from gm_show_*()
+#
+# Revision 1.98  2007/12/11 12:49:26  ncq
+# - explicit signal handling
 #
 # Revision 1.97  2007/11/12 23:05:55  ncq
 # - import extra data from DTOs
