@@ -8,8 +8,8 @@ This is based on seminal work by Ian Haywood <ihaywood@gnu.org>
 """
 ############################################################################
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmPhraseWheel.py,v $
-# $Id: gmPhraseWheel.py,v 1.110 2007/10/29 11:30:21 ncq Exp $
-__version__ = "$Revision: 1.110 $"
+# $Id: gmPhraseWheel.py,v 1.124.2.1 2008/09/09 17:36:24 ncq Exp $
+__version__ = "$Revision: 1.124.2.1 $"
 __author__  = "K.Hilbert <Karsten.Hilbert@gmx.net>, I.Haywood, S.J.Tan <sjtan@bigpond.com>"
 __license__ = "GPL"
 
@@ -26,8 +26,15 @@ import wx.lib.mixins.listctrl as listmixins
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
 from Gnumed.wxpython import gmTimer, gmGuiHelpers
-from Gnumed.pycommon import gmTools, gmSignals, gmDispatcher
+from Gnumed.pycommon import gmTools, gmDispatcher
 
+
+import logging
+_log = logging.getLogger('macosx')
+
+
+color_prw_invalid = 'pink'
+color_prw_valid = None				# this is used by code outside this module
 #============================================================
 # those can be used by the <accepted_chars> phrasewheel parameter
 NUMERIC = '0-9'
@@ -53,10 +60,16 @@ class cPhraseWheelListCtrl(wx.ListCtrl, listmixins.ListCtrlAutoWidthMixin):
 			row_num = self.InsertStringItem(pos, label=item['label'])
 	#--------------------------------------------------------
 	def GetSelectedItemData(self):
-		return self.__data[self.GetFirstSelected()]['data']
+		sel_idx = self.GetFirstSelected()
+		if sel_idx == -1:
+			return None
+		return self.__data[sel_idx]['data']
 	#--------------------------------------------------------
 	def get_selected_item_label(self):
-		return self.__data[self.GetFirstSelected()]['label']
+		sel_idx = self.GetFirstSelected()
+		if sel_idx == -1:
+			return None
+		return self.__data[sel_idx]['label']
 #============================================================
 # FIXME: cols in pick list
 # FIXME: snap_to_basename+set selection
@@ -133,28 +146,36 @@ class cPhraseWheel(wx.TextCtrl):
 	@param matcher: a class used to find matches for the current input
 	@type matcher: a L{match provider<Gnumed.pycommon.gmMatchProvider.cMatchProvider>}
 		instance or C{None}
+
 	@param selection_only: whether free-text can be entered without associated data
 	@type selection_only: boolean
+
 	@param capitalisation_mode: how to auto-capitalize input, valid values
 		are found in L{capitalize()<Gnumed.pycommon.gmTools.capitalize>}
 	@type capitalisation_mode: integer
+
 	@param accepted_chars: a regex pattern defining the characters
 		acceptable in the input string, if None no checking is performed
 	@type accepted_chars: None or a string holding a valid regex pattern
+
 	@param final_regex: when the control loses focus the input is
 		checked against this regular expression
 	@type final_regex: a string holding a valid regex pattern
+
 	@param phrase_separators: if not None, input is split into phrases
 		at boundaries defined by this regex and matching/spellchecking
 		is performed on the phrase the cursor is in only
 	@type phrase_separators: None or a string holding a valid regex pattern
+
 	@param navigate_after_selection: whether or not to immediately
 		navigate to the widget next-in-tab-order after selecting an
 		item from the dropdown picklist
 	@type navigate_after_selection: boolean
+
 	@param speller: if not None used to spellcheck the current input
 		and to retrieve suggested replacements/completions
 	@type speller: None or a L{enchant Dict<enchant>} descendant
+
 	@param picklist_delay: this much time of user inactivity must have
 		passed before the input related smarts kick in and the drop
 		down pick list is shown
@@ -195,44 +216,17 @@ class cPhraseWheel(wx.TextCtrl):
 			kwargs['style'] = kwargs['style'] | wx.TE_PROCESS_TAB
 		except KeyError:
 			kwargs['style'] = wx.TE_PROCESS_TAB
-		wx.TextCtrl.__init__ (self, parent, id, **kwargs)
+		wx.TextCtrl.__init__(self, parent, id, **kwargs)
 
-		# multiple matches dropdown list
-		try:
-			self.__picklist_dropdown = wx.PopupWindow(parent)
-			add_picklist_to_sizer = False
-		except NotImplementedError:
-			# on MacOSX wx.PopupWindow is not implemented
-			self.__picklist_dropdown = wx.Window(parent=self, style = wx.SIMPLE_BORDER)
-			szr_scroll = wx.BoxSizer(wx.VERTICAL)
-			self.__picklist_dropdown.SetSizer(szr_scroll)
-			add_picklist_to_sizer = True
+		self.__non_edit_font = self.GetFont()
+		self.__color_valid = self.GetBackgroundColour()
+		global color_prw_valid
+		if color_prw_valid is None:
+			color_prw_valid = wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW)
 
-		# FIXME: support optional headers
-#		if kwargs['show_list_headers']:
-#			flags = 0
-#		else:
-#			flags = wx.LC_NO_HEADER
-		self._picklist = cPhraseWheelListCtrl (
-			self.__picklist_dropdown,
-			style = wx.LC_NO_HEADER
-		)
-		self._picklist.InsertColumn(0, '')
-
-		if add_picklist_to_sizer:
-			szr_scroll.Add(self._picklist, 1, wx.EXPAND)
-
-		self.__picklist_dropdown.Hide()
-
-		# set event handlers
+		self.__init_dropdown(parent = parent)
 		self.__register_events()
-
-		self.__timer = gmTimer.cTimer (
-			callback = self._on_timer_fired,
-			delay = self.picklist_delay
-		)
-		# initially stopped
-		self.__timer.Stop()
+		self.__init_timer()
 	#--------------------------------------------------------
 	# external API
 	#--------------------------------------------------------
@@ -299,7 +293,7 @@ class cPhraseWheel(wx.TextCtrl):
 
 		for match in matches:
 			if match['data'] == data:
-				self.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
+				self.display_as_valid(valid = True)
 				self.suppress_text_update_smarts = True
 				wx.TextCtrl.SetValue(self, match['label'])
 				self.data = data
@@ -310,7 +304,7 @@ class cPhraseWheel(wx.TextCtrl):
 			return False
 
 		self.data = data
-		self.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
+		self.display_as_valid(valid = True)
 		return True
 	#---------------------------------------------------------
 	def GetData(self):
@@ -319,13 +313,15 @@ class cPhraseWheel(wx.TextCtrl):
 		"""
 		return self.data
 	#---------------------------------------------------------
-	def SetText (self, value=u'', data=None):
+	def SetText (self, value=u'', data=None, suppress_smarts=False):
+
+		self.suppress_text_update_smarts = suppress_smarts
 
 		if data is not None:
 			self.suppress_text_update_smarts = True
 			self.data = data
 		wx.TextCtrl.SetValue(self, value)
-		self.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
+		self.display_as_valid(valid = True)
 
 		# if data already available
 		if self.data is not None:
@@ -347,7 +343,7 @@ class cPhraseWheel(wx.TextCtrl):
 
 		# not found
 		if self.selection_only:
-			self.SetBackgroundColour('pink')
+			self.display_as_valid(valid = False)
 			return False
 
 		return True
@@ -361,6 +357,7 @@ class cPhraseWheel(wx.TextCtrl):
 			self.matcher.unset_context(context=context)
 	#--------------------------------------------------------
 	def enable_default_spellchecker(self):
+		# FIXME: use Debian's wgerman-medical as "personal" wordlist if available
 		try:
 			import enchant
 		except ImportError:
@@ -373,13 +370,75 @@ class cPhraseWheel(wx.TextCtrl):
 			return False
 		return True
 	#--------------------------------------------------------
+	def display_as_valid(self, valid=None):
+		if valid is True:
+			self.SetBackgroundColour(self.__color_valid)
+		elif valid is False:
+			self.SetBackgroundColour(color_prw_invalid)
+		else:
+			raise ArgumentError(u'<valid> must be True or False')
+		self.Refresh()
+	#--------------------------------------------------------
 	# internal API
 	#--------------------------------------------------------
 	# picklist handling
 	#--------------------------------------------------------
+	def __init_dropdown(self, parent = None):
+		szr_dropdown = None
+		try:
+			#raise NotImplementedError		# for testing
+			self.__use_fake_popup = False
+			self.__dropdown_needs_relative_position = False
+			self.__picklist_dropdown = wx.PopupWindow(parent)
+			list_parent = self.__picklist_dropdown
+		except NotImplementedError:
+			self.__use_fake_popup = True
+
+			# on MacOSX wx.PopupWindow is not implemented, so emulate it
+			add_picklist_to_sizer = True
+			szr_dropdown = wx.BoxSizer(wx.VERTICAL)
+
+			# using wx.MiniFrame
+			self.__dropdown_needs_relative_position = False
+			self.__picklist_dropdown = wx.MiniFrame (
+				parent = parent,
+				id = -1,
+				style = wx.SIMPLE_BORDER | wx.FRAME_FLOAT_ON_PARENT | wx.FRAME_NO_TASKBAR | wx.POPUP_WINDOW
+			)
+			scroll_win = wx.ScrolledWindow(parent = self.__picklist_dropdown, style = wx.NO_BORDER)
+			scroll_win.SetSizer(szr_dropdown)
+			list_parent = scroll_win
+
+			# using wx.Window
+			#self.__dropdown_needs_relative_position = True
+			#self.__picklist_dropdown = wx.ScrolledWindow(parent=parent, style = wx.RAISED_BORDER)
+			#self.__picklist_dropdown.SetSizer(szr_dropdown)
+			#list_parent = self.__picklist_dropdown
+
+		self.mac_log('dropdown parent: %s' % self.__picklist_dropdown.GetParent())
+
+		# FIXME: support optional headers
+#		if kwargs['show_list_headers']:
+#			flags = 0
+#		else:
+#			flags = wx.LC_NO_HEADER
+		self._picklist = cPhraseWheelListCtrl (
+			list_parent,
+			style = wx.LC_NO_HEADER
+		)
+		self._picklist.InsertColumn(0, '')
+
+		if szr_dropdown is not None:
+			szr_dropdown.Add(self._picklist, 1, wx.EXPAND)
+
+		self.__picklist_dropdown.Hide()
+	#--------------------------------------------------------
 	def _show_picklist(self):
 		"""Display the pick list."""
 
+		border_width = 4
+		extra_height = 25
+
 		self._hide_picklist()
 
 		# this helps if the current input was already selected from the
@@ -405,36 +464,57 @@ class cPhraseWheel(wx.TextCtrl):
 			rows = 2
 		if rows > 20:				# 20 rows maximum
 			rows = 20
+		self.mac_log('dropdown needs rows: %s' % rows)
 		dropdown_size = self.__picklist_dropdown.GetSize()
 		pw_size = self.GetSize()
 		dropdown_size.SetWidth(pw_size.width)
-		dropdown_size.SetHeight((pw_size.height * rows) + 4)	# adjust for border width
+		dropdown_size.SetHeight (
+			(pw_size.height * rows)
+			+ border_width
+			+ extra_height
+		)
 
 		# recalculate position
 		(pw_x_abs, pw_y_abs) = self.ClientToScreenXY(0,0)
-		new_x = pw_x_abs
-		new_y = pw_y_abs + pw_size.height
+		self.mac_log('phrasewheel position (on screen): x:%s-%s, y:%s-%s' % (pw_x_abs, (pw_x_abs+pw_size.width), pw_y_abs, (pw_y_abs+pw_size.height)))
+		dropdown_new_x = pw_x_abs
+		dropdown_new_y = pw_y_abs + pw_size.height
+		self.mac_log('desired dropdown position (on screen): x:%s-%s, y:%s-%s' % (dropdown_new_x, (dropdown_new_x+dropdown_size.width), dropdown_new_y, (dropdown_new_y+dropdown_size.height)))
+		self.mac_log('desired dropdown size: %s' % dropdown_size)
+
 		# reaches beyond screen ?
-		if (dropdown_size.height + new_y) > self._screenheight:
-			max_height = self._screenheight - new_y - 4
+		if (dropdown_new_y + dropdown_size.height) > self._screenheight:
+			self.mac_log('dropdown extends offscreen (screen max y: %s)' % self._screenheight)
+			max_height = self._screenheight - dropdown_new_y - 4
+			self.mac_log('max dropdown height would be: %s' % max_height)
 			if max_height > ((pw_size.height * 2) + 4):
 				dropdown_size.SetHeight(max_height)
+				self.mac_log('possible dropdown position (on screen): x:%s-%s, y:%s-%s' % (dropdown_new_x, (dropdown_new_x+dropdown_size.width), dropdown_new_y, (dropdown_new_y+dropdown_size.height)))
+				self.mac_log('possible dropdown size: %s' % dropdown_size)
 
 		# now set dimensions
 		self.__picklist_dropdown.SetSize(dropdown_size)
 		self._picklist.SetSize(self.__picklist_dropdown.GetClientSize())
-		self.__picklist_dropdown.MoveXY(new_x, new_y)
+		self.mac_log('pick list size set to: %s' % self.__picklist_dropdown.GetSize())
+		if self.__dropdown_needs_relative_position:
+			dropdown_new_x, dropdown_new_y = self.__picklist_dropdown.GetParent().ScreenToClientXY(dropdown_new_x, dropdown_new_y)
+		self.__picklist_dropdown.MoveXY(dropdown_new_x, dropdown_new_y)
 
 		# select first value
 		self._picklist.Select(0)
 
 		# and show it
 		self.__picklist_dropdown.Show(True)
+
+		dd_tl = self.__picklist_dropdown.ClientToScreenXY(0,0)
+		dd_size = self.__picklist_dropdown.GetSize()
+		dd_br = self.__picklist_dropdown.ClientToScreenXY(dd_size.width, dd_size.height)
+		self.mac_log('dropdown placement now (on screen): x:%s-%s, y:%s-%s' % (dd_tl[0], dd_br[0], dd_tl[1], dd_br[1]))
 	#--------------------------------------------------------
 	def _hide_picklist(self):
 		"""Hide the pick list."""
-		if self.__picklist_dropdown.IsShown():
-			self.__picklist_dropdown.Hide()		# dismiss the dropdown list window
+#		if self.__picklist_dropdown.IsShown():
+		self.__picklist_dropdown.Hide()		# dismiss the dropdown list window
 	#--------------------------------------------------------
 	def __select_picklist_row(self, new_row_idx=None, old_row_idx=None):
 		if old_row_idx is not None:
@@ -447,8 +527,10 @@ class cPhraseWheel(wx.TextCtrl):
 
 		self.input2match = val
 		if self.input2match is None:
-			# get current(ly relevant part of) input
-			if self.__phrase_separators is not None:
+			if self.__phrase_separators is None:
+				self.input2match = self.GetValue().strip()
+			else:
+				# get current(ly relevant part of) input
 				entire_input = self.GetValue()
 				cursor_pos = self.GetInsertionPoint()
 				left_of_cursor = entire_input[:cursor_pos]
@@ -467,8 +549,6 @@ class cPhraseWheel(wx.TextCtrl):
 					phrase_end = len(entire_input) - 1
 				self.right_part = entire_input[phrase_end+1:]
 				self.input2match = entire_input[phrase_start:phrase_end+1]
-			else:
-				self.input2match = self.GetValue().strip()
 
 		# get all currently matching items
 		if self.matcher is not None:
@@ -480,19 +560,19 @@ class cPhraseWheel(wx.TextCtrl):
 			if self.speller is not None:
 				# filter out the last word
 				word = regex.split(self.__speller_word_separators, self.input2match)[-1]
-				if not self.speller.check(word):
-					spells = self.speller.suggest(word)
-					truncated_input2match = self.input2match[:self.input2match.rindex(word)]
-					for spell in spells:
-						self.__current_matches.append({'label': truncated_input2match + spell, 'data': None})
-					self._picklist.SetItems(self.__current_matches)
-
-	#--------------------------------------------------------
-	# internal helpers: GUI
+				if word.strip() != u'':
+					if not self.speller.check(word):
+						spells = self.speller.suggest(word)
+						truncated_input2match = self.input2match[:self.input2match.rindex(word)]
+						for spell in spells:
+							self.__current_matches.append({'label': truncated_input2match + spell, 'data': None})
+						self._picklist.SetItems(self.__current_matches)
 	#--------------------------------------------------------
 	def _picklist_selection2display_string(self):
 		return self._picklist.GetItemText(self._picklist.GetFirstSelected())
 	#--------------------------------------------------------
+	# internal helpers: GUI
+	#--------------------------------------------------------
 	def _on_enter(self):
 		"""Called when the user pressed <ENTER>."""
 		if self.__picklist_dropdown.IsShown():
@@ -514,7 +594,7 @@ class cPhraseWheel(wx.TextCtrl):
 		# select another contextual item)
 		else:
 			self.__timer.Stop()
-			if self.GetValue().strip() == '':
+			if self.GetValue().strip() == u'':
 				self.__update_matches_in_picklist(val='*')
 			else:
 				self.__update_matches_in_picklist()
@@ -530,12 +610,25 @@ class cPhraseWheel(wx.TextCtrl):
 			pass
 	#--------------------------------------------------------
 	def __on_tab(self):
-		if self.__picklist_dropdown.IsShown():
-			if len(self.__current_matches) == 1:
-				self.__select_picklist_row(new_row_idx=0)
-				self._on_list_item_selected()
-				return True
-		return False
+		"""Under certain circumstances takes special action on TAB.
+
+		returns:
+			True: TAB was handled
+			False: TAB was not handled
+		"""
+		if not self.__picklist_dropdown.IsShown():
+			return False
+
+		if len(self.__current_matches) != 1:
+			return False
+
+		if not self.selection_only:
+			return False
+
+		self.__select_picklist_row(new_row_idx=0)
+		self._on_list_item_selected()
+
+		return True
 	#--------------------------------------------------------
 	# internal helpers: logic
 	#--------------------------------------------------------
@@ -590,6 +683,14 @@ class cPhraseWheel(wx.TextCtrl):
 	#------------
 	speller_word_separators = property(_get_speller_word_separators, _set_speller_word_separators)
 	#--------------------------------------------------------
+	def __init_timer(self):
+		self.__timer = gmTimer.cTimer (
+			callback = self._on_timer_fired,
+			delay = self.picklist_delay
+		)
+		# initially stopped
+		self.__timer.Stop()
+	#--------------------------------------------------------
 	def _on_timer_fired(self, cookie):
 		"""Callback for delayed match retrieval timer.
 
@@ -621,9 +722,13 @@ class cPhraseWheel(wx.TextCtrl):
 		"""Gets called when user selected a list item."""
 
 		self._hide_picklist()
-		self.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
+		self.display_as_valid(valid = True)
 
-		self.data = self._picklist.GetSelectedItemData()	# just so that _picklist_selection2display_string could use it
+		data = self._picklist.GetSelectedItemData()	# just so that _picklist_selection2display_string could use it
+		if data is None:
+			return
+
+		self.data = data
 
 		# update our display
 		self.suppress_text_update_smarts = True
@@ -667,9 +772,8 @@ class cPhraseWheel(wx.TextCtrl):
 			if event.ShiftDown():
 				self.Navigate(flags = wx.NavigationKeyEvent.IsBackward)
 				return
-			tab_handled = self.__on_tab()
-			if not tab_handled:
-				self.Navigate(flags = wx.NavigationKeyEvent.IsForward)
+			self.__on_tab()
+			self.Navigate(flags = wx.NavigationKeyEvent.IsForward)
 			return
 
 		# FIXME: need PAGE UP/DOWN//POS1/END here to move in picklist
@@ -730,6 +834,12 @@ class cPhraseWheel(wx.TextCtrl):
 		self._has_focus = True
 		event.Skip()
 
+		self.__non_edit_font = self.GetFont()
+		edit_font = self.GetFont()
+		edit_font.SetPointSize(pointSize = self.__non_edit_font.GetPointSize() + 2)
+		self.SetFont(edit_font)
+		self.Refresh()
+
 		# notify interested parties
 		for callback in self._on_set_focus_callbacks:
 			callback()
@@ -753,6 +863,9 @@ class cPhraseWheel(wx.TextCtrl):
 		# unset selection
 		self.SetSelection(1,1)
 
+		self.SetFont(self.__non_edit_font)
+		self.Refresh()
+
 		# the user may have typed a phrase that is an exact match,
 		# however, just typing it won't associate data from the
 		# picklist, so do that now
@@ -764,19 +877,19 @@ class cPhraseWheel(wx.TextCtrl):
 					if match['label'] == val:
 						self.data = match['data']
 						self.MarkDirty()
-						self.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
+						self.display_as_valid(valid = True)
 						break
 
 		# no exact match found
 		if self.data is None:
 			if self.selection_only:
 				gmDispatcher.send(signal='statustext', msg=self.selection_only_error_msg)
-				self.SetBackgroundColour('pink')
+				self.display_as_valid(valid = False)
 
 		# check value against final_regex if any given
 		if not self.__final_regex.match(self.GetValue().strip()):
 			gmDispatcher.send(signal='statustext', msg=self.final_regex_error_msg % self.__final_regex.pattern)
-			self.SetBackgroundColour('pink')
+			self.display_as_valid(valid = False)
 
 		# notify interested parties
 		for callback in self._on_lose_focus_callbacks:
@@ -784,6 +897,10 @@ class cPhraseWheel(wx.TextCtrl):
 
 		event.Skip()
 		return True
+	#----------------------------------------------------
+	def mac_log(self, msg):
+		if self.__use_fake_popup:
+			_log.debug(msg)
 #--------------------------------------------------------
 # MAIN
 #--------------------------------------------------------
@@ -833,7 +950,7 @@ if __name__ == '__main__':
 
 		mp = gmMatchProvider.cMatchProvider_FixedList(items)
 		# do NOT treat "-" as a word separator here as there are names like "asa-sismussen"
-		mp.setWordSeparators(separators = '[ \t=+&:@]+')
+		mp.word_separators = '[ \t=+&:@]+'
 		global prw
 		prw = cPhraseWheel(parent = app.frame, id = -1)
 		prw.matcher = mp
@@ -909,6 +1026,57 @@ if __name__ == '__main__':
 
 #==================================================
 # $Log: gmPhraseWheel.py,v $
+# Revision 1.124.2.1  2008/09/09 17:36:24  ncq
+# - don't try to access the selected list item if there's no
+#   list - it seems IsShown() isn't really true all the time
+#
+# Revision 1.124  2008/08/15 15:57:37  ncq
+# - enchant doesn't like spellchecking '' anymore
+#
+# Revision 1.123  2008/07/13 16:14:00  ncq
+# - outside code uses color_prw_valid, so leave it and add a comment
+#
+# Revision 1.122  2008/07/07 11:39:21  ncq
+# - separate fake_popup from needs_relative_pos flag
+#
+# Revision 1.121  2008/06/26 17:03:53  ncq
+# - use a wxMiniFrame instead of a wx.Window when emulating wx.PopupWindow
+# - adjust for some extra space needed by the wx.MiniFrame
+#
+# Revision 1.120  2008/06/18 15:48:21  ncq
+# - support valid/invalid coloring via display_as_valid
+# - cleanup init flow
+#
+# Revision 1.119  2008/06/15 20:40:43  ncq
+# - adjust test suite to match provider properties
+#
+# Revision 1.118  2008/06/09 15:36:39  ncq
+# - increase font size by 2 points when editing
+#
+# Revision 1.117  2008/05/14 13:46:37  ncq
+# - better logging
+#
+# Revision 1.116  2008/05/13 14:15:16  ncq
+# - TAB = select-single-match only when selection_only True
+# - improve wxPopupWindow emulation
+#
+# Revision 1.115  2008/05/07 15:21:44  ncq
+# - support suppress smarts argument to SetText
+#
+# Revision 1.114  2008/04/26 16:29:15  ncq
+# - missing if
+#
+# Revision 1.113  2008/04/26 10:06:37  ncq
+# - on MacOSX use relative position for popup window
+#
+# Revision 1.112  2008/04/26 09:30:28  ncq
+# - instrument phrasewheel to exhibit Mac problem
+#   with dropdown placement
+#
+# Revision 1.111  2008/01/30 14:03:42  ncq
+# - use signal names directly
+# - switch to std lib logging
+#
 # Revision 1.110  2007/10/29 11:30:21  ncq
 # - rephrase TODOs
 #
