@@ -4,23 +4,23 @@ Design by Richard Terry and Ian Haywood.
 """
 #====================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmResizingWidgets.py,v $
-# $Id: gmResizingWidgets.py,v 1.48 2007/08/12 00:12:41 ncq Exp $
-__version__ = "$Revision: 1.48 $"
+# $Id: gmResizingWidgets.py,v 1.53.2.1 2008/09/09 17:37:21 ncq Exp $
+__version__ = "$Revision: 1.53.2.1 $"
 __author__ = "Ian Haywood, Karsten Hilbert, Richard Terry"
 __license__ = 'GPL  (details at http://www.gnu.org)'
 
-import sys
+import sys, logging, re as regex
 
 
 import wx
 import wx.stc
 
 
-from Gnumed.pycommon import gmI18N, gmLog, gmDispatcher, gmSignals
+from Gnumed.pycommon import gmI18N, gmDispatcher, gmPG2
 from Gnumed.wxpython import gmGuiHelpers, gmTimer
 
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lInfo, __version__)
+_log = logging.getLogger('gm.ui')
+_log.info(__version__)
 
 STYLE_ERROR=1
 STYLE_TEXT=2
@@ -196,7 +196,7 @@ class cResizingWindow(wx.ScrolledWindow):
 		Overridden by descendants, this function uses AddWidget and Newline to form
 		the outline of the widget
 		"""
-		_log.Log(gmLog.lPanic, '[%s] forgot to override DoLayout()' % self.__class__.__name__)
+		_log.error('[%s] forgot to override DoLayout()' % self.__class__.__name__)
 	#------------------------------------------------
 	def ReSize (self, widget, new_height):
 		"""Called when a child widget has a new height, redoes the layout.
@@ -361,7 +361,9 @@ class cResizingSTC(wx.stc.StyledTextCtrl):
 	def __init__ (self, parent, id, pos=wx.DefaultPosition, size=wx.DefaultSize, style=0, data=None):
 		if not isinstance(parent, cResizingWindow):
 			 raise ValueError, 'parent of %s MUST be a ResizingWindow' % self.__class__.__name__
+
 		wx.stc.StyledTextCtrl.__init__ (self, parent, id, pos, size, style)
+
 		self.SetWrapMode (wx.stc.STC_WRAP_WORD)
 		# FIXME: configure
 		self.StyleSetSpec (STYLE_ERROR, "fore:#7F11010,bold")
@@ -369,11 +371,8 @@ class cResizingSTC(wx.stc.StyledTextCtrl):
 		self.StyleSetChangeable (STYLE_EMBED, 0)
 #		self.StyleSetHotSpot (STYLE_EMBED, 1)
 		self.SetEOLMode (wx.stc.STC_EOL_LF)
-		self.SetModEventMask (wx.stc.STC_MOD_INSERTTEXT | wx.stc.STC_MOD_DELETETEXT | wx.stc.STC_PERFORMED_USER)
 
-		wx.stc.EVT_STC_MODIFIED (self, self.GetId(), self.__on_STC_modified)
-		wx.EVT_KEY_DOWN (self, self.__on_key_down)
-		wx.EVT_KEY_UP (self, self.__OnKeyUp)
+		self.__register_interests()
 
 		self.next_in_tab_order = None
 		self.prev_in_tab_order = None
@@ -383,10 +382,10 @@ class cResizingSTC(wx.stc.StyledTextCtrl):
 		self.__popup_keywords = {}
 
 		# FIXME: delay configurable
-		self.__timer = gmTimer.cTimer (
-			callback = self._on_timer_fired,
-			delay = 300
-		)
+#		self.__timer = gmTimer.cTimer (
+#			callback = self._on_timer_fired,
+#			delay = 300
+#		)
 		self.__matcher = None
 
 		self.__show_list = 1
@@ -395,6 +394,8 @@ class cResizingSTC(wx.stc.StyledTextCtrl):
 		self.no_list = 0			# ??
 
 		self.__data = data			# this is just a placeholder for data to be attached to this STC, will be returned from GetData()
+
+		self.__keyword_separators = regex.compile("[!?'\".,:;)}\]\r\n\s\t]+")
 	#------------------------------------------------
 	# public API
 	#------------------------------------------------
@@ -408,26 +409,12 @@ class cResizingSTC(wx.stc.StyledTextCtrl):
 		wx.stc.StyledTextCtrl.SetText(self, text)
 		self.__show_list = 1
 	#------------------------------------------------
-	def ReplaceText (self, start, end, text, style=-1, space=0):
-		"""
-		Oddly, the otherwise very rich wx.STC API does not provide an
-		easy way to replace text, so we provide it here.
-
-		@param start: the position in the text to start from
-		@param length: the length of the string to replace
-		@param text: the new string
-		@param style: the style for the replaced string
-		"""
-		self.SetTargetStart(start)
-		self.SetTargetEnd(end)
-		self.ReplaceTarget(text)
-		if style > -1:
-			self.StartStyling(start, 0xFF)
-			self.SetStyling(len(text), style)
+	def ReplaceText (self, start, end, text, style=None):
+		self.replace_text(start, end, text, style)
 	#------------------------------------------------
 	def Embed (self, text, data=None):
 		self.no_list = 1
-		self.ReplaceText(self.fragment_start, self.fragment_end, text+';', STYLE_EMBED, 1)
+		self.ReplaceText(self.fragment_start, self.fragment_end, text+';', STYLE_EMBED)
 		self.GotoPos(self.fragment_start+len (text)+1)
 		self.SetFocus()
 #		if data:
@@ -460,14 +447,14 @@ class cResizingSTC(wx.stc.StyledTextCtrl):
 			return
 		# goto last line ?
 		if line == -1:
-			_log.Log(gmLog.lData, 'going to last line in STC')
+			_log.debug('going to last line in STC')
 			last_char_pos = self.GetLength()
 			if x is None:
 				self.GotoPos(last_char_pos)
-				_log.Log(gmLog.lData, 'no X given, use X=%s' % last_char_pos.x)
+				_log.debug('no X given, use X=%s' % last_char_pos.x)
 				return
 			y = self.PointFromPosition(last_char_pos).y
-			_log.Log(gmLog.lData, 'going to given X=%s' % x)
+			_log.debug('going to given X=%s' % x)
 			self.GotoPos(self.PositionFromPoint(wx.Point(x,y)))
 			return
 		# goto last current position
@@ -494,18 +481,72 @@ class cResizingSTC(wx.stc.StyledTextCtrl):
 		"""
 		return self.__data
 	#------------------------------------------------
-	# event handlers
+	def replace_text(self, start=None, end=None, text=None, style=None):
+		"""
+		Oddly, the otherwise very rich wx.STC API does not provide an
+		easy way to replace text, so we provide it here.
+
+		@param start: the position in the text to start from
+		@param length: the length of the string to replace
+		@param text: the new string
+		@param style: the style for the replaced string
+		"""
+		self.SetTargetStart(start)
+		self.SetTargetEnd(end)
+		self.ReplaceTarget(text)
+		if style is not None:
+			self.StartStyling(start, 0xFF)
+			self.SetStyling(len(text), style)
+	#------------------------------------------------
+	def replace_keyword_with_expansion(self, keyword=None, position=None):
+
+		if keyword == u'$$steffi':
+			expansion = u'Hai, play !  Versucht das !  :-)'
+		else:
+			expansion = gmPG2.expand_keyword(keyword = keyword)
+
+		if expansion is None:
+			return
+
+		if expansion == u'':
+			return
+
+		self.replace_text (
+			start = position,
+			end = position + len(keyword),
+			text = expansion
+		)
+
+		self.GotoPos(position + len(expansion) + 1)
+		#wx.stc.StyledTextCtrl.SetFocus(self)
+		cur = self.PointFromPosition(position + len(expansion) + 1)
+		self.__parent.EnsureVisible(self, cur.x, cur.y)
+	#------------------------------------------------
+	# event handling
+	#------------------------------------------------
+	def __register_interests(self):
+		self.SetModEventMask (wx.stc.STC_MOD_INSERTTEXT | wx.stc.STC_MOD_DELETETEXT | wx.stc.STC_PERFORMED_USER)
+
+		wx.stc.EVT_STC_MODIFIED (self, self.GetId(), self.__on_STC_modified)
+
+		wx.EVT_KEY_DOWN (self, self.__on_key_down)
+		wx.EVT_KEY_UP (self, self.__OnKeyUp)
+		wx.EVT_CHAR(self, self.__on_char)
 	#------------------------------------------------
 	def __on_STC_modified(self, event):
+
 		# did the user do anything of note to us ?
 		if not (event.GetModificationType() & (wx.stc.STC_MOD_INSERTTEXT | wx.stc.STC_MOD_DELETETEXT)):
 			event.Skip()
 			return
+
 		last_char_pos = self.GetLength()
+
 		# stop timer if empty
 		if last_char_pos == 0:
-			self.__timer.Stop()
+#			self.__timer.Stop()
 			return
+
 		# do we need to resize ?
 		line_height = self.TextHeight(0)
 		true_txt_height = (self.PointFromPosition(last_char_pos).y - self.PointFromPosition(0).y) + line_height
@@ -539,11 +580,11 @@ class cResizingSTC(wx.stc.StyledTextCtrl):
 		# is currently relevant term a keyword for popping up an edit area or something ?
 		fragment = self.__get_focussed_fragment()
 		if fragment in self.__popup_keywords.keys():
-			self.__timer.Stop()
+#			self.__timer.Stop()
 			self.__handle_keyword(fragment)
 			return
 		# else restart timer for match list
-		self.__timer.Start(oneShot = True)
+#		self.__timer.Start(oneShot = True)
 #		event.Skip()
 		return
 	#------------------------------------------------
@@ -553,43 +594,43 @@ class cResizingSTC(wx.stc.StyledTextCtrl):
 #		if (self.list is not None) and not self.list.alive:
 #			self.list = None # someone else has destroyed our list!
 
-		curs_pos = self.GetCurrentPos()
+#		curs_pos = self.GetCurrentPos()
 
 		# <DOWN>
 		# - if in list: scroll list
 		# - if in last line: goto first line, same character, in next_in_tab_order
 		# - else standard behaviour
-		if event.GetKeyCode() == wx.WXK_DOWN:
+		#if event.GetKeyCode() == wx.WXK_DOWN:
 #			if (self.list is not None) and self.list.alive:
 #				self.list.Down()
 #				return
 #			print "arrow down @ %s (line %s of %s)" % (curs_pos, self.LineFromPosition(curs_pos), self.GetLineCount())
-			if self.LineFromPosition(curs_pos)+1 == self.GetLineCount():
-				if self.next_in_tab_order is not None:
-					curs_coords = self.PointFromPosition(curs_pos)
-					self.next_in_tab_order.SetFocus(x=curs_coords.x, line=1)
-					return
+		#	if self.LineFromPosition(curs_pos)+1 == self.GetLineCount():
+		#		if self.next_in_tab_order is not None:
+		#			curs_coords = self.PointFromPosition(curs_pos)
+		#			self.next_in_tab_order.SetFocus(x=curs_coords.x, line=1)
+		#			return
 
 		# <UP>
 		# - if in list: scroll list
 		# - if in first line: goto last line, same character, in prev_in_tab_order
 		# - else standard behaviour
-		if event.GetKeyCode() == wx.WXK_UP:
-			_log.Log(gmLog.lData, '<UP-ARROW> key press detected')
+		#if event.GetKeyCode() == wx.WXK_UP:
+		#	_log.debug('<UP-ARROW> key press detected')
 #			if (self.list is not None) and self.list.alive:
 #				self.list.Up()
 #				return
-			_log.Log(gmLog.lData, 'pos %s = line %s' % (curs_pos, self.LineFromPosition(curs_pos)))
-			if self.LineFromPosition(curs_pos) == 0:
-				_log.Log(gmLog.lData, 'first line of STC - special handling')
-				if self.prev_in_tab_order is not None:
-					_log.Log(gmLog.lData, 'prev_in_tab_order = %s' % str(self.prev_in_tab_order))
-					curs_coords = self.PointFromPosition(curs_pos)
-					_log.Log(gmLog.lData, 'cursor coordinates in current STC: %s:%s' % (curs_coords.x, curs_coords.y))
-					self.prev_in_tab_order.SetFocus(x=curs_coords.x, line=-1)
-					return
-			else:
-				_log.Log(gmLog.lData, 'not first line of STC - standard handling')
+		#	_log.debug('pos %s = line %s' % (curs_pos, self.LineFromPosition(curs_pos)))
+		#	if self.LineFromPosition(curs_pos) == 0:
+		#		_log.debug('first line of STC - special handling')
+		#		if self.prev_in_tab_order is not None:
+		#			_log.debug('prev_in_tab_order = %s' % str(self.prev_in_tab_order))
+		#			curs_coords = self.PointFromPosition(curs_pos)
+		#			_log.debug('cursor coordinates in current STC: %s:%s' % (curs_coords.x, curs_coords.y))
+		#			self.prev_in_tab_order.SetFocus(x=curs_coords.x, line=-1)
+		#			return
+		#	else:
+		#		_log.debug('not first line of STC - standard handling')
 
 		# <TAB> key
 		# - move to next/prev_in_tab_order
@@ -602,60 +643,27 @@ class cResizingSTC(wx.stc.StyledTextCtrl):
 				if self.next_in_tab_order is not None:
 					self.next_in_tab_order.SetFocus()
 			return
-				# FIXME: why ?
-#				elif self.__parent.complete:
-#					self.__parent.complete()
-
-		# <;>
-		# - do not put into empty field
-		# - do not allow consecutive ';'s
-		if event.GetKeyCode() == ord(';'):
-			if self.GetLength() == 0:
-				return
-			# FIXME: smartup for whitespace after trailing ';'
-			if self.GetCharAt(curs_pos-1) == ord(';'):
-				return
 
 		# <DEL>
 		# - if inside embedded string
 		#	- delete entire string and data dict
 		# - else standard behaviour
-		if event.GetKeyCode() == wx.WXK_DELETE:
-			# FIXME: perhaps add check for regex, too ?
-			if self.GetStyleAt(curs_pos) == STYLE_EMBED:
-				self.DelPhrase(curs_pos)
-				# FIXME: also delete corresponding "additional data" dict ...
-				return
+#		if event.GetKeyCode() == wx.WXK_DELETE:
+#			# FIXME: perhaps add check for regex, too ?
+#			if self.GetStyleAt(curs_pos) == STYLE_EMBED:
+#				self.DelPhrase(curs_pos)
+#				# FIXME: also delete corresponding "additional data" dict ...
+#				return
 
 		# <BACKSPACE>
 		# - if inside embedded string
 		#	- delete entire string and data dict
 		# - else standard behaviour
-		if event.GetKeyCode() == wx.WXK_BACK:
-			# FIXME: perhaps add check for regex, too ?
-			if self.GetStyleAt(curs_pos-1) == STYLE_EMBED:
-				self.DelPhrase (curs_pos-1)
-				# FIXME: also delete corresponding "additional data" dict ...
-				return
-
-		# <ENTER>
-		# - if in list: proxy to list
-		# - in empty widget: go to next in tab order
-		# - after last character in widget:
-		#	- if after ';': go to next in tab order
-		#	- f no ';' there: add one
-#		if event.GetKeyCode() == wx.WXK_RETURN and not event.m_shiftDown:
-#			if self.GetLength() == 0:
-#				if self.next_in_tab_order is not None:
-#					self.next_in_tab_order.SetFocus()
-#				return
-#			if curs_pos == self.GetLength():
-#				# FIXME: make this smarter to deal with whitespace after ';'
-#				if self.GetCharAt(curs_pos-1) == ord(';'):
-#					if self.next_in_tab_order:
-#						self.next_in_tab_order.SetFocus()
-#				else:
-#					self.AddText (';')
+#		if event.GetKeyCode() == wx.WXK_BACK:
+#			# FIXME: perhaps add check for regex, too ?
+#			if self.GetStyleAt(curs_pos-1) == STYLE_EMBED:
+#				self.DelPhrase (curs_pos-1)
+#				# FIXME: also delete corresponding "additional data" dict ...
 #				return
 
 		event.Skip()	# skip to next event handler to keep processing
@@ -665,6 +673,28 @@ class cResizingSTC(wx.stc.StyledTextCtrl):
 			curs_pos = self.PointFromPosition(self.GetCurrentPos())
 			self.__parent.EnsureVisible (self, curs_pos.x, curs_pos.y)
 	#------------------------------------------------
+	def __on_char(self, evt):
+
+		char = unichr(evt.GetUnicodeKey())
+
+		if self.__keyword_separators.match(char) is not None:
+			if self.GetLength() == 1:
+				evt.Skip()
+				return
+
+			line, caret_pos = self.GetCurLine()
+			word = self.__keyword_separators.split(line[:caret_pos])[-1]
+			if (word not in [ r[0] for r in gmPG2.get_text_expansion_keywords() ]) and (word != u'$$steffi'):
+				evt.Skip()
+				return
+
+			start = self.GetCurrentPos() - len(word)
+			wx.CallAfter(self.replace_keyword_with_expansion, word, start)
+			evt.Skip()
+			return
+
+		evt.Skip()
+	#------------------------------------------------
 #	def _cb_on_popup_completion(self, was_cancelled=False):
 #		"""Callback for popup completion.
 #
@@ -799,7 +829,7 @@ class cResizingSTC(wx.stc.StyledTextCtrl):
 				data_sink = self.__popup_keywords[kwd]['widget_data_sink']
 			)
 		except StandardError:
-			_log.LogException('cannot call [%s] on keyword [%s] to create widget' % (create_widget, kwd), sys.exc_info(), verbose=1)
+			_log.exception('cannot call [%s] on keyword [%s] to create widget' % (create_widget, kwd))
 			gmGuiHelpers.gm_show_error (
 				aMessage = _('Cannot invoke [%s] for keyword [%s].') % (create_widget, kwd),
 				aTitle = _('showing keyword popup')
@@ -808,8 +838,8 @@ class cResizingSTC(wx.stc.StyledTextCtrl):
 
 		if not isinstance(popup, wx.Dialog):
 			gmDispatcher.send(signal='statustext', msg=_('Action [%s] on keyword [%s] is invalid.') % (create_widget, kwd))
-			_log.Log(gmLog.lErr, 'keyword [%s] triggered action [%s]' % (kwd, create_widget))
-			_log.Log(gmLog.lErr, 'the result (%s) is not a wx.Dialog subclass instance, however' % str(popup))
+			_log.error('keyword [%s] triggered action [%s]' % (kwd, create_widget))
+			_log.error('the result (%s) is not a wx.Dialog subclass instance, however' % str(popup))
 			return False
 
 		# display widget
@@ -859,8 +889,6 @@ class cResizingSTC(wx.stc.StyledTextCtrl):
 #====================================================================
 if __name__ == '__main__':
 
-	_log.SetAllLogLevels(gmLog.lData)
-
 #	from Gnumed.pycommon.gmMatchProvider import cMatchProvider_FixedList
 #	from Gnumed.pycommon import gmI18N
 
@@ -1050,6 +1078,29 @@ if __name__ == '__main__':
 	app.MainLoop()
 #====================================================================
 # $Log: gmResizingWidgets.py,v $
+# Revision 1.53.2.1  2008/09/09 17:37:21  ncq
+# - do not do anything at all on DEL/BS so as to prevent
+#   weird behaviour reported by user
+#
+# Revision 1.53  2008/08/15 15:58:47  ncq
+# - rip out CURSOR-UP/DOWN based SOAP field jumping by popular demand
+# - lotsa other cleanup
+#
+# Revision 1.52  2008/07/13 16:23:01  ncq
+# - stop setting up unused timer
+#
+# Revision 1.51  2008/07/10 21:01:22  ncq
+# - factor out char handling into __on_char while key handling stays in __on_key_down
+# - add keyword expansion as per user request
+# - a bit of cleanup in advance of the Great SOAP Merger
+#
+# Revision 1.50  2008/03/05 22:30:15  ncq
+# - new style logging
+#
+# Revision 1.49  2008/01/30 14:03:42  ncq
+# - use signal names directly
+# - switch to std lib logging
+#
 # Revision 1.48  2007/08/12 00:12:41  ncq
 # - no more gmSignals.py
 #
