@@ -2,24 +2,26 @@
 """
 #============================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmSOAPWidgets.py,v $
-# $Id: gmSOAPWidgets.py,v 1.95.2.2 2008/02/21 15:46:23 ncq Exp $
-__version__ = "$Revision: 1.95.2.2 $"
+# $Id: gmSOAPWidgets.py,v 1.105 2008/08/08 13:32:59 ncq Exp $
+__version__ = "$Revision: 1.105 $"
 __author__ = "Carlos Moro <cfmoro1976@yahoo.es>, K.Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL"
 
 # std library
-import types
+import types, logging
+
 
 # 3rd party
 import wx
 
-# GNUmed
-from Gnumed.pycommon import gmDispatcher, gmSignals, gmI18N, gmLog, gmExceptions, gmMatchProvider, gmTools
-from Gnumed.wxpython import gmResizingWidgets, gmPhraseWheel, gmEMRStructWidgets, gmGuiHelpers, gmRegetMixin, gmEditArea
-from Gnumed.business import gmPerson, gmEMRStructItems, gmSOAPimporter
 
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lInfo, __version__)
+# GNUmed
+from Gnumed.pycommon import gmDispatcher, gmI18N, gmExceptions, gmMatchProvider, gmTools, gmCfg
+from Gnumed.wxpython import gmResizingWidgets, gmPhraseWheel, gmEMRStructWidgets, gmGuiHelpers, gmRegetMixin, gmEditArea
+from Gnumed.business import gmPerson, gmEMRStructItems, gmSOAPimporter, gmSurgery
+
+_log = logging.getLogger('gm.ui')
+_log.info(__version__)
 
 #============================================================
 def create_issue_popup(parent, pos, size, style, data_sink):
@@ -68,11 +70,11 @@ def create_vacc_popup(parent, pos, size, style, data_sink):
 progress_note_keywords = {
 	's': {
 		'$missing_action': {},
-		'phx': {
+		'phx$': {
 			'widget_factory': create_issue_popup,
 			'widget_data_sink': None
 		},
-		'ea:': {
+		'ea$:': {
 			'widget_factory': create_issue_popup,
 			'widget_data_sink': None
 		},
@@ -106,7 +108,9 @@ progress_note_keywords = {
 }
 #============================================================
 class cProgressNoteInputNotebook(wx.Notebook, gmRegetMixin.cRegetOnPaintMixin):
-	"""Notebook style widget displaying progress note editors.
+	"""A notebook holding panels with progress note editors.
+
+	There is one progress note editor panel for each episode being worked on.
 	"""
 	def __init__(self, parent, id, pos=wx.DefaultPosition, size=wx.DefaultSize):
 		wx.Notebook.__init__ (
@@ -126,8 +130,11 @@ class cProgressNoteInputNotebook(wx.Notebook, gmRegetMixin.cRegetOnPaintMixin):
 	# public API
 	#--------------------------------------------------------
 	def add_editor(self, problem=None, allow_same_problem=False):
-		"""Add a progress note editor page."""
+		"""Add a progress note editor page.
 
+		The way <allow_same_problem> is currently used in callers
+		it only applies to unassociated episodes.
+		"""
 		problem_to_add = problem
 
 		# determine label
@@ -145,7 +152,7 @@ class cProgressNoteInputNotebook(wx.Notebook, gmRegetMixin.cRegetOnPaintMixin):
 			label = problem_to_add['problem']
 			# FIXME: configure maximum length
 			if len(label) > 23:
-				label = label[:20] + '...'
+				label = label[:21] + gmTools.u_ellipsis
 
 		if allow_same_problem:
 			new_page = cResizingSoapPanel(parent = self, problem = problem_to_add)
@@ -195,6 +202,7 @@ class cProgressNoteInputNotebook(wx.Notebook, gmRegetMixin.cRegetOnPaintMixin):
 					self.SetSelection(page_idx)
 					return True
 				continue
+
 		# - add new editor
 		new_page = cResizingSoapPanel(parent = self, problem = problem_to_add)
 		result = self.AddPage (
@@ -202,8 +210,68 @@ class cProgressNoteInputNotebook(wx.Notebook, gmRegetMixin.cRegetOnPaintMixin):
 			text = label,
 			select = True
 		)
+
 		return result
 	#--------------------------------------------------------
+	def close_current_editor(self):
+
+		page_idx = self.GetSelection()
+		page = self.GetPage(page_idx)
+
+		if not page.editor_empty():
+			really_discard = gmGuiHelpers.gm_show_question (
+				_('Are you sure you really want to\n'
+				  'discard this progress note ?\n'
+				),
+				_('Discarding progress note')
+			)
+			if really_discard is False:
+				return
+
+		self.DeletePage(page_idx)
+
+		# always keep one unassociated editor open
+		if self.GetPageCount() == 0:
+			self.add_editor()
+	#--------------------------------------------------------
+	def save_unsaved_soap(self):
+		save_all = False
+		dlg = None
+		for page_idx in range(self.GetPageCount()):
+			page = self.GetPage(page_idx)
+			if page.editor_empty():
+				continue
+
+			if dlg is None:
+				dlg = gmGuiHelpers.c3ButtonQuestionDlg (
+					self, 
+					-1,
+					caption = _('Unsaved progress note'),
+					question = _(
+						'This progress note has not been saved yet.\n'
+						'\n'
+						'Do you want to save it or discard it ?\n\n'
+					),
+					button_defs = [
+						{'label': _('&Save'), 'tooltip': _('Save this progress note'), 'default': True},
+						{'label': _('&Discard'), 'tooltip': _('Discard this progress note'), 'default': False},
+						{'label': _('Save &all'), 'tooltip': _('Save all remaining unsaved progress notes'), 'default': False}
+					]
+				)
+
+			if not save_all:
+				self.ChangeSelection(page_idx)
+				decision = dlg.ShowModal()
+				if decision == wx.ID_NO:
+					_log.info('user requested discarding of unsaved progress note')
+					continue
+				if decision == wx.ID_CANCEL:
+					save_all = True
+			page.save()
+
+		if dlg is not None:
+			dlg.Destroy()
+	#--------------------------------------------------------
 	# internal API
 	#--------------------------------------------------------
 	def __do_layout(self):
@@ -226,17 +294,26 @@ class cProgressNoteInputNotebook(wx.Notebook, gmRegetMixin.cRegetOnPaintMixin):
 		# wxPython events
 
 		# client internal signals
-		gmDispatcher.connect(signal=gmSignals.pre_patient_selection(), receiver=self._on_pre_patient_selection)
-		gmDispatcher.connect(signal=gmSignals.post_patient_selection(), receiver=self._on_post_patient_selection)
-		gmDispatcher.connect(signal=gmSignals.episodes_modified(), receiver=self._on_episodes_modified)
-		gmDispatcher.connect(signal=gmSignals.application_closing(), receiver=self._on_application_closing)
+		gmDispatcher.connect(signal = u'post_patient_selection', receiver=self._on_post_patient_selection)
+#		gmDispatcher.connect(signal = u'application_closing', receiver=self._on_application_closing)
+
+		self.__pat.register_pre_selection_callback(callback = self._pre_selection_callback)
+
+		gmDispatcher.send(signal = u'register_pre_exit_callback', callback = self._pre_exit_callback)
 	#--------------------------------------------------------
-	def _on_pre_patient_selection(self):
-		"""Another patient is about to be activated."""
-#		print "[%s]: another patient is about to become active" % self.__class__.__name__
-#		print "need code to:"
-#		print "- ask user about unsaved progress notes"
-		pass
+	def _pre_selection_callback(self):
+		"""Another patient is about to be activated.
+
+		Patient change will not proceed before this returns.
+		"""
+		self.save_unsaved_soap()
+	#--------------------------------------------------------
+	def _pre_exit_callback(self):
+		"""The client is about to be shut down.
+
+		Shutdown will not proceed before this returns.
+		"""
+		self.save_unsaved_soap()
 	#--------------------------------------------------------
 	def _on_post_patient_selection(self):
 		"""Patient changed."""
@@ -244,31 +321,33 @@ class cProgressNoteInputNotebook(wx.Notebook, gmRegetMixin.cRegetOnPaintMixin):
 		self.add_editor()
 		self._schedule_data_reget()
 	#--------------------------------------------------------
-	def _on_episodes_modified(self):
+#	def _on_application_closing(self):
+#		"""GNUmed is shutting down."""
+#		print "[%s]: the application is closing down" % self.__class__.__name__
+#		print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
+#		print "need to ask user about SOAP saving !"
+#		print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
+	#--------------------------------------------------------
+#	def _on_episodes_modified(self):
 #		print "[%s]: episode modified" % self.__class__.__name__
 #		print "need code to deal with:"
 #		print "- deleted episode that we show so we can notify the user"
 #		print "- renamed episode so we can update our episode label"
 #		self._schedule_data_reget()
-		pass
-	#--------------------------------------------------------
-	def _on_application_closing(self):
-		"""Patient changed."""
-#		print "[%s]: the application is closing down" % self.__class__.__name__
-#		print "need code to:"
-#		print "- ask user about unsaved data"
-		pass
+#		pass
 #============================================================
 class cNotebookedProgressNoteInputPanel(wx.Panel):
-	"""A progress note input panel.
+	"""A panel for entering multiple progress notes in context.
+
+	Expects to be used as a notebook page.
 
 	Left hand side:
 	- problem list (health issues and active episodes)
 
 	Right hand side:
-	- progress note editors notebook
+	- notebook with progress note editors
 
-	Expects to live in a notebook.
+	Listens to patient change signals, thus acts on the current patient.
 	"""
 	#--------------------------------------------------------
 	def __init__(self, parent, id):
@@ -320,7 +399,7 @@ class cNotebookedProgressNoteInputPanel(wx.Panel):
 		list_header = wx.StaticText (
 			parent = PNL_list,
 			id = -1,
-			label = _('Active Problems'),
+			label = _('Active problems'),
 			style = wx.NO_BORDER | wx.ALIGN_CENTRE
 		)
 		# - problem list
@@ -342,23 +421,23 @@ class cNotebookedProgressNoteInputPanel(wx.Panel):
 		self.__soap_notebook = cProgressNoteInputNotebook(PNL_soap_editors, -1)
 		# - buttons
 		self.__BTN_add_unassociated = wx.Button(PNL_soap_editors, -1, _('&New'))
-		self.__BTN_add_unassociated.SetToolTipString(_('add editor for new unassociated progress note'))
-
-		self.__BTN_clear = wx.Button(PNL_soap_editors, -1, _('&Reset'))
-		self.__BTN_clear.SetToolTipString(_('clear progress note editor'))
+		self.__BTN_add_unassociated.SetToolTipString(_('Add editor for a new unassociated progress note.'))
 
 		self.__BTN_save = wx.Button(PNL_soap_editors, -1, _('&Save'))
-		self.__BTN_save.SetToolTipString(_('save progress note into medical record'))
+		self.__BTN_save.SetToolTipString(_('Save progress note into medical record and close this editor.'))
 
-#		self.__BTN_discard = wx.Button(PNL_soap_editors, -1, _('&Discard'))
-#		self.__BTN_discard.SetToolTipString(_('Discard progress note and close editor. This will loose any data you already typed into this editor !'))
+		self.__BTN_clear = wx.Button(PNL_soap_editors, -1, _('&Clear'))
+		self.__BTN_clear.SetToolTipString(_('Clear this progress note editor.'))
+
+		self.__BTN_discard = wx.Button(PNL_soap_editors, -1, _('&Discard'))
+		self.__BTN_discard.SetToolTipString(_('Discard progress note and close this editor. You will loose any data already typed into this editor !'))
 
 		# - arrange
 		szr_btns_right = wx.BoxSizer(wx.HORIZONTAL)
 		szr_btns_right.Add(self.__BTN_add_unassociated, 0, wx.SHAPED)
 		szr_btns_right.Add(self.__BTN_clear, 0, wx.SHAPED)
 		szr_btns_right.Add(self.__BTN_save, 0, wx.SHAPED)
-#		szr_btns_right.Add(self.__BTN_discard, 0, wx.SHAPED)
+		szr_btns_right.Add(self.__BTN_discard, 0, wx.SHAPED)
 
 		szr_right = wx.BoxSizer(wx.VERTICAL)
 		szr_right.Add(self.__soap_notebook, 1, wx.EXPAND)
@@ -419,20 +498,20 @@ class cNotebookedProgressNoteInputPanel(wx.Panel):
 		wx.EVT_LISTBOX_DCLICK(self.__LST_problems, self.__LST_problems.GetId(), self.__on_problem_activated)
 		wx.EVT_BUTTON(self.__BTN_save, self.__BTN_save.GetId(), self.__on_save)
 		wx.EVT_BUTTON(self.__BTN_clear, self.__BTN_clear.GetId(), self.__on_clear)
-#		wx.EVT_BUTTON(self.__BTN_discard, self.__BTN_discard.GetId(), self.__on_discard)
+		wx.EVT_BUTTON(self.__BTN_discard, self.__BTN_discard.GetId(), self.__on_discard)
 		wx.EVT_BUTTON(self.__BTN_add_unassociated, self.__BTN_add_unassociated.GetId(), self.__on_add_unassociated)
 
 		# client internal signals
-		gmDispatcher.connect(signal=gmSignals.post_patient_selection(), receiver=self._on_post_patient_selection)
-		gmDispatcher.connect(signal=gmSignals.episodes_modified(), receiver=self._on_episodes_modified)
-		# FIXME: issues modified missing
+		gmDispatcher.connect(signal='post_patient_selection', receiver=self._on_post_patient_selection)
+		gmDispatcher.connect(signal = 'episode_mod_db', receiver = self._on_episode_issue_mod_db)
+		gmDispatcher.connect(signal = 'health_issue_mod_db', receiver = self._on_episode_issue_mod_db)
 	#--------------------------------------------------------
 	def _on_post_patient_selection(self):
 		"""Patient changed."""
 		if self.GetParent().GetCurrentPage() == self:
 			self.reset_ui_content()
 	#--------------------------------------------------------
-	def _on_episodes_modified(self):
+	def _on_episode_issue_mod_db(self):
 		if self.GetParent().GetCurrentPage() == self:
 			self.__refresh_problem_list()
 	#--------------------------------------------------------
@@ -447,12 +526,34 @@ class cNotebookedProgressNoteInputPanel(wx.Panel):
 
 		Will throw away data !
 		"""
-		self.__soap_notebook.DeletePage(self.__soap_notebook.GetSelection())
+		self.__soap_notebook.close_current_editor()
 	#--------------------------------------------------------
 	def __on_add_unassociated(self, evt):
 		"""Add new editor for as-yet unassociated progress note.
+
+		Clinical logic as per discussion with Jim Busser:
+
+		- if patient has no episodes:
+			- new patient
+			- always allow several NEWs
+		- of patient has episodes:
+			- allow several NEWs per configuration
 		"""
-		self.__soap_notebook.add_editor()
+		emr = self.__pat.get_emr()
+		epis = emr.get_episodes()
+
+		if len(epis) == 0:
+			value = True
+		else:
+			dbcfg = gmCfg.cCfgSQL()
+			value = bool(dbcfg.get2 (
+				option = u'horstspace.soap_editor.allow_same_episode_multiple_times',
+				workplace = gmSurgery.gmCurrentPractice().active_workplace,
+				bias = u'user',
+				default = False
+			))
+
+		self.__soap_notebook.add_editor(allow_same_problem = value)
 	#--------------------------------------------------------
 	def __on_problem_activated(self, event):
 		"""
@@ -490,7 +591,7 @@ class cNotebookedProgressNoteInputPanel(wx.Panel):
 		self.__soap_notebook.DeletePage(page_idx)
 		# always keep one unassociated editor open
 		self.__soap_notebook.add_editor()
-		self.__refresh_problem_list()
+		#self.__refresh_problem_list()
 		return True
 	#--------------------------------------------------------
 	# notebook plugin API
@@ -543,7 +644,7 @@ class cPopupDataHolder:
 			try:
 				saver_func = self.__data_savers[popup_type]
 			except KeyError:
-				_log.LogException('no saver for popup data type [%s] configured' % popup_type)
+				_log.exception('no saver for popup data type [%s] configured', popup_type)
 				return False
 			for desc in self.__data[popup_type].keys():
 				data = self.__data[popup_type][desc]['data']
@@ -560,7 +661,7 @@ class cPopupDataHolder:
 	# def get_data(self, desc=None):
 	# def rename_data(self, old_desc=None, new_desc=None):
 #============================================================
-class cResizingSoapWin (gmResizingWidgets.cResizingWindow):
+class cResizingSoapWin(gmResizingWidgets.cResizingWindow):
 
 	def __init__(self, parent, size, input_defs=None, problem=None):
 		"""Resizing SOAP note input editor.
@@ -658,7 +759,7 @@ class cResizingSoapWin (gmResizingWidgets.cResizingWindow):
 		# - new episode, must get name from narrative (or user)
 		if (self.__problem is None) or (self.__problem['type'] == 'issue'):
 			# work out episode name
-			epic_name = u''
+			epi_name = u''
 			if len(aoe) != 0:
 				epi_name = aoe
 			else:
@@ -669,12 +770,15 @@ class cResizingSoapWin (gmResizingWidgets.cResizingWindow):
 				message = _('Enter a descriptive name for the new episode:'),
 				caption = _('Adding a new episode'),
 				defaultValue = epi_name.replace('\r', '//').replace('\n', '//'),
-				style = wx.OK | wx.CENTRE
+				style = wx.OK | wx.CANCEL | wx.CENTRE
 			)
-			dlg.ShowModal()
+			decision = dlg.ShowModal()
+			if decision != wx.ID_OK:
+				return False
+
 			epi_name = dlg.GetValue().strip()
 			if epi_name == u'':
-				gmGuiHelpers.gm_show_error(_('Cannot save a new episode without a name.'), _('saving progress note'), gmLog.lErr)
+				gmGuiHelpers.gm_show_error(_('Cannot save a new episode without a name.'), _('saving progress note'))
 				return False
 
 			# new unassociated episode
@@ -703,15 +807,14 @@ class cResizingSoapWin (gmResizingWidgets.cResizingWindow):
 		# dump progress note to backend
 		importer = gmSOAPimporter.cSOAPImporter()
 		if not importer.import_soap(progress_note):
-			gmGuiHelpers.gm_show_error(_('Error saving progress note.'), _('saving progress note'), gmLog.lErr)
+			gmGuiHelpers.gm_show_error(_('Error saving progress note.'), _('saving progress note'))
 			return False
 
 		# dump embedded data to backend
 		if not self.__embedded_data_holder.save():
 			gmGuiHelpers.gm_show_error (
 				_('Error saving embedded data.'),
-				_('saving progress note'),
-				gmLog.lErr
+				_('saving progress note')
 			)
 			return False
 		self.__embedded_data_holder.clear()
@@ -720,6 +823,15 @@ class cResizingSoapWin (gmResizingWidgets.cResizingWindow):
 	#--------------------------------------------------------
 	def get_problem(self):
 		return self.__problem
+	#--------------------------------------------------------
+	def is_empty(self):
+		editor_content = self.GetValue()
+
+		for field_content in editor_content.values():
+			if field_content.text.strip() != u'':
+				return False
+
+		return True
 #============================================================
 class cResizingSoapPanel(wx.Panel):
 	"""Basic progress note panel.
@@ -756,14 +868,7 @@ class cResizingSoapPanel(wx.Panel):
 
 		self.__is_saved = False
 		# do layout
-		wx.Panel.__init__ (
-			self,
-			parent,
-			-1,
-			wx.DefaultPosition,
-			wx.DefaultSize,
-			wx.NO_BORDER | wx.TAB_TRAVERSAL
-		)
+		wx.Panel.__init__(self, parent, -1, style = wx.NO_BORDER | wx.TAB_TRAVERSAL)
 		# - editor
 		if input_defs is None:
 			soap_lines = []
@@ -847,8 +952,12 @@ class cResizingSoapPanel(wx.Panel):
 		Check SOAP input widget saved (dumped to backend) state
 		"""
 		return self.__is_saved
+	#--------------------------------------------------------
 	def save(self):
 		return self.__soap_editor.save()
+	#--------------------------------------------------------
+	def editor_empty(self):
+		return self.__soap_editor.is_empty()
 #============================================================
 class cSingleBoxSOAP(wx.TextCtrl):
 	"""if we separate it out like this it can transparently gain features"""
@@ -903,8 +1012,8 @@ class cSingleBoxSOAPPanel(wx.Panel):
 		wx.EVT_BUTTON(self.__BTN_discard, self.__BTN_discard.GetId(), self._on_discard_note)
 
 		# client internal signals
-		gmDispatcher.connect(signal = gmSignals.pre_patient_selection(), receiver = self._save_note)
-		gmDispatcher.connect(signal = gmSignals.application_closing(), receiver = self._save_note)
+		gmDispatcher.connect(signal = 'pre_patient_selection', receiver = self._save_note)
+		gmDispatcher.connect(signal = 'application_closing', receiver = self._save_note)
 
 		return True
 	#--------------------------------------------------------
@@ -928,7 +1037,7 @@ class cSingleBoxSOAPPanel(wx.Panel):
 		# sanity checks
 		if self.__pat is None:
 			return True
-		if not self.__pat.is_connected():
+		if not self.__pat.connected:
 			return True
 		if not self.__soap_box.IsModified():
 			return True
@@ -938,10 +1047,10 @@ class cSingleBoxSOAPPanel(wx.Panel):
 		# now save note
 		emr = self.__pat.get_emr()
 		if emr is None:
-			_log.Log(gmLog.lErr, 'cannot access clinical record of patient')
+			_log.error('cannot access clinical record of patient')
 			return False
 		if not emr.add_clin_narrative(note, soap_cat='s'):
-			_log.Log(gmLog.lErr, 'error saving clinical note')
+			_log.error('error saving clinical note')
 			return False
 		self.__soap_box.SetValue('')
 		return True
@@ -951,8 +1060,7 @@ class cSingleBoxSOAPPanel(wx.Panel):
 if __name__ == "__main__":
 
 	import sys
-	_log = gmLog.gmDefLog
-	_log.SetAllLogLevels(gmLog.lData)
+
 	from Gnumed.pycommon import gmPG2
 	#--------------------------------------------------------
 	def get_narrative(pk_encounter=None, pk_health_issue = None, default_labels=None):
@@ -1042,7 +1150,6 @@ if __name__ == "__main__":
 		application.frame.Show(True)
 		application.MainLoop()
 	#--------------------------------------------------------
-	_log.SetAllLogLevels(gmLog.lData)
 
 	try:
 		# obtain patient
@@ -1100,19 +1207,53 @@ if __name__ == "__main__":
 #		app = wx.PyWidgetTester(size=(600,600))
 #		app.SetWidget(cSingleBoxSOAPPanel, -1)
 #		app.MainLoop()
-		
+
 	except StandardError:
-		_log.LogException("unhandled exception caught !", sys.exc_info(), 1)
+		_log.exception("unhandled exception caught !")
 		# but re-raise them
 		raise
 
 #============================================================
 # $Log: gmSOAPWidgets.py,v $
-# Revision 1.95.2.2  2008/02/21 15:46:23  ncq
-# - fix sometime crash on opening soap editor
+# Revision 1.105  2008/08/08 13:32:59  ncq
+# - factor out save_unsaved_soap()
+# - register and act on pre-exit callback
+# - some cleanup
 #
-# Revision 1.95.2.1  2008/01/16 14:12:09  ncq
-# - do not crash on saving when there's only one editor open for a new episode
+# Revision 1.104  2008/08/05 16:22:17  ncq
+# - cleanup
+#
+# Revision 1.103  2008/07/14 13:48:16  ncq
+# - properly save unsaved soap
+# - cleanup
+#
+# Revision 1.102  2008/07/10 21:04:10  ncq
+# - better comments
+# - experimental pre-selection SOAP saving
+#
+# Revision 1.101  2008/06/28 22:37:39  ncq
+# - visual improvement
+# - improved code comments
+# - add button to discard current editor
+# - prepare for saving soap on patient change/shutdown
+# - allow backing out of saving soap when asked for title of new episode
+#
+# Revision 1.100  2008/03/05 22:30:15  ncq
+# - new style logging
+#
+# Revision 1.99  2008/02/25 17:43:03  ncq
+# - keywords: ea -> ea$, phx -> phx$
+# - fix add_editor()
+# - clinical logic change for adding new editors
+#
+# Revision 1.98  2008/01/14 20:42:26  ncq
+# - don't crash on adding an empty editor
+#
+# Revision 1.97  2008/01/05 16:41:27  ncq
+# - remove logging from gm_show_*()
+#
+# Revision 1.96  2007/12/12 16:24:57  ncq
+# - explicit signals
 #
 # Revision 1.95  2007/09/24 22:05:57  ncq
 # - ask user for episode name when needed
