@@ -2,21 +2,24 @@
 
 #===========================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmTopPanel.py,v $
-# $Id: gmTopPanel.py,v 1.89 2007/12/02 11:00:58 ncq Exp $
-__version__ = "$Revision: 1.89 $"
+# $Id: gmTopPanel.py,v 1.97 2008/08/05 16:22:44 ncq Exp $
+__version__ = "$Revision: 1.97 $"
 __author__  = "R.Terry <rterry@gnumed.net>, I.Haywood <i.haywood@ugrad.unimelb.edu.au>, K.Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL"
 
-import sys, os.path
+
+import sys, os.path, datetime as pyDT, logging
+
 
 import wx
 
-from Gnumed.pycommon import gmGuiBroker, gmPG2, gmSignals, gmDispatcher, gmLog, gmCLI, gmTools
+
+from Gnumed.pycommon import gmGuiBroker, gmPG2, gmDispatcher, gmTools, gmCfg2, gmDateTime
 from Gnumed.business import gmPerson, gmEMRStructItems, gmAllergy
 from Gnumed.wxpython import gmGuiHelpers, gmPatPicWidgets, gmPatSearchWidgets, gmAllergyWidgets
 
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lInfo, __version__)
+_log = logging.getLogger('gm.ui')
+_log.info(__version__)
 
 [	ID_BTN_pat_demographics,
 #	ID_CBOX_consult_type,
@@ -47,7 +50,7 @@ class cMainTopPanel(wx.Panel):
 		self.__register_interests()
 
 		# init plugin toolbars dict
-		self.subbars = {}
+		#self.subbars = {}
 		self.curr_pat = gmPerson.gmCurrentPatient()
 
 		# and actually display ourselves
@@ -100,20 +103,17 @@ class cMainTopPanel(wx.Panel):
 #		self.szr_top_row.Add(self.btn_lock, 0, wxALL, 3)
 
 		#  - patient selector
-		lbl_pat = wx.StaticText (self, -1, _('Patient'), style = wx.ALIGN_CENTER_VERTICAL)
-		lbl_pat.SetFont (wx.Font(12, wx.SWISS, wx.NORMAL, wx.BOLD, False, ''))
-		self.patient_selector = gmPatSearchWidgets.cPatientSelector(self, -1)
-		if self.__gb['main.slave_mode']:
+		self.patient_selector = gmPatSearchWidgets.cActivePatientSelector(self, -1)
+		cfg = gmCfg2.gmCfgData()
+		if cfg.get(option = 'slave'):
 			self.patient_selector.SetEditable(0)
 			self.patient_selector.SetToolTip(None)
 		self.patient_selector.SetFont(wx.Font(12, wx.SWISS, wx.NORMAL, wx.BOLD, False, ''))
-		self.szr_top_row.Add (lbl_pat, 0, wx.ALL, 3)
-		self.szr_top_row.Add (self.patient_selector, 5, wx.BOTTOM, 3)
+
 		#  - age
-		self.txt_age = wx.TextCtrl(self, -1, '', size = (50,-1), style = wx.TE_READONLY)
-		self.txt_age.SetFont (wx.Font(11, wx.SWISS, wx.NORMAL, wx.BOLD, False, ''))
-		self.txt_age.SetBackgroundColour(bg_col)
-		self.szr_top_row.Add (self.txt_age, 0, wx.BOTTOM | wx.LEFT | wx.RIGHT, 3)
+		self.lbl_age = wx.StaticText(self, -1, u'', style = wx.ALIGN_CENTER_VERTICAL)
+		self.lbl_age.SetFont(wx.Font(10, wx.SWISS, wx.NORMAL, wx.BOLD, False, ''))
+
 		#  - allergies (substances only, like "makrolides, penicillins, eggs")
 		self.lbl_allergies = wx.StaticText (self, -1, _('Caveat'), style = wx.ALIGN_CENTER_VERTICAL)
 		self.lbl_allergies.SetFont(wx.Font(12, wx.SWISS, wx.NORMAL, wx.BOLD, False, ''))
@@ -121,10 +121,12 @@ class cMainTopPanel(wx.Panel):
 		self.lbl_allergies.SetForegroundColour(col_brightred)
 		self.txt_allergies = wx.TextCtrl (self, -1, "", style = wx.TE_READONLY)
 		self.txt_allergies.SetFont(wx.Font(10, wx.SWISS, wx.NORMAL, wx.BOLD, False, ''))
-		#self.txt_allergies.SetBackgroundColour(bg_col)
 		self.txt_allergies.SetForegroundColour (col_brightred)
-		self.szr_top_row.Add (self.lbl_allergies, 0, wx.ALL, 3)
-		self.szr_top_row.Add (self.txt_allergies, 6,wx.BOTTOM, 3)
+
+		self.szr_top_row.Add(self.patient_selector, 6, wx.LEFT | wx.BOTTOM, 3)
+		self.szr_top_row.Add(self.lbl_age, 0, wx.ALL, 3)
+		self.szr_top_row.Add(self.lbl_allergies, 0, wx.ALL, 3)
+		self.szr_top_row.Add(self.txt_allergies, 8, wx.BOTTOM, 3)
 
 		# - bottom row
 		# .----------------------------------------------------------.
@@ -249,7 +251,7 @@ class cMainTopPanel(wx.Panel):
 		wx.EVT_LEFT_DCLICK(self.txt_allergies, self._on_allergies_dclicked)
 
 		# client internal signals
-		gmDispatcher.connect(signal=gmSignals.post_patient_selection(), receiver=self._on_post_patient_selection)
+		gmDispatcher.connect(signal = u'post_patient_selection', receiver = self._on_post_patient_selection)
 		gmDispatcher.connect(signal = u'allg_mod_db', receiver = self._update_allergies)
 		gmDispatcher.connect(signal = u'allg_state_mod_db', receiver = self._update_allergies)
 		gmDispatcher.connect(signal = u'name_mod_db', receiver = self._on_name_identity_change)
@@ -262,7 +264,7 @@ class cMainTopPanel(wx.Panel):
 	#----------------------------------------------
 	def _on_allergies_dclicked(self, evt):
 		pat = gmPerson.gmCurrentPatient()
-		if not pat.is_connected():
+		if not pat.connected:
 			gmDispatcher.send('statustext', msg = _('Cannot activate Allergy Manager. No active patient.'))
 			return
 		dlg = gmAllergyWidgets.cAllergyManagerDlg(parent=self, id=-1)
@@ -281,25 +283,24 @@ class cMainTopPanel(wx.Panel):
 #		pc.Centre(wx.BOTH)
 #		pc.Show(1)
 	#----------------------------------------------
-#	def _on_episode_selected(self, evt):
-#		epr = self.curr_pat.get_emr()
-#		if epr is None:
-#			return None
-#		ep_name = evt.GetString()
-#		if not epr.set_active_episode(ep_name):
-#			gmGuiHelpers.gm_show_error (
-#				_('Cannot activate episode [%s].\n'
-#				  'Leaving previous one activated.' % ep_name),
-#				_('selecting active episode'),
-#				gmLog.lErr
-#			)
-	#----------------------------------------------
 	def _on_name_identity_change(self):
 		wx.CallAfter(self.__on_name_identity_change)
 	#----------------------------------------------
 	def __on_name_identity_change(self):
 		self.patient_selector.SetValue(self.curr_pat['description'])
-		self.txt_age.SetValue(self.curr_pat['medical_age'])
+
+		if self.curr_pat['dob'].strftime('%m-%d') == pyDT.datetime.now(tz = gmDateTime.gmCurrentLocalTimezone).strftime('%m-%d'):
+			template = _('%s  %s (%s today !)')
+		else:
+			template = u'%s  %s (%s)'
+		age = template % (
+			gmPerson.map_gender2symbol[self.curr_pat['gender']],
+			self.curr_pat['dob'].strftime('%x'),
+			self.curr_pat['medical_age']
+		)
+		self.lbl_age.SetLabel(age)
+
+		self.Layout()
 	#----------------------------------------------
 	def _on_post_patient_selection(self, **kwargs):
 		# needed because GUI stuff can't be called from a thread (and that's
@@ -307,12 +308,24 @@ class cMainTopPanel(wx.Panel):
 		wx.CallAfter(self.__on_post_patient_selection, **kwargs)
 	#----------------------------------------------
 	def __on_post_patient_selection(self, **kwargs):
-		age = self.curr_pat['medical_age']
+		self.patient_selector.SetValue(self.curr_pat['description'])
+
 		# FIXME: if the age is below, say, 2 hours we should fire
 		# a timer here that updates the age in increments of 1 minute ... :-)
-		self.patient_selector.SetValue(self.curr_pat['description'])
-		self.txt_age.SetValue(age)
+		if self.curr_pat['dob'].strftime('%m-%d') == pyDT.datetime.now(tz = gmDateTime.gmCurrentLocalTimezone).strftime('%m-%d'):
+			template = _('%s  %s (%s today !)')
+		else:
+			template = u'%s  %s (%s)'
+		age = template % (
+			gmPerson.map_gender2symbol[self.curr_pat['gender']],
+			self.curr_pat['dob'].strftime('%x'),
+			self.curr_pat['medical_age']
+		)
+		self.lbl_age.SetLabel(age)
+
 		self.__update_allergies()
+
+		self.Layout()
 	#-------------------------------------------------------
 	def __on_display_demographics(self, evt):
 		print "display patient demographic window now"
@@ -330,7 +343,7 @@ class cMainTopPanel(wx.Panel):
 			return True
 
 		tmp = []
-		tt = u'%-70s\n\n' % _('Allergies:')
+		tt = u'%-70s\n' % u' '
 		for allergy in allergies:
 			if allergy['type'] == 'allergy':
 				tmp.append(allergy['descriptor'][:10])
@@ -340,7 +353,7 @@ class cMainTopPanel(wx.Panel):
 				certainty = _('likely')
 			reaction = gmTools.coalesce(allergy['reaction'], _('reaction not recorded'))
 			if len(reaction) > 50:
-				reaction = reaction[:50] + u'...'
+				reaction = reaction[:50] + gmTools.u_ellipsis
 			tt += u'%s - %s (%s): %s\n' % (
 				allergy['descriptor'],
 				allergy['l10n_type'],
@@ -364,75 +377,75 @@ class cMainTopPanel(wx.Panel):
 		"""
 		self.szr_bottom_row.Prepend(widget, 0, wx.ALL, 0)
 	#-------------------------------------------------------
-	def CreateBar(self):
-		"""Creates empty toolbar suited for adding to top panel."""
-		bar = wx.ToolBar (
-			self.pnl_bottom_row,
-			-1,
-			size = self.pnl_bottom_row.GetClientSize(),
-			style = wx.TB_HORIZONTAL | wx.NO_BORDER | wx.TB_FLAT
-		)
-		return bar
+#	def CreateBar(self):
+#		"""Creates empty toolbar suited for adding to top panel."""
+#		bar = wx.ToolBar (
+#			self.pnl_bottom_row,
+#			-1,
+#			size = self.pnl_bottom_row.GetClientSize(),
+#			style = wx.TB_HORIZONTAL | wx.NO_BORDER | wx.TB_FLAT
+#		)
+#		return bar
 	#-------------------------------------------------------
-	def AddBar(self, key=None, bar=None):
-		"""Creates and returns a new empty toolbar, referenced by key.
-
-		Key should correspond to the notebook page number as defined
-		by the notebook (see gmPlugin.py), so that gmGuiMain can
-		display the toolbar with the notebook
-		"""
-		bar.SetToolBitmapSize((16,16))
-		self.subbars[key] = bar
-		if len(self.subbars) == 1:
-			bar.Show(1)
-			self.__current = key
-		else:
-			bar.Hide()
-		return True
+#	def AddBar(self, key=None, bar=None):
+#		"""Creates and returns a new empty toolbar, referenced by key.
+#
+#		Key should correspond to the notebook page number as defined
+#		by the notebook (see gmPlugin.py), so that gmGuiMain can
+#		display the toolbar with the notebook
+#		"""
+#		bar.SetToolBitmapSize((16,16))
+#		self.subbars[key] = bar
+#		if len(self.subbars) == 1:
+#			bar.Show(1)
+#			self.__current = key
+#		else:
+#			bar.Hide()
+#		return True
 	#-------------------------------------------------------
-	def ReFit (self):
-		"""Refits the toolbar after its been changed
-		"""
-		tw = 0
-		th = 0
-		# get maximum size for the toolbar
-		for i in self.subbars.values ():
-			ntw, nth = i.GetSizeTuple ()
-			if ntw > tw:
-				tw = ntw
-			if nth > th:
-				th = nth
-		#import pdb
-		#pdb.set_trace ()
-		sz = wx.Size (tw, th)
-		self.pnl_bottom_row.SetSize(sz)
-		for i in self.subbars.values():
-			i.SetSize (sz)
-		self.szr_main.Layout()
-		self.szr_main.Fit(self)
+#	def ReFit (self):
+#		"""Refits the toolbar after its been changed
+#		"""
+#		tw = 0
+#		th = 0
+#		# get maximum size for the toolbar
+#		for i in self.subbars.values ():
+#			ntw, nth = i.GetSizeTuple ()
+#			if ntw > tw:
+#				tw = ntw
+#			if nth > th:
+#				th = nth
+#		#import pdb
+#		#pdb.set_trace ()
+#		sz = wx.Size (tw, th)
+#		self.pnl_bottom_row.SetSize(sz)
+#		for i in self.subbars.values():
+#			i.SetSize (sz)
+#		self.szr_main.Layout()
+#		self.szr_main.Fit(self)
 	#-------------------------------------------------------
-	def ShowBar (self, key):
-		"""Displays the named toolbar.
-		"""
-		self.subbars[self.__current].Hide()
-		try:
-			self.subbars[key].Show(1)
-			self.__current = key
-		except KeyError:
-			gmLog.gmDefLog.LogException("cannot show undefined toolbar [%s]" % key, sys.exc_info(), verbose=1)
+#	def ShowBar (self, key):
+#		"""Displays the named toolbar.
+#		"""
+#		self.subbars[self.__current].Hide()
+#		try:
+#			self.subbars[key].Show(1)
+#			self.__current = key
+#		except KeyError:
+#			_log.exception("cannot show undefined toolbar [%s]" % key)
 	#-------------------------------------------------------
-	def DeleteBar (self, key):
-		"""Removes a toolbar.
-		"""
-		try:
-			self.subbars[key].Destroy()
-			del self.subbars[key]
-			# FIXME: ??
-			if self.__current == key and len(self.subbars):
-				self.__current = self.subbars.keys()[0]
-				self.subbars[self.__current].Show(1)
-		except KeyError:
-			gmLog.gmDefLog.LogException("cannot delete undefined toolbar [%s]" % key, sys.exc_info(), verbose=1)
+#	def DeleteBar (self, key):
+#		"""Removes a toolbar.
+#		"""
+#		try:
+#			self.subbars[key].Destroy()
+#			del self.subbars[key]
+#			# FIXME: ??
+#			if self.__current == key and len(self.subbars):
+#				self.__current = self.subbars.keys()[0]
+#				self.subbars[self.__current].Show(1)
+#		except KeyError:
+#			_log.exception("cannot delete undefined toolbar [%s]" % key)
 
 #===========================================================	
 if __name__ == "__main__":
@@ -442,6 +455,33 @@ if __name__ == "__main__":
 	app.MainLoop()
 #===========================================================
 # $Log: gmTopPanel.py,v $
+# Revision 1.97  2008/08/05 16:22:44  ncq
+# - improve dob display and allergies tooltip
+#
+# Revision 1.96  2008/07/10 08:41:09  ncq
+# - comment out toolbar handling
+# - pat.is_connected -> connected property
+#
+# Revision 1.95  2008/03/09 20:18:56  ncq
+# - use cActivePatientSelector()
+#
+# Revision 1.94  2008/03/05 22:30:15  ncq
+# - new style logging
+#
+# Revision 1.93  2008/01/27 21:20:58  ncq
+# - no more label "Patient"
+# - make age field a dynamically adjusting static text, include gender and DOB
+# - remind of birthday
+#
+# Revision 1.92  2007/12/23 22:04:19  ncq
+# - no more gmCLI
+#
+# Revision 1.91  2007/12/23 20:29:57  ncq
+# - use gmCfg2
+#
+# Revision 1.90  2007/12/11 12:49:26  ncq
+# - explicit signal handling
+#
 # Revision 1.89  2007/12/02 11:00:58  ncq
 # - update age, too, on pat identity change
 #
