@@ -1,7 +1,4 @@
 #!/usr/bin/env python
-#===========================================================
-# gnumed.py - launcher for the main GNUmed GUI client module
-#===========================================================
 
 __doc__ = """GNUmed client launcher.
 
@@ -25,7 +22,7 @@ care of all the pre- and post-GUI runtime environment setup.
  standard textdomain is, of course, "gnumed.mo".
 --log-file=<file>
  Use this to change the name of the log file.
- See gmLog.py to find out where the standard log file would
+ See gmLog2.py to find out where the standard log file would
  end up.
 --conf-file=<file>
  Use configuration file <file> instead of searching for it in
@@ -37,30 +34,31 @@ care of all the pre- and post-GUI runtime environment setup.
 --override-schema-check
  Continue loading the client even if the database schema version
  and the client software version cannot be verified to be compatible.
+--local-import
+ Adjust the PYTHONPATH such that GNUmed can be run from a local source tree.
 --help, -h, or -?
  Show this help.
 """
 #==========================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gnumed.py,v $
-# $Id: gnumed.py,v 1.123.2.1 2008/02/21 17:18:45 ncq Exp $
-__version__ = "$Revision: 1.123.2.1 $"
+# $Id: gnumed.py,v 1.142.2.1 2008/08/31 14:25:41 ncq Exp $
+__version__ = "$Revision: 1.142.2.1 $"
 __author__  = "H. Herb <hherb@gnumed.net>, K. Hilbert <Karsten.Hilbert@gmx.net>, I. Haywood <i.haywood@ugrad.unimelb.edu.au>"
 __license__ = "GPL (details at http://www.gnu.org)"
 
-
 # standard library
-import sys, os, os.path, signal, warnings
+import sys, os, os.path, signal, logging
 
 
-# 1) don't run as module
+# do not run as module
 if __name__ != "__main__":
-	print "GNUmed startup: This should not be imported as a module !"
-	print "---------------------------------------------------------"
+	print "GNUmed startup: This is not intended to be imported as a module !"
+	print "-----------------------------------------------------------------"
 	print __doc__
-	sys.exit(0)
+	sys.exit(1)
 
 
-# 2) advise not to run as root
+# do not run as root
 if os.name in ['posix'] and os.geteuid() == 0:
 	print """
 GNUmed startup: GNUmed should not be run as root.
@@ -72,26 +70,28 @@ against. Please run GNUmed as a non-root user.
 """
 	sys.exit(1)
 
-
-# Python 2.3 on Mandrake turns True/False deprecation warnings
-# into exceptions, so revert them to warnings again
-warnings.filterwarnings("default", "Use\sPython.s\sFalse\sinstead", DeprecationWarning)
-warnings.filterwarnings("default", "Use\sPython.s\sTrue\sinstead", DeprecationWarning)
-
-
+#----------------------------------------------------------
 _log = None
-#_cfg = None
-_email_logger = None
-gmLog = None
-_old_sig_hup = None
+_cfg = None
 _old_sig_term = None
+_known_short_options = u'h?'
+_known_long_options = [
+	u'debug',
+	u'slave',
+	u'profile=',
+	u'text-domain=',
+	u'log-file=',
+	u'conf-file=',
+	u'lang-gettext=',
+	u'override-schema-check',
+	u'local-import',
+	u'help'
+]
 
-#==========================================================
-# convenience functions
-#==========================================================
-def setup_console_exption_handler():
-	import_error_sermon = """
-CRITICAL ERROR: Cannot load GNUmed Python modules ! - Program halted.
+import_error_sermon = """
+GNUmed startup: Cannot load GNUmed Python modules !
+---------------------------------------------------
+CRITICAL ERROR: Program halted.
 
 Please make sure you have:
 
@@ -109,96 +109,95 @@ did run gnumed/check-prerequisites.sh with good results.
 
 If you still encounter errors after checking the above
 requirements please ask on the mailing list.
-""" % '\n '.join(sys.path)
+"""
 
-	try:
-		from Gnumed.pycommon import gmTools
-	except ImportError:
-		sys.exit(import_error_sermon)
 
-	sys.excepthook = gmTools.handle_uncaught_exception
+missing_config_file = """
+GNUmed startup: Missing configuration file.
+-------------------------------------------
+
+You explicitely specified a configuration file
+on the command line:
+
+	--conf-file=%s
+
+The file does not exist, however.
+"""
+#==========================================================
+# convenience functions
+#==========================================================
+def setup_python_path():
+
+	if not u'--local-import' in sys.argv:
+		return
+
+	print "GNUmed startup: Running from local source tree."
+	print "-----------------------------------------------"
+
+	local_python_base_dir = os.path.dirname (
+		os.path.abspath(os.path.join(sys.argv[0], '..', '..'))
+	)
+
+	# does the path exist at all, physically ?
+	# note that broken links are reported as True
+	if not os.path.lexists(os.path.join(local_python_base_dir, 'Gnumed')):
+		src = os.path.join(local_python_base_dir, 'client')
+		dst = os.path.join(local_python_base_dir, 'Gnumed')
+		print "Creating module import symlink ..."
+		print '', dst, '=>'
+		print '    =>', src
+		os.symlink(src, dst)
+
+	print "Adjusting PYTHONPATH ..."
+	sys.path.insert(0, local_python_base_dir)
 #==========================================================
 def setup_logging():
-	from Gnumed.pycommon import gmLog as _gmLog
+	try:
+		from Gnumed.pycommon import gmLog2 as _gmLog2
+	except ImportError:
+		sys.exit(import_error_sermon % '\n '.join(sys.path))
+
+	global gmLog2
+	gmLog2 = _gmLog2
 
-	global gmLog
-	gmLog = _gmLog
 	global _log
-	_log = gmLog.gmDefLog
-
-	# always start with debugging enabled
-	_log.SetAllLogLevels(gmLog.lData)
-
-	return True
+	_log = logging.getLogger('gm.launcher')
 #==========================================================
-def setup_locale():
-	gmI18N.activate_locale()
+def setup_console_exception_handler():
+	from Gnumed.pycommon.gmTools import handle_uncaught_exception_console
 
-	td = None
-	if gmCLI.has_arg('--text-domain'):
-		td = gmCLI.arg['--text-domain']
-
-	l = None
-	if gmCLI.has_arg('--lang-gettext'):
-		l = gmCLI.arg['--lang-gettext']
-
-	gmI18N.install_domain(domain = td, language = l)
-
-	return True
+	sys.excepthook = handle_uncaught_exception_console
 #==========================================================
-def setup_paths():
-	"""Create needed paths in user home directory."""
-
-	from Gnumed.pycommon import gmTools
-
-	gmTools.mkdir(os.path.expanduser(os.path.join('~', '.gnumed', 'scripts')))
-	gmTools.mkdir(os.path.expanduser(os.path.join('~', '.gnumed', 'spellcheck')))
-	gmTools.mkdir(os.path.expanduser(os.path.join('~', '.gnumed', 'tmp', 'docs')))
-	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed', 'export', 'docs')))
-	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed', 'export', 'xDT')))
-	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed', 'export', 'EMR')))
-	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed', 'xDT')))
-	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed', 'logs')))
-
-	paths = gmTools.gmPaths()
-#==========================================================
-def setup_date_time():
-	from Gnumed.pycommon import gmDateTime
-
-	gmDateTime.init()
-#==========================================================
-def setup_cfg_files():
-	from Gnumed.pycommon import gmCfg, gmNull
-
-	if isinstance(gmCfg.gmDefCfgFile, gmNull.cNull):
-		if gmCfg.create_default_cfg_file():
-			# now that we got the file we can reopen it as a config file :-)
-			try:
-				f = gmCfg.cCfgFile()
-			except:
-				print "GNUmed startup: Cannot open or create config file by any means."
-				print "GNUmed startup: Please see the log for details."
-				_log.LogException('unhandled exception', sys.exc_info(), verbose=0)
-				sys.exit(0)
-			gmCfg.gmDefCfgFile = f
-		else:
-			print "GNUmed startup: Cannot open or create config file by any means.\nPlease see the log for details."
-			sys.exit(0)
+def setup_cli():
+	from Gnumed.pycommon import gmCfg2
 
 	global _cfg
-	_cfg = gmCfg.gmDefCfgFile
+	_cfg = gmCfg2.gmCfgData()
+	_cfg.add_cli (
+		short_options = _known_short_options,
+		long_options = _known_long_options
+	)
+
+	val = _cfg.get(option = '--debug', source_order = [('cli', 'return')])
+	if val is None:
+		val = False
+	_cfg.set_option (
+		option = u'debug',
+		value = val
+	)
+
+	val = _cfg.get(option = '--slave', source_order = [('cli', 'return')])
+	if val is None:
+		val = False
+	_cfg.set_option (
+		option = u'slave',
+		value = val
+	)
 #==========================================================
-def handle_sig_hup(signum, frame):
-	print "SIGHUP caught"
-	print signum
-	print frame
-	if _old_sig_hup in [None, signal.SIG_IGN]:
-		sys.exit(signal.SIGHUP)
-	else:
-		_old_sig_hup(signum, frame)
-#----------------------------------------------------------
 def handle_sig_term(signum, frame):
-	print 'GNUmed: SIGTERM (SIG%s) caught, shutting down ...' % signum
+	_log.critical('SIGTERM (SIG%s) received, shutting down ...' % signum)
+	gmLog2.flush()
+	print 'GNUmed: SIGTERM (SIG%s) received, shutting down ...' % signum
 	if frame is not None:
 		print '%s::%s@%s' % (frame.f_code.co_filename, frame.f_code.co_name, frame.f_lineno)
 
@@ -210,81 +209,201 @@ def handle_sig_term(signum, frame):
 		_old_sig_term(signum, frame)
 #----------------------------------------------------------
 def setup_signal_handlers():
-#	global _old_sig_hup
-#	old_sig_hup = signal.signal(signal.SIGHUP, handle_sig_hup)
 	global _old_sig_term
 	old_sig_term = signal.signal(signal.SIGTERM, handle_sig_term)
 #==========================================================
-def log_object_refcounts():
-	if not gmCLI.has_arg('--debug'):
-		return
-
-	import types
-
-	def get_refcounts():
-		refcount = {}
-		# collect all classes
-		for module in sys.modules.values():
-			for sym in dir(module):
-				obj = getattr(module, sym)
-				if type(obj) is types.ClassType:
-					refcount[obj] = sys.getrefcount(obj)
-		# sort by refcount
-		pairs = map(lambda x: (x[1],x[0]), refcount.items())
-		pairs.sort()
-		pairs.reverse()
-		return pairs
-
-	rcfile = open('./gm-refcount.lst', 'wb')
-#	for refcount, class_ in get_refcounts():
-#		if not class_.__name__.startswith('wx'):
-#			rcfile.write('%10d %s\n' % (refcount, class_.__name__))
-	rcfile.close()
+#def setup_legacy_logging():
+#	gmLog.gmDefLog.SetAllLogLevels(gmLog.lData)
+#==========================================================
+def setup_locale():
+	gmI18N.activate_locale()
+	td = _cfg.get(option = '--text-domain', source_order = [('cli', 'return')])
+	l =  _cfg.get(option = '--lang-gettext', source_order = [('cli', 'return')])
+	gmI18N.install_domain(domain = td, language = l)
 
+	# make sure we re-get the default encoding
+	# in case it changed
+	gmLog2.set_string_encoding()
 #==========================================================
 def check_help_request():
-	if gmCLI.has_arg("--help") or gmCLI.has_arg("-h") or gmCLI.has_arg("-?"):
+	src = [(u'cli', u'return')]
+
+	help_requested = (
+		_cfg.get(option = u'--help', source_order = src) or
+		_cfg.get(option = u'-h', source_order = src) or
+		_cfg.get(option = u'-?', source_order = src)
+	)
+
+	if help_requested:
 		print _(
 			'Help requested\n'
 			'--------------'
 		)
 		print __doc__
 		sys.exit(0)
+#==========================================================
+def setup_paths_and_files():
+	"""Create needed paths in user home directory."""
+
+	gmTools.mkdir(os.path.expanduser(os.path.join('~', '.gnumed', 'scripts')))
+	gmTools.mkdir(os.path.expanduser(os.path.join('~', '.gnumed', 'spellcheck')))
+	gmTools.mkdir(os.path.expanduser(os.path.join('~', '.gnumed', 'tmp')))
+	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed', 'export', 'docs')))
+	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed', 'export', 'xDT')))
+	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed', 'export', 'EMR')))
+	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed', 'xDT')))
+	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed', 'logs')))
+
+	paths = gmTools.gmPaths(app_name = u'gnumed')
+
+	open(os.path.expanduser(os.path.join('~', '.gnumed', 'gnumed.conf')), 'a+').close()
+#==========================================================
+def setup_date_time():
+	gmDateTime.init()
+#==========================================================
+def setup_cfg():
+
+	enc = gmI18N.get_encoding()
+	paths = gmTools.gmPaths(app_name = u'gnumed')
+
+	# the current working dir
+	_cfg.add_file_source (
+		source = u'workbase',
+		file = os.path.join(paths.working_dir, 'gnumed.conf'),
+		encoding = enc
+	)
+	# /etc/gnumed/
+	_cfg.add_file_source (
+		source = u'system',
+		file = os.path.join(paths.system_config_dir, 'gnumed-client.conf'),
+		encoding = enc
+	)
+	# ~/.gnumed/
+	_cfg.add_file_source (
+		source = u'user',
+		file = os.path.join(paths.user_config_dir, 'gnumed.conf'),
+		encoding = enc
+	)
+	# CVS/tgz tree .../gnumed/client/ (IOW a local installation)
+	_cfg.add_file_source (
+		source = u'local',
+		file = os.path.join(paths.local_base_dir, 'gnumed.conf'),
+		encoding = enc
+	)
+	# --conf-file=
+	fname = _cfg.get(option = u'--conf-file', source_order = [(u'cli', u'return')])
+	if fname is None:
+		_cfg.add_file_source(source = u'explicit')
+	else:
+		_cfg.add_file_source (
+			source = u'explicit',
+			file = fname,
+			encoding = enc
+		)
+		if _cfg.source_files['explicit'] is None:
+			sys.exit(missing_config_file % fname)
+
+	found_any_file = False
+	for f in _cfg.source_files.values():
+		if f is not None:
+			found_any_file = True
+			break
+	if not found_any_file:
+		sys.exit('Cannot find any configuration files.')
+
+	# mime type handling sources
+	fname = u'mime_type2file_extension.conf'
+	_cfg.add_file_source (
+		source = u'user-mime',
+		file = os.path.join(paths.user_config_dir, fname),
+		encoding = enc
+	)
+	_cfg.add_file_source (
+		source = u'system-mime',
+		file = os.path.join(paths.system_config_dir, fname),
+		encoding = enc
+	)
+#==========================================================
+def setup_backend():
+	# set up database connection timezone
+	timezone = _cfg.get (
+		group = u'backend',
+		option = 'client timezone',
+		source_order = [
+			('explicit', 'return'),
+			('workbase', 'return'),
+			('local', 'return'),
+			('user', 'return'),
+			('system', 'return')
+		]
+	)
+	if timezone is not None:
+		gmPG2.set_default_client_timezone(timezone)
+#==========================================================
+def shutdown_backend():
+	gmPG2.shutdown()
+#==========================================================
+def shutdown_logging():
+
+	if _cfg.get(option = u'debug'):
+		import types
+
+#		def get_refcounts():
+#			refcount = {}
+#			# collect all classes
+#			for module in sys.modules.values():
+#				for sym in dir(module):
+#					obj = getattr(module, sym)
+#					if type(obj) is types.ClassType:
+#						refcount[obj] = sys.getrefcount(obj)
+#			# sort by refcount
+#			pairs = map(lambda x: (x[1],x[0]), refcount.items())
+#			pairs.sort()
+#			pairs.reverse()
+#			return pairs
+
+#		rcfile = open('./gm-refcount.lst', 'wb')
+#		for refcount, class_ in get_refcounts():
+#			if not class_.__name__.startswith('wx'):
+#				rcfile.write('%10d %s\n' % (refcount, class_.__name__))
+#		rcfile.close()
+
+	# do not choke on Windows
+	logging.raiseExceptions = False
 
 #==========================================================
 # main - launch the GNUmed wxPython GUI client
 #----------------------------------------------------------
-import logging
-logging.raiseExceptions = False
-
-setup_console_exption_handler()
-setup_signal_handlers()
+setup_python_path()
 setup_logging()
 
-from Gnumed.pycommon import gmCLI, gmI18N
+_log.info('Starting up as main module (%s).', __version__)
+_log.info('Python %s on %s (%s)', sys.version, sys.platform, os.name)
 
+setup_console_exception_handler()
+setup_cli()
+setup_signal_handlers()
+
+from Gnumed.pycommon import gmI18N, gmTools, gmDateTime, gmHooks
+
+#setup_legacy_logging()
 setup_locale()
 check_help_request()
-
-_log.Log(gmLog.lInfo, 'Starting up as main module (%s).' % __version__)
-_log.Log(gmLog.lInfo, 'command line is: %s' % str(gmCLI.arg))
-_log.Log(gmLog.lInfo, 'Python %s on %s (%s)' % (sys.version, sys.platform, os.name))
-
-setup_paths()
+setup_paths_and_files()
 setup_date_time()
-#setup_cfg_files()
+setup_cfg()
+
+from Gnumed.pycommon import gmPG2
+
+setup_backend()
 
-# import more of our stuff
-from Gnumed.pycommon import gmGuiBroker, gmHooks
 
 gmHooks.run_hook_script(hook = u'startup-before-GUI')
 
-# now actually run GNUmed
 from Gnumed.wxpython import gmGuiMain
-# do we do profiling ?
-if gmCLI.has_arg('--profile'):
-	profile_file = gmCLI.arg['--profile']
-	_log.Log(gmLog.lInfo, 'writing profiling data into %s' % profile_file)
+profile_file = _cfg.get(option = u'--profile', source_order = [(u'cli', u'return')])
+if profile_file is not None:
+	_log.info('writing profiling data into %s', profile_file)
 	import profile
 	profile.run('gmGuiMain.main()', profile_file)
 else:
@@ -292,14 +411,76 @@ else:
 
 gmHooks.run_hook_script(hook = u'shutdown-post-GUI')
 
-log_object_refcounts()
 
-_log.Log(gmLog.lInfo, 'Normally shutting down as main module.')
+shutdown_backend()
+_log.info('Normally shutting down as main module.')
+shutdown_logging()
 
 #==========================================================
 # $Log: gnumed.py,v $
-# Revision 1.123.2.1  2008/02/21 17:18:45  ncq
-# - silence exception generated by logging bug on windows
+# Revision 1.142.2.1  2008/08/31 14:25:41  ncq
+# - explicitely detect and fail when no config file whatsoever is found
+#
+# Revision 1.142  2008/08/05 12:47:14  ncq
+# - support --local-import
+# - some cleanup
+#
+# Revision 1.141  2008/07/10 21:08:16  ncq
+# - call path detection with app name
+#
+# Revision 1.140  2008/06/11 19:12:13  ncq
+# - add a comment
+#
+# Revision 1.139  2008/05/29 13:32:22  ncq
+# - signal handlers cleanup
+#
+# Revision 1.138  2008/04/13 14:40:17  ncq
+# - no old style logging anymore
+#
+# Revision 1.137  2008/03/06 21:30:49  ncq
+# - comment out legacy logging
+# - shutdown_backend()
+# - shutdown_logging()
+#
+# Revision 1.136  2008/03/06 18:29:30  ncq
+# - standard lib logging only
+#
+# Revision 1.135  2008/02/25 17:43:40  ncq
+# - try suppressing the Python-on-Windows logging bug
+#
+# Revision 1.134  2008/01/30 14:10:40  ncq
+# - cleanup
+#
+# Revision 1.133  2008/01/14 20:45:45  ncq
+# - use set_string_encoding()
+#
+# Revision 1.132  2008/01/05 22:32:33  ncq
+# - setup_backend()
+#
+# Revision 1.131  2007/12/26 21:06:38  ncq
+# - don't use gmTools too early
+#
+# Revision 1.130  2007/12/26 20:59:41  ncq
+# - fix faulty parens
+#
+# Revision 1.129  2007/12/26 20:53:15  ncq
+# - set_option does not take None, so use gmTools.coalesce()
+#
+# Revision 1.128  2007/12/26 20:18:44  ncq
+# - add = to long options where needed
+#
+# Revision 1.127  2007/12/26 12:36:37  ncq
+# - missing :
+# - extra or
+#
+# Revision 1.126  2007/12/23 21:15:26  ncq
+# - add setup_backend()
+#
+# Revision 1.125  2007/12/23 20:56:32  ncq
+# - cleanup++
+#
+# Revision 1.124  2007/12/12 16:26:10  ncq
+# - a whole bunch of cleanup, particularly related to logging
 #
 # Revision 1.123  2007/10/21 20:21:17  ncq
 # - no more mandatory global config file
