@@ -11,21 +11,23 @@ TODO:
 """
 #=============================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gui/gmXdtViewer.py,v $
-# $Id: gmXdtViewer.py,v 1.29 2007/08/29 14:43:43 ncq Exp $
-__version__ = "$Revision: 1.29 $"
+# $Id: gmXdtViewer.py,v 1.37 2008/03/06 18:32:31 ncq Exp $
+__version__ = "$Revision: 1.37 $"
 __author__ = "S.Hilbert, K.Hilbert"
 
-import sys, os, os.path, codecs
+import sys, os, os.path, codecs, logging
+
 
 import wx
 
+
 from Gnumed.wxpython import gmGuiHelpers, gmPlugin
-from Gnumed.pycommon import gmLog, gmI18N, gmDispatcher, gmSignals
+from Gnumed.pycommon import gmI18N, gmDispatcher
 from Gnumed.business import gmXdtMappings, gmXdtObjects
 from Gnumed.wxGladeWidgets import wxgXdtListPnl
 
-_log = gmLog.gmDefLog
-_log.Log(gmLog.lInfo, __version__)
+_log = logging.getLogger('gm.ui')
+_log.info(__version__)
 
 #=============================================================================
 # FIXME: this belongs elsewhere under wxpython/
@@ -84,8 +86,7 @@ class cXdtListPnl(wxgXdtListPnl.wxgXdtListPnl):
 			gmGuiHelpers.gm_show_error (
 				_('Cannot access xDT file\n\n'
 				  ' [%s]'),
-				_('loading xDT file'),
-				gmLog.lErr
+				_('loading xDT file')
 			)
 			return False
 		f.close()
@@ -94,7 +95,7 @@ class cXdtListPnl(wxgXdtListPnl.wxgXdtListPnl):
 		if encoding is None:
 			encoding = 'utf8'
 			gmDispatcher.send(signal = 'statustext', msg = _('Encoding missing in xDT file. Assuming [%s].') % encoding)
-			_log.Log(gmLog.lWarn, 'xDT file [%s] does not define an encoding, assuming [%s]' % (filename, encoding))
+			_log.warning('xDT file [%s] does not define an encoding, assuming [%s]' % (filename, encoding))
 
 		try:
 			xdt_file = codecs.open(filename=filename, mode='rU', encoding=encoding, errors='replace')
@@ -102,8 +103,7 @@ class cXdtListPnl(wxgXdtListPnl.wxgXdtListPnl):
 			gmGuiHelpers.gm_show_error (
 				_('Cannot access xDT file\n\n'
 				  ' [%s]'),
-				_('loading xDT file'),
-				gmLog.lErr
+				_('loading xDT file')
 			)
 			return False
 
@@ -229,7 +229,7 @@ class gmXdtViewerPanel(wx.Panel):
 	#-------------------------------------------------------------------------
 	def __decode_xdt(self):
 		if self.filename is None:
-			_log.Log(gmLog.lErr, "Need name of file to parse !")
+			_log.error("Need name of file to parse !")
 			return None
 
 		xDTFile = fileinput.input(self.filename)
@@ -357,7 +357,7 @@ class gmXdtViewerPanel(wx.Panel):
 class gmXdtViewer(gmPlugin.cNotebookPlugin):
 	"""Plugin to encapsulate xDT list-in-panel viewer"""
 
-	tab_name = _('xDT Viewer')
+	tab_name = _('xDT viewer')
 
 	def name(self):
 		return gmXdtViewer.tab_name
@@ -375,7 +375,10 @@ class gmXdtViewer(gmPlugin.cNotebookPlugin):
 # main
 #------------------------------------------------------
 if __name__ == '__main__':
-	from Gnumed.pycommon import gmCLI
+	from Gnumed.pycommon import gmCfg2
+
+	cfg = gmCfg2.gmCfgData()
+	cfg.add_cli(long_options=['xdt-file='])
 	#---------------------
 	# set up dummy app
 	class TestApp (wx.App):
@@ -383,21 +386,21 @@ if __name__ == '__main__':
 
 			fname = ""
 			# has the user manually supplied a config file on the command line ?
-			if gmCLI.has_arg('--xdt-file'):
-				fname = gmCLI.arg['--xdt-file']
-				_log.Log(gmLog.lData, 'XDT file is [%s]' % fname)
+			fname = cfg.get(option = '--xdt-file', source_order = [('cli', 'return')])
+			if fname is not None:
+				_log.debug('XDT file is [%s]' % fname)
 				# file valid ?
 				if not os.access(fname, os.R_OK):
 					title = _('Opening xDT file')
 					msg = _('Cannot open xDT file.\n'
 							'[%s]') % fname
-					gmGuiHelpers.gm_show_error(msg, title, gmLog.lErr)
+					gmGuiHelpers.gm_show_error(msg, title)
 					return False
 			else:
 				title = _('Opening xDT file')
 				msg = _('You must provide an xDT file on the command line.\n'
 						'Format: --xdt-file=<file>')
-				gmGuiHelpers.gm_show_error(msg, title, gmLog.lWarn)
+				gmGuiHelpers.gm_show_error(msg, title)
 				return False
 
 			frame = wx.Frame(
@@ -415,11 +418,36 @@ if __name__ == '__main__':
 		app = TestApp ()
 		app.MainLoop ()
 	except StandardError:
-		_log.LogException('Unhandled exception.', sys.exc_info(), verbose=1)
+		_log.exception('Unhandled exception.')
 		raise
 
 #=============================================================================
 # $Log: gmXdtViewer.py,v $
+# Revision 1.37  2008/03/06 18:32:31  ncq
+# - standard lib logging only
+#
+# Revision 1.36  2008/01/30 14:03:42  ncq
+# - use signal names directly
+# - switch to std lib logging
+#
+# Revision 1.35  2008/01/28 16:15:50  ncq
+# - fix typo
+#
+# Revision 1.34  2008/01/22 12:26:24  ncq
+# - better tab names
+#
+# Revision 1.33  2008/01/05 16:42:02  ncq
+# - no more logging in gm_show_*()
+#
+# Revision 1.32  2007/12/26 20:19:34  ncq
+# - add = to long options
+#
+# Revision 1.31  2007/12/26 14:35:51  ncq
+# - move to gmLog2/gmCfg2
+#
+# Revision 1.30  2007/12/11 12:49:26  ncq
+# - explicit signal handling
+#
 # Revision 1.29  2007/08/29 14:43:43  ncq
 # - show xDT field ID in third column to enable more comfortable debugging
 #
